@@ -13,18 +13,18 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from ironic_lib import metrics_utils
-import pecan
-from pecan import rest
-from six.moves import http_client
-import wsme
-from wsme import types as wtypes
+from http import client as http_client
 
+from ironic_lib import metrics_utils
+from pecan import rest
+
+from ironic import api
 from ironic.api.controllers import base
 from ironic.api.controllers import link
 from ironic.api.controllers.v1 import types
 from ironic.api.controllers.v1 import utils as api_utils
 from ironic.api import expose
+from ironic.api import types as atypes
 from ironic.common import exception
 from ironic.common.i18n import _
 from ironic.common import policy
@@ -72,63 +72,63 @@ def hide_fields_in_newer_versions(obj):
     matches or exceeds the versions when these fields were introduced.
     """
     if not api_utils.allow_storage_interface():
-        obj.default_storage_interface = wsme.Unset
-        obj.enabled_storage_interfaces = wsme.Unset
+        obj.default_storage_interface = atypes.Unset
+        obj.enabled_storage_interfaces = atypes.Unset
 
     if not api_utils.allow_rescue_interface():
-        obj.default_rescue_interface = wsme.Unset
-        obj.enabled_rescue_interfaces = wsme.Unset
+        obj.default_rescue_interface = atypes.Unset
+        obj.enabled_rescue_interfaces = atypes.Unset
 
     if not api_utils.allow_bios_interface():
-        obj.default_bios_interface = wsme.Unset
-        obj.enabled_bios_interfaces = wsme.Unset
+        obj.default_bios_interface = atypes.Unset
+        obj.enabled_bios_interfaces = atypes.Unset
 
 
-class Driver(base.APIBase):
+class Driver(base.Base):
     """API representation of a driver."""
 
-    name = wtypes.text
+    name = str
     """The name of the driver"""
 
-    hosts = [wtypes.text]
+    hosts = [str]
     """A list of active conductors that support this driver"""
 
-    type = wtypes.text
+    type = str
     """Whether the driver is classic or dynamic (hardware type)"""
 
-    links = wsme.wsattr([link.Link], readonly=True)
+    links = None
     """A list containing self and bookmark links"""
 
-    properties = wsme.wsattr([link.Link], readonly=True)
+    properties = None
     """A list containing links to driver properties"""
 
     """Default interface for a hardware type"""
-    default_bios_interface = wtypes.text
-    default_boot_interface = wtypes.text
-    default_console_interface = wtypes.text
-    default_deploy_interface = wtypes.text
-    default_inspect_interface = wtypes.text
-    default_management_interface = wtypes.text
-    default_network_interface = wtypes.text
-    default_power_interface = wtypes.text
-    default_raid_interface = wtypes.text
-    default_rescue_interface = wtypes.text
-    default_storage_interface = wtypes.text
-    default_vendor_interface = wtypes.text
+    default_bios_interface = str
+    default_boot_interface = str
+    default_console_interface = str
+    default_deploy_interface = str
+    default_inspect_interface = str
+    default_management_interface = str
+    default_network_interface = str
+    default_power_interface = str
+    default_raid_interface = str
+    default_rescue_interface = str
+    default_storage_interface = str
+    default_vendor_interface = str
 
     """A list of enabled interfaces for a hardware type"""
-    enabled_bios_interfaces = [wtypes.text]
-    enabled_boot_interfaces = [wtypes.text]
-    enabled_console_interfaces = [wtypes.text]
-    enabled_deploy_interfaces = [wtypes.text]
-    enabled_inspect_interfaces = [wtypes.text]
-    enabled_management_interfaces = [wtypes.text]
-    enabled_network_interfaces = [wtypes.text]
-    enabled_power_interfaces = [wtypes.text]
-    enabled_raid_interfaces = [wtypes.text]
-    enabled_rescue_interfaces = [wtypes.text]
-    enabled_storage_interfaces = [wtypes.text]
-    enabled_vendor_interfaces = [wtypes.text]
+    enabled_bios_interfaces = [str]
+    enabled_boot_interfaces = [str]
+    enabled_console_interfaces = [str]
+    enabled_deploy_interfaces = [str]
+    enabled_inspect_interfaces = [str]
+    enabled_management_interfaces = [str]
+    enabled_network_interfaces = [str]
+    enabled_power_interfaces = [str]
+    enabled_raid_interfaces = [str]
+    enabled_rescue_interfaces = [str]
+    enabled_storage_interfaces = [str]
+    enabled_vendor_interfaces = [str]
 
     @staticmethod
     def convert_with_links(name, hosts, detail=False, interface_info=None):
@@ -146,23 +146,23 @@ class Driver(base.APIBase):
         driver.name = name
         driver.hosts = hosts
         driver.links = [
-            link.Link.make_link('self',
-                                pecan.request.public_url,
-                                'drivers', name),
-            link.Link.make_link('bookmark',
-                                pecan.request.public_url,
-                                'drivers', name,
-                                bookmark=True)
+            link.make_link('self',
+                           api.request.public_url,
+                           'drivers', name),
+            link.make_link('bookmark',
+                           api.request.public_url,
+                           'drivers', name,
+                           bookmark=True)
         ]
         if api_utils.allow_links_node_states_and_driver_properties():
             driver.properties = [
-                link.Link.make_link('self',
-                                    pecan.request.public_url,
-                                    'drivers', name + "/properties"),
-                link.Link.make_link('bookmark',
-                                    pecan.request.public_url,
-                                    'drivers', name + "/properties",
-                                    bookmark=True)
+                link.make_link('self',
+                               api.request.public_url,
+                               'drivers', name + "/properties"),
+                link.make_link('bookmark',
+                               api.request.public_url,
+                               'drivers', name + "/properties",
+                               bookmark=True)
             ]
 
         if api_utils.allow_dynamic_drivers():
@@ -172,7 +172,7 @@ class Driver(base.APIBase):
             if detail:
                 if interface_info is None:
                     # TODO(jroll) objectify this
-                    interface_info = (pecan.request.dbapi
+                    interface_info = (api.request.dbapi
                                       .list_hardware_type_interfaces([name]))
                 for iface_type in driver_base.ALL_INTERFACES:
                     default = None
@@ -209,7 +209,7 @@ class Driver(base.APIBase):
         return sample
 
 
-class DriverList(base.APIBase):
+class DriverList(base.Base):
     """API representation of a list of drivers."""
 
     drivers = [Driver]
@@ -233,7 +233,7 @@ class DriverList(base.APIBase):
         # This is checked in Driver.convert_with_links(), however also
         # checking here can save us a DB query.
         if api_utils.allow_dynamic_drivers() and detail:
-            iface_info = pecan.request.dbapi.list_hardware_type_interfaces(
+            iface_info = api.request.dbapi.list_hardware_type_interfaces(
                 list(hardware_types))
         else:
             iface_info = []
@@ -268,7 +268,7 @@ class DriverPassthruController(rest.RestController):
     }
 
     @METRICS.timer('DriverPassthruController.methods')
-    @expose.expose(wtypes.text, wtypes.text)
+    @expose.expose(str, str)
     def methods(self, driver_name):
         """Retrieve information about vendor methods of the given driver.
 
@@ -278,20 +278,20 @@ class DriverPassthruController(rest.RestController):
         :raises: DriverNotFound if the driver name is invalid or the
                  driver cannot be loaded.
         """
-        cdict = pecan.request.context.to_policy_values()
+        cdict = api.request.context.to_policy_values()
         policy.authorize('baremetal:driver:vendor_passthru', cdict, cdict)
 
         if driver_name not in _VENDOR_METHODS:
-            topic = pecan.request.rpcapi.get_topic_for_driver(driver_name)
-            ret = pecan.request.rpcapi.get_driver_vendor_passthru_methods(
-                pecan.request.context, driver_name, topic=topic)
+            topic = api.request.rpcapi.get_topic_for_driver(driver_name)
+            ret = api.request.rpcapi.get_driver_vendor_passthru_methods(
+                api.request.context, driver_name, topic=topic)
             _VENDOR_METHODS[driver_name] = ret
 
         return _VENDOR_METHODS[driver_name]
 
     @METRICS.timer('DriverPassthruController._default')
-    @expose.expose(wtypes.text, wtypes.text, wtypes.text,
-                   body=wtypes.text)
+    @expose.expose(str, str, str,
+                   body=str)
     def _default(self, driver_name, method, data=None):
         """Call a driver API extension.
 
@@ -300,10 +300,10 @@ class DriverPassthruController(rest.RestController):
                        implementation.
         :param data: body of data to supply to the specified method.
         """
-        cdict = pecan.request.context.to_policy_values()
+        cdict = api.request.context.to_policy_values()
         policy.authorize('baremetal:driver:vendor_passthru', cdict, cdict)
 
-        topic = pecan.request.rpcapi.get_topic_for_driver(driver_name)
+        topic = api.request.rpcapi.get_topic_for_driver(driver_name)
         return api_utils.vendor_passthru(driver_name, method, topic, data=data,
                                          driver_passthru=True)
 
@@ -315,7 +315,7 @@ class DriverRaidController(rest.RestController):
     }
 
     @METRICS.timer('DriverRaidController.logical_disk_properties')
-    @expose.expose(types.jsontype, wtypes.text)
+    @expose.expose(types.jsontype, str)
     def logical_disk_properties(self, driver_name):
         """Returns the logical disk properties for the driver.
 
@@ -329,7 +329,7 @@ class DriverRaidController(rest.RestController):
         :raises: DriverNotFound, if driver is not loaded on any of the
             conductors.
         """
-        cdict = pecan.request.context.to_policy_values()
+        cdict = api.request.context.to_policy_values()
         policy.authorize('baremetal:driver:get_raid_logical_disk_properties',
                          cdict, cdict)
 
@@ -337,10 +337,10 @@ class DriverRaidController(rest.RestController):
             raise exception.NotAcceptable()
 
         if driver_name not in _RAID_PROPERTIES:
-            topic = pecan.request.rpcapi.get_topic_for_driver(driver_name)
+            topic = api.request.rpcapi.get_topic_for_driver(driver_name)
             try:
-                info = pecan.request.rpcapi.get_raid_logical_disk_properties(
-                    pecan.request.context, driver_name, topic=topic)
+                info = api.request.rpcapi.get_raid_logical_disk_properties(
+                    api.request.context, driver_name, topic=topic)
             except exception.UnsupportedDriverExtension as e:
                 # Change error code as 404 seems appropriate because RAID is a
                 # standard interface and all drivers might not have it.
@@ -364,14 +364,14 @@ class DriversController(rest.RestController):
     }
 
     @METRICS.timer('DriversController.get_all')
-    @expose.expose(DriverList, wtypes.text, types.boolean)
+    @expose.expose(DriverList, str, types.boolean)
     def get_all(self, type=None, detail=None):
         """Retrieve a list of drivers."""
-        # FIXME(deva): formatting of the auto-generated REST API docs
+        # FIXME(tenbrae): formatting of the auto-generated REST API docs
         #              will break from a single-line doc string.
         #              This is a result of a bug in sphinxcontrib-pecanwsme
         # https://github.com/dreamhost/sphinxcontrib-pecanwsme/issues/8
-        cdict = pecan.request.context.to_policy_values()
+        cdict = api.request.context.to_policy_values()
         policy.authorize('baremetal:driver:get', cdict, cdict)
 
         api_utils.check_allow_driver_detail(detail)
@@ -382,7 +382,7 @@ class DriversController(rest.RestController):
                 'if specified.'))
 
         if type is None or type == 'dynamic':
-            hw_type_dict = pecan.request.dbapi.get_active_hardware_type_dict()
+            hw_type_dict = api.request.dbapi.get_active_hardware_type_dict()
         else:
             # NOTE(dtantsur): we don't support classic drivers starting with
             # the Rocky release.
@@ -390,17 +390,17 @@ class DriversController(rest.RestController):
         return DriverList.convert_with_links(hw_type_dict, detail=detail)
 
     @METRICS.timer('DriversController.get_one')
-    @expose.expose(Driver, wtypes.text)
+    @expose.expose(Driver, str)
     def get_one(self, driver_name):
         """Retrieve a single driver."""
         # NOTE(russell_h): There is no way to make this more efficient than
         # retrieving a list of drivers using the current sqlalchemy schema, but
         # this path must be exposed for Pecan to route any paths we might
         # choose to expose below it.
-        cdict = pecan.request.context.to_policy_values()
+        cdict = api.request.context.to_policy_values()
         policy.authorize('baremetal:driver:get', cdict, cdict)
 
-        hw_type_dict = pecan.request.dbapi.get_active_hardware_type_dict()
+        hw_type_dict = api.request.dbapi.get_active_hardware_type_dict()
         for name, hosts in hw_type_dict.items():
             if name == driver_name:
                 return Driver.convert_with_links(name, list(hosts),
@@ -409,7 +409,7 @@ class DriversController(rest.RestController):
         raise exception.DriverNotFound(driver_name=driver_name)
 
     @METRICS.timer('DriversController.properties')
-    @expose.expose(wtypes.text, wtypes.text)
+    @expose.expose(str, str)
     def properties(self, driver_name):
         """Retrieve property information of the given driver.
 
@@ -419,13 +419,13 @@ class DriversController(rest.RestController):
         :raises: DriverNotFound (HTTP 404) if the driver name is invalid or
                  the driver cannot be loaded.
         """
-        cdict = pecan.request.context.to_policy_values()
+        cdict = api.request.context.to_policy_values()
         policy.authorize('baremetal:driver:get_properties', cdict, cdict)
 
         if driver_name not in _DRIVER_PROPERTIES:
-            topic = pecan.request.rpcapi.get_topic_for_driver(driver_name)
-            properties = pecan.request.rpcapi.get_driver_properties(
-                pecan.request.context, driver_name, topic=topic)
+            topic = api.request.rpcapi.get_topic_for_driver(driver_name)
+            properties = api.request.rpcapi.get_driver_properties(
+                api.request.context, driver_name, topic=topic)
             _DRIVER_PROPERTIES[driver_name] = properties
 
         return _DRIVER_PROPERTIES[driver_name]
