@@ -14,16 +14,14 @@
 #    under the License.
 
 import datetime
+from http import client as http_client
 
 from ironic_lib import metrics_utils
 from oslo_log import log
 from oslo_utils import uuidutils
-import pecan
 from pecan import rest
-from six.moves import http_client
-import wsme
-from wsme import types as wtypes
 
+from ironic import api
 from ironic.api.controllers import base
 from ironic.api.controllers import link
 from ironic.api.controllers.v1 import collection
@@ -31,6 +29,7 @@ from ironic.api.controllers.v1 import notification_utils as notify
 from ironic.api.controllers.v1 import types
 from ironic.api.controllers.v1 import utils as api_utils
 from ironic.api import expose
+from ironic.api import types as atypes
 from ironic.common import exception
 from ironic.common.i18n import _
 from ironic.common import policy
@@ -47,18 +46,21 @@ _DEFAULT_RETURN_FIELDS = ('uuid', 'address')
 def hide_fields_in_newer_versions(obj):
     # if requested version is < 1.18, hide internal_info field
     if not api_utils.allow_port_internal_info():
-        obj.internal_info = wsme.Unset
+        obj.internal_info = atypes.Unset
     # if requested version is < 1.19, hide local_link_connection and
     # pxe_enabled fields
     if not api_utils.allow_port_advanced_net_fields():
-        obj.pxe_enabled = wsme.Unset
-        obj.local_link_connection = wsme.Unset
+        obj.pxe_enabled = atypes.Unset
+        obj.local_link_connection = atypes.Unset
     # if requested version is < 1.24, hide portgroup_uuid field
     if not api_utils.allow_portgroups_subcontrollers():
-        obj.portgroup_uuid = wsme.Unset
+        obj.portgroup_uuid = atypes.Unset
     # if requested version is < 1.34, hide physical_network field.
     if not api_utils.allow_port_physical_network():
-        obj.physical_network = wsme.Unset
+        obj.physical_network = atypes.Unset
+    # if requested version is < 1.53, hide is_smartnic field.
+    if not api_utils.allow_port_is_smartnic():
+        obj.is_smartnic = atypes.Unset
 
 
 class Port(base.APIBase):
@@ -80,7 +82,7 @@ class Port(base.APIBase):
                 # FIXME(comstud): One should only allow UUID here, but
                 # there seems to be a bug in that tests are passing an
                 # ID. See bug #1301046 for more details.
-                node = objects.Node.get(pecan.request.context, value)
+                node = objects.Node.get(api.request.context, value)
                 self._node_uuid = node.uuid
                 # NOTE(lucasagomes): Create the node_id attribute on-the-fly
                 #                    to satisfy the api -> rpc object
@@ -91,8 +93,8 @@ class Port(base.APIBase):
                 # response for a POST request to create a Port
                 e.code = http_client.BAD_REQUEST  # BadRequest
                 raise
-        elif value == wtypes.Unset:
-            self._node_uuid = wtypes.Unset
+        elif value == atypes.Unset:
+            self._node_uuid = atypes.Unset
 
     def _get_portgroup_uuid(self):
         return self._portgroup_uuid
@@ -100,10 +102,10 @@ class Port(base.APIBase):
     def _set_portgroup_uuid(self, value):
         if value and self._portgroup_uuid != value:
             if not api_utils.allow_portgroups_subcontrollers():
-                self._portgroup_uuid = wtypes.Unset
+                self._portgroup_uuid = atypes.Unset
                 return
             try:
-                portgroup = objects.Portgroup.get(pecan.request.context, value)
+                portgroup = objects.Portgroup.get(api.request.context, value)
                 if portgroup.node_id != self.node_id:
                     raise exception.BadRequest(_('Port can not be added to a '
                                                  'portgroup belonging to a '
@@ -118,8 +120,8 @@ class Port(base.APIBase):
                 # response for a POST request to create a Port
                 e.code = http_client.BAD_REQUEST  # BadRequest
                 raise e
-        elif value == wtypes.Unset:
-            self._portgroup_uuid = wtypes.Unset
+        elif value == atypes.Unset:
+            self._portgroup_uuid = atypes.Unset
         elif value is None and api_utils.allow_portgroups_subcontrollers():
             # This is to output portgroup_uuid field if API version allows this
             self._portgroup_uuid = None
@@ -127,21 +129,21 @@ class Port(base.APIBase):
     uuid = types.uuid
     """Unique UUID for this port"""
 
-    address = wsme.wsattr(types.macaddress, mandatory=True)
+    address = atypes.wsattr(types.macaddress, mandatory=True)
     """MAC Address for this port"""
 
-    extra = {wtypes.text: types.jsontype}
+    extra = {str: types.jsontype}
     """This port's meta data"""
 
-    internal_info = wsme.wsattr({wtypes.text: types.jsontype}, readonly=True)
+    internal_info = atypes.wsattr({str: types.jsontype}, readonly=True)
     """This port's internal information maintained by ironic"""
 
-    node_uuid = wsme.wsproperty(types.uuid, _get_node_uuid, _set_node_uuid,
-                                mandatory=True)
+    node_uuid = atypes.wsproperty(types.uuid, _get_node_uuid, _set_node_uuid,
+                                  mandatory=True)
     """The UUID of the node this port belongs to"""
 
-    portgroup_uuid = wsme.wsproperty(types.uuid, _get_portgroup_uuid,
-                                     _set_portgroup_uuid, mandatory=False)
+    portgroup_uuid = atypes.wsproperty(types.uuid, _get_portgroup_uuid,
+                                       _set_portgroup_uuid, mandatory=False)
     """The UUID of the portgroup this port belongs to"""
 
     pxe_enabled = types.boolean
@@ -150,12 +152,15 @@ class Port(base.APIBase):
     local_link_connection = types.locallinkconnectiontype
     """The port binding profile for the port"""
 
-    physical_network = wtypes.StringType(max_length=64)
+    physical_network = atypes.StringType(max_length=64)
     """The name of the physical network to which this port is connected."""
 
-    links = wsme.wsattr([link.Link], readonly=True)
+    links = None
     """A list containing a self link and associated port links"""
 
+    is_smartnic = types.boolean
+    """Indicates whether this port is a Smart NIC port."""
+
     def __init__(self, **kwargs):
         self.fields = []
         fields = list(objects.Port.fields)
@@ -169,14 +174,14 @@ class Port(base.APIBase):
             # Add fields we expose.
             if hasattr(self, field):
                 self.fields.append(field)
-                setattr(self, field, kwargs.get(field, wtypes.Unset))
+                setattr(self, field, kwargs.get(field, atypes.Unset))
 
         # NOTE(lucasagomes): node_id is an attribute created on-the-fly
         # by _set_node_uuid(), it needs to be present in the fields so
         # that as_dict() will contain node_id field when converting it
         # before saving it in the database.
         self.fields.append('node_id')
-        setattr(self, 'node_uuid', kwargs.get('node_id', wtypes.Unset))
+        setattr(self, 'node_uuid', kwargs.get('node_id', atypes.Unset))
 
         # NOTE: portgroup_id is an attribute created on-the-fly
         # by _set_portgroup_uuid(), it needs to be present in the fields so
@@ -184,42 +189,53 @@ class Port(base.APIBase):
         # before saving it in the database.
         self.fields.append('portgroup_id')
         setattr(self, 'portgroup_uuid', kwargs.get('portgroup_id',
-                                                   wtypes.Unset))
-
-    @staticmethod
-    def _convert_with_links(port, url, fields=None):
-        # NOTE(lucasagomes): Since we are able to return a specified set of
-        # fields the "uuid" can be unset, so we need to save it in another
-        # variable to use when building the links
-        port_uuid = port.uuid
-        if fields is not None:
-            port.unset_fields_except(fields)
-
-        # never expose the node_id attribute
-        port.node_id = wtypes.Unset
-
-        # never expose the portgroup_id attribute
-        port.portgroup_id = wtypes.Unset
-
-        port.links = [link.Link.make_link('self', url,
-                                          'ports', port_uuid),
-                      link.Link.make_link('bookmark', url,
-                                          'ports', port_uuid,
-                                          bookmark=True)
-                      ]
-        return port
+                                                   atypes.Unset))
 
     @classmethod
-    def convert_with_links(cls, rpc_port, fields=None):
+    def convert_with_links(cls, rpc_port, fields=None, sanitize=True):
         port = Port(**rpc_port.as_dict())
 
+        port._validate_fields(fields)
+
+        url = api.request.public_url
+
+        port.links = [link.make_link('self', url,
+                                     'ports', port.uuid),
+                      link.make_link('bookmark', url,
+                                     'ports', port.uuid,
+                                     bookmark=True)
+                      ]
+
+        if not sanitize:
+            return port
+
+        port.sanitize(fields=fields)
+
+        return port
+
+    def _validate_fields(self, fields=None):
         if fields is not None:
-            api_utils.check_for_invalid_fields(fields, port.as_dict())
+            api_utils.check_for_invalid_fields(fields, self.as_dict())
 
-        hide_fields_in_newer_versions(port)
+    def sanitize(self, fields=None):
+        """Removes sensitive and unrequested data.
 
-        return cls._convert_with_links(port, pecan.request.public_url,
-                                       fields=fields)
+        Will only keep the fields specified in the ``fields`` parameter.
+
+        :param fields:
+            list of fields to preserve, or ``None`` to preserve them all
+        :type fields: list of str
+        """
+        hide_fields_in_newer_versions(self)
+
+        if fields is not None:
+            self.unset_fields_except(fields)
+
+        # never expose the node_id attribute
+        self.node_id = atypes.Unset
+
+        # never expose the portgroup_id attribute
+        self.portgroup_id = atypes.Unset
 
     @classmethod
     def sample(cls, expand=True):
@@ -234,7 +250,8 @@ class Port(base.APIBase):
                      local_link_connection={
                          'switch_info': 'host', 'port_id': 'Gig0/1',
                          'switch_id': 'aa:bb:cc:dd:ee:ff'},
-                     physical_network='physnet1')
+                     physical_network='physnet1',
+                     is_smartnic=False)
         # NOTE(lucasagomes): node_uuid getter() method look at the
         # _node_uuid variable
         sample._node_uuid = '7ae81bb3-dec3-4289-8d6c-da80bd8001ae'
@@ -268,7 +285,8 @@ class PortCollection(collection.Collection):
         collection.ports = []
         for rpc_port in rpc_ports:
             try:
-                port = Port.convert_with_links(rpc_port, fields=fields)
+                port = Port.convert_with_links(rpc_port, fields=fields,
+                                               sanitize=False)
             except exception.NodeNotFound:
                 # NOTE(dtantsur): node was deleted after we fetched the port
                 # list, meaning that the port was also deleted. Skip it.
@@ -282,11 +300,17 @@ class PortCollection(collection.Collection):
                 LOG.debug('Removing port group UUID from port %s as the port '
                           'group was deleted', rpc_port.uuid)
                 rpc_port.portgroup_id = None
-                port = Port.convert_with_links(rpc_port, fields=fields)
+                port = Port.convert_with_links(rpc_port, fields=fields,
+                                               sanitize=False)
 
             collection.ports.append(port)
 
-        collection.next = collection.get_next(limit, url=url, **kwargs)
+        collection.next = collection.get_next(limit, url=url, fields=fields,
+                                              **kwargs)
+
+        for item in collection.ports:
+            item.sanitize(fields=fields)
+
         return collection
 
     @classmethod
@@ -314,14 +338,38 @@ class PortsController(rest.RestController):
 
     def _get_ports_collection(self, node_ident, address, portgroup_ident,
                               marker, limit, sort_key, sort_dir,
-                              resource_url=None, fields=None, detail=None):
+                              resource_url=None, fields=None, detail=None,
+                              project=None):
+        """Retrieve a collection of ports.
+
+        :param node_ident: UUID or name of a node, to get only ports for that
+                           node.
+        :param address: MAC address of a port, to get the port which has
+                        this MAC address.
+        :param portgroup_ident: UUID or name of a portgroup, to get only ports
+                                for that portgroup.
+        :param marker: pagination marker for large data sets.
+        :param limit: maximum number of resources to return in a single result.
+                      This value cannot be larger than the value of max_limit
+                      in the [api] section of the ironic configuration, or only
+                      max_limit resources will be returned.
+        :param sort_key: column to sort results by. Default: id.
+        :param sort_dir: direction to sort. "asc" or "desc". Default: asc.
+        :param resource_url: Optional, base url to be used for links
+        :param fields: Optional, a list with a specified set of fields
+            of the resource to be returned.
+        :param detail: Optional, show detailed list of ports
+        :param project: Optional, filter by project
+        :returns: a list of ports.
+
+        """
 
         limit = api_utils.validate_limit(limit)
         sort_dir = api_utils.validate_sort_dir(sort_dir)
 
         marker_obj = None
         if marker:
-            marker_obj = objects.Port.get_by_uuid(pecan.request.context,
+            marker_obj = objects.Port.get_by_uuid(api.request.context,
                                                   marker)
 
         if sort_key in self.invalid_sort_key_list:
@@ -341,27 +389,29 @@ class PortsController(rest.RestController):
             #                 for that column. This will get cleaned up
             #                 as we move to the object interface.
             portgroup = api_utils.get_rpc_portgroup(portgroup_ident)
-            ports = objects.Port.list_by_portgroup_id(pecan.request.context,
+            ports = objects.Port.list_by_portgroup_id(api.request.context,
                                                       portgroup.id, limit,
                                                       marker_obj,
                                                       sort_key=sort_key,
-                                                      sort_dir=sort_dir)
+                                                      sort_dir=sort_dir,
+                                                      project=project)
         elif node_ident:
             # FIXME(comstud): Since all we need is the node ID, we can
             #                 make this more efficient by only querying
             #                 for that column. This will get cleaned up
             #                 as we move to the object interface.
             node = api_utils.get_rpc_node(node_ident)
-            ports = objects.Port.list_by_node_id(pecan.request.context,
+            ports = objects.Port.list_by_node_id(api.request.context,
                                                  node.id, limit, marker_obj,
                                                  sort_key=sort_key,
-                                                 sort_dir=sort_dir)
+                                                 sort_dir=sort_dir,
+                                                 project=project)
         elif address:
-            ports = self._get_ports_by_address(address)
+            ports = self._get_ports_by_address(address, project=project)
         else:
-            ports = objects.Port.list(pecan.request.context, limit,
+            ports = objects.Port.list(api.request.context, limit,
                                       marker_obj, sort_key=sort_key,
-                                      sort_dir=sort_dir)
+                                      sort_dir=sort_dir, project=project)
         parameters = {}
 
         if detail is not None:
@@ -374,16 +424,18 @@ class PortsController(rest.RestController):
                                                  sort_dir=sort_dir,
                                                  **parameters)
 
-    def _get_ports_by_address(self, address):
+    def _get_ports_by_address(self, address, project=None):
         """Retrieve a port by its address.
 
         :param address: MAC address of a port, to get the port which has
                         this MAC address.
+        :param project: Optional, filter by project
         :returns: a list with the port, or an empty list if no port is found.
 
         """
         try:
-            port = objects.Port.get_by_address(pecan.request.context, address)
+            port = objects.Port.get_by_address(api.request.context, address,
+                                               project=project)
             return [port]
         except exception.PortNotFound:
             return []
@@ -408,11 +460,22 @@ class PortsController(rest.RestController):
         if ('physical_network' in fields
                 and not api_utils.allow_port_physical_network()):
             raise exception.NotAcceptable()
+        if ('is_smartnic' in fields
+                and not api_utils.allow_port_is_smartnic()):
+            raise exception.NotAcceptable()
+        if ('local_link_connection/network_type' in fields
+                and not api_utils.allow_local_link_connection_network_type()):
+            raise exception.NotAcceptable()
+        if (isinstance(fields, dict)
+                and fields.get('local_link_connection') is not None):
+            if (not api_utils.allow_local_link_connection_network_type()
+                    and 'network_type' in fields['local_link_connection']):
+                raise exception.NotAcceptable()
 
     @METRICS.timer('PortsController.get_all')
     @expose.expose(PortCollection, types.uuid_or_name, types.uuid,
-                   types.macaddress, types.uuid, int, wtypes.text,
-                   wtypes.text, types.listtype, types.uuid_or_name,
+                   types.macaddress, types.uuid, int, str,
+                   str, types.listtype, types.uuid_or_name,
                    types.boolean)
     def get_all(self, node=None, node_uuid=None, address=None, marker=None,
                 limit=None, sort_key='id', sort_dir='asc', fields=None,
@@ -441,8 +504,7 @@ class PortsController(rest.RestController):
                                    for that portgroup.
         :raises: NotAcceptable, HTTPNotFound
         """
-        cdict = pecan.request.context.to_policy_values()
-        policy.authorize('baremetal:port:get', cdict, cdict)
+        project = api_utils.check_port_list_policy()
 
         api_utils.check_allow_specify_fields(fields)
         self._check_allowed_port_fields(fields)
@@ -465,12 +527,12 @@ class PortsController(rest.RestController):
         return self._get_ports_collection(node_uuid or node, address,
                                           portgroup, marker, limit, sort_key,
                                           sort_dir, fields=fields,
-                                          detail=detail)
+                                          detail=detail, project=project)
 
     @METRICS.timer('PortsController.detail')
     @expose.expose(PortCollection, types.uuid_or_name, types.uuid,
-                   types.macaddress, types.uuid, int, wtypes.text,
-                   wtypes.text, types.uuid_or_name)
+                   types.macaddress, types.uuid, int, str,
+                   str, types.uuid_or_name)
     def detail(self, node=None, node_uuid=None, address=None, marker=None,
                limit=None, sort_key='id', sort_dir='asc', portgroup=None):
         """Retrieve a list of ports with detail.
@@ -495,8 +557,7 @@ class PortsController(rest.RestController):
         :param sort_dir: direction to sort. "asc" or "desc". Default: asc.
         :raises: NotAcceptable, HTTPNotFound
         """
-        cdict = pecan.request.context.to_policy_values()
-        policy.authorize('baremetal:port:get', cdict, cdict)
+        project = api_utils.check_port_list_policy()
 
         self._check_allowed_port_fields([sort_key])
         if portgroup and not api_utils.allow_portgroups_subcontrollers():
@@ -511,14 +572,15 @@ class PortsController(rest.RestController):
                 raise exception.NotAcceptable()
 
         # NOTE(lucasagomes): /detail should only work against collections
-        parent = pecan.request.path.split('/')[:-1][-1]
+        parent = api.request.path.split('/')[:-1][-1]
         if parent != "ports":
             raise exception.HTTPNotFound()
 
         resource_url = '/'.join(['ports', 'detail'])
         return self._get_ports_collection(node_uuid or node, address,
                                           portgroup, marker, limit, sort_key,
-                                          sort_dir, resource_url)
+                                          sort_dir, resource_url,
+                                          project=project)
 
     @METRICS.timer('PortsController.get_one')
     @expose.expose(Port, types.uuid, types.listtype)
@@ -530,16 +592,15 @@ class PortsController(rest.RestController):
             of the resource to be returned.
         :raises: NotAcceptable, HTTPNotFound
         """
-        cdict = pecan.request.context.to_policy_values()
-        policy.authorize('baremetal:port:get', cdict, cdict)
-
         if self.parent_node_ident or self.parent_portgroup_ident:
             raise exception.OperationNotPermitted()
 
+        rpc_port, rpc_node = api_utils.check_port_policy_and_retrieve(
+            'baremetal:port:get', port_uuid)
+
         api_utils.check_allow_specify_fields(fields)
         self._check_allowed_port_fields(fields)
 
-        rpc_port = objects.Port.get_by_uuid(pecan.request.context, port_uuid)
         return Port.convert_with_links(rpc_port, fields=fields)
 
     @METRICS.timer('PortsController.post')
@@ -550,17 +611,23 @@ class PortsController(rest.RestController):
         :param port: a port within the request body.
         :raises: NotAcceptable, HTTPNotFound, Conflict
         """
-        context = pecan.request.context
-        cdict = context.to_policy_values()
-        policy.authorize('baremetal:port:create', cdict, cdict)
-
         if self.parent_node_ident or self.parent_portgroup_ident:
             raise exception.OperationNotPermitted()
 
+        context = api.request.context
+        cdict = context.to_policy_values()
+        policy.authorize('baremetal:port:create', cdict, cdict)
+
         pdict = port.as_dict()
         self._check_allowed_port_fields(pdict)
 
-        create_remotely = pecan.request.rpcapi.can_send_create_port()
+        if (port.is_smartnic and not types.locallinkconnectiontype
+                .validate_for_smart_nic(port.local_link_connection)):
+            raise exception.Invalid(
+                "Smart NIC port must have port_id "
+                "and hostname in local_link_connection")
+
+        create_remotely = api.request.rpcapi.can_send_create_port()
         if (not create_remotely and pdict.get('portgroup_uuid')):
             # NOTE(mgoddard): In RPC API v1.41, port creation was moved to the
             # conductor service to facilitate validation of the physical
@@ -604,20 +671,20 @@ class PortsController(rest.RestController):
             # the RPCAPI will reject the create_port method, so we need to
             # create the port locally.
             if create_remotely:
-                topic = pecan.request.rpcapi.get_topic_for(rpc_node)
-                new_port = pecan.request.rpcapi.create_port(context, rpc_port,
-                                                            topic)
+                topic = api.request.rpcapi.get_topic_for(rpc_node)
+                new_port = api.request.rpcapi.create_port(context, rpc_port,
+                                                          topic)
             else:
                 rpc_port.create()
                 new_port = rpc_port
         notify.emit_end_notification(context, new_port, 'create',
                                      **notify_extra)
         # Set the HTTP Location Header
-        pecan.response.location = link.build_url('ports', new_port.uuid)
+        api.response.location = link.build_url('ports', new_port.uuid)
         return Port.convert_with_links(new_port)
 
     @METRICS.timer('PortsController.patch')
-    @wsme.validate(types.uuid, [PortPatchType])
+    @expose.validate(types.uuid, [PortPatchType])
     @expose.expose(Port, types.uuid, body=[PortPatchType])
     def patch(self, port_uuid, patch):
         """Update an existing port.
@@ -626,38 +693,35 @@ class PortsController(rest.RestController):
         :param patch: a json PATCH document to apply to this port.
         :raises: NotAcceptable, HTTPNotFound
         """
-        context = pecan.request.context
-        cdict = context.to_policy_values()
-        policy.authorize('baremetal:port:update', cdict, cdict)
-
         if self.parent_node_ident or self.parent_portgroup_ident:
             raise exception.OperationNotPermitted()
 
+        rpc_port, rpc_node = api_utils.check_port_policy_and_retrieve(
+            'baremetal:port:update', port_uuid)
+
+        context = api.request.context
         fields_to_check = set()
         for field in (self.advanced_net_fields
-                      + ['portgroup_uuid', 'physical_network']):
+                      + ['portgroup_uuid', 'physical_network',
+                         'is_smartnic', 'local_link_connection/network_type']):
             field_path = '/%s' % field
             if (api_utils.get_patch_values(patch, field_path)
                     or api_utils.is_path_removed(patch, field_path)):
                 fields_to_check.add(field)
         self._check_allowed_port_fields(fields_to_check)
 
-        rpc_port = objects.Port.get_by_uuid(context, port_uuid)
-        try:
-            port_dict = rpc_port.as_dict()
-            # NOTE(lucasagomes):
-            # 1) Remove node_id because it's an internal value and
-            #    not present in the API object
-            # 2) Add node_uuid
-            port_dict['node_uuid'] = port_dict.pop('node_id', None)
-            # NOTE(vsaienko):
-            # 1) Remove portgroup_id because it's an internal value and
-            #    not present in the API object
-            # 2) Add portgroup_uuid
-            port_dict['portgroup_uuid'] = port_dict.pop('portgroup_id', None)
-            port = Port(**api_utils.apply_jsonpatch(port_dict, patch))
-        except api_utils.JSONPATCH_EXCEPTIONS as e:
-            raise exception.PatchError(patch=patch, reason=e)
+        port_dict = rpc_port.as_dict()
+        # NOTE(lucasagomes):
+        # 1) Remove node_id because it's an internal value and
+        #    not present in the API object
+        # 2) Add node_uuid
+        port_dict['node_uuid'] = port_dict.pop('node_id', None)
+        # NOTE(vsaienko):
+        # 1) Remove portgroup_id because it's an internal value and
+        #    not present in the API object
+        # 2) Add portgroup_uuid
+        port_dict['portgroup_uuid'] = port_dict.pop('portgroup_id', None)
+        port = Port(**api_utils.apply_jsonpatch(port_dict, patch))
 
         api_utils.handle_patch_port_like_extra_vif(rpc_port, port, patch)
 
@@ -671,20 +735,19 @@ class PortsController(rest.RestController):
             except AttributeError:
                 # Ignore fields that aren't exposed in the API
                 continue
-            if patch_val == wtypes.Unset:
+            if patch_val == atypes.Unset:
                 patch_val = None
             if rpc_port[field] != patch_val:
                 rpc_port[field] = patch_val
 
-        rpc_node = objects.Node.get_by_id(context, rpc_port.node_id)
         if (rpc_node.provision_state == ir_states.INSPECTING
                 and api_utils.allow_inspect_wait_state()):
             msg = _('Cannot update port "%(port)s" on "%(node)s" while it is '
                     'in state "%(state)s".') % {'port': rpc_port.uuid,
                                                 'node': rpc_node.uuid,
                                                 'state': ir_states.INSPECTING}
-            raise wsme.exc.ClientSideError(msg,
-                                           status_code=http_client.CONFLICT)
+            raise exception.ClientSideError(msg,
+                                            status_code=http_client.CONFLICT)
 
         notify_extra = {'node_uuid': rpc_node.uuid,
                         'portgroup_uuid': port.portgroup_uuid}
@@ -692,9 +755,9 @@ class PortsController(rest.RestController):
                                        **notify_extra)
         with notify.handle_error_notification(context, rpc_port, 'update',
                                               **notify_extra):
-            topic = pecan.request.rpcapi.get_topic_for(rpc_node)
-            new_port = pecan.request.rpcapi.update_port(context, rpc_port,
-                                                        topic)
+            topic = api.request.rpcapi.get_topic_for(rpc_node)
+            new_port = api.request.rpcapi.update_port(context, rpc_port,
+                                                      topic)
 
         api_port = Port.convert_with_links(new_port)
         notify.emit_end_notification(context, new_port, 'update',
@@ -710,15 +773,13 @@ class PortsController(rest.RestController):
         :param port_uuid: UUID of a port.
         :raises: OperationNotPermitted, HTTPNotFound
         """
-        context = pecan.request.context
-        cdict = context.to_policy_values()
-        policy.authorize('baremetal:port:delete', cdict, cdict)
-
         if self.parent_node_ident or self.parent_portgroup_ident:
             raise exception.OperationNotPermitted()
 
-        rpc_port = objects.Port.get_by_uuid(context, port_uuid)
-        rpc_node = objects.Node.get_by_id(context, rpc_port.node_id)
+        rpc_port, rpc_node = api_utils.check_port_policy_and_retrieve(
+            'baremetal:port:delete', port_uuid)
+
+        context = api.request.context
 
         portgroup_uuid = None
         if rpc_port.portgroup_id:
@@ -732,7 +793,7 @@ class PortsController(rest.RestController):
                                        **notify_extra)
         with notify.handle_error_notification(context, rpc_port, 'delete',
                                               **notify_extra):
-            topic = pecan.request.rpcapi.get_topic_for(rpc_node)
-            pecan.request.rpcapi.destroy_port(context, rpc_port, topic)
+            topic = api.request.rpcapi.get_topic_for(rpc_node)
+            api.request.rpcapi.destroy_port(context, rpc_port, topic)
         notify.emit_end_notification(context, rpc_port, 'delete',
                                      **notify_extra)
