@@ -11,13 +11,11 @@
 #    under the License.
 
 import datetime
+from http import client as http_client
 
 from ironic_lib import metrics_utils
 from oslo_utils import uuidutils
 import pecan
-from six.moves import http_client
-import wsme
-from wsme import types as wtypes
 
 from ironic import api
 from ironic.api.controllers import base
@@ -28,6 +26,7 @@ from ironic.api.controllers.v1 import port
 from ironic.api.controllers.v1 import types
 from ironic.api.controllers.v1 import utils as api_utils
 from ironic.api import expose
+from ironic.api import types as atypes
 from ironic.common import exception
 from ironic.common.i18n import _
 from ironic.common import policy
@@ -55,7 +54,7 @@ class Portgroup(base.APIBase):
     def _set_node_uuid(self, value):
         if value and self._node_uuid != value:
             if not api_utils.allow_portgroups():
-                self._node_uuid = wtypes.Unset
+                self._node_uuid = atypes.Unset
                 return
             try:
                 node = objects.Node.get(api.request.context, value)
@@ -69,44 +68,44 @@ class Portgroup(base.APIBase):
                 # response for a POST request to create a Portgroup
                 e.code = http_client.BAD_REQUEST
                 raise e
-        elif value == wtypes.Unset:
-            self._node_uuid = wtypes.Unset
+        elif value == atypes.Unset:
+            self._node_uuid = atypes.Unset
 
     uuid = types.uuid
     """Unique UUID for this portgroup"""
 
-    address = wsme.wsattr(types.macaddress)
+    address = atypes.wsattr(types.macaddress)
     """MAC Address for this portgroup"""
 
-    extra = {wtypes.text: types.jsontype}
+    extra = {str: types.jsontype}
     """This portgroup's meta data"""
 
-    internal_info = wsme.wsattr({wtypes.text: types.jsontype}, readonly=True)
+    internal_info = atypes.wsattr({str: types.jsontype}, readonly=True)
     """This portgroup's internal info"""
 
-    node_uuid = wsme.wsproperty(types.uuid, _get_node_uuid, _set_node_uuid,
-                                mandatory=True)
+    node_uuid = atypes.wsproperty(types.uuid, _get_node_uuid, _set_node_uuid,
+                                  mandatory=True)
     """The UUID of the node this portgroup belongs to"""
 
-    name = wsme.wsattr(wtypes.text)
+    name = atypes.wsattr(str)
     """The logical name for this portgroup"""
 
-    links = wsme.wsattr([link.Link], readonly=True)
+    links = None
     """A list containing a self link and associated portgroup links"""
 
     standalone_ports_supported = types.boolean
     """Indicates whether ports of this portgroup may be used as
        single NIC ports"""
 
-    mode = wsme.wsattr(wtypes.text)
+    mode = atypes.wsattr(str)
     """The mode for this portgroup. See linux bonding
     documentation for details:
     https://www.kernel.org/doc/Documentation/networking/bonding.txt"""
 
-    properties = {wtypes.text: types.jsontype}
+    properties = {str: types.jsontype}
     """This portgroup's properties"""
 
-    ports = wsme.wsattr([link.Link], readonly=True)
+    ports = None
     """Links to the collection of ports of this portgroup"""
 
     def __init__(self, **kwargs):
@@ -120,34 +119,34 @@ class Portgroup(base.APIBase):
             if not hasattr(self, field):
                 continue
             self.fields.append(field)
-            setattr(self, field, kwargs.get(field, wtypes.Unset))
+            setattr(self, field, kwargs.get(field, atypes.Unset))
 
         # NOTE: node_id is an attribute created on-the-fly
         # by _set_node_uuid(), it needs to be present in the fields so
         # that as_dict() will contain node_id field when converting it
         # before saving it in the database.
         self.fields.append('node_id')
-        setattr(self, 'node_uuid', kwargs.get('node_id', wtypes.Unset))
+        setattr(self, 'node_uuid', kwargs.get('node_id', atypes.Unset))
 
     @staticmethod
     def _convert_with_links(portgroup, url, fields=None):
         """Add links to the portgroup."""
         if fields is None:
             portgroup.ports = [
-                link.Link.make_link('self', url, 'portgroups',
-                                    portgroup.uuid + "/ports"),
-                link.Link.make_link('bookmark', url, 'portgroups',
-                                    portgroup.uuid + "/ports", bookmark=True)
+                link.make_link('self', url, 'portgroups',
+                               portgroup.uuid + "/ports"),
+                link.make_link('bookmark', url, 'portgroups',
+                               portgroup.uuid + "/ports", bookmark=True)
             ]
 
         # never expose the node_id attribute
-        portgroup.node_id = wtypes.Unset
+        portgroup.node_id = atypes.Unset
 
-        portgroup.links = [link.Link.make_link('self', url,
-                                               'portgroups', portgroup.uuid),
-                           link.Link.make_link('bookmark', url,
-                                               'portgroups', portgroup.uuid,
-                                               bookmark=True)
+        portgroup.links = [link.make_link('self', url,
+                                          'portgroups', portgroup.uuid),
+                           link.make_link('bookmark', url,
+                                          'portgroups', portgroup.uuid,
+                                          bookmark=True)
                            ]
         return portgroup
 
@@ -183,7 +182,7 @@ class Portgroup(base.APIBase):
             self.unset_fields_except(fields)
 
         # never expose the node_id attribute
-        self.node_id = wtypes.Unset
+        self.node_id = atypes.Unset
 
     @classmethod
     def sample(cls, expand=True):
@@ -233,7 +232,8 @@ class PortgroupCollection(collection.Collection):
         collection.portgroups = [Portgroup.convert_with_links(p, fields=fields,
                                                               sanitize=False)
                                  for p in rpc_portgroups]
-        collection.next = collection.get_next(limit, url=url, **kwargs)
+        collection.next = collection.get_next(limit, url=url, fields=fields,
+                                              **kwargs)
 
         for item in collection.portgroups:
             item.sanitize(fields=fields)
@@ -358,7 +358,7 @@ class PortgroupsController(pecan.rest.RestController):
 
     @METRICS.timer('PortgroupsController.get_all')
     @expose.expose(PortgroupCollection, types.uuid_or_name, types.macaddress,
-                   types.uuid, int, wtypes.text, wtypes.text, types.listtype,
+                   types.uuid, int, str, str, types.listtype,
                    types.boolean)
     def get_all(self, node=None, address=None, marker=None,
                 limit=None, sort_key='id', sort_dir='asc', fields=None,
@@ -399,7 +399,7 @@ class PortgroupsController(pecan.rest.RestController):
 
     @METRICS.timer('PortgroupsController.detail')
     @expose.expose(PortgroupCollection, types.uuid_or_name, types.macaddress,
-                   types.uuid, int, wtypes.text, wtypes.text)
+                   types.uuid, int, str, str)
     def detail(self, node=None, address=None, marker=None,
                limit=None, sort_key='id', sort_dir='asc'):
         """Retrieve a list of portgroups with detail.
@@ -475,15 +475,15 @@ class PortgroupsController(pecan.rest.RestController):
             raise exception.OperationNotPermitted()
 
         if (not api_utils.allow_portgroup_mode_properties()
-                and (portgroup.mode is not wtypes.Unset
-                     or portgroup.properties is not wtypes.Unset)):
+                and (portgroup.mode is not atypes.Unset
+                     or portgroup.properties is not atypes.Unset)):
             raise exception.NotAcceptable()
 
         if (portgroup.name
                 and not api_utils.is_valid_logical_name(portgroup.name)):
             error_msg = _("Cannot create portgroup with invalid name "
                           "'%(name)s'") % {'name': portgroup.name}
-            raise wsme.exc.ClientSideError(
+            raise exception.ClientSideError(
                 error_msg, status_code=http_client.BAD_REQUEST)
 
         pg_dict = portgroup.as_dict()
@@ -510,7 +510,7 @@ class PortgroupsController(pecan.rest.RestController):
         return Portgroup.convert_with_links(new_portgroup)
 
     @METRICS.timer('PortgroupsController.patch')
-    @wsme.validate(types.uuid_or_name, [PortgroupPatchType])
+    @expose.validate(types.uuid_or_name, [PortgroupPatchType])
     @expose.expose(Portgroup, types.uuid_or_name, body=[PortgroupPatchType])
     def patch(self, portgroup_ident, patch):
         """Update an existing portgroup.
@@ -543,7 +543,7 @@ class PortgroupsController(pecan.rest.RestController):
                               " invalid name '%(name)s'") % {'portgroup':
                                                              portgroup_ident,
                                                              'name': name}
-                raise wsme.exc.ClientSideError(
+                raise exception.ClientSideError(
                     error_msg, status_code=http_client.BAD_REQUEST)
 
         portgroup_dict = rpc_portgroup.as_dict()
@@ -565,7 +565,7 @@ class PortgroupsController(pecan.rest.RestController):
             except AttributeError:
                 # Ignore fields that aren't exposed in the API
                 continue
-            if patch_val == wtypes.Unset:
+            if patch_val == atypes.Unset:
                 patch_val = None
             if rpc_portgroup[field] != patch_val:
                 rpc_portgroup[field] = patch_val
@@ -577,8 +577,8 @@ class PortgroupsController(pecan.rest.RestController):
                     '"%(node)s" while it is in state "%(state)s".') % {
                 'portgroup': rpc_portgroup.uuid, 'node': rpc_node.uuid,
                 'state': ir_states.INSPECTING}
-            raise wsme.exc.ClientSideError(msg,
-                                           status_code=http_client.CONFLICT)
+            raise exception.ClientSideError(msg,
+                                            status_code=http_client.CONFLICT)
 
         notify.emit_start_notification(context, rpc_portgroup, 'update',
                                        node_uuid=rpc_node.uuid)
