@@ -23,8 +23,8 @@ CONF = cfg.CONF
 BASE_VERSION = 1
 
 # Here goes a short log of changes in every version.
-# Refer to doc/source/dev/webapi-version-history.rst for a detailed explanation
-# of what each version contains.
+# Refer to doc/source/contributor/webapi-version-history.rst for a detailed
+# explanation of what each version contains.
 #
 # v1.0: corresponds to Juno API, not supported since Kilo
 # v1.1: API at the point in time when versioning support was added,
@@ -84,6 +84,28 @@ BASE_VERSION = 1
 # v1.44: Add node deploy_step field
 # v1.45: reset_interfaces parameter to node's PATCH
 # v1.46: Add conductor_group to the node object.
+# v1.47: Add automated_clean to the node object.
+# v1.48: Add protected to the node object.
+# v1.49: Add conductor to the node object and /v1/conductors.
+# v1.50: Add owner to the node object.
+# v1.51: Add description to the node object.
+# v1.52: Add allocation API.
+# v1.53: Add support for Smart NIC port
+# v1.54: Add events support.
+# v1.55: Add deploy templates API.
+# v1.56: Add support for building configdrives.
+# v1.57: Add support for updating an exisiting allocation.
+# v1.58: Add support for backfilling allocations.
+# v1.59: Add support vendor data in configdrives.
+# v1.60: Add owner to the allocation object.
+# v1.61: Add retired and retired_reason to the node object.
+# v1.62: Add agent_token support for agent communication.
+# v1.63: Add support for indicators
+# v1.64: Add network_type to port.local_link_connection
+# v1.65: Add lessee to the node object.
+# v1.66: Add support for node network_data field.
+# v1.67: Add support for port_uuid/portgroup_uuid in node vif_attach
+# v1.68: Add agent_verify_ca to heartbeat.
 
 MINOR_0_JUNO = 0
 MINOR_1_INITIAL_VERSION = 1
@@ -132,6 +154,28 @@ MINOR_43_ENABLE_DETAIL_QUERY = 43
 MINOR_44_NODE_DEPLOY_STEP = 44
 MINOR_45_RESET_INTERFACES = 45
 MINOR_46_NODE_CONDUCTOR_GROUP = 46
+MINOR_47_NODE_AUTOMATED_CLEAN = 47
+MINOR_48_NODE_PROTECTED = 48
+MINOR_49_CONDUCTORS = 49
+MINOR_50_NODE_OWNER = 50
+MINOR_51_NODE_DESCRIPTION = 51
+MINOR_52_ALLOCATION = 52
+MINOR_53_PORT_SMARTNIC = 53
+MINOR_54_EVENTS = 54
+MINOR_55_DEPLOY_TEMPLATES = 55
+MINOR_56_BUILD_CONFIGDRIVE = 56
+MINOR_57_ALLOCATION_UPDATE = 57
+MINOR_58_ALLOCATION_BACKFILL = 58
+MINOR_59_CONFIGDRIVE_VENDOR_DATA = 59
+MINOR_60_ALLOCATION_OWNER = 60
+MINOR_61_NODE_RETIRED = 61
+MINOR_62_AGENT_TOKEN = 62
+MINOR_63_INDICATORS = 63
+MINOR_64_LOCAL_LINK_CONNECTION_NETWORK_TYPE = 64
+MINOR_65_NODE_LESSEE = 65
+MINOR_66_NODE_NETWORK_DATA = 66
+MINOR_67_NODE_VIF_ATTACH_PORT = 67
+MINOR_68_HEARTBEAT_VERIFY_CA = 68
 
 # When adding another version, update:
 # - MINOR_MAX_VERSION
@@ -139,7 +183,7 @@ MINOR_46_NODE_CONDUCTOR_GROUP = 46
 #   explanation of what changed in the new version
 # - common/release_mappings.py, RELEASE_MAPPING['master']['api']
 
-MINOR_MAX_VERSION = MINOR_46_NODE_CONDUCTOR_GROUP
+MINOR_MAX_VERSION = MINOR_68_HEARTBEAT_VERIFY_CA
 
 # String representations of the minor and maximum versions
 _MIN_VERSION_STRING = '{}.{}'.format(BASE_VERSION, MINOR_1_INITIAL_VERSION)
