@@ -13,14 +13,11 @@
 #    under the License.
 
 import datetime
+from http import client as http_client
 
 from ironic_lib import metrics_utils
 from oslo_utils import uuidutils
 from pecan import rest
-import six
-from six.moves import http_client
-import wsme
-from wsme import types as wtypes
 
 from ironic import api
 from ironic.api.controllers import base
@@ -30,6 +27,7 @@ from ironic.api.controllers.v1 import notification_utils as notify
 from ironic.api.controllers.v1 import types
 from ironic.api.controllers.v1 import utils as api_utils
 from ironic.api import expose
+from ironic.api import types as atypes
 from ironic.common import exception
 from ironic.common.i18n import _
 from ironic.common import policy
@@ -57,10 +55,10 @@ class VolumeTarget(base.APIBase):
     def _set_node_identifiers(self, value):
         """Set both UUID and ID of a node for VolumeTarget object
 
-        :param value: UUID, ID of a node, or wtypes.Unset
+        :param value: UUID, ID of a node, or atypes.Unset
         """
-        if value == wtypes.Unset:
-            self._node_uuid = wtypes.Unset
+        if value == atypes.Unset:
+            self._node_uuid = atypes.Unset
         elif value and self._node_uuid != value:
             try:
                 node = objects.Node.get(api.request.context, value)
@@ -77,26 +75,26 @@ class VolumeTarget(base.APIBase):
     uuid = types.uuid
     """Unique UUID for this volume target"""
 
-    volume_type = wsme.wsattr(wtypes.text, mandatory=True)
+    volume_type = atypes.wsattr(str, mandatory=True)
     """The volume_type of volume target"""
 
-    properties = {wtypes.text: types.jsontype}
+    properties = {str: types.jsontype}
     """The properties for this volume target"""
 
-    boot_index = wsme.wsattr(int, mandatory=True)
+    boot_index = atypes.wsattr(int, mandatory=True)
     """The boot_index of volume target"""
 
-    volume_id = wsme.wsattr(wtypes.text, mandatory=True)
+    volume_id = atypes.wsattr(str, mandatory=True)
     """The volume_id for this volume target"""
 
-    extra = {wtypes.text: types.jsontype}
+    extra = {str: types.jsontype}
     """The metadata for this volume target"""
 
-    node_uuid = wsme.wsproperty(types.uuid, _get_node_uuid,
-                                _set_node_identifiers, mandatory=True)
+    node_uuid = atypes.wsproperty(types.uuid, _get_node_uuid,
+                                  _set_node_identifiers, mandatory=True)
     """The UUID of the node this volume target belongs to"""
 
-    links = wsme.wsattr([link.Link], readonly=True)
+    links = None
     """A list containing a self link and associated volume target links"""
 
     def __init__(self, **kwargs):
@@ -107,7 +105,7 @@ class VolumeTarget(base.APIBase):
             if not hasattr(self, field):
                 continue
             self.fields.append(field)
-            setattr(self, field, kwargs.get(field, wtypes.Unset))
+            setattr(self, field, kwargs.get(field, atypes.Unset))
 
         # NOTE(smoriya): node_id is an attribute created on-the-fly
         # by _set_node_uuid(), it needs to be present in the fields so
@@ -121,18 +119,18 @@ class VolumeTarget(base.APIBase):
         # secondary identifier in case RPC volume target object dictionary
         # was passed to the constructor.
         self.node_uuid = kwargs.get('node_uuid') or kwargs.get('node_id',
-                                                               wtypes.Unset)
+                                                               atypes.Unset)
 
     @staticmethod
     def _convert_with_links(target, url):
 
-        target.links = [link.Link.make_link('self', url,
-                                            'volume/targets',
-                                            target.uuid),
-                        link.Link.make_link('bookmark', url,
-                                            'volume/targets',
-                                            target.uuid,
-                                            bookmark=True)
+        target.links = [link.make_link('self', url,
+                                       'volume/targets',
+                                       target.uuid),
+                        link.make_link('bookmark', url,
+                                       'volume/targets',
+                                       target.uuid,
+                                       bookmark=True)
                         ]
         return target
 
@@ -166,7 +164,7 @@ class VolumeTarget(base.APIBase):
             self.unset_fields_except(fields)
 
         # never expose the node_id attribute
-        self.node_id = wtypes.Unset
+        self.node_id = atypes.Unset
 
     @classmethod
     def sample(cls, expand=True):
@@ -217,7 +215,8 @@ class VolumeTargetCollection(collection.Collection):
             for p in rpc_targets]
         if detail:
             kwargs['detail'] = detail
-        collection.next = collection.get_next(limit, url=url, **kwargs)
+        collection.next = collection.get_next(limit, url=url, fields=fields,
+                                              **kwargs)
         for target in collection.targets:
             target.sanitize(fields)
         return collection
@@ -279,7 +278,7 @@ class VolumeTargetsController(rest.RestController):
 
     @METRICS.timer('VolumeTargetsController.get_all')
     @expose.expose(VolumeTargetCollection, types.uuid_or_name, types.uuid,
-                   int, wtypes.text, wtypes.text, types.listtype,
+                   int, str, str, types.listtype,
                    types.boolean)
     def get_all(self, node=None, marker=None, limit=None, sort_key='id',
                 sort_dir='asc', fields=None, detail=None):
@@ -391,7 +390,7 @@ class VolumeTargetsController(rest.RestController):
         return VolumeTarget.convert_with_links(new_target)
 
     @METRICS.timer('VolumeTargetsController.patch')
-    @wsme.validate(types.uuid, [VolumeTargetPatchType])
+    @expose.validate(types.uuid, [VolumeTargetPatchType])
     @expose.expose(VolumeTarget, types.uuid,
                    body=[VolumeTargetPatchType])
     def patch(self, target_uuid, patch):
@@ -428,7 +427,7 @@ class VolumeTargetsController(rest.RestController):
         for value in values:
             if not uuidutils.is_uuid_like(value):
                 message = _("Expected a UUID for node_uuid, but received "
-                            "%(uuid)s.") % {'uuid': six.text_type(value)}
+                            "%(uuid)s.") % {'uuid': str(value)}
                 raise exception.InvalidUUID(message=message)
 
         rpc_target = objects.VolumeTarget.get_by_uuid(context, target_uuid)
@@ -448,7 +447,7 @@ class VolumeTargetsController(rest.RestController):
             except AttributeError:
                 # Ignore fields that aren't exposed in the API
                 continue
-            if patch_val == wtypes.Unset:
+            if patch_val == atypes.Unset:
                 patch_val = None
             if rpc_target[field] != patch_val:
                 rpc_target[field] = patch_val
