@@ -19,12 +19,19 @@
 # concurrency models can cause undefined behavior and potentially API timeouts.
 import os
 
-os.environ['EVENTLET_NO_GREENDNS'] = 'yes'  # noqa E402
+os.environ['EVENTLET_NO_GREENDNS'] = 'yes'
 
 import eventlet
 
 eventlet.monkey_patch(os=False)
+# Monkey patch the original current_thread to use the up-to-date _active
+# global variable. See https://bugs.launchpad.net/bugs/1863021 and
+# https://github.com/eventlet/eventlet/issues/592
+# all the noqa below are for I202 due to 'import eventlet' above
+import __original_module_threading as orig_threading  # noqa
+import threading  # noqa
+orig_threading.current_thread.__globals__['_active'] = threading._active
 
-import oslo_i18n as i18n  # noqa for I202 due to 'import eventlet' above
+from ironic.common import i18n  # noqa
 
 i18n.install('ironic')
