@@ -14,130 +14,18 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""Ironic base exception handling.
+"""Ironic specific exceptions list."""
 
-SHOULD include dedicated exception logging.
-
-"""
-
-import collections
+from http import client as http_client
 
+from ironic_lib.exception import IronicException
 from oslo_log import log as logging
-from oslo_serialization import jsonutils
-import six
-from six.moves import http_client
 
 from ironic.common.i18n import _
-from ironic.conf import CONF
 
 LOG = logging.getLogger(__name__)
 
 
-def _ensure_exception_kwargs_serializable(exc_class_name, kwargs):
-    """Ensure that kwargs are serializable
-
-    Ensure that all kwargs passed to exception constructor can be passed over
-    RPC, by trying to convert them to JSON, or, as a last resort, to string.
-    If it is not possible, unserializable kwargs will be removed, letting the
-    receiver to handle the exception string as it is configured to.
-
-    :param exc_class_name: an IronicException class name.
-    :param kwargs: a dictionary of keyword arguments passed to the exception
-        constructor.
-    :returns: a dictionary of serializable keyword arguments.
-    """
-    serializers = [(jsonutils.dumps, _('when converting to JSON')),
-                   (six.text_type, _('when converting to string'))]
-    exceptions = collections.defaultdict(list)
-    serializable_kwargs = {}
-    for k, v in kwargs.items():
-        for serializer, msg in serializers:
-            try:
-                serializable_kwargs[k] = serializer(v)
-                exceptions.pop(k, None)
-                break
-            except Exception as e:
-                exceptions[k].append(
-                    '(%(serializer_type)s) %(e_type)s: %(e_contents)s' %
-                    {'serializer_type': msg, 'e_contents': e,
-                     'e_type': e.__class__.__name__})
-    if exceptions:
-        LOG.error("One or more arguments passed to the %(exc_class)s "
-                  "constructor as kwargs can not be serialized. The "
-                  "serialized arguments: %(serialized)s. These "
-                  "unserialized kwargs were dropped because of the "
-                  "exceptions encountered during their "
-                  "serialization:\n%(errors)s",
-                  dict(errors=';\n'.join("%s: %s" % (k, '; '.join(v))
-                                         for k, v in exceptions.items()),
-                       exc_class=exc_class_name,
-                       serialized=serializable_kwargs))
-        # We might be able to actually put the following keys' values into
-        # format string, but there is no guarantee, drop it just in case.
-        for k in exceptions:
-            del kwargs[k]
-    return serializable_kwargs
-
-
-class IronicException(Exception):
-    """Base Ironic Exception
-
-    To correctly use this class, inherit from it and define
-    a '_msg_fmt' property. That message will get printf'd
-    with the keyword arguments provided to the constructor.
-
-    If you need to access the message from an exception you should use
-    six.text_type(exc)
-
-    """
-    _msg_fmt = _("An unknown exception occurred.")
-    code = http_client.INTERNAL_SERVER_ERROR
-    safe = False
-
-    def __init__(self, message=None, **kwargs):
-
-        self.kwargs = _ensure_exception_kwargs_serializable(
-            self.__class__.__name__, kwargs)
-
-        if 'code' not in self.kwargs:
-            try:
-                self.kwargs['code'] = self.code
-            except AttributeError:
-                pass
-        else:
-            self.code = int(kwargs['code'])
-
-        if not message:
-            try:
-                message = self._msg_fmt % kwargs
-
-            except Exception as e:
-                # kwargs doesn't match a variable in self._msg_fmt
-                # log the issue and the kwargs
-                prs = ', '.join('%s: %s' % pair for pair in kwargs.items())
-                LOG.exception('Exception in string format operation '
-                              '(arguments %s)', prs)
-                if CONF.fatal_exception_format_errors:
-                    raise e
-                else:
-                    # at least get the core self._msg_fmt out if something
-                    # happened
-                    message = self._msg_fmt
-
-        super(IronicException, self).__init__(message)
-
-    def __str__(self):
-        """Encode to utf-8 then wsme api can consume it as well."""
-        if not six.PY3:
-            return six.text_type(self.args[0]).encode('utf-8')
-
-        return self.args[0]
-
-    def __unicode__(self):
-        """Return a unicode representation of the exception message."""
-        return six.text_type(self.args[0])
-
-
 class NotAuthorized(IronicException):
     _msg_fmt = _("Not authorized.")
     code = http_client.FORBIDDEN
@@ -163,7 +51,8 @@ class TemporaryFailure(IronicException):
 
 
 class NotAcceptable(IronicException):
-    # TODO(deva): We need to set response headers in the API for this exception
+    # TODO(tenbrae): We need to set response headers in the API
+    # for this exception
     _msg_fmt = _("Request not acceptable.")
     code = http_client.NOT_ACCEPTABLE
 
@@ -408,6 +297,10 @@ class InvalidIPv4Address(IronicException):
     _msg_fmt = _("Invalid IPv4 address %(ip_address)s.")
 
 
+class InvalidIPAddress(IronicException):
+    _msg_fmt = _("Invalid IP address %(ip_address)s.")
+
+
 class FailedToUpdateMacOnPort(IronicException):
     _msg_fmt = _("Update MAC address on port: %(port_id)s failed.")
 
@@ -674,34 +567,11 @@ class DirectoryNotWritable(IronicException):
     _msg_fmt = _("Directory %(dir)s is not writable.")
 
 
-class UcsOperationError(DriverOperationError):
-    _msg_fmt = _("Cisco UCS client: operation %(operation)s failed for node"
-                 " %(node)s. Reason: %(error)s")
-
-
-class UcsConnectionError(IronicException):
-    _msg_fmt = _("Cisco UCS client: connection failed for node "
-                 "%(node)s. Reason: %(error)s")
-
-
 class ImageUploadFailed(IronicException):
     _msg_fmt = _("Failed to upload %(image_name)s image to web server "
                  "%(web_server)s, reason: %(reason)s")
 
 
-class CIMCException(DriverOperationError):
-    _msg_fmt = _("Cisco IMC exception occurred for node %(node)s: %(error)s")
-
-
-class OneViewError(DriverOperationError):
-    _msg_fmt = _("OneView exception occurred. Error: %(error)s")
-
-
-class OneViewInvalidNodeParameter(OneViewError):
-    _msg_fmt = _("Error while obtaining OneView info from node %(node_uuid)s. "
-                 "Error: %(error)s")
-
-
 class NodeTagNotFound(IronicException):
     _msg_fmt = _("Node %(node_id)s doesn't have a tag '%(tag)s'")
 
@@ -795,3 +665,151 @@ class DatabaseVersionTooOld(IronicException):
 
 class AgentConnectionFailed(IronicException):
     _msg_fmt = _("Connection to agent failed: %(reason)s")
+
+
+class AgentCommandTimeout(IronicException):
+    _msg_fmt = _("Timeout executing command %(command)s on node %(node)s")
+
+
+class NodeProtected(HTTPForbidden):
+    _msg_fmt = _("Node %(node)s is protected and cannot be undeployed, "
+                 "rebuilt or deleted")
+
+
+class AllocationNotFound(NotFound):
+    _msg_fmt = _("Allocation %(allocation)s could not be found.")
+
+
+class AllocationDuplicateName(Conflict):
+    _msg_fmt = _("An allocation with name %(name)s already exists.")
+
+
+class AllocationAlreadyExists(Conflict):
+    _msg_fmt = _("An allocation with UUID %(uuid)s already exists.")
+
+
+class AllocationFailed(IronicException):
+    _msg_fmt = _("Failed to process allocation %(uuid)s: %(error)s.")
+
+
+class DeployTemplateDuplicateName(Conflict):
+    _msg_fmt = _("A deploy template with name %(name)s already exists.")
+
+
+class DeployTemplateAlreadyExists(Conflict):
+    _msg_fmt = _("A deploy template with UUID %(uuid)s already exists.")
+
+
+class DeployTemplateNotFound(NotFound):
+    _msg_fmt = _("Deploy template %(template)s could not be found.")
+
+
+class InvalidDeployTemplate(Invalid):
+    _msg_fmt = _("Deploy template invalid: %(err)s.")
+
+
+class IBMCError(DriverOperationError):
+    _msg_fmt = _("IBMC exception occurred on node %(node)s. Error: %(error)s")
+
+
+class IBMCConnectionError(IBMCError):
+    _msg_fmt = _("IBMC connection failed for node %(node)s: %(error)s")
+
+
+class ClientSideError(RuntimeError):
+    def __init__(self, msg=None, status_code=400, faultcode='Client'):
+        self.msg = msg
+        self.code = status_code
+        self.faultcode = faultcode
+        super(ClientSideError, self).__init__(self.faultstring)
+
+    @property
+    def faultstring(self):
+        if self.msg is None:
+            return str(self)
+        elif isinstance(self.msg, str):
+            return self.msg
+        else:
+            return str(self.msg)
+
+
+class NodeIsRetired(Invalid):
+    _msg_fmt = _("The %(op)s operation can't be performed on node "
+                 "%(node)s because it is retired.")
+
+
+class NoFreeIPMITerminalPorts(TemporaryFailure):
+    _msg_fmt = _("Unable to allocate a free port on host %(host)s for IPMI "
+                 "terminal, not enough free ports.")
+
+
+class InvalidInput(ClientSideError):
+    def __init__(self, fieldname, value, msg=''):
+        self.fieldname = fieldname
+        self.value = value
+        super(InvalidInput, self).__init__(msg)
+
+    @property
+    def faultstring(self):
+        return _(
+            "Invalid input for field/attribute %(fieldname)s. "
+            "Value: '%(value)s'. %(msg)s"
+        ) % {
+            'fieldname': self.fieldname,
+            'value': self.value,
+            'msg': self.msg
+        }
+
+
+class UnknownArgument(ClientSideError):
+    def __init__(self, argname, msg=''):
+        self.argname = argname
+        super(UnknownArgument, self).__init__(msg)
+
+    @property
+    def faultstring(self):
+        return _('Unknown argument: "%(argname)s"%(msg)s') % {
+            'argname': self.argname,
+            'msg': self.msg and ": " + self.msg or ""
+        }
+
+
+class MissingArgument(ClientSideError):
+    def __init__(self, argname, msg=''):
+        self.argname = argname
+        super(MissingArgument, self).__init__(msg)
+
+    @property
+    def faultstring(self):
+        return _('Missing argument: "%(argname)s"%(msg)s') % {
+            'argname': self.argname,
+            'msg': self.msg and ": " + self.msg or ""
+        }
+
+
+class UnknownAttribute(ClientSideError):
+    def __init__(self, fieldname, attributes, msg=''):
+        self.fieldname = fieldname
+        self.attributes = attributes
+        self.msg = msg
+        super(UnknownAttribute, self).__init__(self.msg)
+
+    @property
+    def faultstring(self):
+        error = _("Unknown attribute for argument %(argn)s: %(attrs)s")
+        if len(self.attributes) > 1:
+            error = _("Unknown attributes for argument %(argn)s: %(attrs)s")
+        str_attrs = ", ".join(self.attributes)
+        return error % {'argn': self.fieldname, 'attrs': str_attrs}
+
+    def add_fieldname(self, name):
+        """Add a fieldname to concatenate the full name.
+
+        Add a fieldname so that the whole hierarchy is displayed. Successive
+        calls to this method will prepend ``name`` to the hierarchy of names.
+        """
+        if self.fieldname is not None:
+            self.fieldname = "{}.{}".format(name, self.fieldname)
+        else:
+            self.fieldname = name
+        super(UnknownAttribute, self).__init__(self.msg)
