@@ -36,12 +36,12 @@ default_policies = [
     policy.RuleDefault('admin_api',
                        'role:admin or role:administrator',
                        description='Legacy rule for cloud admin access'),
-    # is_public_api is set in the environment from AuthTokenMiddleware
+    # is_public_api is set in the environment from AuthPublicRoutes
     policy.RuleDefault('public_api',
                        'is_public_api:True',
                        description='Internal flag for public API routes'),
     # Generic default to hide passwords in node driver_info
-    # NOTE(deva): the 'show_password' policy setting hides secrets in
+    # NOTE(tenbrae): the 'show_password' policy setting hides secrets in
     #             driver_info. However, the name exists for legacy
     #             purposes and can not be changed. Changing it will cause
     #             upgrade problems for any operators who have customized
@@ -63,9 +63,18 @@ default_policies = [
     policy.RuleDefault('is_admin',
                        'rule:admin_api or (rule:is_member and role:baremetal_admin)',  # noqa
                        description='Full read/write API access'),
+    policy.RuleDefault('is_node_owner',
+                       'project_id:%(node.owner)s',
+                       description='Owner of node'),
+    policy.RuleDefault('is_node_lessee',
+                       'project_id:%(node.lessee)s',
+                       description='Lessee of node'),
+    policy.RuleDefault('is_allocation_owner',
+                       'project_id:%(allocation.owner)s',
+                       description='Owner of allocation'),
 ]
 
-# NOTE(deva): to follow policy-in-code spec, we define defaults for
+# NOTE(tenbrae): to follow policy-in-code spec, we define defaults for
 #             the granular policies in code, rather than in policy.json.
 #             All of these may be overridden by configuration, but we can
 #             depend on their existence throughout the code.
@@ -79,15 +88,40 @@ node_policies = [
     policy.DocumentedRuleDefault(
         'baremetal:node:get',
         'rule:is_admin or rule:is_observer',
-        'Retrieve Node records',
+        'Retrieve a single Node record',
+        [{'path': '/nodes/{node_ident}', 'method': 'GET'}]),
+    policy.DocumentedRuleDefault(
+        'baremetal:node:list',
+        'rule:baremetal:node:get',
+        'Retrieve multiple Node records, filtered by owner',
         [{'path': '/nodes', 'method': 'GET'},
-         {'path': '/nodes/detail', 'method': 'GET'},
-         {'path': '/nodes/{node_ident}', 'method': 'GET'}]),
+         {'path': '/nodes/detail', 'method': 'GET'}]),
+    policy.DocumentedRuleDefault(
+        'baremetal:node:list_all',
+        'rule:baremetal:node:get',
+        'Retrieve multiple Node records',
+        [{'path': '/nodes', 'method': 'GET'},
+         {'path': '/nodes/detail', 'method': 'GET'}]),
     policy.DocumentedRuleDefault(
         'baremetal:node:update',
         'rule:is_admin',
         'Update Node records',
         [{'path': '/nodes/{node_ident}', 'method': 'PATCH'}]),
+    policy.DocumentedRuleDefault(
+        'baremetal:node:update_extra',
+        'rule:baremetal:node:update',
+        'Update Node extra field',
+        [{'path': '/nodes/{node_ident}', 'method': 'PATCH'}]),
+    policy.DocumentedRuleDefault(
+        'baremetal:node:update_instance_info',
+        'rule:baremetal:node:update',
+        'Update Node instance_info field',
+        [{'path': '/nodes/{node_ident}', 'method': 'PATCH'}]),
+    policy.DocumentedRuleDefault(
+        'baremetal:node:update_owner_provisioned',
+        'rule:is_admin',
+        'Update Node owner even when Node is provisioned',
+        [{'path': '/nodes/{node_ident}', 'method': 'PATCH'}]),
     policy.DocumentedRuleDefault(
         'baremetal:node:delete',
         'rule:is_admin',
@@ -126,6 +160,23 @@ node_policies = [
         [{'path': '/nodes/{node_ident}/management/boot_device',
           'method': 'PUT'}]),
 
+    policy.DocumentedRuleDefault(
+        'baremetal:node:get_indicator_state',
+        'rule:is_admin or rule:is_observer',
+        'Retrieve Node indicators and their states',
+        [{'path': '/nodes/{node_ident}/management/indicators/'
+                  '{component}/{indicator}',
+          'method': 'GET'},
+         {'path': '/nodes/{node_ident}/management/indicators',
+          'method': 'GET'}]),
+    policy.DocumentedRuleDefault(
+        'baremetal:node:set_indicator_state',
+        'rule:is_admin',
+        'Change Node indicator state',
+        [{'path': '/nodes/{node_ident}/management/indicators/'
+                  '{component}/{indicator}',
+          'method': 'PUT'}]),
+
     policy.DocumentedRuleDefault(
         'baremetal:node:inject_nmi',
         'rule:is_admin',
@@ -213,14 +264,24 @@ port_policies = [
         'baremetal:port:get',
         'rule:is_admin or rule:is_observer',
         'Retrieve Port records',
-        [{'path': '/ports', 'method': 'GET'},
-         {'path': '/ports/detail', 'method': 'GET'},
-         {'path': '/ports/{port_id}', 'method': 'GET'},
+        [{'path': '/ports/{port_id}', 'method': 'GET'},
          {'path': '/nodes/{node_ident}/ports', 'method': 'GET'},
          {'path': '/nodes/{node_ident}/ports/detail', 'method': 'GET'},
          {'path': '/portgroups/{portgroup_ident}/ports', 'method': 'GET'},
          {'path': '/portgroups/{portgroup_ident}/ports/detail',
           'method': 'GET'}]),
+    policy.DocumentedRuleDefault(
+        'baremetal:port:list',
+        'rule:baremetal:port:get',
+        'Retrieve multiple Port records, filtered by owner',
+        [{'path': '/ports', 'method': 'GET'},
+         {'path': '/ports/detail', 'method': 'GET'}]),
+    policy.DocumentedRuleDefault(
+        'baremetal:port:list_all',
+        'rule:baremetal:port:get',
+        'Retrieve multiple Port records',
+        [{'path': '/ports', 'method': 'GET'},
+         {'path': '/ports/detail', 'method': 'GET'}]),
     policy.DocumentedRuleDefault(
         'baremetal:port:create',
         'rule:is_admin',
@@ -395,6 +456,91 @@ volume_policies = [
           'method': 'PATCH'}]),
 ]
 
+conductor_policies = [
+    policy.DocumentedRuleDefault(
+        'baremetal:conductor:get',
+        'rule:is_admin or rule:is_observer',
+        'Retrieve Conductor records',
+        [{'path': '/conductors', 'method': 'GET'},
+         {'path': '/conductors/{hostname}', 'method': 'GET'}]),
+]
+
+allocation_policies = [
+    policy.DocumentedRuleDefault(
+        'baremetal:allocation:get',
+        'rule:is_admin or rule:is_observer',
+        'Retrieve Allocation records',
+        [{'path': '/allocations/{allocation_id}', 'method': 'GET'},
+         {'path': '/nodes/{node_ident}/allocation', 'method': 'GET'}]),
+    policy.DocumentedRuleDefault(
+        'baremetal:allocation:list',
+        'rule:baremetal:allocation:get',
+        'Retrieve multiple Allocation records, filtered by owner',
+        [{'path': '/allocations', 'method': 'GET'}]),
+    policy.DocumentedRuleDefault(
+        'baremetal:allocation:list_all',
+        'rule:baremetal:allocation:get',
+        'Retrieve multiple Allocation records',
+        [{'path': '/allocations', 'method': 'GET'}]),
+    policy.DocumentedRuleDefault(
+        'baremetal:allocation:create',
+        'rule:is_admin',
+        'Create Allocation records',
+        [{'path': '/allocations', 'method': 'POST'}]),
+    policy.DocumentedRuleDefault(
+        'baremetal:allocation:create_restricted',
+        'rule:baremetal:allocation:create',
+        'Create Allocation records that are restricted to an owner',
+        [{'path': '/allocations', 'method': 'POST'}]),
+    policy.DocumentedRuleDefault(
+        'baremetal:allocation:delete',
+        'rule:is_admin',
+        'Delete Allocation records',
+        [{'path': '/allocations/{allocation_id}', 'method': 'DELETE'},
+         {'path': '/nodes/{node_ident}/allocation', 'method': 'DELETE'}]),
+    policy.DocumentedRuleDefault(
+        'baremetal:allocation:update',
+        'rule:is_admin',
+        'Change name and extra fields of an allocation',
+        [{'path': '/allocations/{allocation_id}', 'method': 'PATCH'}]),
+]
+
+event_policies = [
+    policy.DocumentedRuleDefault(
+        'baremetal:events:post',
+        'rule:is_admin',
+        'Post events',
+        [{'path': '/events', 'method': 'POST'}])
+]
+
+
+deploy_template_policies = [
+    policy.DocumentedRuleDefault(
+        'baremetal:deploy_template:get',
+        'rule:is_admin or rule:is_observer',
+        'Retrieve Deploy Template records',
+        [{'path': '/deploy_templates', 'method': 'GET'},
+         {'path': '/deploy_templates/{deploy_template_ident}',
+          'method': 'GET'}]),
+    policy.DocumentedRuleDefault(
+        'baremetal:deploy_template:create',
+        'rule:is_admin',
+        'Create Deploy Template records',
+        [{'path': '/deploy_templates', 'method': 'POST'}]),
+    policy.DocumentedRuleDefault(
+        'baremetal:deploy_template:delete',
+        'rule:is_admin',
+        'Delete Deploy Template records',
+        [{'path': '/deploy_templates/{deploy_template_ident}',
+          'method': 'DELETE'}]),
+    policy.DocumentedRuleDefault(
+        'baremetal:deploy_template:update',
+        'rule:is_admin',
+        'Update Deploy Template records',
+        [{'path': '/deploy_templates/{deploy_template_ident}',
+          'method': 'PATCH'}]),
+]
+
 
 def list_policies():
     policies = itertools.chain(
@@ -406,7 +552,11 @@ def list_policies():
         driver_policies,
         vendor_passthru_policies,
         utility_policies,
-        volume_policies
+        volume_policies,
+        conductor_policies,
+        allocation_policies,
+        event_policies,
+        deploy_template_policies,
     )
     return policies
 
@@ -431,7 +581,7 @@ def init_enforcer(policy_file=None, rules=None,
     if _ENFORCER:
         return
 
-    # NOTE(deva): Register defaults for policy-in-code here so that they are
+    # NOTE(tenbrae): Register defaults for policy-in-code here so that they are
     # loaded exactly once - when this module-global is initialized.
     # Defining these in the relevant API modules won't work
     # because API classes lack singletons and don't use globals.
@@ -472,7 +622,7 @@ def get_oslo_policy_enforcer():
     return get_enforcer()
 
 
-# NOTE(deva): We can't call these methods from within decorators because the
+# NOTE(tenbrae): We can't call these methods from within decorators because the
 # 'target' and 'creds' parameter must be fetched from the call time
 # context-local pecan.request magic variable, but decorators are compiled
 # at module-load time.
@@ -483,9 +633,9 @@ def authorize(rule, target, creds, *args, **kwargs):
 
     Checks authorization of a rule against the target and credentials, and
     raises an exception if the rule is not defined.
-    Always returns true if CONF.auth_strategy == noauth.
+    Always returns true if CONF.auth_strategy is not keystone.
     """
-    if CONF.auth_strategy == 'noauth':
+    if CONF.auth_strategy != 'keystone':
         return True
     enforcer = get_enforcer()
     try:
