@@ -13,15 +13,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo_config import cfg
 import oslo_messaging as messaging
+from oslo_messaging.rpc import dispatcher
+from osprofiler import profiler
 
 from ironic.common import context as ironic_context
 from ironic.common import exception
+from ironic.conf import CONF
 
 
-CONF = cfg.CONF
-
 TRANSPORT = None
 NOTIFICATION_TRANSPORT = None
 SENSORS_NOTIFIER = None
@@ -32,24 +32,16 @@ ALLOWED_EXMODS = [
 ]
 EXTRA_EXMODS = []
 
-TRANSPORT_ALIASES = {
-    'ironic.rpc.impl_kombu': 'rabbit',
-    'ironic.rpc.impl_qpid': 'qpid',
-    'ironic.rpc.impl_zmq': 'zmq',
-}
-
 
 def init(conf):
     global TRANSPORT, NOTIFICATION_TRANSPORT
     global SENSORS_NOTIFIER, VERSIONED_NOTIFIER
     exmods = get_allowed_exmods()
-    TRANSPORT = messaging.get_transport(conf,
-                                        allowed_remote_exmods=exmods,
-                                        aliases=TRANSPORT_ALIASES)
+    TRANSPORT = messaging.get_rpc_transport(conf,
+                                            allowed_remote_exmods=exmods)
     NOTIFICATION_TRANSPORT = messaging.get_notification_transport(
         conf,
-        allowed_remote_exmods=exmods,
-        aliases=TRANSPORT_ALIASES)
+        allowed_remote_exmods=exmods)
 
     serializer = RequestContextSerializer(messaging.JsonPayloadSerializer())
     SENSORS_NOTIFIER = messaging.Notifier(NOTIFICATION_TRANSPORT,
@@ -59,10 +51,10 @@ def init(conf):
                                                 serializer=serializer,
                                                 driver='noop')
     else:
-        VERSIONED_NOTIFIER = messaging.Notifier(NOTIFICATION_TRANSPORT,
-                                                serializer=serializer,
-                                                topics=['ironic_versioned_'
-                                                        'notifications'])
+        VERSIONED_NOTIFIER = messaging.Notifier(
+            NOTIFICATION_TRANSPORT,
+            serializer=serializer,
+            topics=CONF.versioned_notifications_topics)
 
 
 def cleanup():
@@ -82,14 +74,6 @@ def set_defaults(control_exchange):
     messaging.set_transport_defaults(control_exchange)
 
 
-def add_extra_exmods(*args):
-    EXTRA_EXMODS.extend(args)
-
-
-def clear_extra_exmods():
-    del EXTRA_EXMODS[:]
-
-
 def get_allowed_exmods():
     return ALLOWED_EXMODS + EXTRA_EXMODS
 
@@ -110,14 +94,26 @@ class RequestContextSerializer(messaging.Serializer):
         return self._base.deserialize_entity(context, entity)
 
     def serialize_context(self, context):
-        return context.to_dict()
+        _context = context.to_dict()
+        prof = profiler.get()
+        if prof:
+            trace_info = {
+                "hmac_key": prof.hmac_key,
+                "base_id": prof.get_base_id(),
+                "parent_id": prof.get_id()
+            }
+            _context.update({"trace_info": trace_info})
+        return _context
 
     def deserialize_context(self, context):
+        trace_info = context.pop("trace_info", None)
+        if trace_info:
+            profiler.init(**trace_info)
         return ironic_context.RequestContext.from_dict(context)
 
 
 def get_transport_url(url_str=None):
-    return messaging.TransportURL.parse(CONF, url_str, TRANSPORT_ALIASES)
+    return messaging.TransportURL.parse(CONF, url_str)
 
 
 def get_client(target, version_cap=None, serializer=None):
@@ -132,11 +128,13 @@ def get_client(target, version_cap=None, serializer=None):
 def get_server(target, endpoints, serializer=None):
     assert TRANSPORT is not None
     serializer = RequestContextSerializer(serializer)
+    access_policy = dispatcher.DefaultRPCAccessPolicy
     return messaging.get_rpc_server(TRANSPORT,
                                     target,
                                     endpoints,
                                     executor='eventlet',
-                                    serializer=serializer)
+                                    serializer=serializer,
+                                    access_policy=access_policy)
 
 
 def get_sensors_notifier(service=None, host=None, publisher_id=None):
