@@ -22,6 +22,7 @@ import contextlib
 import datetime
 import errno
 import hashlib
+import ipaddress
 import os
 import re
 import shutil
@@ -31,10 +32,10 @@ import jinja2
 from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
+from oslo_utils import fileutils
 from oslo_utils import netutils
 from oslo_utils import timeutils
 import pytz
-import six
 
 from ironic.common import exception
 from ironic.common.i18n import _
@@ -42,6 +43,15 @@ from ironic.conf import CONF
 
 LOG = logging.getLogger(__name__)
 
+DATE_RE = r'(?P<year>-?\d{4,})-(?P<month>\d{2})-(?P<day>\d{2})'
+TIME_RE = r'(?P<hour>\d{2}):(?P<min>\d{2}):(?P<sec>\d{2})' + \
+          r'(\.(?P<sec_frac>\d+))?'
+TZ_RE = r'((?P<tz_sign>[+-])(?P<tz_hour>\d{2}):(?P<tz_min>\d{2}))' + \
+        r'|(?P<tz_z>Z)'
+
+DATETIME_RE = re.compile(
+    '%sT%s(%s)?' % (DATE_RE, TIME_RE, TZ_RE))
+
 warn_deprecated_extra_vif_port_id = False
 
 
@@ -91,7 +101,7 @@ def is_valid_datapath_id(datapath_id):
 
     """
     m = "^[0-9a-f]{16}$"
-    return (isinstance(datapath_id, six.string_types)
+    return (isinstance(datapath_id, str)
             and re.match(m, datapath_id.lower()))
 
 
@@ -113,7 +123,7 @@ def is_valid_logical_name(hostname):
 
         ALPHA / DIGIT / "-" / "." / "_" / "~"
     """
-    if not isinstance(hostname, six.string_types) or len(hostname) > 255:
+    if not isinstance(hostname, str) or len(hostname) > 255:
         return False
 
     return _is_valid_logical_name_re.match(hostname) is not None
@@ -135,7 +145,7 @@ def is_hostname_safe(hostname):
     :param hostname: The hostname to be validated.
     :returns: True if valid. False if not.
     """
-    if not isinstance(hostname, six.string_types) or len(hostname) > 255:
+    if not isinstance(hostname, str) or len(hostname) > 255:
         return False
 
     return _is_hostname_safe_re.match(hostname) is not None
@@ -152,7 +162,7 @@ def is_valid_no_proxy(no_proxy):
         (with optional :port).
     :returns: True if no_proxy is valid, False otherwise.
     """
-    if not isinstance(no_proxy, six.string_types):
+    if not isinstance(no_proxy, str):
         return False
     hostname_re = re.compile('(?!-)[A-Z\\d-]{1,63}(?<!-)$', re.IGNORECASE)
     for hostname in no_proxy.split(','):
@@ -218,8 +228,7 @@ def _get_hash_object(hash_algo_name):
     :raises: InvalidParameterValue, on unsupported or invalid input.
     :returns: a hash object based on the given named algorithm.
     """
-    algorithms = (hashlib.algorithms_guaranteed if six.PY3
-                  else hashlib.algorithms)
+    algorithms = hashlib.algorithms_guaranteed
     if hash_algo_name not in algorithms:
         msg = (_("Unsupported/Invalid hash name '%s' provided.")
                % hash_algo_name)
@@ -229,42 +238,19 @@ def _get_hash_object(hash_algo_name):
     return getattr(hashlib, hash_algo_name)()
 
 
-def hash_file(file_like_object, hash_algo='md5'):
-    """Generate a hash for the contents of a file.
-
-    It returns a hash of the file object as a string of double length,
-    containing only hexadecimal digits. It supports all the algorithms
-    hashlib does.
-    :param file_like_object: file like object whose hash to be calculated.
-    :param hash_algo: name of the hashing strategy, default being 'md5'.
-    :raises: InvalidParameterValue, on unsupported or invalid input.
-    :returns: a condensed digest of the bytes of contents.
-    """
-    checksum = _get_hash_object(hash_algo)
-    while True:
-        chunk = file_like_object.read(32768)
-        if not chunk:
-            break
-        encoded_chunk = (chunk.encode(encoding='utf-8')
-                         if isinstance(chunk, six.string_types) else chunk)
-        checksum.update(encoded_chunk)
-    return checksum.hexdigest()
-
-
-def file_has_content(path, content, hash_algo='md5'):
+def file_has_content(path, content, hash_algo='sha256'):
     """Checks that content of the file is the same as provided reference.
 
     :param path: path to file
     :param content: reference content to check against
-    :param hash_algo: hashing algo from hashlib to use, default is 'md5'
+    :param hash_algo: hashing algo from hashlib to use, default is 'sha256'
     :returns: True if the hash of reference content is the same as
         the hash of file's content, False otherwise
     """
-    with open(path, 'rb') as existing:
-        file_hash_hex = hash_file(existing, hash_algo=hash_algo)
+    file_hash_hex = fileutils.compute_file_checksum(path, algorithm=hash_algo)
     ref_hash = _get_hash_object(hash_algo)
     encoded_content = (content.encode(encoding='utf-8')
-                       if isinstance(content, six.string_types) else content)
+                       if isinstance(content, str) else content)
     ref_hash.update(encoded_content)
     return file_hash_hex == ref_hash.hexdigest()
 
@@ -316,7 +302,7 @@ def safe_rstrip(value, chars=None):
     :return: Stripped value.
 
     """
-    if not isinstance(value, six.string_types):
+    if not isinstance(value, str):
         LOG.warning("Failed to remove trailing character. Returning "
                     "original object. Supplied object is not a string: "
                     "%s,", value)
@@ -496,7 +482,12 @@ def render_template(template, params, is_file=True):
     else:
         tmpl_name = 'template'
         loader = jinja2.DictLoader({tmpl_name: template})
-    env = jinja2.Environment(loader=loader)
+    # NOTE(pas-ha) bandit does not seem to cope with such syntaxis
+    # and still complains with B701 for that line
+    # NOTE(pas-ha) not using default_for_string=False as we set the name
+    # of the template above for strings too.
+    env = jinja2.Environment(loader=loader,  # nosec B701
+                             autoescape=jinja2.select_autoescape())
     tmpl = env.get_template(tmpl_name)
     return tmpl.render(params, enumerate=enumerate)
 
@@ -537,7 +528,7 @@ def parse_instance_info_capabilities(node):
         raise exception.InvalidParameterValue(error_msg)
 
     capabilities = node.instance_info.get('capabilities', {})
-    if isinstance(capabilities, six.string_types):
+    if isinstance(capabilities, str):
         try:
             capabilities = jsonutils.loads(capabilities)
         except (ValueError, TypeError):
@@ -550,7 +541,42 @@ def parse_instance_info_capabilities(node):
 
 
 def validate_conductor_group(conductor_group):
-    if not isinstance(conductor_group, six.string_types):
+    if not isinstance(conductor_group, str):
         raise exception.InvalidConductorGroup(group=conductor_group)
     if not re.match(r'^[a-zA-Z0-9_\-\.]*$', conductor_group):
         raise exception.InvalidConductorGroup(group=conductor_group)
+
+
+def set_node_nested_field(node, collection, field, value):
+    """Set a value in a dictionary field of a node.
+
+    :param node: Node object.
+    :param collection: Name of the field with the dictionary.
+    :param field: Nested field name.
+    :param value: New value.
+    """
+    col = getattr(node, collection)
+    col[field] = value
+    setattr(node, collection, col)
+
+
+def pop_node_nested_field(node, collection, field, default=None):
+    """Pop a value from a dictionary field of a node.
+
+    :param node: Node object.
+    :param collection: Name of the field with the dictionary.
+    :param field: Nested field name.
+    :param default: The default value to return.
+    :return: The removed value or the default.
+    """
+    col = getattr(node, collection)
+    result = col.pop(field, default)
+    setattr(node, collection, col)
+    return result
+
+
+def wrap_ipv6(ip):
+    """Wrap the address in square brackets if it's an IPv6 address."""
+    if ipaddress.ip_address(ip).version == 6:
+        return "[%s]" % ip
+    return ip
