@@ -25,10 +25,12 @@ import oslo_messaging as messaging
 from ironic.common import exception
 from ironic.common import hash_ring
 from ironic.common.i18n import _
+from ironic.common.json_rpc import client as json_rpc
 from ironic.common import release_mappings as versions
 from ironic.common import rpc
 from ironic.conductor import manager
 from ironic.conf import CONF
+from ironic.db import api as dbapi
 from ironic.objects import base as objects_base
 
 
@@ -96,13 +98,19 @@ class ConductorAPI(object):
     |    1.45 - Added continue_node_deploy
     |    1.46 - Added reset_interfaces to update_node
     |    1.47 - Added support for conductor groups
+    |    1.48 - Added allocation API
+    |    1.49 - Added get_node_with_token and agent_token argument to
+                heartbeat
+    |    1.50 - Added set_indicator_state, get_indicator_state and
+    |           get_supported_indicators.
+    |    1.51 - Added agent_verify_ca to heartbeat.
 
     """
 
     # NOTE(rloo): This must be in sync with manager.ConductorManager's.
     # NOTE(pas-ha): This also must be in sync with
     #               ironic.common.release_mappings.RELEASE_MAPPING['master']
-    RPC_API_VERSION = '1.47'
+    RPC_API_VERSION = '1.51'
 
     def __init__(self, topic=None):
         super(ConductorAPI, self).__init__()
@@ -110,19 +118,43 @@ class ConductorAPI(object):
         if self.topic is None:
             self.topic = manager.MANAGER_TOPIC
 
-        target = messaging.Target(topic=self.topic,
-                                  version='1.0')
         serializer = objects_base.IronicObjectSerializer()
         release_ver = versions.RELEASE_MAPPING.get(CONF.pin_release_version)
         version_cap = (release_ver['rpc'] if release_ver
                        else self.RPC_API_VERSION)
-        self.client = rpc.get_client(target, version_cap=version_cap,
-                                     serializer=serializer)
+
+        if CONF.rpc_transport == 'json-rpc':
+            self.client = json_rpc.Client(serializer=serializer,
+                                          version_cap=version_cap)
+            self.topic = ''
+        else:
+            target = messaging.Target(topic=self.topic, version='1.0')
+            self.client = rpc.get_client(target, version_cap=version_cap,
+                                         serializer=serializer)
 
         use_groups = self.client.can_send_version('1.47')
-        # NOTE(deva): this is going to be buggy
+        # NOTE(tenbrae): this is going to be buggy
         self.ring_manager = hash_ring.HashRingManager(use_groups=use_groups)
 
+    def get_conductor_for(self, node):
+        """Get the conductor which the node is mapped to.
+
+        :param node: a node object.
+        :returns: the conductor hostname.
+        :raises: NoValidHost
+
+        """
+        try:
+            ring = self.ring_manager.get_ring(node.driver,
+                                              node.conductor_group)
+            dest = ring.get_nodes(node.uuid.encode('utf-8'))
+            return dest.pop()
+        except exception.DriverNotFound:
+            reason = (_('No conductor service registered which supports '
+                        'driver %(driver)s for conductor group "%(group)s".') %
+                      {'driver': node.driver, 'group': node.conductor_group})
+            raise exception.NoValidHost(reason=reason)
+
     def get_topic_for(self, node):
         """Get the RPC topic for the conductor service the node is mapped to.
 
@@ -131,23 +163,18 @@ class ConductorAPI(object):
         :raises: NoValidHost
 
         """
-        self.ring_manager.reset()
-
-        # There are no conductors, temporary failure - 503 Service Unavailable
-        if not self.ring_manager.ring:
-            raise exception.TemporaryFailure()
+        hostname = self.get_conductor_for(node)
+        return '%s.%s' % (self.topic, hostname)
 
+    def get_random_topic(self):
+        """Get an RPC topic for a random conductor service."""
+        conductors = dbapi.get_instance().get_online_conductors()
         try:
-            ring = self.ring_manager.get_ring(node.driver,
-                                              node.conductor_group)
-            dest = ring.get_nodes(node.uuid.encode('utf-8'),
-                                  replicas=CONF.hash_distribution_replicas)
-            return '%s.%s' % (self.topic, dest.pop())
-        except exception.DriverNotFound:
-            reason = (_('No conductor service registered which supports '
-                        'driver %(driver)s for conductor group "%(group)s".') %
-                      {'driver': node.driver, 'group': node.conductor_group})
-            raise exception.NoValidHost(reason=reason)
+            hostname = random.choice(conductors)
+        except IndexError:
+            # There are no conductors - return 503 Service Unavailable
+            raise exception.TemporaryFailure()
+        return '%s.%s' % (self.topic, hostname)
 
     def get_topic_for_driver(self, driver_name):
         """Get RPC topic name for a conductor supporting the given driver.
@@ -166,10 +193,19 @@ class ConductorAPI(object):
         # does not take groups into account.
         local_ring_manager = hash_ring.HashRingManager(use_groups=False,
                                                        cache=False)
-        ring = local_ring_manager.get_ring(driver_name, '')
+        try:
+            ring = local_ring_manager.get_ring(driver_name, '')
+        except exception.TemporaryFailure:
+            # NOTE(dtantsur): even if no conductors are registered, it makes
+            # sense to report 404 on any driver request.
+            raise exception.DriverNotFound(_("No conductors registered."))
         host = random.choice(list(ring.nodes))
         return self.topic + "." + host
 
+    def get_current_topic(self):
+        """Get RPC topic name for the current conductor."""
+        return self.topic + "." + CONF.host
+
     def can_send_create_port(self):
         """Return whether the RPCAPI supports the create_port method."""
         return self.client.can_send_version("1.41")
@@ -684,6 +720,89 @@ class ConductorAPI(object):
         return cctxt.call(context, 'get_supported_boot_devices',
                           node_id=node_id)
 
+    def set_indicator_state(self, context, node_id, component,
+                            indicator, state, topic=None):
+        """Set node hardware components indicator to the desired state.
+
+        :param context: request context.
+        :param node_id: node id or uuid.
+        :param component: The hardware component, one of
+            :mod:`ironic.common.components`.
+        :param indicator: Indicator IDs, as
+            reported by `get_supported_indicators`)
+        :param state: Indicator state, one of
+            mod:`ironic.common.indicator_states`.
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: NodeLocked if node is locked by another conductor.
+        :raises: UnsupportedDriverExtension if the node's driver doesn't
+                 support management.
+        :raises: InvalidParameterValue when the wrong driver info is
+                 specified or an invalid boot device is specified.
+        :raises: MissingParameterValue if missing supplied info.
+
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.50')
+        return cctxt.call(context, 'set_indicator_state', node_id=node_id,
+                          component=component, indicator=indicator,
+                          state=state)
+
+    def get_indicator_state(self, context, node_id, component, indicator,
+                            topic=None):
+        """Get node hardware component indicator state.
+
+        :param context: request context.
+        :param node_id: node id or uuid.
+        :param component: The hardware component, one of
+            :mod:`ironic.common.components`.
+        :param indicator: Indicator IDs, as
+            reported by `get_supported_indicators`)
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: NodeLocked if node is locked by another conductor.
+        :raises: UnsupportedDriverExtension if the node's driver doesn't
+                 support management.
+        :raises: InvalidParameterValue when the wrong driver info is
+                 specified.
+        :raises: MissingParameterValue if missing supplied info.
+        :returns: Indicator state, one of
+            mod:`ironic.common.indicator_states`.
+
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.50')
+        return cctxt.call(context, 'get_indicator_state', node_id=node_id,
+                          component=component, indicator=indicator)
+
+    def get_supported_indicators(self, context, node_id,
+                                 component=None, topic=None):
+        """Get node hardware components and their indicators.
+
+        :param context: request context.
+        :param node_id: node id or uuid.
+        :param component: The hardware component, one of
+            :mod:`ironic.common.components`.
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: NodeLocked if node is locked by another conductor.
+        :raises: UnsupportedDriverExtension if the node's driver doesn't
+                 support management.
+        :raises: InvalidParameterValue when the wrong driver info is
+                 specified.
+        :raises: MissingParameterValue if missing supplied info.
+        :returns: A dictionary of hardware components
+            (:mod:`ironic.common.components`) as keys with indicator IDs
+            as values.
+
+                ::
+
+                    {
+                        'chassis': ['enclosure-0'],
+                        'system': ['blade-A']
+                        'drive': ['ssd0']
+                    }
+
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.50')
+        return cctxt.call(context, 'get_supported_indicators', node_id=node_id,
+                          component=component)
+
     def inspect_hardware(self, context, node_id, topic=None):
         """Signals the conductor service to perform hardware introspection.
 
@@ -784,20 +903,29 @@ class ConductorAPI(object):
                           node_id=node_id, clean_steps=clean_steps)
 
     def heartbeat(self, context, node_id, callback_url, agent_version,
-                  topic=None):
+                  agent_token=None, agent_verify_ca=None, topic=None):
         """Process a node heartbeat.
 
         :param context: request context.
         :param node_id: node ID or UUID.
         :param callback_url: URL to reach back to the ramdisk.
         :param topic: RPC topic. Defaults to self.topic.
+        :param agent_token: randomly generated validation token.
         :param agent_version: the version of the agent that is heartbeating
+        :param agent_verify_ca: TLS certificate for the agent.
+        :raises: InvalidParameterValue if an invalid agent token is received.
         """
         new_kws = {}
         version = '1.34'
         if self.client.can_send_version('1.42'):
             version = '1.42'
             new_kws['agent_version'] = agent_version
+        if self.client.can_send_version('1.49'):
+            version = '1.49'
+            new_kws['agent_token'] = agent_token
+        if self.client.can_send_version('1.51'):
+            version = '1.51'
+            new_kws['agent_verify_ca'] = agent_verify_ca
         cctxt = self.client.prepare(topic=topic or self.topic, version=version)
         return cctxt.call(context, 'heartbeat', node_id=node_id,
                           callback_url=callback_url, **new_kws)
@@ -1084,3 +1212,38 @@ class ConductorAPI(object):
         cctxt = self.client.prepare(topic=topic or self.topic, version='1.44')
         return cctxt.call(context, 'remove_node_traits', node_id=node_id,
                           traits=traits)
+
+    def create_allocation(self, context, allocation, topic=None):
+        """Create an allocation.
+
+        :param context: request context.
+        :param allocation: an allocation object.
+        :param topic: RPC topic. Defaults to self.topic.
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.48')
+        return cctxt.call(context, 'create_allocation', allocation=allocation)
+
+    def destroy_allocation(self, context, allocation, topic=None):
+        """Delete an allocation.
+
+        :param context: request context.
+        :param allocation: an allocation object.
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: InvalidState if the associated node is in the wrong provision
+            state to perform deallocation.
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.48')
+        return cctxt.call(context, 'destroy_allocation', allocation=allocation)
+
+    def get_node_with_token(self, context, node_id, topic=None):
+        """Request the node from the conductor with an agent token
+
+        :param context: request context.
+        :param node_id: node ID or UUID.
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: NodeLocked if node is locked by another conductor.
+
+        :returns: A Node object with agent token.
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.49')
+        return cctxt.call(context, 'get_node_with_token', node_id=node_id)
