@@ -20,7 +20,6 @@ from ironic.conf import ansible
 from ironic.conf import api
 from ironic.conf import audit
 from ironic.conf import cinder
-from ironic.conf import cisco
 from ironic.conf import conductor
 from ironic.conf import console
 from ironic.conf import database
@@ -30,15 +29,17 @@ from ironic.conf import dhcp
 from ironic.conf import drac
 from ironic.conf import glance
 from ironic.conf import healthcheck
+from ironic.conf import ibmc
 from ironic.conf import ilo
 from ironic.conf import inspector
 from ironic.conf import ipmi
 from ironic.conf import irmc
 from ironic.conf import iscsi
+from ironic.conf import json_rpc
 from ironic.conf import metrics
 from ironic.conf import metrics_statsd
 from ironic.conf import neutron
-from ironic.conf import oneview
+from ironic.conf import nova
 from ironic.conf import pxe
 from ironic.conf import redfish
 from ironic.conf import service_catalog
@@ -53,7 +54,6 @@ ansible.register_opts(CONF)
 api.register_opts(CONF)
 audit.register_opts(CONF)
 cinder.register_opts(CONF)
-cisco.register_opts(CONF)
 conductor.register_opts(CONF)
 console.register_opts(CONF)
 database.register_opts(CONF)
@@ -63,15 +63,17 @@ drac.register_opts(CONF)
 dhcp.register_opts(CONF)
 glance.register_opts(CONF)
 healthcheck.register_opts(CONF)
+ibmc.register_opts(CONF)
 ilo.register_opts(CONF)
 inspector.register_opts(CONF)
 ipmi.register_opts(CONF)
 irmc.register_opts(CONF)
 iscsi.register_opts(CONF)
+json_rpc.register_opts(CONF)
 metrics.register_opts(CONF)
 metrics_statsd.register_opts(CONF)
 neutron.register_opts(CONF)
-oneview.register_opts(CONF)
+nova.register_opts(CONF)
 pxe.register_opts(CONF)
 redfish.register_opts(CONF)
 service_catalog.register_opts(CONF)
