@@ -30,22 +30,12 @@ opts = [
     cfg.IntOpt('heartbeat_interval',
                default=10,
                help=_('Seconds between conductor heart beats.')),
-    cfg.URIOpt('api_url',
-               schemes=('http', 'https'),
-               deprecated_for_removal=True,
-               deprecated_reason=_("Use [service_catalog]endpoint_override "
-                                   "option instead if required to use "
-                                   "a specific ironic api address, "
-                                   "for example in noauth mode."),
-               help=_('URL of Ironic API service. If not set ironic can '
-                      'get the current value from the keystone service '
-                      'catalog. If set, the value must start with either '
-                      'http:// or https://.')),
     cfg.IntOpt('heartbeat_timeout',
                default=60,
                # We're using timedelta which can overflow if somebody sets this
                # too high, so limit to a sane value of 10 years.
                max=315576000,
+               mutable=True,
                help=_('Maximum time (in seconds) since the last check-in '
                       'of a conductor. A conductor is considered inactive '
                       'when this time has been exceeded.')),
@@ -70,10 +60,12 @@ opts = [
                       'in seconds. Set to 0 to disable checks.')),
     cfg.IntOpt('deploy_callback_timeout',
                default=1800,
+               min=0,
                help=_('Timeout (seconds) to wait for a callback from '
                       'a deploy ramdisk. Set to 0 to disable timeout.')),
     cfg.BoolOpt('force_power_state_during_sync',
                 default=True,
+                mutable=True,
                 help=_('During sync_power_state, should the hardware power '
                        'state be set to the state recorded in the database '
                        '(True) or should the database be updated based on '
@@ -158,10 +150,12 @@ opts = [
                       'configdrive_use_object_store is True.')),
     cfg.IntOpt('inspect_wait_timeout',
                default=1800,
+               min=0,
                help=_('Timeout (seconds) for waiting for node inspection. '
                       '0 - unlimited.')),
     cfg.BoolOpt('automated_clean',
                 default=True,
+                mutable=True,
                 help=_('Enables or disables automated cleaning. Automated '
                        'cleaning is a configurable set of steps, '
                        'such as erasing disk drives, that are performed on '
@@ -192,6 +186,7 @@ opts = [
                        'maintenance will make the process continue.')),
     cfg.IntOpt('clean_callback_timeout',
                default=1800,
+               min=0,
                help=_('Timeout (seconds) to wait for a callback from the '
                       'ramdisk doing the cleaning. If the timeout is reached '
                       'the node will be put in the "clean failed" provision '
@@ -206,10 +201,12 @@ opts = [
     cfg.IntOpt('soft_power_off_timeout',
                default=600,
                min=1,
+               mutable=True,
                help=_('Timeout (in seconds) of soft reboot and soft power '
                       'off operation. This value always has to be positive.')),
     cfg.IntOpt('power_state_change_timeout',
-               min=2, default=30,
+               min=2, default=60,
+               mutable=True,
                help=_('Number of seconds to wait for power operations to '
                       'complete, i.e., so that a baremetal node is in the '
                       'desired power state. If timed out, the power operation '
@@ -252,6 +249,20 @@ opts = [
                mutable=True,
                help=_('Glance ID, http:// or file:// URL of the initramfs of '
                       'the default rescue image.')),
+    cfg.StrOpt('rescue_password_hash_algorithm',
+               default='sha256',
+               choices=['sha256', 'sha512'],
+               mutable=True,
+               help=_('Password hash algorithm to be used for the rescue '
+                      'password.')),
+    cfg.BoolOpt('require_rescue_password_hashed',
+                # TODO(TheJulia): Change this to True in Victoria.
+                default=False,
+                mutable=True,
+                help=_('Option to cause the conductor to not fallback to '
+                       'an un-hashed version of the rescue password, '
+                       'permitting rescue with older ironic-python-agent '
+                       'ramdisks.')),
     cfg.StrOpt('bootloader',
                mutable=True,
                help=_('Glance ID, http:// or file:// URL of the EFI system '
