@@ -17,6 +17,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import hashlib
 import os
 import socket
 import tempfile
@@ -58,10 +59,15 @@ api_opts = [
         default='keystone',
         choices=[('noauth', _('no authentication')),
                  ('keystone', _('use the Identity service for '
-                                'authentication'))],
+                                'authentication')),
+                 ('http_basic', _('HTTP basic authentication'))],
         help=_('Authentication strategy used by ironic-api. "noauth" should '
                'not be used in a production environment because all '
                'authentication will be disabled.')),
+    cfg.StrOpt('http_basic_auth_user_file',
+               default='/etc/ironic/htpasswd',
+               help=_('Path to Apache format user authentication file used '
+                      'when auth_strategy=http_basic')),
     cfg.BoolOpt('debug_tracebacks_in_api',
                 default=False,
                 help=_('Return server tracebacks in the API response for any '
@@ -72,20 +78,12 @@ api_opts = [
                 help=_('Enable pecan debug mode. WARNING: this is insecure '
                        'and should not be used in a production environment.')),
     cfg.StrOpt('default_resource_class',
+               mutable=True,
                help=_('Resource class to use for new nodes when no resource '
                       'class is provided in the creation request.')),
 ]
 
 driver_opts = [
-    cfg.ListOpt('enabled_drivers',
-                default=[],
-                help=_('This option is left for a start up check only. '
-                       'Any non-empty value will prevent the conductor '
-                       'from starting.'),
-                deprecated_for_removal=True,
-                deprecated_reason=_('Hardware types should be used instead '
-                                    'of classic drivers. They are enabled '
-                                    'via the enabled_hardware_types option.')),
     cfg.ListOpt('enabled_hardware_types',
                 default=['ipmi'],
                 help=_('Specify the list of hardware types to load during '
@@ -113,7 +111,7 @@ driver_opts = [
     cfg.StrOpt('default_console_interface',
                help=_DEFAULT_IFACE_HELP.format('console')),
     cfg.ListOpt('enabled_deploy_interfaces',
-                default=['iscsi', 'direct'],
+                default=['direct'],
                 help=_ENABLED_IFACE_HELP.format('deploy')),
     cfg.StrOpt('default_deploy_interface',
                help=_DEFAULT_IFACE_HELP.format('deploy')),
@@ -151,6 +149,7 @@ driver_opts = [
                 default=['cinder', 'noop'],
                 help=_ENABLED_IFACE_HELP.format('storage')),
     cfg.StrOpt('default_storage_interface',
+               default='noop',
                help=_DEFAULT_IFACE_HELP.format('storage')),
     cfg.ListOpt('enabled_vendor_interfaces',
                 default=['ipmitool', 'no-vendor'],
@@ -160,12 +159,9 @@ driver_opts = [
 ]
 
 exc_log_opts = [
-    cfg.BoolOpt('fatal_exception_format_errors',
-                default=False,
-                help=_('Used if there is a formatting error when generating '
-                       'an exception message (a programming error). If True, '
-                       'raise an exception; if False, use the unformatted '
-                       'message.')),
+    cfg.IntOpt('log_in_db_max_size', default=4096,
+               help=_('Max number of characters of any node '
+                      'last_error/maintenance_reason pushed to database.'))
 ]
 
 hash_opts = [
@@ -183,22 +179,26 @@ hash_opts = [
                       'smooth in most cases. The default is suitable for up '
                       'to a few hundred conductors. Configuring for too many '
                       'partitions has a negative impact on CPU usage.')),
-    cfg.IntOpt('hash_distribution_replicas',
-               default=1,
-               help=_('[Experimental Feature] '
-                      'Number of hosts to map onto each hash partition. '
-                      'Setting this to more than one will cause additional '
-                      'conductor services to prepare deployment environments '
-                      'and potentially allow the Ironic cluster to recover '
-                      'more quickly if a conductor instance is terminated.')),
     cfg.IntOpt('hash_ring_reset_interval',
-               default=180,
-               help=_('Interval (in seconds) between hash ring resets.')),
+               default=15,
+               help=_('Time (in seconds) after which the hash ring is '
+                      'considered outdated and is refreshed on the next '
+                      'access.')),
+    cfg.StrOpt('hash_ring_algorithm',
+               default='md5',
+               advanced=True,
+               choices=hashlib.algorithms_guaranteed,
+               help=_('Hash function to use when building the hash ring. '
+                      'If running on a FIPS system, do not use md5. '
+                      'WARNING: all ironic services in a cluster MUST use '
+                      'the same algorithm at all times. Changing the '
+                      'algorithm requires an offline update.')),
 ]
 
 image_opts = [
     cfg.BoolOpt('force_raw_images',
                 default=True,
+                mutable=True,
                 help=_('If True, convert backing images to "raw" disk image '
                        'format.')),
     cfg.StrOpt('isolinux_bin',
@@ -208,6 +208,10 @@ image_opts = [
                default=os.path.join('$pybasedir',
                                     'common/isolinux_config.template'),
                help=_('Template file for isolinux configuration file.')),
+    cfg.StrOpt('grub_config_path',
+               default='/boot/grub/grub.cfg',
+               help=_('GRUB2 configuration file location on the UEFI ISO '
+                      'images produced by ironic.')),
     cfg.StrOpt('grub_config_template',
                default=os.path.join('$pybasedir',
                                     'common/grub_conf.template'),
@@ -218,11 +222,24 @@ image_opts = [
                       'looked for in '
                       '"/usr/lib/syslinux/modules/bios/ldlinux.c32" and '
                       '"/usr/share/syslinux/ldlinux.c32".')),
+    cfg.StrOpt('esp_image',
+               help=_('Path to EFI System Partition image file. This file is '
+                      'recommended for creating UEFI bootable ISO images '
+                      'efficiently. ESP image should contain a '
+                      'FAT12/16/32-formatted file system holding EFI boot '
+                      'loaders (e.g. GRUB2) for each hardware architecture '
+                      'ironic needs to boot. This option is only used when '
+                      'neither ESP nor ISO deploy image is configured to '
+                      'the node being deployed in which case ironic will '
+                      'attempt to fetch ESP image from the configured '
+                      'location or extract ESP image from UEFI-bootable '
+                      'deploy ISO image.')),
 ]
 
 img_cache_opts = [
     cfg.BoolOpt('parallel_image_downloads',
                 default=False,
+                mutable=True,
                 help=_('Run image downloads and raw format conversions in '
                        'parallel.')),
 ]
@@ -231,15 +248,27 @@ netconf_opts = [
     cfg.StrOpt('my_ip',
                default=netutils.get_my_ipv4(),
                sample_default='127.0.0.1',
-               help=_('IP address of this host. If unset, will determine the '
-                      'IP programmatically. If unable to do so, will use '
-                      '"127.0.0.1".')),
+               help=_('IPv4 address of this host. If unset, will determine '
+                      'the IP programmatically. If unable to do so, will use '
+                      '"127.0.0.1". NOTE: This field does accept an IPv6 '
+                      'address as an override for templates and URLs, '
+                      'however it is recommended that [DEFAULT]my_ipv6 '
+                      'is used along with DNS names for service URLs for '
+                      'dual-stack environments.')),
+    cfg.StrOpt('my_ipv6',
+               default=None,
+               sample_default='2001:db8::1',
+               help=_('IP address of this host using IPv6. This value must '
+                      'be supplied via the configuration and cannot be '
+                      'adequately programmatically determined like the '
+                      '[DEFAULT]my_ip parameter for IPv4.')),
 ]
 
-# NOTE(mariojv) By default, accessing this option when it's unset will return
-# None, indicating no notifications will be sent. oslo.config returns None by
-# default for options without set defaults that aren't required.
 notification_opts = [
+    # NOTE(mariojv) By default, accessing this option when it's unset will
+    # return None, indicating no notifications will be sent. oslo.config
+    # returns None by default for options without set defaults that aren't
+    # required.
     cfg.StrOpt('notification_level',
                choices=[('debug', _('"debug" level')),
                         ('info', _('"info" level')),
@@ -248,7 +277,22 @@ notification_opts = [
                         ('critical', _('"critical" level'))],
                help=_('Specifies the minimum level for which to send '
                       'notifications. If not set, no notifications will '
-                      'be sent. The default is for this option to be unset.'))
+                      'be sent. The default is for this option to be unset.')),
+    cfg.ListOpt(
+        'versioned_notifications_topics',
+        default=['ironic_versioned_notifications'],
+        help=_("""
+Specifies the topics for the versioned notifications issued by Ironic.
+
+The default value is fine for most deployments and rarely needs to be changed.
+However, if you have a third-party service that consumes versioned
+notifications, it might be worth getting a topic for that service.
+Ironic will send a message containing a versioned notification payload to each
+topic queue in this list.
+
+The list of versioned notifications is visible in
+https://docs.openstack.org/ironic/latest/admin/notifications.html
+""")),
 ]
 
 path_opts = [
@@ -269,6 +313,7 @@ path_opts = [
 portgroup_opts = [
     cfg.StrOpt(
         'default_portgroup_mode', default='active-backup',
+        mutable=True,
         help=_(
             'Default mode for portgroups. Allowed values can be found in the '
             'linux kernel documentation on bonding: '
@@ -297,6 +342,12 @@ service_opts = [
                       'When doing a rolling upgrade from version N to version '
                       'N+1, set (to pin) this to N. To unpin (default), leave '
                       'it unset and the latest versions will be used.')),
+    cfg.StrOpt('rpc_transport',
+               default='oslo',
+               choices=[('oslo', _('use oslo.messaging transport')),
+                        ('json-rpc', _('use JSON RPC transport'))],
+               help=_('Which RPC transport implementation to use between '
+                      'conductor and API services')),
 ]
 
 utils_opts = [
@@ -306,11 +357,31 @@ utils_opts = [
                       'running commands as root.')),
     cfg.StrOpt('tempdir',
                default=tempfile.gettempdir(),
-               sample_default='/tmp',
+               sample_default=tempfile.gettempdir(),
                help=_('Temporary working directory, default is Python temp '
                       'dir.')),
 ]
 
+cert_verify_opts = [
+    cfg.StrOpt('webserver_verify_ca',
+               default='True',
+               mutable=True,
+               help=_('CA certificates to be used for certificate '
+                      'verification. This can be either a Boolean value '
+                      'or a path to a CA_BUNDLE file.'
+                      'If set to True, the certificates present in the '
+                      'standard path are used to verify the host '
+                      'certificates.'
+                      'If set to False, the conductor will ignore verifying '
+                      'the SSL certificate presented by the host.'
+                      'If it"s a path, conductor uses the specified '
+                      'certificate for SSL verification. If the path does '
+                      'not exist, the behavior is same as when this value '
+                      'is set to True i.e the certificates present in the '
+                      'standard path are used for SSL verification.'
+                      'Defaults to True.')),
+]
+
 
 def register_opts(conf):
     conf.register_opts(api_opts)
@@ -325,3 +396,4 @@ def register_opts(conf):
     conf.register_opts(portgroup_opts)
     conf.register_opts(service_opts)
     conf.register_opts(utils_opts)
+    conf.register_opts(cert_verify_opts)
