@@ -40,14 +40,6 @@ opts = [
                        'is set to False, then Ironic will use Swift '
                        'to host the floppy images and generated '
                        'boot_iso.')),
-    cfg.IntOpt('clean_priority_erase_devices',
-               deprecated_for_removal=True,
-               deprecated_reason=_('This configuration option is duplicated '
-                                   'by [deploy] erase_devices_priority, '
-                                   'please use that instead.'),
-               help=_('Priority for erase devices clean step. If unset, '
-                      'it defaults to 10. If set to 0, the step will be '
-                      'disabled and will not run during cleaning.')),
     cfg.IntOpt('clean_priority_reset_ilo',
                default=0,
                help=_('Priority for reset_ilo clean step.')),
@@ -70,25 +62,55 @@ opts = [
                       'step requires "ilo_change_password" parameter to be '
                       'updated in nodes\'s driver_info with the new '
                       'password.')),
-    cfg.IntOpt('power_retry',
-               default=6,
-               help=_('Number of times a power operation needs to be '
-                      'retried')),
     cfg.IntOpt('power_wait',
                default=2,
                help=_('Amount of time in seconds to wait in between power '
                       'operations')),
+    cfg.IntOpt('oob_erase_devices_job_status_interval',
+               min=10,
+               default=300,
+               help=_('Interval (in seconds) between periodic erase-devices '
+                      'status checks to determine whether the asynchronous '
+                      'out-of-band erase-devices was successfully finished or '
+                      'not. On an average, a 300GB HDD with default pattern '
+                      '"overwrite" would take approximately 9 hours and '
+                      '300GB SSD with default pattern "block" would take '
+                      'approx. 30 seconds to complete sanitize disk erase.')),
     cfg.StrOpt('ca_file',
+               deprecated_for_removal=True,
+               deprecated_reason=_('Its being replaced by new configuration '
+                                   'parameter "verify_ca".'),
                help=_('CA certificate file to validate iLO.')),
+    cfg.StrOpt('verify_ca',
+               default='True',
+               help=_('CA certificate to validate iLO. This can be either '
+                      'a Boolean value, a path to a CA_BUNDLE file or '
+                      'directory with certificates of trusted CAs. If set '
+                      'to True the driver will verify the host certificates; '
+                      'if False the driver will ignore verifying the SSL '
+                      'certificate. If it\'s a path the driver will use the '
+                      'specified certificate or one of the certificates in '
+                      'the directory. Defaults to True.')),
     cfg.StrOpt('default_boot_mode',
                default='auto',
-               choices=['auto', 'bios', 'uefi'],
+               choices=[('auto', _('based on boot mode settings on the '
+                                   'system')),
+                        ('bios', _('BIOS boot mode')),
+                        ('uefi', _('UEFI boot mode'))],
                help=_('Default boot mode to be used in provisioning when '
                       '"boot_mode" capability is not provided in the '
                       '"properties/capabilities" of the node. The default is '
                       '"auto" for backward compatibility. When "auto" is '
                       'specified, default boot mode will be selected based '
                       'on boot mode settings on the system.')),
+    cfg.IntOpt('file_permission',
+               default=0o644,
+               help=_('File permission for swift-less image hosting with the '
+                      'octal permission representation of file access '
+                      'permissions. This setting defaults to ``644``, '
+                      'or as the octal number ``0o644`` in Python. '
+                      'This setting must be set to the octal number '
+                      'representation, meaning starting with ``0o``.')),
 ]
 
 
