@@ -12,6 +12,8 @@
 
 import itertools
 
+from oslo_log import log
+
 import ironic.conf
 
 _default_opt_lists = [
@@ -24,6 +26,7 @@ _default_opt_lists = [
     ironic.conf.default.netconf_opts,
     ironic.conf.default.notification_opts,
     ironic.conf.default.path_opts,
+    ironic.conf.default.portgroup_opts,
     ironic.conf.default.service_opts,
     ironic.conf.default.utils_opts,
 ]
@@ -31,11 +34,10 @@ _default_opt_lists = [
 _opts = [
     ('DEFAULT', itertools.chain(*_default_opt_lists)),
     ('agent', ironic.conf.agent.opts),
-    ('amt', ironic.conf.amt.opts),
+    ('ansible', ironic.conf.ansible.opts),
     ('api', ironic.conf.api.opts),
     ('audit', ironic.conf.audit.opts),
-    ('cimc', ironic.conf.cisco.cimc_opts),
-    ('cisco_ucs', ironic.conf.cisco.ucsm_opts),
+    ('cinder', ironic.conf.cinder.list_opts()),
     ('conductor', ironic.conf.conductor.opts),
     ('console', ironic.conf.console.opts),
     ('database', ironic.conf.database.opts),
@@ -43,24 +45,23 @@ _opts = [
     ('dhcp', ironic.conf.dhcp.opts),
     ('drac', ironic.conf.drac.opts),
     ('glance', ironic.conf.glance.list_opts()),
-    ('iboot', ironic.conf.iboot.opts),
+    ('healthcheck', ironic.conf.healthcheck.opts),
     ('ilo', ironic.conf.ilo.opts),
     ('inspector', ironic.conf.inspector.list_opts()),
     ('ipmi', ironic.conf.ipmi.opts),
     ('irmc', ironic.conf.irmc.opts),
     ('iscsi', ironic.conf.iscsi.opts),
-    ('keystone', ironic.conf.keystone.opts),
+    ('json_rpc', ironic.conf.json_rpc.list_opts()),
     ('metrics', ironic.conf.metrics.opts),
     ('metrics_statsd', ironic.conf.metrics_statsd.opts),
     ('neutron', ironic.conf.neutron.list_opts()),
-    ('oneview', ironic.conf.oneview.opts),
+    ('nova', ironic.conf.nova.list_opts()),
     ('pxe', ironic.conf.pxe.opts),
-    ('seamicro', ironic.conf.seamicro.opts),
+    ('redfish', ironic.conf.redfish.opts),
     ('service_catalog', ironic.conf.service_catalog.list_opts()),
     ('snmp', ironic.conf.snmp.opts),
-    ('ssh', ironic.conf.ssh.opts),
     ('swift', ironic.conf.swift.list_opts()),
-    ('virtualbox', ironic.conf.virtualbox.opts),
+    ('xclarity', ironic.conf.xclarity.opts),
 ]
 
 
@@ -80,3 +81,26 @@ def list_opts():
     :returns: a list of (group, options) tuples
     """
     return _opts
+
+
+def update_opt_defaults():
+    log.set_defaults(
+        default_log_levels=[
+            'amqp=WARNING',
+            'amqplib=WARNING',
+            'qpid.messaging=INFO',
+            # This comes in two flavors
+            'oslo.messaging=INFO',
+            'oslo_messaging=INFO',
+            'sqlalchemy=WARNING',
+            'stevedore=INFO',
+            'eventlet.wsgi.server=INFO',
+            'iso8601=WARNING',
+            'requests=WARNING',
+            'glanceclient=WARNING',
+            'urllib3.connectionpool=WARNING',
+            'keystonemiddleware.auth_token=INFO',
+            'keystoneauth.session=INFO',
+            'openstack=WARNING',
+        ]
+    )
