@@ -23,9 +23,11 @@ from ironic.common.i18n import _
 opts = [
     cfg.StrOpt('pxe_append_params',
                default='nofb nomodeset vga=normal',
+               mutable=True,
                help=_('Additional append parameters for baremetal PXE boot.')),
     cfg.StrOpt('default_ephemeral_format',
                default='ext4',
+               mutable=True,
                help=_('Default file system format for ephemeral partition, '
                       'if one is created.')),
     cfg.StrOpt('images_path',
@@ -36,7 +38,7 @@ opts = [
                default='/var/lib/ironic/master_images',
                help=_('On the ironic-conductor node, directory where master '
                       'instance images are stored on disk. '
-                      'Setting to <None> disables image caching.')),
+                      'Setting to the empty string disables image caching.')),
     cfg.IntOpt('image_cache_size',
                default=20480,
                help=_('Maximum size (in MiB) of cache for master images, '
@@ -50,16 +52,26 @@ opts = [
     cfg.StrOpt('pxe_config_template',
                default=os.path.join(
                    '$pybasedir', 'drivers/modules/pxe_config.template'),
+               mutable=True,
                help=_('On ironic-conductor node, template file for PXE '
-                      'configuration.')),
+                      'loader configuration.')),
+    cfg.StrOpt('ipxe_config_template',
+               default=os.path.join(
+                   '$pybasedir', 'drivers/modules/ipxe_config.template'),
+               mutable=True,
+               help=_('On ironic-conductor node, template file for iPXE '
+                      'operations.')),
     cfg.StrOpt('uefi_pxe_config_template',
                default=os.path.join(
                    '$pybasedir',
                    'drivers/modules/pxe_grub_config.template'),
+               mutable=True,
                help=_('On ironic-conductor node, template file for PXE '
-                      'configuration for UEFI boot loader.')),
+                      'configuration for UEFI boot loader. Generally this '
+                      'is used for GRUB specific templates.')),
     cfg.DictOpt('pxe_config_template_by_arch',
                 default={},
+                mutable=True,
                 help=_('On ironic-conductor node, template file for PXE '
                        'configuration per node architecture. '
                        'For example: '
@@ -76,7 +88,7 @@ opts = [
                default='/tftpboot/master_images',
                help=_('On ironic-conductor node, directory where master TFTP '
                       'images are stored on disk. '
-                      'Setting to <None> disables image caching.')),
+                      'Setting to the empty string disables image caching.')),
     cfg.IntOpt('dir_permission',
                help=_("The permission that will be applied to the TFTP "
                       "folders upon creation. This should be set to the "
@@ -95,20 +107,29 @@ opts = [
     cfg.StrOpt('pxe_config_subdir',
                default='pxelinux.cfg',
                help=_('Directory in which to create symbolic links which '
-                      'represent the MAC or IP address of the the ports on '
+                      'represent the MAC or IP address of the ports on '
                       'a node and allow boot loaders to load the PXE '
                       'file for the node. This directory name is relative '
                       'to the PXE or iPXE folders.')),
     cfg.StrOpt('uefi_pxe_bootfile_name',
                default='bootx64.efi',
                help=_('Bootfile DHCP parameter for UEFI boot mode.')),
+    cfg.StrOpt('ipxe_bootfile_name',
+               default='undionly.kpxe',
+               help=_('Bootfile DHCP parameter.')),
+    cfg.StrOpt('uefi_ipxe_bootfile_name',
+               default='ipxe.efi',
+               help=_('Bootfile DHCP parameter for UEFI boot mode. If you '
+                      'experience problems with booting using it, try '
+                      'snponly.efi.')),
     cfg.DictOpt('pxe_bootfile_name_by_arch',
                 default={},
                 help=_('Bootfile DHCP parameter per node architecture. '
                        'For example: aarch64:grubaa64.efi')),
-    cfg.BoolOpt('ipxe_enabled',
-                default=False,
-                help=_('Enable iPXE boot.')),
+    cfg.DictOpt('ipxe_bootfile_name_by_arch',
+                default={},
+                help=_('Bootfile DHCP parameter per node architecture. '
+                       'For example: aarch64:ipxe_aa64.efi')),
     cfg.StrOpt('ipxe_boot_script',
                default=os.path.join(
                    '$pybasedir', 'drivers/modules/boot.ipxe'),
@@ -118,22 +139,42 @@ opts = [
                default=0,
                help=_('Timeout value (in seconds) for downloading an image '
                       'via iPXE. Defaults to 0 (no timeout)')),
+    cfg.IntOpt('boot_retry_timeout',
+               min=60,
+               help=_('Timeout (in seconds) after which PXE boot should be '
+                      'retried. Must be less than [conductor]'
+                      'deploy_callback_timeout. Disabled by default.')),
+    cfg.IntOpt('boot_retry_check_interval',
+               default=90, min=1,
+               help=_('Interval (in seconds) between periodic checks on PXE '
+                      'boot retry. Has no effect if boot_retry_timeout '
+                      'is not set.')),
     cfg.StrOpt('ip_version',
                default='4',
                choices=[('4', _('IPv4')),
                         ('6', _('IPv6'))],
+               mutable=True,
                help=_('The IP version that will be used for PXE booting. '
                       'Defaults to 4. EXPERIMENTAL')),
     cfg.BoolOpt('ipxe_use_swift',
                 default=False,
+                mutable=True,
                 help=_("Download deploy and rescue images directly from swift "
                        "using temporary URLs. "
                        "If set to false (default), images are downloaded "
                        "to the ironic-conductor node and served over its "
                        "local HTTP server. "
-                       "Applicable only when 'ipxe_enabled' option is "
-                       "set to true.")),
-
+                       "Applicable only when 'ipxe' compatible boot interface "
+                       "is used.")),
+    cfg.BoolOpt('enable_netboot_fallback',
+                default=False,
+                mutable=True,
+                help=_('If True, generate a PXE environment even for nodes '
+                       'that use local boot. This is useful when the driver '
+                       'cannot switch nodes to local boot, e.g. with SNMP '
+                       'or with Redfish on machines that cannot do persistent '
+                       'boot. Mostly useful for standalone ironic since '
+                       'Neutron will prevent incorrect PXE boot.')),
 ]
 
 
