@@ -18,6 +18,7 @@ Ansible deploy interface
 import json
 import os
 import shlex
+from urllib import parse as urlparse
 
 from ironic_lib import metrics_utils
 from ironic_lib import utils as irlib_utils
@@ -26,8 +27,6 @@ from oslo_log import log
 from oslo_utils import strutils
 from oslo_utils import units
 import retrying
-import six
-import six.moves.urllib.parse as urlparse
 import yaml
 
 from ironic.common import dhcp_factory
@@ -36,11 +35,12 @@ from ironic.common.i18n import _
 from ironic.common import images
 from ironic.common import states
 from ironic.common import utils
+from ironic.conductor import steps as conductor_steps
 from ironic.conductor import task_manager
 from ironic.conductor import utils as manager_utils
 from ironic.conf import CONF
 from ironic.drivers import base
-from ironic.drivers.modules import agent_base_vendor as agent_base
+from ironic.drivers.modules import agent_base
 from ironic.drivers.modules import deploy_utils
 
 
@@ -73,6 +73,8 @@ OPTIONAL_PROPERTIES = {
     'ansible_clean_steps_config': _('Name of the file inside the '
                                     '"ansible_playbooks_path" folder with '
                                     'cleaning steps configuration. Optional.'),
+    'ansible_python_interpreter': _('Absolute path to the python interpreter '
+                                    'on the managed machines. Optional.'),
 }
 
 COMMON_PROPERTIES = OPTIONAL_PROPERTIES
@@ -101,6 +103,11 @@ def _parse_ansible_driver_info(node, action='deploy'):
     return os.path.basename(playbook), user, key
 
 
+def _get_python_interpreter(node):
+    return node.driver_info.get('ansible_python_interpreter',
+                                CONF.ansible.default_python_interpreter)
+
+
 def _get_configdrive_path(basename):
     return os.path.join(CONF.tempdir, basename + '.cndrive')
 
@@ -126,6 +133,9 @@ def _run_playbook(node, name, extra_vars, key, tags=None, notags=None):
     playbook = os.path.join(root, name)
     inventory = os.path.join(root, 'inventory')
     ironic_vars = {'ironic': extra_vars}
+    python_interpreter = _get_python_interpreter(node)
+    if python_interpreter:
+        ironic_vars['ansible_python_interpreter'] = python_interpreter
     args = [CONF.ansible.ansible_playbook_script, playbook,
             '-i', inventory,
             '-e', json.dumps(ironic_vars),
@@ -219,19 +229,14 @@ def _parse_partitioning_info(node):
 
 def _parse_root_device_hints(node):
     """Convert string with hints to dict. """
-    root_device = node.properties.get('root_device')
-    if not root_device:
+    parsed_hints = deploy_utils.get_root_device_for_deploy(node)
+    if not parsed_hints:
         return {}
-    try:
-        parsed_hints = irlib_utils.parse_root_device_hints(root_device)
-    except ValueError as e:
-        raise exception.InvalidParameterValue(
-            _('Failed to validate the root device hints for node %(node)s. '
-              'Error: %(error)s') % {'node': node.uuid, 'error': e})
+
     root_device_hints = {}
     advanced = {}
     for hint, value in parsed_hints.items():
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             if value.startswith('== '):
                 root_device_hints[hint] = int(value[3:])
             elif value.startswith('s== '):
@@ -266,7 +271,6 @@ def _prepare_variables(task):
     for i_key, i_value in i_info.items():
         if i_key.startswith('image_'):
             image[i_key[6:]] = i_value
-    image['mem_req'] = _calculate_memory_req(task)
 
     checksum = image.get('checksum')
     if checksum:
@@ -371,9 +375,13 @@ def _get_clean_steps(node, interface=None, override_priorities=None):
     return steps
 
 
-class AnsibleDeploy(agent_base.HeartbeatMixin, base.DeployInterface):
+class AnsibleDeploy(agent_base.HeartbeatMixin,
+                    agent_base.AgentOobStepsMixin,
+                    base.DeployInterface):
     """Interface for deploy-related actions."""
 
+    has_decomposed_deploy_steps = True
+
     def __init__(self):
         super(AnsibleDeploy, self).__init__()
         # NOTE(pas-ha) overriding agent creation as we won't be
@@ -418,8 +426,10 @@ class AnsibleDeploy(agent_base.HeartbeatMixin, base.DeployInterface):
                   {'node': node.uuid, 'ip': node_address})
         variables = _prepare_variables(task)
         if not node.driver_internal_info.get('is_whole_disk_image'):
-            variables.update(_parse_partitioning_info(task.node))
-        playbook, user, key = _parse_ansible_driver_info(task.node)
+            variables.update(_parse_partitioning_info(node))
+        if node.target_raid_config:
+            variables.update({'raid_config': node.target_raid_config})
+        playbook, user, key = _parse_ansible_driver_info(node)
         node_list = [(node.uuid, node_address, user, node.extra)]
         extra_vars = _prepare_extra_vars(node_list, variables=variables)
 
@@ -432,15 +442,43 @@ class AnsibleDeploy(agent_base.HeartbeatMixin, base.DeployInterface):
     @task_manager.require_exclusive_lock
     def deploy(self, task):
         """Perform a deployment to a node."""
+        self._required_image_info(task)
         manager_utils.node_power_action(task, states.REBOOT)
         return states.DEPLOYWAIT
 
+    def process_next_step(self, task, step_type):
+        """Start the next clean/deploy step if the previous one is complete.
+
+        :param task: a TaskManager instance
+        :param step_type: "clean" or "deploy"
+        """
+        # Run the next step as soon as agent heartbeats in deploy.deploy
+        if step_type == 'deploy' and self.in_core_deploy_step(task):
+            manager_utils.notify_conductor_resume_deploy(task)
+
+    @staticmethod
+    def _required_image_info(task):
+        """Gather and save needed image info while the context is good.
+
+        Gather image info that will be needed later, during the
+        write_image execution, where the context won't be the same
+        anymore, since coming from the server's heartbeat.
+        """
+        node = task.node
+        i_info = node.instance_info
+        i_info['image_mem_req'] = _calculate_memory_req(task)
+        node.instance_info = i_info
+        node.save()
+
     @METRICS.timer('AnsibleDeploy.tear_down')
     @task_manager.require_exclusive_lock
     def tear_down(self, task):
         """Tear down a previous deployment on the task's node."""
         manager_utils.node_power_action(task, states.POWER_OFF)
+        power_state_to_restore = manager_utils.power_on_node_if_needed(task)
         task.driver.network.unconfigure_tenant_networks(task)
+        manager_utils.restore_power_state_if_needed(
+            task, power_state_to_restore)
         return states.DELETED
 
     @METRICS.timer('AnsibleDeploy.prepare')
@@ -451,7 +489,11 @@ class AnsibleDeploy(agent_base.HeartbeatMixin, base.DeployInterface):
         if node.provision_state == states.DEPLOYING:
             # adding network-driver dependent provisioning ports
             manager_utils.node_power_action(task, states.POWER_OFF)
+            power_state_to_restore = (
+                manager_utils.power_on_node_if_needed(task))
             task.driver.network.add_provisioning_network(task)
+            manager_utils.restore_power_state_if_needed(
+                task, power_state_to_restore)
         if node.provision_state not in [states.ACTIVE, states.ADOPTING]:
             node.instance_info = deploy_utils.build_instance_info_for_deploy(
                 task)
@@ -503,22 +545,22 @@ class AnsibleDeploy(agent_base.HeartbeatMixin, base.DeployInterface):
         node_address = _get_node_ip(task)
 
         node_list = [(node.uuid, node_address, user, node.extra)]
-        extra_vars = _prepare_extra_vars(node_list)
+
+        if node.target_raid_config:
+            variables = {'raid_config': node.target_raid_config}
+            extra_vars = _prepare_extra_vars(node_list, variables=variables)
+        else:
+            extra_vars = _prepare_extra_vars(node_list)
 
         LOG.debug('Starting cleaning step %(step)s on node %(node)s',
                   {'node': node.uuid, 'step': stepname})
         step_tags = step['args'].get('tags', [])
-        try:
-            _run_playbook(node, playbook, extra_vars, key, tags=step_tags)
-        except exception.InstanceDeployFailure as e:
-            LOG.error("Ansible failed cleaning step %(step)s "
-                      "on node %(node)s.",
-                      {'node': node.uuid, 'step': stepname})
-            manager_utils.cleaning_error_handler(task, six.text_type(e))
-        else:
-            LOG.info('Ansible completed cleaning step %(step)s '
-                     'on node %(node)s.',
-                     {'node': node.uuid, 'step': stepname})
+        LOG.debug("Detected tags from cleaning step: %(tags)s",
+                  {'tags': step_tags})
+        _run_playbook(node, playbook, extra_vars, key, tags=step_tags)
+        LOG.info('Ansible completed cleaning step %(step)s '
+                 'on node %(node)s.',
+                 {'node': node.uuid, 'step': stepname})
 
     @METRICS.timer('AnsibleDeploy.prepare_cleaning')
     def prepare_cleaning(self, task):
@@ -530,11 +572,14 @@ class AnsibleDeploy(agent_base.HeartbeatMixin, base.DeployInterface):
         :returns: None or states.CLEANWAIT for async prepare.
         """
         node = task.node
-        manager_utils.set_node_cleaning_steps(task)
+        conductor_steps.set_node_cleaning_steps(task)
         if not node.driver_internal_info['clean_steps']:
             # no clean steps configured, nothing to do.
             return
+        power_state_to_restore = manager_utils.power_on_node_if_needed(task)
         task.driver.network.add_cleaning_network(task)
+        manager_utils.restore_power_state_if_needed(
+            task, power_state_to_restore)
         boot_opt = deploy_utils.build_agent_options(node)
         task.driver.boot.prepare_ramdisk(task, boot_opt)
         manager_utils.node_power_action(task, states.REBOOT)
@@ -550,41 +595,35 @@ class AnsibleDeploy(agent_base.HeartbeatMixin, base.DeployInterface):
         """
         manager_utils.node_power_action(task, states.POWER_OFF)
         task.driver.boot.clean_up_ramdisk(task)
+        power_state_to_restore = manager_utils.power_on_node_if_needed(task)
         task.driver.network.remove_cleaning_network(task)
+        manager_utils.restore_power_state_if_needed(
+            task, power_state_to_restore)
 
-    @METRICS.timer('AnsibleDeploy.continue_deploy')
-    def continue_deploy(self, task):
+    @METRICS.timer('AnsibleDeploy.write_image')
+    @base.deploy_step(priority=80)
+    def write_image(self, task):
         # NOTE(pas-ha) the lock should be already upgraded in heartbeat,
         # just setting its purpose for better logging
         task.upgrade_lock(purpose='deploy')
-        task.process_event('resume')
         # NOTE(pas-ha) this method is called from heartbeat processing only,
         # so we are sure we need this particular method, not the general one
         node_address = _get_node_ip(task)
         self._ansible_deploy(task, node_address)
-        self.reboot_to_instance(task)
-
-    @METRICS.timer('AnsibleDeploy.reboot_to_instance')
-    def reboot_to_instance(self, task):
-        node = task.node
-        LOG.info('Ansible complete deploy on node %s', node.uuid)
-
-        LOG.debug('Rebooting node %s to instance', node.uuid)
+        LOG.info('Ansible complete deploy on node %s', task.node.uuid)
         manager_utils.node_set_boot_device(task, 'disk', persistent=True)
-        self.reboot_and_finish_deploy(task)
-        task.driver.boot.clean_up_ramdisk(task)
 
-        if not node.deploy_step:
-            # TODO(rloo): delete this 'if' part after deprecation period, when
-            # we expect all (out-of-tree) drivers to support deploy steps.
-            # After which we will always notify_conductor_resume_deploy().
-            task.process_event('done')
-            LOG.info('Deployment to node %s done', task.node.uuid)
-        else:
-            manager_utils.notify_conductor_resume_deploy(task)
+    @METRICS.timer('AnsibleDeploy.tear_down_agent')
+    @base.deploy_step(priority=40)
+    @task_manager.require_exclusive_lock
+    def tear_down_agent(self, task):
+        """A deploy step to tear down the agent.
 
-    @METRICS.timer('AnsibleDeploy.reboot_and_finish_deploy')
-    def reboot_and_finish_deploy(self, task):
+        Shuts down the machine and removes it from the provisioning
+        network.
+
+        :param task: a TaskManager object containing the node
+        """
         wait = CONF.ansible.post_deploy_get_power_state_retry_interval * 1000
         attempts = CONF.ansible.post_deploy_get_power_state_retries + 1
 
@@ -622,9 +661,6 @@ class AnsibleDeploy(agent_base.HeartbeatMixin, base.DeployInterface):
                     manager_utils.node_power_action(task, states.POWER_OFF)
             else:
                 manager_utils.node_power_action(task, states.POWER_OFF)
-            task.driver.network.remove_provisioning_network(task)
-            task.driver.network.configure_tenant_networks(task)
-            manager_utils.node_power_action(task, states.POWER_ON)
         except Exception as e:
             msg = (_('Error rebooting node %(node)s after deploy. '
                      'Error: %(error)s') %
