@@ -26,11 +26,10 @@ import uuid
 from oslo_concurrency import lockutils
 from oslo_log import log as logging
 from oslo_utils import fileutils
-import six
 
 from ironic.common import exception
 from ironic.common.glance_service import service_utils
-from ironic.common.i18n import _LI, _LW
+from ironic.common.i18n import _
 from ironic.common import image_service
 from ironic.common import images
 from ironic.common import utils
@@ -82,7 +81,7 @@ class ImageCache(object):
         if self.master_dir is None:
             # NOTE(ghe): We don't share images between instances/hosts
             if not CONF.parallel_image_downloads:
-                with lockutils.lock(img_download_lock_name, 'ironic-'):
+                with lockutils.lock(img_download_lock_name):
                     _fetch(ctx, href, dest_path, force_raw)
             else:
                 _fetch(ctx, href, dest_path, force_raw)
@@ -93,20 +92,21 @@ class ImageCache(object):
         # NOTE(vdrok): File name is converted to UUID if it's not UUID already,
         # so that two images with same file names do not collide
         if service_utils.is_glance_image(href):
-            master_file_name = service_utils.parse_image_ref(href)[0]
+            master_file_name = service_utils.parse_image_id(href)
         else:
-            # NOTE(vdrok): Doing conversion of href in case it's unicode
-            # string, UUID cannot be generated for unicode strings on python 2.
-            href_encoded = href.encode('utf-8') if six.PY2 else href
-            master_file_name = str(uuid.uuid5(uuid.NAMESPACE_URL,
-                                              href_encoded))
+            master_file_name = str(uuid.uuid5(uuid.NAMESPACE_URL, href))
+        # NOTE(kaifeng) The ".converted" suffix acts as an indicator that the
+        # image cached has gone through the conversion logic.
+        if force_raw:
+            master_file_name = master_file_name + '.converted'
+
         master_path = os.path.join(self.master_dir, master_file_name)
 
         if CONF.parallel_image_downloads:
             img_download_lock_name = 'download-image:%s' % master_file_name
 
         # TODO(dtantsur): lock expiration time
-        with lockutils.lock(img_download_lock_name, 'ironic-'):
+        with lockutils.lock(img_download_lock_name):
             # NOTE(vdrok): After rebuild requested image can change, so we
             # should ensure that dest_path and master_path (if exists) are
             # pointing to the same file and their content is up to date
@@ -123,15 +123,14 @@ class ImageCache(object):
 
             if cache_up_to_date:
                 # NOTE(dtantsur): ensure we're not in the middle of clean up
-                with lockutils.lock('master_image', 'ironic-'):
+                with lockutils.lock('master_image'):
                     os.link(master_path, dest_path)
                 LOG.debug("Master cache hit for image %(href)s",
                           {'href': href})
                 return
 
-            LOG.info(_LI("Master cache miss for image %(href)s, "
-                         "starting download"),
-                     {'href': href})
+            LOG.info("Master cache miss for image %(href)s, "
+                     "starting download", {'href': href})
             self._download_image(
                 href, master_path, dest_path, ctx=ctx, force_raw=force_raw)
 
@@ -150,6 +149,9 @@ class ImageCache(object):
         :param ctx: context
         :param force_raw: boolean value, whether to convert the image to raw
                           format
+        :raise ImageDownloadFailed: when the image cache and the image HTTP or
+                                    TFTP location are on different file system,
+                                    causing hard link to fail.
         """
         # TODO(ghe): timeout and retry for downloads
         # TODO(ghe): logging when image cannot be created
@@ -162,10 +164,17 @@ class ImageCache(object):
             # will have link count >1 at any moment, so won't be cleaned up
             os.link(tmp_path, master_path)
             os.link(master_path, dest_path)
+        except OSError as exc:
+            msg = (_("Could not link image %(img_href)s from %(src_path)s "
+                     "to %(dst_path)s, error: %(exc)s") %
+                   {'img_href': href, 'src_path': master_path,
+                    'dst_path': dest_path, 'exc': exc})
+            LOG.error(msg)
+            raise exception.ImageDownloadFailed(msg)
         finally:
             utils.rmtree_without_raise(tmp_dir)
 
-    @lockutils.synchronized('master_image', 'ironic-')
+    @lockutils.synchronized('master_image')
     def clean_up(self, amount=None):
         """Clean up directory with images, keeping cache of the latest images.
 
@@ -180,7 +189,7 @@ class ImageCache(object):
         if self.master_dir is None:
             return
 
-        LOG.debug("Starting clean up for master image cache %(dir)s" %
+        LOG.debug("Starting clean up for master image cache %(dir)s",
                   {'dir': self.master_dir})
 
         amount_copy = amount
@@ -190,11 +199,10 @@ class ImageCache(object):
             return
         amount = self._clean_up_ensure_cache_size(survived, amount)
         if amount is not None and amount > 0:
-            LOG.warning(
-                _LW("Cache clean up was unable to reclaim %(required)d "
-                    "MiB of disk space, still %(left)d MiB required"),
-                {'required': amount_copy / 1024 / 1024,
-                 'left': amount / 1024 / 1024})
+            LOG.warning("Cache clean up was unable to reclaim %(required)d "
+                        "MiB of disk space, still %(left)d MiB required",
+                        {'required': amount_copy / 1024 / 1024,
+                         'left': amount / 1024 / 1024})
 
     def _clean_up_too_old(self, listing, amount):
         """Clean up stage 1: drop images that are older than TTL.
@@ -218,8 +226,8 @@ class ImageCache(object):
                 try:
                     os.unlink(file_name)
                 except EnvironmentError as exc:
-                    LOG.warning(_LW("Unable to delete file %(name)s from "
-                                    "master image cache: %(exc)s"),
+                    LOG.warning("Unable to delete file %(name)s from "
+                                "master image cache: %(exc)s",
                                 {'name': file_name, 'exc': exc})
                 else:
                     if amount is not None:
@@ -251,14 +259,14 @@ class ImageCache(object):
                          for f in os.listdir(self.master_dir))
         total_size = sum(os.path.getsize(f)
                          for f in total_listing)
-        while listing and (total_size > self._cache_size or
-                           (amount is not None and amount > 0)):
+        while listing and (total_size > self._cache_size
+                           or (amount is not None and amount > 0)):
             file_name, last_used, stat = listing.pop()
             try:
                 os.unlink(file_name)
             except EnvironmentError as exc:
-                LOG.warning(_LW("Unable to delete file %(name)s from "
-                                "master image cache: %(exc)s"),
+                LOG.warning("Unable to delete file %(name)s from "
+                            "master image cache: %(exc)s",
                             {'name': file_name, 'exc': exc})
             else:
                 total_size -= stat.st_size
@@ -266,9 +274,9 @@ class ImageCache(object):
                     amount -= stat.st_size
 
         if total_size > self._cache_size:
-            LOG.info(_LI("After cleaning up cache dir %(dir)s "
-                         "cache size %(actual)d is still larger than "
-                         "threshold %(expected)d"),
+            LOG.info("After cleaning up cache dir %(dir)s "
+                     "cache size %(actual)d is still larger than "
+                     "threshold %(expected)d",
                      {'dir': self.master_dir, 'actual': total_size,
                       'expected': self._cache_size})
         return max(amount, 0) if amount is not None else 0
@@ -305,9 +313,10 @@ def _fetch(context, image_href, path, force_raw=False):
     # Notes(yjiang5): If glance can provide the virtual size information,
     # then we can firstly clean cache and then invoke images.fetch().
     if force_raw:
-        required_space = images.converted_size(path_tmp)
-        directory = os.path.dirname(path_tmp)
-        _clean_up_caches(directory, required_space)
+        if images.force_raw_will_convert(image_href, path_tmp):
+            required_space = images.converted_size(path_tmp)
+            directory = os.path.dirname(path_tmp)
+            _clean_up_caches(directory, required_space)
         images.image_to_raw(image_href, path, path_tmp)
     else:
         os.rename(path_tmp, path)
@@ -319,7 +328,7 @@ def _clean_up_caches(directory, amount):
     :param directory: the directory (of the cache) to be freed up.
     :param amount: amount of space to reclaim.
     :raises: InsufficientDiskSpace exception, if we cannot free up enough space
-    after trying all the caches.
+             after trying all the caches.
     """
     free = _free_disk_space_for(directory)
 
@@ -355,9 +364,9 @@ def clean_up_caches(ctx, directory, images_info):
     :param ctx: context
     :param directory: the directory (of the cache) to be freed up.
     :param images_info: a list of tuples of the form (image_uuid,path)
-        for which space is to be created in cache.
+                        for which space is to be created in cache.
     :raises: InsufficientDiskSpace exception, if we cannot free up enough space
-    after trying all the caches.
+             after trying all the caches.
     """
     total_size = sum(images.download_size(ctx, uuid)
                      for (uuid, path) in images_info)
@@ -392,17 +401,17 @@ def _delete_master_path_if_stale(master_path, href, ctx):
         if not img_mtime:
             # This means that href is not a glance image and doesn't have an
             # updated_at attribute
-            LOG.warning(_LW("Image service couldn't determine last "
-                            "modification time of %(href)s, considering "
-                            "cached image up to date."), {'href': href})
+            LOG.warning("Image service couldn't determine last "
+                        "modification time of %(href)s, considering "
+                        "cached image up to date.", {'href': href})
             return True
         master_mtime = utils.unix_file_modification_datetime(master_path)
         if img_mtime <= master_mtime:
             return True
         # Delete image from cache as it is outdated
-        LOG.info(_LI('Image %(href)s was last modified at %(remote_time)s. '
-                     'Deleting the cached copy "%(cached_file)s since it was '
-                     'last modified at %(local_time)s and may be outdated.'),
+        LOG.info('Image %(href)s was last modified at %(remote_time)s. '
+                 'Deleting the cached copy "%(cached_file)s since it was '
+                 'last modified at %(local_time)s and may be outdated.',
                  {'href': href, 'remote_time': img_mtime,
                   'local_time': master_mtime, 'cached_file': master_path})
 
@@ -423,8 +432,8 @@ def _delete_dest_path_if_stale(master_path, dest_path):
         # Image not cached, re-download
         return False
     master_path_exists = os.path.exists(master_path)
-    if (not master_path_exists or
-            os.stat(master_path).st_ino != os.stat(dest_path).st_ino):
+    if (not master_path_exists
+            or os.stat(master_path).st_ino != os.stat(dest_path).st_ino):
         # Image exists in cache, but dest_path out of date
         os.unlink(dest_path)
         return False
