@@ -36,13 +36,13 @@ import subprocess
 import tempfile
 import time
 
+from eventlet.green import subprocess as green_subprocess
 from ironic_lib import metrics_utils
 from ironic_lib import utils as ironic_utils
 from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import strutils
-import six
 
 from ironic.common import boot_devices
 from ironic.common import exception
@@ -71,6 +71,9 @@ REQUIRED_PROPERTIES = {
 }
 OPTIONAL_PROPERTIES = {
     'ipmi_password': _("password. Optional."),
+    'ipmi_hex_kg_key': _('Kg key for IPMIv2 authentication. '
+                         'The key is expected in hexadecimal format. '
+                         'Optional.'),
     'ipmi_port': _("remote IPMI RMCP port. Optional."),
     'ipmi_priv_level': _("privilege level; default is ADMINISTRATOR. One of "
                          "%s. Optional.") % ', '.join(VALID_PRIV_LEVELS),
@@ -97,7 +100,20 @@ OPTIONAL_PROPERTIES = {
                                 "is turned on, eg. because the BMC is not "
                                 "capable of remembering the selected boot "
                                 "device across power cycles; default value "
-                                "is False. Optional.")
+                                "is False. Optional."),
+    'ipmi_disable_boot_timeout': _('By default ironic will send a raw IPMI '
+                                   'command to disable the 60 second timeout '
+                                   'for booting. Setting this option to '
+                                   'False will NOT send that command on '
+                                   'this node. The '
+                                   '[ipmi]disable_boot_timeout will be '
+                                   'used if this option is not set. '
+                                   'Optional.'),
+    'ipmi_cipher_suite': _('The number of a cipher suite to use. Only 3 '
+                           '(AES-128 with SHA1) or 17 (AES-128 with SHA256) '
+                           'should ever be used; 17 is preferred. The '
+                           'default value depends on the ipmitool version, '
+                           'some recent versions have switched from 3 to 17.'),
 }
 COMMON_PROPERTIES = REQUIRED_PROPERTIES.copy()
 COMMON_PROPERTIES.update(OPTIONAL_PROPERTIES)
@@ -140,7 +156,7 @@ IPMITOOL_RETRYABLE_FAILURES = ['insufficient resources for session',
 
 # NOTE(lucasagomes): A mapping for the boot devices and their hexadecimal
 # value. For more information about these values see the "Set System Boot
-# Options Command" section of the link below (page 418)
+# Options Command" section 28.13 of the link below (page 392)
 # http://www.intel.com/content/www/us/en/servers/ipmi/ipmi-second-gen-interface-spec-v2-rev1-1.html  # noqa
 BOOT_DEVICE_HEXA_MAP = {
     boot_devices.PXE: '0x04',
@@ -149,6 +165,50 @@ BOOT_DEVICE_HEXA_MAP = {
     boot_devices.BIOS: '0x18',
     boot_devices.SAFE: '0x0c'
 }
+# NOTE(TheJulia): Inline notes for ipmi raw boot device commands.
+# Per spec (intel 2nd gen ipmi interface, v2, rev1-1)
+#  - bit 7 is CMOS clear
+#  - bit 6 is Keyboard lock
+#  - bit 5-2 is the boot device selector
+#    * where 0000b is to do nothing
+#    * where 0001b is to force pxe
+#    * where 0010b is boot from hard drive
+#    * And anything above 1100b is listed reserved
+#      which is 0x30 for ipmitool raw commands!
+#  - bit 1 is screen blank
+#  - bit 0 is lock out reset button
+#
+# Effectively this results in
+# 00000100b == 0x04 for booting from PXE
+# 00001000b == 0x08 for booting from the primary hard disk.
+# 00100000b == 0x20 is remotely connected virtual media
+# 00100100b == 0x24 which is primary remote boot media as set
+#                   via the OEM initiator mailbox information.
+#                   Like.. iscsi?
+#                   However! Supermicro uses this as the UEFI
+#                   hard disk. See: https://www.supermicro.com/support/faqs/faq.cfm?faq=25559  # noqa
+# 00010000b == 0x30 Enters the reserved territory, and should not
+#                   be expected in use.
+
+
+def _vendor_aware_boot_device_map(task):
+    """Returns an altered vendor boot device map based upon vendor."""
+    node = task.node
+    vendor = node.properties.get('vendor', None)
+    boot_dev_map = BOOT_DEVICE_HEXA_MAP.copy()
+    boot_mode = boot_mode_utils.get_boot_mode(node)
+    if vendor:
+        vendor = str(vendor).lower()
+        if boot_mode == 'uefi' and vendor == "supermicro":
+            # This difference is only known on UEFI mode for supermicro
+            # hardware.
+            boot_dev_map[boot_devices.DISK] = '0x24'
+        # NOTE(TheJulia): Similar differences may exist with Cisco UCS
+        # hardware when using IPMI, however at present we don't know
+        # what the setting would be.
+        # NOTE(TheJulia) I've observed "mc info" manufacter name of a cisco
+        # c-series machine to return "Unknown (0x168B)"
+    return boot_dev_map
 
 
 def _check_option_support(options):
@@ -263,6 +323,7 @@ def _parse_driver_info(node):
 
     """
     info = node.driver_info or {}
+    internal_info = node.driver_internal_info or {}
     bridging_types = ['single', 'dual']
     missing_info = [key for key in REQUIRED_PROPERTIES if not info.get(key)]
     if missing_info:
@@ -272,9 +333,11 @@ def _parse_driver_info(node):
 
     address = info.get('ipmi_address')
     username = info.get('ipmi_username')
-    password = six.text_type(info.get('ipmi_password', ''))
+    password = str(info.get('ipmi_password', ''))
+    hex_kg_key = info.get('ipmi_hex_kg_key')
     dest_port = info.get('ipmi_port')
-    port = info.get('ipmi_terminal_port')
+    port = (info.get('ipmi_terminal_port')
+            or internal_info.get('allocated_ipmi_terminal_port'))
     priv_level = info.get('ipmi_priv_level', 'ADMINISTRATOR')
     bridging_type = info.get('ipmi_bridging', 'no')
     local_address = info.get('ipmi_local_address')
@@ -284,6 +347,7 @@ def _parse_driver_info(node):
     target_address = info.get('ipmi_target_address')
     protocol_version = str(info.get('ipmi_protocol_version', '2.0'))
     force_boot_device = info.get('ipmi_force_boot_device', False)
+    cipher_suite = info.get('ipmi_cipher_suite')
 
     if not username:
         LOG.warning('ipmi_username is not defined or empty for node %s: '
@@ -352,11 +416,26 @@ def _parse_driver_info(node):
             " can be one of %(valid_levels)s") %
             {'priv_level': priv_level, 'valid_levels': valid_priv_lvls})
 
+    if hex_kg_key and len(hex_kg_key) % 2 != 0:
+        raise exception.InvalidParameterValue(_(
+            "Number of ipmi_hex_kg_key characters is not even"))
+
+    if cipher_suite is not None:
+        try:
+            int(cipher_suite)
+        except (TypeError, ValueError):
+            raise exception.InvalidParameterValue(_(
+                "Invalid cipher suite %s, expected a number") % cipher_suite)
+        if protocol_version == '1.5':
+            raise exception.InvalidParameterValue(_(
+                "Cipher suites cannot be used with IPMI 1.5"))
+
     return {
         'address': address,
         'dest_port': dest_port,
         'username': username,
         'password': password,
+        'hex_kg_key': hex_kg_key,
         'port': port,
         'uuid': node.uuid,
         'priv_level': priv_level,
@@ -367,44 +446,64 @@ def _parse_driver_info(node):
         'target_address': target_address,
         'protocol_version': protocol_version,
         'force_boot_device': force_boot_device,
+        'cipher_suite': cipher_suite,
     }
 
 
-def _exec_ipmitool(driver_info, command, check_exit_code=None):
+def _get_ipmitool_args(driver_info, pw_file=None):
+    ipmi_version = ('lanplus'
+                    if driver_info['protocol_version'] == '2.0'
+                    else 'lan')
+
+    args = ['ipmitool',
+            '-I', ipmi_version,
+            '-H', driver_info['address'],
+            '-L', driver_info['priv_level']
+            ]
+
+    for field, arg in [('dest_port', '-p'),
+                       ('username', '-U'),
+                       ('hex_kg_key', '-y'),
+                       ('cipher_suite', '-C')]:
+        if driver_info[field]:
+            args.append(arg)
+            args.append(driver_info[field])
+
+    for name, option in BRIDGING_OPTIONS:
+        if driver_info[name] is not None:
+            args.append(option)
+            args.append(driver_info[name])
+
+    if pw_file:
+        args.append('-f')
+        args.append(pw_file)
+
+    if CONF.ipmi.debug:
+        args.append('-v')
+
+    # ensure all arguments are strings
+    args = [str(arg) for arg in args]
+
+    return args
+
+
+def _exec_ipmitool(driver_info, command, check_exit_code=None,
+                   kill_on_timeout=False):
     """Execute the ipmitool command.
 
     :param driver_info: the ipmitool parameters for accessing a node.
     :param command: the ipmitool command to be executed.
     :param check_exit_code: Single bool, int, or list of allowed exit codes.
+    :param kill_on_timeout: if `True`, kill unresponsive ipmitool on
+        `min_command_interval` timeout. Default is `False`. Makes no
+        effect on Windows.
     :returns: (stdout, stderr) from executing the command.
     :raises: PasswordFileFailedToCreate from creating or writing to the
              temporary file.
     :raises: processutils.ProcessExecutionError from executing the command.
 
     """
-    ipmi_version = ('lanplus'
-                    if driver_info['protocol_version'] == '2.0'
-                    else 'lan')
-    args = ['ipmitool',
-            '-I',
-            ipmi_version,
-            '-H',
-            driver_info['address'],
-            '-L', driver_info['priv_level']
-            ]
-
-    if driver_info['dest_port']:
-        args.append('-p')
-        args.append(driver_info['dest_port'])
-
-    if driver_info['username']:
-        args.append('-U')
-        args.append(driver_info['username'])
-
-    for name, option in BRIDGING_OPTIONS:
-        if driver_info[name] is not None:
-            args.append(option)
-            args.append(driver_info[name])
+    args = _get_ipmitool_args(driver_info)
 
     timeout = CONF.ipmi.command_retry_timeout
 
@@ -413,16 +512,27 @@ def _exec_ipmitool(driver_info, command, check_exit_code=None):
 
     if _is_option_supported('timing'):
         args.append('-R')
-        args.append(str(num_tries))
+        if CONF.ipmi.use_ipmitool_retries:
+            args.append(str(num_tries))
+        else:
+            args.append('1')
 
         args.append('-N')
         args.append(str(CONF.ipmi.min_command_interval))
 
+    extra_args = {}
+
+    if kill_on_timeout:
+        extra_args['timeout'] = timeout
+
+    if check_exit_code is not None:
+        extra_args['check_exit_code'] = check_exit_code
+
     end_time = (time.time() + timeout)
 
     while True:
         num_tries = num_tries - 1
-        # NOTE(deva): ensure that no communications are sent to a BMC more
+        # NOTE(tenbrae): ensure that no communications are sent to a BMC more
         #             often than once every min_command_interval seconds.
         time_till_next_poll = CONF.ipmi.min_command_interval - (
             time.time() - LAST_CMD_TIME.get(driver_info['address'], 0))
@@ -431,9 +541,6 @@ def _exec_ipmitool(driver_info, command, check_exit_code=None):
         # Resetting the list that will be utilized so the password arguments
         # from any previous execution are preserved.
         cmd_args = args[:]
-        extra_args = {}
-        if check_exit_code is not None:
-            extra_args['check_exit_code'] = check_exit_code
         # 'ipmitool' command will prompt password if there is no '-f'
         # option, we set it to '\0' to write a password file to support
         # empty password
@@ -446,11 +553,17 @@ def _exec_ipmitool(driver_info, command, check_exit_code=None):
                 return out, err
             except processutils.ProcessExecutionError as e:
                 with excutils.save_and_reraise_exception() as ctxt:
-                    err_list = [x for x in IPMITOOL_RETRYABLE_FAILURES
-                                if x in six.text_type(e)]
+                    err_list = [
+                        x for x in (
+                            IPMITOOL_RETRYABLE_FAILURES
+                            + CONF.ipmi.additional_retryable_ipmi_errors)
+                        if x in str(e)]
+                    # If Ironic is doing retries then retry all errors
+                    retry_failures = (err_list
+                                      or not CONF.ipmi.use_ipmitool_retries)
                     if ((time.time() > end_time)
                         or (num_tries == 0)
-                        or not err_list):
+                        or not retry_failures):
                         LOG.error('IPMI Error while attempting "%(cmd)s" '
                                   'for node %(node)s. Error: %(error)s',
                                   {'node': driver_info['uuid'],
@@ -500,7 +613,10 @@ def _set_and_wait(task, power_action, driver_info, timeout=None):
     try:
         _exec_ipmitool(driver_info, cmd)
     except (exception.PasswordFileFailedToCreate,
-            processutils.ProcessExecutionError) as e:
+            processutils.ProcessExecutionError,
+            subprocess.TimeoutExpired,
+            # https://github.com/eventlet/eventlet/issues/624
+            green_subprocess.TimeoutExpired) as e:
         LOG.warning("IPMI power action %(cmd)s failed for node %(node_id)s "
                     "with error: %(error)s.",
                     {'node_id': driver_info['uuid'], 'cmd': cmd, 'error': e})
@@ -559,7 +675,8 @@ def _power_status(driver_info):
     """
     cmd = "power status"
     try:
-        out_err = _exec_ipmitool(driver_info, cmd)
+        out_err = _exec_ipmitool(
+            driver_info, cmd, kill_on_timeout=CONF.ipmi.kill_on_timeout)
     except (exception.PasswordFileFailedToCreate,
             processutils.ProcessExecutionError) as e:
         LOG.warning("IPMI power status failed for node %(node_id)s with "
@@ -581,6 +698,9 @@ def _process_sensor(sensor_data):
     for field in sensor_data_fields:
         if not field:
             continue
+        if field.startswith('<<'):
+            # This is debug data, and can be safely ignored for this.
+            continue
         kv_value = field.split(':')
         if len(kv_value) != 2:
             continue
@@ -744,6 +864,26 @@ def _constructor_checks(driver):
     _check_temp_dir()
 
 
+def _allocate_port(task, host=None):
+    node = task.node
+    dii = node.driver_internal_info or {}
+    allocated_port = console_utils.acquire_port(host=host)
+    dii['allocated_ipmi_terminal_port'] = allocated_port
+    node.driver_internal_info = dii
+    node.save()
+    return allocated_port
+
+
+def _release_allocated_port(task):
+    node = task.node
+    dii = node.driver_internal_info or {}
+    allocated_port = dii.pop('allocated_ipmi_terminal_port', None)
+    if allocated_port:
+        node.driver_internal_info = dii
+        node.save()
+        console_utils.release_port(allocated_port)
+
+
 class IPMIPower(base.PowerInterface):
 
     def __init__(self):
@@ -764,7 +904,7 @@ class IPMIPower(base.PowerInterface):
 
         """
         _parse_driver_info(task.node)
-        # NOTE(deva): don't actually touch the BMC in validate because it is
+        # NOTE(tenbrae): don't actually touch the BMC in validate because it is
         #             called too often, and BMCs are too fragile.
         #             This is a temporary measure to mitigate problems while
         #             1314954 and 1314961 are resolved.
@@ -781,6 +921,25 @@ class IPMIPower(base.PowerInterface):
             call).
 
         """
+        # NOTE(TheJulia): Temporary until we promote detect_vendor as
+        # a higher level management method and able to automatically
+        # run detection upon either power state sync or in the enrollment
+        # to management step.
+        try:
+            properties = task.node.properties
+            if not properties.get('vendor'):
+                # We have no vendor stored, so we'll go ahead and
+                # call to store it.
+                vendor = task.driver.management.detect_vendor(task)
+                if vendor:
+                    task.upgrade_lock()
+                    props = task.node.properties
+                    props['vendor'] = vendor
+                    task.node.properties = props
+                    task.node.save()
+        except exception.UnsupportedDriverExtension:
+            pass
+
         driver_info = _parse_driver_info(task.node)
         return _power_status(driver_info)
 
@@ -912,15 +1071,31 @@ class IPMIManagement(base.ManagementInterface):
             raise exception.InvalidParameterValue(_(
                 "Invalid boot device %s specified.") % device)
 
-        # note(JayF): IPMI spec indicates unless you send these raw bytes the
-        # boot device setting times out after 60s. Since it's possible it
-        # could be >60s before a node is rebooted, we should always send them.
-        # This mimics pyghmi's current behavior, and the "option=timeout"
-        # setting on newer ipmitool binaries.
-        timeout_disable = "0x00 0x08 0x03 0x08"
-        send_raw(task, timeout_disable)
+        # NOTE(tonyb): Some BMCs do not implement Option 0x03, such as OpenBMC
+        # and will error when we try to set this.  Resulting in an abort.  If
+        # the BMC doesn't support this timeout there isn't a need to disable
+        # it.  Let's use a driver option to signify that.
+        # NOTE(kaifeng) [ipmi]disable_boot_timeout provides default value if
+        # driver_info/ipmi_disable_boot_timeout is not set.
+        idt = task.node.driver_info.get('ipmi_disable_boot_timeout')
+        if idt is None:
+            idt = CONF.ipmi.disable_boot_timeout
+        if strutils.bool_from_string(idt):
+            # note(JayF): IPMI spec indicates unless you send these raw bytes
+            # the boot device setting times out after 60s. Since it's possible
+            # it could be >60s before a node is rebooted, we should always send
+            # them.  This mimics pyghmi's current behavior, and the
+            # "option=timeout" setting on newer ipmitool binaries.
+            timeout_disable = "0x00 0x08 0x03 0x08"
+            send_raw(task, timeout_disable)
+        else:
+            LOG.info('For node %(node_uuid)s, '
+                     'driver_info[\'ipmi_disable_boot_timeout\'] is set '
+                     'to False, so not sending ipmi boot-timeout-disable',
+                     {'node_uuid', task.node.uuid})
 
-        if task.node.driver_info.get('ipmi_force_boot_device', False):
+        ifbd = task.node.driver_info.get('ipmi_force_boot_device', False)
+        if strutils.bool_from_string(ifbd):
             driver_utils.force_persistent_boot(task,
                                                device,
                                                persistent)
@@ -928,7 +1103,7 @@ class IPMIManagement(base.ManagementInterface):
             # persistent or we do not have admin rights.
             persistent = False
 
-        # FIXME(lucasagomes): Older versions of the ipmitool utility
+        # NOTE(lucasagomes): Older versions of the ipmitool utility
         # are not able to set the options "efiboot" and "persistent"
         # at the same time, combining other options seems to work fine,
         # except efiboot. Newer versions of ipmitool (1.8.17) does fix
@@ -938,19 +1113,37 @@ class IPMIManagement(base.ManagementInterface):
         # uefi mode, this will work with newer and older versions of the
         # ipmitool utility. Also see:
         # https://bugs.launchpad.net/ironic/+bug/1611306
-        boot_mode = boot_mode_utils.get_boot_mode_for_deploy(task.node)
-        if persistent and boot_mode == 'uefi':
-            raw_cmd = ('0x00 0x08 0x05 0xe0 %s 0x00 0x00 0x00' %
-                       BOOT_DEVICE_HEXA_MAP[device])
+        # NOTE(TheJulia): Previously we added raw device support because
+        # most distributions are carrying older downstream patched versions
+        # of ipmitool where "efiboot" and "persistent" do not work. However,
+        # using the embedded support assumes that every vendor out there uses
+        # the same device numbers all the time. Reality is not so kind.
+        # It should also be noted that the ipmitool chassis bootparm get
+        # command also just interprets back the same fields, so if the vendor
+        # has set a different value as default, then ipmitool is not going
+        # to understand it and may be listing the wrong boot flag as a result.
+        # See: https://storyboard.openstack.org/#!/story/2008241
+        boot_mode = boot_mode_utils.get_boot_mode(task.node)
+        if boot_mode == 'uefi':
+            # Long story short: UEFI was added to IPMI after the final spec
+            # release occured. This means BMCs may actually NEED to be
+            # explicitly told if the boot is persistant because the
+            # BMC may return a value which is explicitly parsed as
+            # no change, BUT the BMC may treat that as operational default.
+            efi_persistence = '0xe0' if persistent else '0xa0'
+            # 0xe0 is persistent UEFI boot, 0xa0 is one-time UEFI boot.
+            boot_device_map = _vendor_aware_boot_device_map(task)
+            raw_cmd = ('0x00 0x08 0x05 %s %s 0x00 0x00 0x00' %
+                       (efi_persistence, boot_device_map[device]))
             send_raw(task, raw_cmd)
             return
 
         options = []
         if persistent:
             options.append('persistent')
-        if boot_mode == 'uefi':
-            options.append('efiboot')
-
+        # NOTE(TheJulia): Once upon a time we had efiboot here. It doesn't
+        # work in all cases and directs us to unhappy places if the values
+        # are incorrect.
         cmd = "chassis bootdev %s" % device
         if options:
             cmd = cmd + " options=%s" % ','.join(options)
@@ -986,8 +1179,11 @@ class IPMIManagement(base.ManagementInterface):
         """
         driver_info = task.node.driver_info
         driver_internal_info = task.node.driver_internal_info
+        ifbd = driver_info.get('ipmi_force_boot_device', False)
 
-        if (driver_info.get('ipmi_force_boot_device', False)
+        driver_info = _parse_driver_info(task.node)
+
+        if (strutils.bool_from_string(ifbd)
                 and driver_internal_info.get('persistent_boot_device')
                 and driver_internal_info.get('is_next_boot_persistent', True)):
             return {
@@ -996,7 +1192,6 @@ class IPMIManagement(base.ManagementInterface):
             }
 
         cmd = "chassis bootparam get 5"
-        driver_info = _parse_driver_info(task.node)
         response = {'boot_device': None, 'persistent': None}
 
         try:
@@ -1008,7 +1203,9 @@ class IPMIManagement(base.ManagementInterface):
                         'Error: %(error)s',
                         {'node': driver_info['uuid'], 'cmd': cmd, 'error': e})
             raise exception.IPMIFailure(cmd=cmd)
-
+        # FIXME(TheJulia): This whole thing needs to be refactored to be based
+        # upon the response generated by the "Boot parameter data".
+        # See: https://storyboard.openstack.org/#!/story/2008241
         re_obj = re.search('Boot Device Selector : (.+)?\n', out)
         if re_obj:
             boot_selector = re_obj.groups('')[0]
@@ -1027,6 +1224,37 @@ class IPMIManagement(base.ManagementInterface):
         response['persistent'] = 'Options apply to all future boots' in out
         return response
 
+    @METRICS.timer('IPMIManagement.detect_vendor')
+    def detect_vendor(self, task):
+        """Detects, stores, and returns the hardware vendor.
+
+        If the Node object ``properties`` field does not already contain
+        a ``vendor`` field, then this method is intended to query
+        Detects the BMC hardware vendor and stores the returned value
+        with-in the Node object ``properties`` field if detected.
+
+        :param task: A task from TaskManager.
+        :raises: InvalidParameterValue if an invalid component, indicator
+            or state is specified.
+        :raises: MissingParameterValue if a required parameter is missing
+        :returns: String representing the BMC reported Vendor or
+                  Manufacturer, otherwise returns None.
+        """
+        try:
+            driver_info = _parse_driver_info(task.node)
+            out, err = _exec_ipmitool(driver_info, "mc info")
+            re_obj = re.search("Manufacturer Name .*: (.+)", out)
+            if re_obj:
+                bmc_vendor = str(re_obj.groups('')[0]).lower().split(':')
+                # Pull unparsed data and save the vendor
+                return bmc_vendor[-1]
+        except (exception.PasswordFileFailedToCreate,
+                processutils.ProcessExecutionError) as e:
+            LOG.warning('IPMI get boot device failed to detect vendor '
+                        'of bmc for %(node)s. Error %(err)s',
+                        {'node': task.node.uuid,
+                         'err': e})
+
     @METRICS.timer('IPMIManagement.get_sensors_data')
     def get_sensors_data(self, task):
         """Get sensors data.
@@ -1044,7 +1272,8 @@ class IPMIManagement(base.ManagementInterface):
         # extended sensor informations
         cmd = "sdr -v"
         try:
-            out, err = _exec_ipmitool(driver_info, cmd)
+            out, err = _exec_ipmitool(
+                driver_info, cmd, kill_on_timeout=CONF.ipmi.kill_on_timeout)
         except (exception.PasswordFileFailedToCreate,
                 processutils.ProcessExecutionError) as e:
             raise exception.FailedToGetSensorData(node=task.node.uuid,
@@ -1190,10 +1419,10 @@ class IPMIConsole(base.ConsoleInterface):
 
         """
         driver_info = _parse_driver_info(task.node)
-        if not driver_info['port']:
+        if not driver_info['port'] and CONF.console.port_range is None:
             raise exception.MissingParameterValue(_(
-                "Missing 'ipmi_terminal_port' parameter in node's"
-                " driver_info."))
+                "Either missing 'ipmi_terminal_port' parameter in node's "
+                "driver_info or [console]port_range is not configured"))
 
         if driver_info['protocol_version'] != '2.0':
             raise exception.InvalidParameterValue(_(
@@ -1208,13 +1437,7 @@ class IPMIConsole(base.ConsoleInterface):
         :param pw_file: password file to be used in ipmitool command
         :returns: returns a command string for ipmitool
         """
-        user = driver_info.get('username')
-        user = ' -U {}'.format(user) if user else ''
-        return ("ipmitool -H %(address)s -I lanplus"
-                "%(user)s -f %(pwfile)s"
-                % {'address': driver_info['address'],
-                   'user': user,
-                   'pwfile': pw_file})
+        return ' '.join(_get_ipmitool_args(driver_info, pw_file=pw_file))
 
     def _start_console(self, driver_info, start_method):
         """Start a remote console for the node.
@@ -1232,15 +1455,8 @@ class IPMIConsole(base.ConsoleInterface):
         pw_file = console_utils.make_persistent_password_file(
             path, driver_info['password'] or '\0')
         ipmi_cmd = self._get_ipmi_cmd(driver_info, pw_file)
+        ipmi_cmd += ' sol activate'
 
-        for name, option in BRIDGING_OPTIONS:
-            if driver_info[name] is not None:
-                ipmi_cmd = " ".join([ipmi_cmd,
-                                     option, driver_info[name]])
-
-        if CONF.debug:
-            ipmi_cmd += " -v"
-        ipmi_cmd += " sol activate"
         try:
             start_method(driver_info['uuid'], driver_info['port'], ipmi_cmd)
         except (exception.ConsoleError, exception.ConsoleSubprocessFailed):
@@ -1278,6 +1494,9 @@ class IPMIShellinaboxConsole(IPMIConsole):
         :raises: ConsoleSubprocessFailed when invoking the subprocess failed
         """
         driver_info = _parse_driver_info(task.node)
+        if not driver_info['port']:
+            driver_info['port'] = _allocate_port(task)
+
         self._start_console(driver_info,
                             console_utils.start_shellinabox_console)
 
@@ -1293,6 +1512,7 @@ class IPMIShellinaboxConsole(IPMIConsole):
         finally:
             ironic_utils.unlink_without_raise(
                 _console_pwfile_path(task.node.uuid))
+        _release_allocated_port(task)
 
     @METRICS.timer('IPMIShellinaboxConsole.get_console')
     def get_console(self, task):
@@ -1318,6 +1538,10 @@ class IPMISocatConsole(IPMIConsole):
         :raises: ConsoleSubprocessFailed when invoking the subprocess failed
         """
         driver_info = _parse_driver_info(task.node)
+        if not driver_info['port']:
+            driver_info['port'] = _allocate_port(
+                task, host=CONF.console.socat_address)
+
         try:
             self._exec_stop_console(driver_info)
         except OSError:
@@ -1341,6 +1565,7 @@ class IPMISocatConsole(IPMIConsole):
             ironic_utils.unlink_without_raise(
                 _console_pwfile_path(task.node.uuid))
         self._exec_stop_console(driver_info)
+        _release_allocated_port(task)
 
     def _exec_stop_console(self, driver_info):
         cmd = "sol deactivate"
