@@ -15,7 +15,7 @@
 
 import collections
 
-from neutronclient.common import exceptions as neutron_exceptions
+from openstack.connection import exceptions as openstack_exc
 from oslo_config import cfg
 from oslo_log import log
 
@@ -83,7 +83,7 @@ def _is_port_physnet_allowed(port, physnets):
             or port.physical_network in physnets)
 
 
-def _get_free_portgroups_and_ports(task, vif_id, physnets):
+def _get_free_portgroups_and_ports(task, vif_id, physnets, vif_info={}):
     """Get free portgroups and ports.
 
     It only returns ports or portgroups that can be used for attachment of
@@ -95,6 +95,8 @@ def _get_free_portgroups_and_ports(task, vif_id, physnets):
         attached. This is governed by the segments of the VIF's network. An
         empty set indicates that the ports' physical networks should be
         ignored.
+    :param vif_info: dict that may contain extra information, such as
+        port_uuid
     :returns: list of free ports and portgroups.
     :raises: VifAlreadyAttached, if vif_id is attached to any of the
         node's ports or portgroups.
@@ -109,9 +111,18 @@ def _get_free_portgroups_and_ports(task, vif_id, physnets):
     # at least one port with vif already attached to it
     non_usable_portgroups = set()
 
+    port_uuid = None
+    portgroup_uuid = None
+    if 'port_uuid' in vif_info:
+        port_uuid = vif_info['port_uuid']
+    elif 'portgroup_uuid' in vif_info:
+        portgroup_uuid = vif_info['portgroup_uuid']
+
     for p in task.ports:
+        # If port_uuid is specified in vif_info, check id
         # Validate that port has needed information
-        if not neutron.validate_port_info(task.node, p):
+        if ((port_uuid and port_uuid != p.uuid)
+            or not neutron.validate_port_info(task.node, p)):
             continue
         if _vif_attached(p, vif_id):
             # Consider such portgroup unusable. The fact that we can have None
@@ -120,27 +131,30 @@ def _get_free_portgroups_and_ports(task, vif_id, physnets):
             continue
         if not _is_port_physnet_allowed(p, physnets):
             continue
-        if p.portgroup_id is None:
-            # ports without portgroup_id are always considered candidates
+        if p.portgroup_id is None and not portgroup_uuid:
             free_port_like_objs.append(p)
         else:
             ports_by_portgroup[p.portgroup_id].append(p)
 
-    for pg in task.portgroups:
-        if _vif_attached(pg, vif_id):
-            continue
-        if pg.id in non_usable_portgroups:
-            # This portgroup has vifs attached to its ports, consider its
-            # ports instead to avoid collisions
-            free_port_like_objs.extend(ports_by_portgroup[pg.id])
-        # Also ignore empty portgroups
-        elif ports_by_portgroup[pg.id]:
-            free_port_like_objs.append(pg)
+    if not port_uuid:
+        for pg in task.portgroups:
+            # if portgroup_uuid is specified in vif_info, check id
+            if ((portgroup_uuid and portgroup_uuid != pg.uuid)
+                or _vif_attached(pg, vif_id)):
+                continue
+            if pg.id in non_usable_portgroups:
+                # This portgroup has vifs attached to its ports, consider its
+                # ports instead to avoid collisions
+                if not portgroup_uuid:
+                    free_port_like_objs.extend(ports_by_portgroup[pg.id])
+            # Also ignore empty portgroups
+            elif ports_by_portgroup[pg.id]:
+                free_port_like_objs.append(pg)
 
     return free_port_like_objs
 
 
-def get_free_port_like_object(task, vif_id, physnets):
+def get_free_port_like_object(task, vif_id, physnets, vif_info={}):
     """Find free port-like object (portgroup or port) VIF will be attached to.
 
     Ensures that the VIF is not already attached to this node.  When selecting
@@ -160,6 +174,8 @@ def get_free_port_like_object(task, vif_id, physnets):
         attached. This is governed by the segments of the VIF's network. An
         empty set indicates that the ports' physical networks should be
         ignored.
+    :param vif_info: dict that may contain extra information, such as
+        port_uuid
     :raises: VifAlreadyAttached, if VIF is already attached to the node.
     :raises: NoFreePhysicalPorts, if there is no port-like object VIF can be
         attached to.
@@ -167,8 +183,8 @@ def get_free_port_like_object(task, vif_id, physnets):
              has ports which are not all assigned the same physical network.
     :returns: port-like object VIF will be attached to.
     """
-    free_port_like_objs = _get_free_portgroups_and_ports(task, vif_id,
-                                                         physnets)
+    free_port_like_objs = _get_free_portgroups_and_ports(
+        task, vif_id, physnets, vif_info)
 
     if not free_port_like_objs:
         raise exception.NoFreePhysicalPorts(vif=vif_id)
@@ -208,8 +224,8 @@ def plug_port_to_tenant_network(task, port_like_obj, client=None):
     :param task: A TaskManager instance.
     :param port_like_obj: port-like object to plug.
     :param client: Neutron client instance.
-    :raises NetworkError: if failed to update Neutron port.
-    :raises VifNotAttached if tenant VIF is not associated with port_like_obj.
+    :raises: NetworkError if failed to update Neutron port.
+    :raises: VifNotAttached if tenant VIF is not associated with port_like_obj.
     """
 
     node = task.node
@@ -251,31 +267,42 @@ def plug_port_to_tenant_network(task, port_like_obj, client=None):
     # NOTE(sambetts) Only update required binding: attributes,
     # because other port attributes may have been set by the user or
     # nova.
-    body = {
-        'port': {
-            'binding:vnic_type': neutron.VNIC_BAREMETAL,
-            'binding:host_id': node.uuid,
-        }
-    }
+    port_attrs = {'binding:vnic_type': neutron.VNIC_BAREMETAL,
+                  'binding:host_id': node.uuid}
+    # NOTE(kaifeng) Only update mac address when it's available
+    if port_like_obj.address:
+        port_attrs['mac_address'] = port_like_obj.address
     binding_profile = {'local_link_information': local_link_info}
     if local_group_info:
         binding_profile['local_group_information'] = local_group_info
-    body['port']['binding:profile'] = binding_profile
+    port_attrs['binding:profile'] = binding_profile
 
     if client_id_opt:
-        body['port']['extra_dhcp_opts'] = [client_id_opt]
+        port_attrs['extra_dhcp_opts'] = [client_id_opt]
+
+    is_smart_nic = neutron.is_smartnic_port(port_like_obj)
+    if is_smart_nic:
+        link_info = local_link_info[0]
+        LOG.debug('Setting hostname as host_id in case of Smart NIC, '
+                  'port %(port_id)s, hostname %(hostname)s',
+                  {'port_id': vif_id, 'hostname': link_info['hostname']})
+        port_attrs['binding:host_id'] = link_info['hostname']
+        port_attrs['binding:vnic_type'] = neutron.VNIC_SMARTNIC
 
     if not client:
         client = neutron.get_client(context=task.context)
 
+    if is_smart_nic:
+        neutron.wait_for_host_agent(client, port_attrs['binding:host_id'])
+
     try:
-        client.update_port(vif_id, body)
-    except neutron_exceptions.ConnectionFailed as e:
+        neutron.update_neutron_port(task.context, vif_id, port_attrs)
+        if is_smart_nic:
+            neutron.wait_for_port_status(client, vif_id, 'ACTIVE')
+    except openstack_exc.OpenStackCloudException as e:
         msg = (_('Could not add public network VIF %(vif)s '
                  'to node %(node)s, possible network issue. %(exc)s') %
-               {'vif': vif_id,
-                'node': node.uuid,
-                'exc': e})
+               {'vif': vif_id, 'node': node.uuid, 'exc': e})
         LOG.error(msg)
         raise exception.NetworkError(msg)
 
@@ -374,6 +401,7 @@ class VIFPortIDMixin(object):
         return (p_obj.internal_info.get('cleaning_vif_port_id')
                 or p_obj.internal_info.get('provisioning_vif_port_id')
                 or p_obj.internal_info.get('rescuing_vif_port_id')
+                or p_obj.internal_info.get('inspection_vif_port_id')
                 or self._get_vif_id_by_port_like_obj(p_obj) or None)
 
 
@@ -410,8 +438,7 @@ class NeutronVIFPortIDMixin(VIFPortIDMixin):
             original_port = objects.Port.get_by_id(context, port_obj.id)
             updated_client_id = port_obj.extra.get('client-id')
 
-            if (original_port.extra.get('client-id')
-                != updated_client_id):
+            if original_port.extra.get('client-id') != updated_client_id:
                 # DHCP Option with opt_value=None will remove it
                 # from the neutron port
                 if vif:
@@ -428,8 +455,7 @@ class NeutronVIFPortIDMixin(VIFPortIDMixin):
                         "No VIF found for instance %(instance)s "
                         "port %(port)s when attempting to update port "
                         "client-id.",
-                        {'port': port_uuid,
-                         'instance': node.instance_uuid})
+                        {'port': port_uuid, 'instance': node.instance_uuid})
 
         if portgroup_obj and ((set(port_obj.obj_what_changed())
                               & {'pxe_enabled', 'portgroup_id'}) or vif):
@@ -505,8 +531,8 @@ class NeutronVIFPortIDMixin(VIFPortIDMixin):
 
         :param task: A TaskManager instance.
         :param vif_info: a dictionary of information about a VIF.
-             It must have an 'id' key, whose value is a unique
-             identifier for that VIF.
+                         It must have an 'id' key, whose value is a unique
+                         identifier for that VIF.
         :raises: NetworkError, VifAlreadyAttached, NoFreePhysicalPorts
         :raises: PortgroupPhysnetInconsistent if one of the node's portgroups
                  has ports which are not all assigned the same physical
@@ -535,7 +561,8 @@ class NeutronVIFPortIDMixin(VIFPortIDMixin):
                     raise exception.VifInvalidForAttach(
                         node=task.node.uuid, vif=vif_id, reason=reason)
 
-        port_like_obj = get_free_port_like_object(task, vif_id, physnets)
+        port_like_obj = get_free_port_like_object(
+            task, vif_id, physnets, vif_info)
 
         # Address is optional for portgroups
         if port_like_obj.address:
@@ -562,7 +589,7 @@ class NeutronVIFPortIDMixin(VIFPortIDMixin):
         :param task: A TaskManager instance.
         :param vif_id: A VIF ID to detach
         :raises: VifNotAttached if VIF not attached.
-        :raises: NetworkError: if unbind Neutron port failed.
+        :raises: NetworkError if unbind Neutron port failed.
         """
         # NOTE(mgoddard): Lookup the port first to check that the VIF is
         # attached, and fail if not.
@@ -575,3 +602,51 @@ class NeutronVIFPortIDMixin(VIFPortIDMixin):
         # DELETING state.
         if task.node.provision_state in [states.ACTIVE, states.DELETING]:
             neutron.unbind_neutron_port(vif_id, context=task.context)
+
+    def get_node_network_data(self, task):
+        """Get network configuration data for node ports.
+
+        Pull network data from ironic node object if present, otherwise
+        collect it for Neutron VIFs.
+
+        :param task: A TaskManager instance.
+        :raises: InvalidParameterValue, if the network interface configuration
+            is invalid.
+        :raises: MissingParameterValue, if some parameters are missing.
+        :returns: a dict holding network configuration information adhearing
+            Nova network metadata layout (`network_data.json`).
+        """
+        # NOTE(etingof): static network data takes precedence
+        network_data = (
+            super(NeutronVIFPortIDMixin, self).get_node_network_data(task))
+        if network_data:
+            return network_data
+
+        node = task.node
+
+        LOG.debug('Gathering network data from ports of node '
+                  '%(node)s', {'node': node.uuid})
+
+        network_data = collections.defaultdict(list)
+
+        for port_obj in task.ports:
+            vif_port_id = self.get_current_vif(task, port_obj)
+
+            LOG.debug('Considering node %(node)s port %(port)s, VIF %(vif)s',
+                      {'node': node.uuid, 'port': port_obj.uuid,
+                       'vif': vif_port_id})
+
+            if not vif_port_id:
+                continue
+
+            port_network_data = neutron.get_neutron_port_data(
+                port_obj.uuid, vif_port_id, context=task.context)
+
+            for field, field_data in port_network_data.items():
+                if field_data:
+                    network_data[field].extend(field_data)
+
+        LOG.debug('Collected network data for node %(node)s: %(data)s',
+                  {'node': node.uuid, 'data': network_data})
+
+        return network_data
