@@ -12,7 +12,8 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-
+from oslo_config import cfg
+from oslo_log import log
 from oslo_utils import strutils
 from oslo_utils import uuidutils
 from oslo_utils import versionutils
@@ -29,6 +30,9 @@ from ironic.objects import notification
 
 REQUIRED_INT_PROPERTIES = ['local_gb', 'cpus', 'memory_mb']
 
+CONF = cfg.CONF
+LOG = log.getLogger(__name__)
+
 
 @base.IronicObjectRegistry.register
 class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
@@ -64,7 +68,15 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
     # Version 1.25: Add fault field
     # Version 1.26: Add deploy_step field
     # Version 1.27: Add conductor_group field
-    VERSION = '1.27'
+    # Version 1.28: Add automated_clean field
+    # Version 1.29: Add protected and protected_reason fields
+    # Version 1.30: Add owner field
+    # Version 1.31: Add allocation_id field
+    # Version 1.32: Add description field
+    # Version 1.33: Add retired and retired_reason fields
+    # Version 1.34: Add lessee field
+    # Version 1.35: Add network_data field
+    VERSION = '1.35'
 
     dbapi = db_api.get_instance()
 
@@ -130,6 +142,10 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         'inspection_started_at': object_fields.DateTimeField(nullable=True),
 
         'extra': object_fields.FlexibleDictField(nullable=True),
+        'automated_clean': objects.fields.BooleanField(nullable=True),
+        'protected': objects.fields.BooleanField(),
+        'protected_reason': object_fields.StringField(nullable=True),
+        'allocation_id': object_fields.IntegerField(nullable=True),
 
         'bios_interface': object_fields.StringField(nullable=True),
         'boot_interface': object_fields.StringField(nullable=True),
@@ -144,6 +160,12 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         'storage_interface': object_fields.StringField(nullable=True),
         'vendor_interface': object_fields.StringField(nullable=True),
         'traits': object_fields.ObjectField('TraitList', nullable=True),
+        'owner': object_fields.StringField(nullable=True),
+        'lessee': object_fields.StringField(nullable=True),
+        'description': object_fields.StringField(nullable=True),
+        'retired': objects.fields.BooleanField(nullable=True),
+        'retired_reason': object_fields.StringField(nullable=True),
+        'network_data': object_fields.FlexibleDictField(nullable=True),
     }
 
     def as_dict(self, secure=False):
@@ -153,6 +175,8 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
                 d.get('driver_info', {}), "******")
             d['instance_info'] = strutils.mask_dict_password(
                 d.get('instance_info', {}), "******")
+            d['driver_internal_info'] = strutils.mask_dict_password(
+                d.get('driver_internal_info', {}), "******")
         return d
 
     def _validate_property_values(self, properties):
@@ -405,12 +429,18 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
                         object, e.g.: Node(context)
         :raises: InvalidParameterValue if some property values are invalid.
         """
+
+        for attr_name in ('last_error', 'maintenance_reason'):
+            attr_value = getattr(self, attr_name, '')
+            if (attr_value and isinstance(attr_value, str)
+                    and len(attr_value) > CONF.log_in_db_max_size):
+                LOG.info('Truncating too long %s to %s characters for node %s',
+                         attr_name, CONF.log_in_db_max_size, self.uuid)
+                setattr(self, attr_name,
+                        attr_value[0:CONF.log_in_db_max_size])
+
         updates = self.do_version_changes_for_db()
         self._validate_property_values(updates.get('properties'))
-        if 'driver' in updates and 'driver_internal_info' not in updates:
-            # Clean driver_internal_info when changes driver
-            self.driver_internal_info = {}
-            updates = self.do_version_changes_for_db()
         self._validate_and_remove_traits(updates)
         self._validate_and_format_conductor_group(updates)
         db_node = self.dbapi.update_node(self.uuid, updates)
@@ -488,18 +518,6 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         node = cls._from_db_object(context, cls(), db_node)
         return node
 
-    def _convert_fault_field(self, target_version,
-                             remove_unavailable_fields=True):
-        fault_is_set = self.obj_attr_is_set('fault')
-        if target_version >= (1, 25):
-            if not fault_is_set:
-                self.fault = None
-        elif fault_is_set:
-            if remove_unavailable_fields:
-                delattr(self, 'fault')
-            elif self.fault is not None:
-                self.fault = None
-
     def _convert_deploy_step_field(self, target_version,
                                    remove_unavailable_fields=True):
         # NOTE(rloo): Typically we set the value to None. However,
@@ -529,6 +547,41 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
             elif self.conductor_group:
                 self.conductor_group = ''
 
+    def _convert_network_data_field(self, target_version,
+                                    remove_unavailable_fields=True):
+        # NOTE(etingof): The default value for `network_data` is an empty
+        # dict. Therefore we can't use generic version adjustment
+        # routine.
+        field_is_set = self.obj_attr_is_set('network_data')
+        if target_version >= (1, 35):
+            if not field_is_set:
+                self.network_data = {}
+        elif field_is_set:
+            if remove_unavailable_fields:
+                delattr(self, 'network_data')
+            elif self.network_data:
+                self.network_data = {}
+
+    # NOTE (yolanda): new method created to avoid repeating code in
+    # _convert_to_version, and to avoid pep8 too complex error
+    def _adjust_field_to_version(self, field_name, field_default_value,
+                                 target_version, major, minor,
+                                 remove_unavailable_fields):
+        field_is_set = self.obj_attr_is_set(field_name)
+        if target_version >= (major, minor):
+            # target version supports the major/minor specified
+            if not field_is_set:
+                # set it to its default value if it is not set
+                setattr(self, field_name, field_default_value)
+        elif field_is_set:
+            # target version does not support the field, and it is set
+            if remove_unavailable_fields:
+                # (De)serialising: remove unavailable fields
+                delattr(self, field_name)
+            elif getattr(self, field_name) is not field_default_value:
+                # DB: set unavailable field to the default value
+                setattr(self, field_name, field_default_value)
+
     def _convert_to_version(self, target_version,
                             remove_unavailable_fields=True):
         """Convert to the target version.
@@ -552,6 +605,22 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
             this, it should be removed.
         Version 1.27: conductor_group field was added. For versions prior to
             this, it should be removed.
+        Version 1.28: automated_clean was added. For versions prior to this, it
+            should be set to None (or removed).
+        Version 1.29: protected was added. For versions prior to this, it
+            should be set to False (or removed).
+        Version 1.30: owner was added. For versions prior to this, it should be
+            set to None or removed.
+        Version 1.31: allocation_id was added. For versions prior to this, it
+            should be set to None (or removed).
+        Version 1.32: description was added. For versions prior to this, it
+            should be set to None (or removed).
+        Version 1.33: retired was added. For versions prior to this, it
+            should be set to False (or removed).
+        Version 1.34: lessee was added. For versions prior to this, it should
+            be set to None or removed.
+        Version 1.35: network_data was added. For versions prior to this, it
+            should be set to empty dict (or removed).
 
         :param target_version: the desired version of the object
         :param remove_unavailable_fields: True to remove fields that are
@@ -560,62 +629,41 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
             values; set this to False for DB interactions.
         """
         target_version = versionutils.convert_version_to_tuple(target_version)
-        # Convert the rescue_interface field.
-        rescue_iface_is_set = self.obj_attr_is_set('rescue_interface')
-        if target_version >= (1, 22):
-            # Target version supports rescue_interface.
-            if not rescue_iface_is_set:
-                # Set it to its default value if it is not set.
-                self.rescue_interface = None
-        elif rescue_iface_is_set:
-            # Target version does not support rescue_interface, and it is set.
-            if remove_unavailable_fields:
-                # (De)serialising: remove unavailable fields.
-                delattr(self, 'rescue_interface')
-            elif self.rescue_interface is not None:
-                # DB: set unavailable field to the default of None.
-                self.rescue_interface = None
 
-        traits_is_set = self.obj_attr_is_set('traits')
-        if target_version >= (1, 23):
-            # Target version supports traits.
-            if not traits_is_set:
-                self.traits = None
-        elif traits_is_set:
-            if remove_unavailable_fields:
-                delattr(self, 'traits')
-            elif self.traits is not None:
-                self.traits = None
+        # Convert the different fields depending on version
+        fields = [('rescue_interface', 22), ('traits', 23),
+                  ('bios_interface', 24), ('fault', 25),
+                  ('automated_clean', 28), ('protected_reason', 29),
+                  ('owner', 30), ('allocation_id', 31), ('description', 32),
+                  ('retired_reason', 33), ('lessee', 34)]
 
-        bios_iface_is_set = self.obj_attr_is_set('bios_interface')
-        if target_version >= (1, 24):
-            # Target version supports bios_interface.
-            if not bios_iface_is_set:
-                # Set it to its default value if it is not set.
-                self.bios_interface = None
-        elif bios_iface_is_set:
-            # Target version does not support bios_interface, and it is set.
-            if remove_unavailable_fields:
-                # (De)serialising: remove unavailable fields.
-                delattr(self, 'bios_interface')
-            elif self.bios_interface is not None:
-                # DB: set unavailable field to the default of None.
-                self.bios_interface = None
+        for name, minor in fields:
+            self._adjust_field_to_version(name, None, target_version,
+                                          1, minor, remove_unavailable_fields)
+
+        # NOTE(dtantsur): the default is False for protected
+        self._adjust_field_to_version('protected', False, target_version,
+                                      1, 29, remove_unavailable_fields)
 
-        self._convert_fault_field(target_version, remove_unavailable_fields)
         self._convert_deploy_step_field(target_version,
                                         remove_unavailable_fields)
         self._convert_conductor_group_field(target_version,
                                             remove_unavailable_fields)
 
+        self._adjust_field_to_version('retired', False, target_version,
+                                      1, 33, remove_unavailable_fields)
+
+        self._convert_network_data_field(target_version,
+                                         remove_unavailable_fields)
+
 
 @base.IronicObjectRegistry.register
 class NodePayload(notification.NotificationPayloadBase):
     """Base class used for all notification payloads about a Node object."""
     # NOTE: This payload does not include the Node fields "chassis_id",
     # "driver_info", "driver_internal_info", "instance_info", "raid_config",
-    # "reservation", or "target_raid_config". These were excluded for reasons
-    # including:
+    # "network_data", "reservation", or "target_raid_config". These were
+    # excluded for reasons including:
     # - increased complexity needed for creating the payload
     # - sensitive information in the fields that shouldn't be exposed to
     #   external services
@@ -626,6 +674,7 @@ class NodePayload(notification.NotificationPayloadBase):
         'console_enabled': ('node', 'console_enabled'),
         'created_at': ('node', 'created_at'),
         'deploy_step': ('node', 'deploy_step'),
+        'description': ('node', 'description'),
         'driver': ('node', 'driver'),
         'extra': ('node', 'extra'),
         'inspection_finished_at': ('node', 'inspection_finished_at'),
@@ -648,11 +697,17 @@ class NodePayload(notification.NotificationPayloadBase):
         'rescue_interface': ('node', 'rescue_interface'),
         'storage_interface': ('node', 'storage_interface'),
         'vendor_interface': ('node', 'vendor_interface'),
+        'owner': ('node', 'owner'),
+        'lessee': ('node', 'lessee'),
         'power_state': ('node', 'power_state'),
         'properties': ('node', 'properties'),
+        'protected': ('node', 'protected'),
+        'protected_reason': ('node', 'protected_reason'),
         'provision_state': ('node', 'provision_state'),
         'provision_updated_at': ('node', 'provision_updated_at'),
         'resource_class': ('node', 'resource_class'),
+        'retired': ('node', 'retired'),
+        'retired_reason': ('node', 'retired_reason'),
         'target_power_state': ('node', 'target_power_state'),
         'target_provision_state': ('node', 'target_provision_state'),
         'updated_at': ('node', 'updated_at'),
@@ -671,13 +726,19 @@ class NodePayload(notification.NotificationPayloadBase):
     # Version 1.8: Add bios interface field exposed via API.
     # Version 1.9: Add deploy_step field exposed via API.
     # Version 1.10: Add conductor_group field exposed via API.
-    VERSION = '1.10'
+    # Version 1.11: Add protected and protected_reason fields exposed via API.
+    # Version 1.12: Add node owner field.
+    # Version 1.13: Add description field.
+    # Version 1.14: Add retired and retired_reason fields exposed via API.
+    # Version 1.15: Add node lessee field.
+    VERSION = '1.15'
     fields = {
         'clean_step': object_fields.FlexibleDictField(nullable=True),
         'conductor_group': object_fields.StringField(nullable=True),
         'console_enabled': object_fields.BooleanField(nullable=True),
         'created_at': object_fields.DateTimeField(nullable=True),
         'deploy_step': object_fields.FlexibleDictField(nullable=True),
+        'description': object_fields.StringField(nullable=True),
         'driver': object_fields.StringField(nullable=True),
         'extra': object_fields.FlexibleDictField(nullable=True),
         'inspection_finished_at': object_fields.DateTimeField(nullable=True),
@@ -700,11 +761,17 @@ class NodePayload(notification.NotificationPayloadBase):
         'storage_interface': object_fields.StringField(nullable=True),
         'vendor_interface': object_fields.StringField(nullable=True),
         'name': object_fields.StringField(nullable=True),
+        'owner': object_fields.StringField(nullable=True),
+        'lessee': object_fields.StringField(nullable=True),
         'power_state': object_fields.StringField(nullable=True),
         'properties': object_fields.FlexibleDictField(nullable=True),
+        'protected': object_fields.BooleanField(nullable=True),
+        'protected_reason': object_fields.StringField(nullable=True),
         'provision_state': object_fields.StringField(nullable=True),
         'provision_updated_at': object_fields.DateTimeField(nullable=True),
         'resource_class': object_fields.StringField(nullable=True),
+        'retired': object_fields.BooleanField(nullable=True),
+        'retired_reason': object_fields.StringField(nullable=True),
         'target_power_state': object_fields.StringField(nullable=True),
         'target_provision_state': object_fields.StringField(nullable=True),
         'traits': object_fields.ListOfStringsField(nullable=True),
@@ -748,7 +815,12 @@ class NodeSetPowerStatePayload(NodePayload):
     # Version 1.8: Parent NodePayload version 1.8
     # Version 1.9: Parent NodePayload version 1.9
     # Version 1.10: Parent NodePayload version 1.10
-    VERSION = '1.10'
+    # Version 1.11: Parent NodePayload version 1.11
+    # Version 1.12: Parent NodePayload version 1.12
+    # Version 1.13: Parent NodePayload version 1.13
+    # Version 1.14: Parent NodePayload version 1.14
+    # Version 1.15: Parent NodePayload version 1.15
+    VERSION = '1.15'
 
     fields = {
         # "to_power" indicates the future target_power_state of the node. A
@@ -799,7 +871,12 @@ class NodeCorrectedPowerStatePayload(NodePayload):
     # Version 1.8: Parent NodePayload version 1.8
     # Version 1.9: Parent NodePayload version 1.9
     # Version 1.10: Parent NodePayload version 1.10
-    VERSION = '1.10'
+    # Version 1.11: Parent NodePayload version 1.11
+    # Version 1.12: Parent NodePayload version 1.12
+    # Version 1.13: Parent NodePayload version 1.13
+    # Version 1.14: Parent NodePayload version 1.14
+    # Version 1.15: Parent NodePayload version 1.15
+    VERSION = '1.15'
 
     fields = {
         'from_power': object_fields.StringField(nullable=True)
@@ -834,13 +911,21 @@ class NodeSetProvisionStatePayload(NodePayload):
     # Version 1.8: Parent NodePayload version 1.8
     # Version 1.9: Parent NodePayload version 1.9
     # Version 1.10: Parent NodePayload version 1.10
-    VERSION = '1.10'
+    # Version 1.11: Parent NodePayload version 1.11
+    # Version 1.12: Parent NodePayload version 1.12
+    # Version 1.13: Parent NodePayload version 1.13
+    # Version 1.14: Parent NodePayload version 1.14
+    # Version 1.15: Parent NodePayload version 1.15
+    # Version 1.16: add driver_internal_info
+    VERSION = '1.16'
 
     SCHEMA = dict(NodePayload.SCHEMA,
-                  **{'instance_info': ('node', 'instance_info')})
+                  **{'instance_info': ('node', 'instance_info'),
+                     'driver_internal_info': ('node', 'driver_internal_info')})
 
     fields = {
         'instance_info': object_fields.FlexibleDictField(nullable=True),
+        'driver_internal_info': object_fields.FlexibleDictField(nullable=True),
         'event': object_fields.StringField(nullable=True),
         'previous_provision_state': object_fields.StringField(nullable=True),
         'previous_target_provision_state':
@@ -876,7 +961,12 @@ class NodeCRUDPayload(NodePayload):
     # Version 1.6: Parent NodePayload version 1.8
     # Version 1.7: Parent NodePayload version 1.9
     # Version 1.8: Parent NodePayload version 1.10
-    VERSION = '1.8'
+    # Version 1.9: Parent NodePayload version 1.11
+    # Version 1.10: Parent NodePayload version 1.12
+    # Version 1.11: Parent NodePayload version 1.13
+    # Version 1.12: Parent NodePayload version 1.14
+    # Version 1.13: Parent NodePayload version 1.15
+    VERSION = '1.13'
 
     SCHEMA = dict(NodePayload.SCHEMA,
                   **{'instance_info': ('node', 'instance_info'),
