@@ -16,21 +16,20 @@ Tests for the API /volume connectors/ methods.
 """
 
 import datetime
+from http import client as http_client
+from unittest import mock
+from urllib import parse as urlparse
 
-import mock
 from oslo_config import cfg
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
-import six
-from six.moves import http_client
-from six.moves.urllib import parse as urlparse
-from wsme import types as wtypes
 
 from ironic.api.controllers import base as api_base
 from ironic.api.controllers import v1 as api_v1
 from ironic.api.controllers.v1 import notification_utils
 from ironic.api.controllers.v1 import utils as api_utils
 from ironic.api.controllers.v1 import volume_connector as api_volume_connector
+from ironic.api import types as atypes
 from ironic.common import exception
 from ironic.conductor import rpcapi
 from ironic import objects
@@ -55,7 +54,7 @@ class TestVolumeConnectorObject(base.TestCase):
         connector_dict = apiutils.volume_connector_post_data(node_id=None)
         del connector_dict['extra']
         connector = api_volume_connector.VolumeConnector(**connector_dict)
-        self.assertEqual(wtypes.Unset, connector.extra)
+        self.assertEqual(atypes.Unset, connector.extra)
 
 
 class TestListVolumeConnectors(test_api_base.BaseApiTest):
@@ -115,7 +114,7 @@ class TestListVolumeConnectors(test_api_base.BaseApiTest):
             '/volume/connectors/%s?fields=%s' % (connector.uuid, fields),
             headers=self.headers)
         # We always append "links"
-        self.assertItemsEqual(['connector_id', 'extra', 'links'], data)
+        self.assertCountEqual(['connector_id', 'extra', 'links'], data)
 
     def test_get_collection_custom_fields(self):
         fields = 'uuid,extra'
@@ -132,7 +131,7 @@ class TestListVolumeConnectors(test_api_base.BaseApiTest):
         self.assertEqual(3, len(data['connectors']))
         for connector in data['connectors']:
             # We always append "links"
-            self.assertItemsEqual(['uuid', 'extra', 'links'], connector)
+            self.assertCountEqual(['uuid', 'extra', 'links'], connector)
 
     def test_get_custom_fields_invalid_fields(self):
         connector = obj_utils.create_test_volume_connector(
@@ -222,7 +221,7 @@ class TestListVolumeConnectors(test_api_base.BaseApiTest):
         self.assertEqual(len(connectors), len(data['connectors']))
 
         uuids = [n['uuid'] for n in data['connectors']]
-        six.assertCountEqual(self, connectors, uuids)
+        self.assertCountEqual(connectors, uuids)
 
     def test_links(self):
         uuid = uuidutils.generate_uuid()
@@ -234,9 +233,9 @@ class TestListVolumeConnectors(test_api_base.BaseApiTest):
         self.assertIn('links', data)
         self.assertEqual(2, len(data['links']))
         self.assertIn(uuid, data['links'][0]['href'])
-        for l in data['links']:
-            bookmark = l['rel'] == 'bookmark'
-            self.assertTrue(self.validate_link(l['href'], bookmark=bookmark,
+        for link in data['links']:
+            bookmark = link['rel'] == 'bookmark'
+            self.assertTrue(self.validate_link(link['href'], bookmark=bookmark,
                                                headers=self.headers))
 
     def test_collection_links(self):
@@ -273,6 +272,47 @@ class TestListVolumeConnectors(test_api_base.BaseApiTest):
         next_marker = data['connectors'][-1]['uuid']
         self.assertIn(next_marker, data['next'])
 
+    def test_collection_links_custom_fields(self):
+        cfg.CONF.set_override('max_limit', 3, 'api')
+        connectors = []
+        fields = 'uuid,extra'
+        for i in range(5):
+            connector = obj_utils.create_test_volume_connector(
+                self.context, node_id=self.node.id,
+                uuid=uuidutils.generate_uuid(),
+                connector_id='test-connector_id-%s' % i)
+            connectors.append(connector.uuid)
+
+        data = self.get_json(
+            '/volume/connectors?fields=%s' % fields,
+            headers=self.headers)
+
+        self.assertEqual(3, len(data['connectors']))
+        self.assertIn('volume/connectors', data['next'])
+
+        next_marker = data['connectors'][-1]['uuid']
+        self.assertIn(next_marker, data['next'])
+        self.assertIn('fields', data['next'])
+
+    def test_get_collection_pagination_no_uuid(self):
+        fields = 'connector_id'
+        limit = 2
+        connectors = []
+        for id_ in range(3):
+            volume_connector = obj_utils.create_test_volume_connector(
+                self.context,
+                node_id=self.node.id,
+                connector_id='test-connector_id-%s' % id_,
+                uuid=uuidutils.generate_uuid())
+            connectors.append(volume_connector)
+
+        data = self.get_json(
+            '/volume/connectors?fields=%s&limit=%s' % (fields, limit),
+            headers=self.headers)
+
+        self.assertEqual(limit, len(data['connectors']))
+        self.assertIn('marker=%s' % connectors[limit - 1].uuid, data['next'])
+
     def test_collection_links_detail(self):
         connectors = []
         for id_ in range(5):
