@@ -16,21 +16,20 @@ Tests for the API /volume targets/ methods.
 """
 
 import datetime
+from http import client as http_client
+from unittest import mock
+from urllib import parse as urlparse
 
-import mock
 from oslo_config import cfg
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
-import six
-from six.moves import http_client
-from six.moves.urllib import parse as urlparse
-from wsme import types as wtypes
 
 from ironic.api.controllers import base as api_base
 from ironic.api.controllers import v1 as api_v1
 from ironic.api.controllers.v1 import notification_utils
 from ironic.api.controllers.v1 import utils as api_utils
 from ironic.api.controllers.v1 import volume_target as api_volume_target
+from ironic.api import types as atypes
 from ironic.common import exception
 from ironic.conductor import rpcapi
 from ironic import objects
@@ -55,7 +54,7 @@ class TestVolumeTargetObject(base.TestCase):
         target_dict = apiutils.volume_target_post_data(node_id=None)
         del target_dict['extra']
         target = api_volume_target.VolumeTarget(**target_dict)
-        self.assertEqual(wtypes.Unset, target.extra)
+        self.assertEqual(atypes.Unset, target.extra)
 
 
 class TestListVolumeTargets(test_api_base.BaseApiTest):
@@ -115,7 +114,7 @@ class TestListVolumeTargets(test_api_base.BaseApiTest):
             '/volume/targets/%s?fields=%s' % (target.uuid, fields),
             headers=self.headers)
         # We always append "links"
-        self.assertItemsEqual(['boot_index', 'extra', 'links'], data)
+        self.assertCountEqual(['boot_index', 'extra', 'links'], data)
 
     def test_get_collection_custom_fields(self):
         fields = 'uuid,extra'
@@ -131,7 +130,7 @@ class TestListVolumeTargets(test_api_base.BaseApiTest):
         self.assertEqual(3, len(data['targets']))
         for target in data['targets']:
             # We always append "links"
-            self.assertItemsEqual(['uuid', 'extra', 'links'], target)
+            self.assertCountEqual(['uuid', 'extra', 'links'], target)
 
     def test_get_custom_fields_invalid_fields(self):
         target = obj_utils.create_test_volume_target(
@@ -212,7 +211,7 @@ class TestListVolumeTargets(test_api_base.BaseApiTest):
         self.assertEqual(len(targets), len(data['targets']))
 
         uuids = [n['uuid'] for n in data['targets']]
-        six.assertCountEqual(self, targets, uuids)
+        self.assertCountEqual(targets, uuids)
 
     def test_links(self):
         uuid = uuidutils.generate_uuid()
@@ -224,9 +223,9 @@ class TestListVolumeTargets(test_api_base.BaseApiTest):
         self.assertIn('links', data)
         self.assertEqual(2, len(data['links']))
         self.assertIn(uuid, data['links'][0]['href'])
-        for l in data['links']:
-            bookmark = l['rel'] == 'bookmark'
-            self.assertTrue(self.validate_link(l['href'], bookmark=bookmark,
+        for link in data['links']:
+            bookmark = link['rel'] == 'bookmark'
+            self.assertTrue(self.validate_link(link['href'], bookmark=bookmark,
                                                headers=self.headers))
 
     def test_collection_links(self):
@@ -258,6 +257,41 @@ class TestListVolumeTargets(test_api_base.BaseApiTest):
         self.assertIn(next_marker, data['next'])
         self.assertIn('volume/targets', data['next'])
 
+    def test_collection_links_custom_fields(self):
+        fields = 'uuid,extra'
+        cfg.CONF.set_override('max_limit', 3, 'api')
+        targets = []
+        for id_ in range(5):
+            target = obj_utils.create_test_volume_target(
+                self.context, node_id=self.node.id,
+                uuid=uuidutils.generate_uuid(), boot_index=id_)
+            targets.append(target.uuid)
+        data = self.get_json('/volume/targets?fields=%s' % fields,
+                             headers=self.headers)
+        self.assertEqual(3, len(data['targets']))
+
+        next_marker = data['targets'][-1]['uuid']
+        self.assertIn(next_marker, data['next'])
+        self.assertIn('volume/targets', data['next'])
+        self.assertIn('fields', data['next'])
+
+    def test_get_collection_pagination_no_uuid(self):
+        fields = 'boot_index'
+        limit = 2
+        targets = []
+        for id_ in range(3):
+            target = obj_utils.create_test_volume_target(
+                self.context, node_id=self.node.id,
+                uuid=uuidutils.generate_uuid(), boot_index=id_)
+            targets.append(target)
+
+        data = self.get_json(
+            '/volume/targets?fields=%s&limit=%s' % (fields, limit),
+            headers=self.headers)
+
+        self.assertEqual(limit, len(data['targets']))
+        self.assertIn('marker=%s' % targets[limit - 1].uuid, data['next'])
+
     def test_collection_links_detail(self):
         targets = []
         for id_ in range(5):
