@@ -15,22 +15,22 @@
 
 
 import datetime
+import importlib
 import time
+from unittest import mock
 
 from glanceclient import client as glance_client
 from glanceclient import exc as glance_exc
-from keystoneauth1 import loading as kaloading
-import mock
+from keystoneauth1 import loading as ks_loading
 from oslo_config import cfg
 from oslo_utils import uuidutils
+import retrying
 import testtools
 
 from ironic.common import context
 from ironic.common import exception
-from ironic.common.glance_service import base_image_service
+from ironic.common.glance_service import image_service
 from ironic.common.glance_service import service_utils
-from ironic.common.glance_service.v2 import image_service as glance_v2
-from ironic.common import image_service as service
 from ironic.tests import base
 from ironic.tests.unit import stubs
 
@@ -48,7 +48,6 @@ class NullWriter(object):
 class TestGlanceSerializer(testtools.TestCase):
     def test_serialize(self):
         metadata = {'name': 'image1',
-                    'is_public': True,
                     'foo': 'bar',
                     'properties': {
                         'prop1': 'propvalue1',
@@ -62,7 +61,6 @@ class TestGlanceSerializer(testtools.TestCase):
 
         expected = {
             'name': 'image1',
-            'is_public': True,
             'foo': 'bar',
             'properties': {'prop1': 'propvalue1',
                            'mappings': [
@@ -95,17 +93,13 @@ class TestGlanceImageService(base.TestCase):
         self.context = context.RequestContext(auth_token=True)
         self.context.user_id = 'fake'
         self.context.project_id = 'fake'
-        self.service = service.GlanceImageService(self.client, 2, self.context)
-
-        self.config(glance_api_servers=['http://localhost'], group='glance')
-        self.config(auth_strategy='keystone', group='glance')
+        self.service = image_service.GlanceImageService(self.client,
+                                                        self.context)
 
     @staticmethod
     def _make_fixture(**kwargs):
         fixture = {'name': None,
-                   'properties': {},
-                   'status': "active",
-                   'is_public': None}
+                   'status': "active"}
         fixture.update(kwargs)
         return stubs.FakeImage(fixture)
 
@@ -128,25 +122,30 @@ class TestGlanceImageService(base.TestCase):
 
     def test_show_passes_through_to_client(self):
         image_id = uuidutils.generate_uuid()
-        image = self._make_fixture(name='image1', is_public=True,
-                                   id=image_id)
+        image = self._make_fixture(name='image1', id=image_id)
         expected = {
+            'checksum': None,
+            'container_format': None,
+            'created_at': None,
+            'deleted': None,
+            'deleted_at': None,
+            'disk_format': None,
+            'file': None,
             'id': image_id,
-            'name': 'image1',
-            'is_public': True,
-            'size': None,
             'min_disk': None,
             'min_ram': None,
-            'disk_format': None,
-            'container_format': None,
-            'checksum': None,
-            'created_at': None,
-            'updated_at': None,
-            'deleted_at': None,
-            'deleted': None,
-            'status': "active",
-            'properties': {},
+            'name': 'image1',
             'owner': None,
+            'properties': {},
+            'protected': None,
+            'schema': None,
+            'size': None,
+            'status': "active",
+            'tags': None,
+            'updated_at': None,
+            'visibility': None,
+            'os_hash_algo': None,
+            'os_hash_value': None,
         }
         with mock.patch.object(self.service, 'call', return_value=image,
                                autospec=True):
@@ -172,14 +171,13 @@ class TestGlanceImageService(base.TestCase):
 
     def test_show_raises_when_image_not_active(self):
         image_id = uuidutils.generate_uuid()
-        image = self._make_fixture(name='image1', is_public=True,
-                                   id=image_id, status="queued")
+        image = self._make_fixture(name='image1', id=image_id, status="queued")
         with mock.patch.object(self.service, 'call', return_value=image,
                                autospec=True):
             self.assertRaises(exception.ImageUnacceptable,
                               self.service.show, image_id)
 
-    @mock.patch.object(time, 'sleep', autospec=True)
+    @mock.patch.object(retrying.time, 'sleep', autospec=True)
     def test_download_with_retries(self, mock_sleep):
         tries = [0]
 
@@ -196,18 +194,22 @@ class TestGlanceImageService(base.TestCase):
         stub_context = context.RequestContext(auth_token=True)
         stub_context.user_id = 'fake'
         stub_context.project_id = 'fake'
-        stub_service = service.GlanceImageService(stub_client, 1, stub_context)
+        stub_service = image_service.GlanceImageService(stub_client,
+                                                        stub_context)
         image_id = uuidutils.generate_uuid()
         writer = NullWriter()
 
         # When retries are disabled, we should get an exception
-        self.config(glance_num_retries=0, group='glance')
+        self.config(num_retries=0, group='glance')
         self.assertRaises(exception.GlanceConnectionFailed,
                           stub_service.download, image_id, writer)
 
         # Now lets enable retries. No exception should happen now.
+        self.config(num_retries=1, group='glance')
+        importlib.reload(image_service)
+        stub_service = image_service.GlanceImageService(stub_client,
+                                                        stub_context)
         tries = [0]
-        self.config(glance_num_retries=1, group='glance')
         stub_service.download(image_id, writer)
         self.assertTrue(mock_sleep.called)
 
@@ -242,16 +244,15 @@ class TestGlanceImageService(base.TestCase):
         stub_context.project_id = 'fake'
         stub_client = MyGlanceStubClient()
 
-        stub_service = service.GlanceImageService(stub_client,
-                                                  context=stub_context,
-                                                  version=2)
+        stub_service = image_service.GlanceImageService(stub_client,
+                                                        context=stub_context)
         image_id = uuidutils.generate_uuid()
 
         self.config(allowed_direct_url_schemes=['file'], group='glance')
 
-        # patching open in base_image_service module namespace
+        # patching open in image_service module namespace
         # to make call-spec assertions
-        with mock.patch('ironic.common.glance_service.base_image_service.open',
+        with mock.patch('ironic.common.glance_service.image_service.open',
                         new=mock.mock_open(), create=True) as mock_ironic_open:
             with open('/whatever/target', 'w') as mock_target_fd:
                 stub_service.download(image_id, mock_target_fd)
@@ -279,7 +280,8 @@ class TestGlanceImageService(base.TestCase):
         stub_context = context.RequestContext(auth_token=True)
         stub_context.user_id = 'fake'
         stub_context.project_id = 'fake'
-        stub_service = service.GlanceImageService(stub_client, 1, stub_context)
+        stub_service = image_service.GlanceImageService(stub_client,
+                                                        stub_context)
         image_id = uuidutils.generate_uuid()
         writer = NullWriter()
         self.assertRaises(exception.ImageNotAuthorized, stub_service.download,
@@ -295,7 +297,8 @@ class TestGlanceImageService(base.TestCase):
         stub_context = context.RequestContext(auth_token=True)
         stub_context.user_id = 'fake'
         stub_context.project_id = 'fake'
-        stub_service = service.GlanceImageService(stub_client, 1, stub_context)
+        stub_service = image_service.GlanceImageService(stub_client,
+                                                        stub_context)
         image_id = uuidutils.generate_uuid()
         writer = NullWriter()
         self.assertRaises(exception.ImageNotAuthorized, stub_service.download,
@@ -311,7 +314,8 @@ class TestGlanceImageService(base.TestCase):
         stub_context = context.RequestContext(auth_token=True)
         stub_context.user_id = 'fake'
         stub_context.project_id = 'fake'
-        stub_service = service.GlanceImageService(stub_client, 1, stub_context)
+        stub_service = image_service.GlanceImageService(stub_client,
+                                                        stub_context)
         image_id = uuidutils.generate_uuid()
         writer = NullWriter()
         self.assertRaises(exception.ImageNotFound, stub_service.download,
@@ -327,7 +331,8 @@ class TestGlanceImageService(base.TestCase):
         stub_context = context.RequestContext(auth_token=True)
         stub_context.user_id = 'fake'
         stub_context.project_id = 'fake'
-        stub_service = service.GlanceImageService(stub_client, 1, stub_context)
+        stub_service = image_service.GlanceImageService(stub_client,
+                                                        stub_context)
         image_id = uuidutils.generate_uuid()
         writer = NullWriter()
         self.assertRaises(exception.ImageNotFound, stub_service.download,
@@ -346,10 +351,10 @@ class CheckImageServiceTestCase(base.TestCase):
     def setUp(self):
         super(CheckImageServiceTestCase, self).setUp()
         self.context = context.RequestContext(global_request_id='global')
-        self.service = service.GlanceImageService(None, 1, self.context)
+        self.service = image_service.GlanceImageService(None, self.context)
         # NOTE(pas-ha) register keystoneauth dynamic options manually
-        plugin = kaloading.get_plugin_loader('password')
-        opts = kaloading.get_auth_plugin_conf_options(plugin)
+        plugin = ks_loading.get_plugin_loader('password')
+        opts = ks_loading.get_auth_plugin_conf_options(plugin)
         self.cfg_fixture.register_opts(opts, group='glance')
         self.config(auth_type='password',
                     auth_url='viking',
@@ -359,9 +364,8 @@ class CheckImageServiceTestCase(base.TestCase):
                     service_type='image',
                     region_name='SomeRegion',
                     interface='internal',
-                    auth_strategy='keystone',
                     group='glance')
-        base_image_service._GLANCE_SESSION = None
+        image_service._GLANCE_SESSION = None
 
     def test_check_image_service_client_already_set(self, mock_gclient,
                                                     mock_sess, mock_adapter,
@@ -371,7 +375,7 @@ class CheckImageServiceTestCase(base.TestCase):
 
         self.service.client = True
 
-        wrapped_func = base_image_service.check_image_service(func)
+        wrapped_func = image_service.check_image_service(func)
         self.assertTrue(wrapped_func(self.service))
         self.assertEqual(0, mock_gclient.call_count)
         self.assertEqual(0, mock_sess.call_count)
@@ -381,7 +385,7 @@ class CheckImageServiceTestCase(base.TestCase):
 
     def _assert_client_call(self, mock_gclient, url, user=False):
         mock_gclient.assert_called_once_with(
-            1,
+            2,
             session=mock.sentinel.session,
             global_request_id='global',
             auth=mock.sentinel.sauth if user else mock.sentinel.auth,
@@ -398,7 +402,7 @@ class CheckImageServiceTestCase(base.TestCase):
         uuid = uuidutils.generate_uuid()
         params = {'image_href': uuid}
 
-        wrapped_func = base_image_service.check_image_service(func)
+        wrapped_func = image_service.check_image_service(func)
         self.assertEqual(((), params), wrapped_func(self.service, **params))
         self._assert_client_call(mock_gclient, 'glance_url')
         mock_auth.assert_called_once_with('glance')
@@ -422,7 +426,7 @@ class CheckImageServiceTestCase(base.TestCase):
         uuid = uuidutils.generate_uuid()
         params = {'image_href': uuid}
 
-        wrapped_func = base_image_service.check_image_service(func)
+        wrapped_func = image_service.check_image_service(func)
         self.assertEqual(((), params), wrapped_func(self.service, **params))
         self._assert_client_call(mock_gclient, 'glance_url', user=True)
         mock_sess.assert_called_once_with('glance')
@@ -433,51 +437,22 @@ class CheckImageServiceTestCase(base.TestCase):
                                            mock.sentinel.auth)
         mock_auth.assert_called_once_with('glance')
 
-    def test_check_image_service__deprecated_opts(self, mock_gclient,
-                                                  mock_sess, mock_adapter,
-                                                  mock_sauth, mock_auth):
-        def func(service, *args, **kwargs):
-            return args, kwargs
-
-        mock_adapter.return_value = adapter = mock.Mock()
-        adapter.get_endpoint.return_value = 'glance_url'
-        uuid = uuidutils.generate_uuid()
-        params = {'image_href': uuid}
-        self.config(glance_api_servers='https://localhost:1234',
-                    glance_api_insecure=True,
-                    glance_cafile='cafile',
-                    region_name=None,
-                    group='glance')
-
-        wrapped_func = base_image_service.check_image_service(func)
-        self.assertEqual(((), params), wrapped_func(self.service, **params))
-        self.assertEqual('https://localhost:1234',
-                         base_image_service.CONF.glance.endpoint_override)
-        self._assert_client_call(mock_gclient, 'glance_url')
-        mock_sess.assert_called_once_with('glance', insecure=True,
-                                          cacert='cafile')
-        mock_adapter.assert_called_once_with(
-            'glance', session=mock.sentinel.session,
-            auth=mock.sentinel.auth)
-        self.assertEqual(0, mock_sauth.call_count)
-        mock_auth.assert_called_once_with('glance')
-
     def test_check_image_service__no_auth(self, mock_gclient, mock_sess,
                                           mock_adapter, mock_sauth, mock_auth):
         def func(service, *args, **kwargs):
             return args, kwargs
 
         self.config(endpoint_override='foo',
-                    auth_strategy='noauth',
+                    auth_type='none',
                     group='glance')
         mock_adapter.return_value = adapter = mock.Mock()
         adapter.get_endpoint.return_value = 'foo'
         uuid = uuidutils.generate_uuid()
         params = {'image_href': uuid}
 
-        wrapped_func = base_image_service.check_image_service(func)
+        wrapped_func = image_service.check_image_service(func)
         self.assertEqual(((), params), wrapped_func(self.service, **params))
-        self.assertEqual('none', base_image_service.CONF.glance.auth_type)
+        self.assertEqual('none', image_service.CONF.glance.auth_type)
         self._assert_client_call(mock_gclient, 'foo')
         mock_sess.assert_called_once_with('glance')
         mock_adapter.assert_called_once_with('glance',
@@ -504,7 +479,7 @@ class TestGlanceSwiftTempURL(base.TestCase):
         client = stubs.StubGlanceClient()
         self.context = context.RequestContext()
         self.context.auth_token = 'fake'
-        self.service = service.GlanceImageService(client, 2, self.context)
+        self.service = image_service.GlanceImageService(client, self.context)
         self.config(swift_temp_url_key='correcthorsebatterystaple',
                     group='glance')
         self.config(swift_endpoint_url='https://swift.example.com',
@@ -769,8 +744,8 @@ class TestSwiftTempUrlCache(base.TestCase):
                     group='glance')
         self.config(swift_store_multiple_containers_seed=0,
                     group='glance')
-        self.glance_service = service.GlanceImageService(client, version=2,
-                                                         context=self.context)
+        self.glance_service = image_service.GlanceImageService(
+            client, context=self.context)
 
     @mock.patch('swiftclient.utils.generate_temp_url', autospec=True)
     def test_add_items_to_cache(self, tempurl_mock):
@@ -819,8 +794,8 @@ class TestSwiftTempUrlCache(base.TestCase):
             {'uuid': fake_image['id'], 'exp_time': exp_time}
         )
         self.glance_service._cache[fake_image['id']] = (
-            glance_v2.TempUrlCacheElement(url=temp_url,
-                                          url_expires_at=exp_time)
+            image_service.TempUrlCacheElement(url=temp_url,
+                                              url_expires_at=exp_time)
         )
 
         cleanup_mock = mock.Mock()
@@ -847,7 +822,7 @@ class TestSwiftTempUrlCache(base.TestCase):
         )
         query = '?temp_url_sig=hmacsig&temp_url_expires=%s'
         self.glance_service._cache[fake_image['id']] = (
-            glance_v2.TempUrlCacheElement(
+            image_service.TempUrlCacheElement(
                 url=(CONF.glance.swift_endpoint_url + path
                      + query % old_exp_time),
                 url_expires_at=old_exp_time)
@@ -876,21 +851,21 @@ class TestSwiftTempUrlCache(base.TestCase):
 
     def test_remove_expired_items_from_cache(self):
         expired_items = {
-            uuidutils.generate_uuid(): glance_v2.TempUrlCacheElement(
+            uuidutils.generate_uuid(): image_service.TempUrlCacheElement(
                 'fake-url-1',
                 int(time.time()) - 10
             ),
-            uuidutils.generate_uuid(): glance_v2.TempUrlCacheElement(
+            uuidutils.generate_uuid(): image_service.TempUrlCacheElement(
                 'fake-url-2',
                 int(time.time()) + 90  # Agent won't be able to start in time
             )
         }
         valid_items = {
-            uuidutils.generate_uuid(): glance_v2.TempUrlCacheElement(
+            uuidutils.generate_uuid(): image_service.TempUrlCacheElement(
                 'fake-url-3',
                 int(time.time()) + 1000
             ),
-            uuidutils.generate_uuid(): glance_v2.TempUrlCacheElement(
+            uuidutils.generate_uuid(): image_service.TempUrlCacheElement(
                 'fake-url-4',
                 int(time.time()) + 2000
             )
@@ -979,20 +954,6 @@ class TestServiceUtils(base.TestCase):
                           service_utils.parse_image_id,
                           u'http://spam.ham/eggs')
 
-    def test_get_glance_api_server_fail(self):
-        self.assertRaises(exception.InvalidImageRef,
-                          service_utils.get_glance_api_server,
-                          u'http://spam.ham/eggs')
-
-    # TODO(pas-ha) remove in Rocky
-    def test_get_glance_api_server(self):
-        self.config(glance_api_servers='http://spam:1234, https://ham',
-                    group='glance')
-        api_servers = {service_utils.get_glance_api_server(
-            uuidutils.generate_uuid()) for i in range(2)}
-        self.assertEqual({'http://spam:1234', 'https://ham'},
-                         api_servers)
-
     def test_is_glance_image(self):
         image_href = u'uui\u0111'
         self.assertFalse(service_utils.is_glance_image(image_href))
@@ -1004,17 +965,3 @@ class TestServiceUtils(base.TestCase):
         self.assertFalse(service_utils.is_glance_image(image_href))
         image_href = None
         self.assertFalse(service_utils.is_glance_image(image_href))
-
-    def test_is_image_href_ordinary_file_name_true(self):
-        image = u"\u0111eploy.iso"
-        result = service_utils.is_image_href_ordinary_file_name(image)
-        self.assertTrue(result)
-
-    def test_is_image_href_ordinary_file_name_false(self):
-        for image in ('733d1c44-a2ea-414b-aca7-69decf20d810',
-                      u'glance://\u0111eploy_iso',
-                      u'http://\u0111eploy_iso',
-                      u'https://\u0111eploy_iso',
-                      u'file://\u0111eploy_iso',):
-            result = service_utils.is_image_href_ordinary_file_name(image)
-            self.assertFalse(result)
