@@ -15,16 +15,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import builtins
+import io
 import os
 import shutil
+from unittest import mock
 
 from ironic_lib import disk_utils
 from ironic_lib import utils as ironic_utils
-import mock
 from oslo_concurrency import processutils
 from oslo_config import cfg
-import six
-import six.moves.builtins as __builtin__
 
 from ironic.common import exception
 from ironic.common.glance_service import service_utils as glance_utils
@@ -33,10 +33,6 @@ from ironic.common import images
 from ironic.common import utils
 from ironic.tests import base
 
-if six.PY3:
-    import io
-    file = io.BytesIO
-
 CONF = cfg.CONF
 
 
@@ -46,9 +42,9 @@ class IronicImagesTestCase(base.TestCase):
         pass
 
     @mock.patch.object(image_service, 'get_image_service', autospec=True)
-    @mock.patch.object(__builtin__, 'open', autospec=True)
+    @mock.patch.object(builtins, 'open', autospec=True)
     def test_fetch_image_service(self, open_mock, image_service_mock):
-        mock_file_handle = mock.MagicMock(spec=file)
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'file'
         open_mock.return_value = mock_file_handle
 
@@ -62,10 +58,10 @@ class IronicImagesTestCase(base.TestCase):
 
     @mock.patch.object(image_service, 'get_image_service', autospec=True)
     @mock.patch.object(images, 'image_to_raw', autospec=True)
-    @mock.patch.object(__builtin__, 'open', autospec=True)
+    @mock.patch.object(builtins, 'open', autospec=True)
     def test_fetch_image_service_force_raw(self, open_mock, image_to_raw_mock,
                                            image_service_mock):
-        mock_file_handle = mock.MagicMock(spec=file)
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'file'
         open_mock.return_value = mock_file_handle
 
@@ -254,34 +250,28 @@ class IronicImagesTestCase(base.TestCase):
 
 class FsImageTestCase(base.TestCase):
 
+    @mock.patch.object(builtins, 'open', autospec=True)
     @mock.patch.object(shutil, 'copyfile', autospec=True)
     @mock.patch.object(os, 'makedirs', autospec=True)
-    @mock.patch.object(os.path, 'dirname', autospec=True)
-    @mock.patch.object(os.path, 'exists', autospec=True)
-    def test__create_root_fs(self, path_exists_mock,
-                             dirname_mock, mkdir_mock, cp_mock):
-
-        def path_exists_mock_func(path):
-            return path == 'root_dir'
-
+    def test__create_root_fs(self, mkdir_mock, cp_mock, open_mock):
         files_info = {
             'a1': 'b1',
             'a2': 'b2',
-            'a3': 'sub_dir/b3'}
+            'a3': 'sub_dir/b3',
+            b'a4': 'b4'}
 
-        path_exists_mock.side_effect = path_exists_mock_func
-        dirname_mock.side_effect = ['root_dir', 'root_dir', 'root_dir/sub_dir',
-                                    'root_dir/sub_dir']
         images._create_root_fs('root_dir', files_info)
+
         cp_mock.assert_any_call('a1', 'root_dir/b1')
         cp_mock.assert_any_call('a2', 'root_dir/b2')
         cp_mock.assert_any_call('a3', 'root_dir/sub_dir/b3')
 
-        path_exists_mock.assert_any_call('root_dir/sub_dir')
-        dirname_mock.assert_any_call('root_dir/b1')
-        dirname_mock.assert_any_call('root_dir/b2')
-        dirname_mock.assert_any_call('root_dir/sub_dir/b3')
-        mkdir_mock.assert_called_once_with('root_dir/sub_dir')
+        open_mock.assert_called_once_with('root_dir/b4', 'wb')
+        fp = open_mock.return_value.__enter__.return_value
+        fp.write.assert_called_once_with(b'a4')
+
+        mkdir_mock.assert_any_call('root_dir', exist_ok=True)
+        mkdir_mock.assert_any_call('root_dir/sub_dir', exist_ok=True)
 
     @mock.patch.object(images, '_create_root_fs', autospec=True)
     @mock.patch.object(utils, 'tempdir', autospec=True)
@@ -294,7 +284,7 @@ class FsImageTestCase(base.TestCase):
             self, mkfs_mock, mount_mock, umount_mock, dd_mock, write_mock,
             tempdir_mock, create_root_fs_mock):
 
-        mock_file_handle = mock.MagicMock(spec=file)
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'tempdir'
         tempdir_mock.return_value = mock_file_handle
 
@@ -329,7 +319,7 @@ class FsImageTestCase(base.TestCase):
             self, mkfs_mock, mount_mock, umount_mock, dd_mock,
             tempdir_mock, create_root_fs_mock):
 
-        mock_file_handle = mock.MagicMock(spec=file)
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'tempdir'
         tempdir_mock.return_value = mock_file_handle
         files_info = {'a': 'b'}
@@ -353,7 +343,7 @@ class FsImageTestCase(base.TestCase):
     def test_create_vfat_image_mkfs_fails(self, mkfs_mock, dd_mock,
                                           tempdir_mock):
 
-        mock_file_handle = mock.MagicMock(spec=file)
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'tempdir'
         tempdir_mock.return_value = mock_file_handle
 
@@ -371,7 +361,7 @@ class FsImageTestCase(base.TestCase):
             self, mkfs_mock, mount_mock, umount_mock, dd_mock,
             tempdir_mock, create_root_fs_mock):
 
-        mock_file_handle = mock.MagicMock(spec=file)
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'tempdir'
         tempdir_mock.return_value = mock_file_handle
         umount_mock.side_effect = processutils.ProcessExecutionError
@@ -438,9 +428,9 @@ class FsImageTestCase(base.TestCase):
     @mock.patch.object(os.path, 'relpath', autospec=True)
     @mock.patch.object(os, 'walk', autospec=True)
     @mock.patch.object(utils, 'mount', autospec=True)
-    def test__mount_deploy_iso_fail_no_efibootimg(self, mount_mock,
-                                                  walk_mock, relpath_mock,
-                                                  umount_mock):
+    def test__mount_deploy_iso_fail_no_esp_imageimg(self, mount_mock,
+                                                    walk_mock, relpath_mock,
+                                                    umount_mock):
         walk_mock.return_value = [('/tmpdir1/EFI/ubuntu', [], ['grub.cfg']),
                                   ('/tmpdir1/isolinux', [],
                                    ['isolinux.bin', 'isolinux.cfg'])]
@@ -489,26 +479,22 @@ class FsImageTestCase(base.TestCase):
     @mock.patch.object(images, '_mount_deploy_iso', autospec=True)
     @mock.patch.object(utils, 'tempdir', autospec=True)
     @mock.patch.object(images, '_generate_cfg', autospec=True)
-    def test_create_isolinux_image_for_uefi(
+    def test_create_esp_image_for_uefi_with_deploy_iso(
             self, gen_cfg_mock, tempdir_mock, mount_mock, execute_mock,
             write_to_file_mock, create_root_fs_mock, umount_mock):
 
         files_info = {
             'path/to/kernel': 'vmlinuz',
             'path/to/ramdisk': 'initrd',
-            CONF.isolinux_bin: 'isolinux/isolinux.bin',
-            'path/to/grub': 'relpath/to/grub.cfg',
-            'sourceabspath/to/efiboot.img': 'path/to/efiboot.img'
+            'sourceabspath/to/efiboot.img': 'path/to/efiboot.img',
+            'path/to/grub': 'relpath/to/grub.cfg'
         }
-        cfg = "cfg"
-        cfg_file = 'tmpdir/isolinux/isolinux.cfg'
+
         grubcfg = "grubcfg"
         grub_file = 'tmpdir/relpath/to/grub.cfg'
-        gen_cfg_mock.side_effect = cfg, grubcfg
+        gen_cfg_mock.side_effect = (grubcfg,)
 
         params = ['a=b', 'c']
-        isolinux_options = {'kernel': '/vmlinuz',
-                            'ramdisk': '/initrd'}
         grub_options = {'linux': '/vmlinuz',
                         'initrd': '/initrd'}
 
@@ -517,34 +503,74 @@ class FsImageTestCase(base.TestCase):
             'path/to/grub': 'relpath/to/grub.cfg'}
         grub_rel_path = 'relpath/to/grub.cfg'
         e_img_rel_path = 'path/to/efiboot.img'
-        mock_file_handle = mock.MagicMock(spec=file)
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'tmpdir'
-        mock_file_handle1 = mock.MagicMock(spec=file)
+        mock_file_handle1 = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle1.__enter__.return_value = 'mountdir'
         tempdir_mock.side_effect = mock_file_handle, mock_file_handle1
         mount_mock.return_value = (uefi_path_info,
                                    e_img_rel_path, grub_rel_path)
 
-        images.create_isolinux_image_for_uefi('tgt_file', 'path/to/deploy_iso',
-                                              'path/to/kernel',
-                                              'path/to/ramdisk',
-                                              kernel_params=params)
+        images.create_esp_image_for_uefi('tgt_file',
+                                         'path/to/kernel',
+                                         'path/to/ramdisk',
+                                         deploy_iso='path/to/deploy_iso',
+                                         kernel_params=params)
         mount_mock.assert_called_once_with('path/to/deploy_iso', 'mountdir')
         create_root_fs_mock.assert_called_once_with('tmpdir', files_info)
-        gen_cfg_mock.assert_any_call(params, CONF.isolinux_config_template,
-                                     isolinux_options)
-        write_to_file_mock.assert_any_call(cfg_file, cfg)
         gen_cfg_mock.assert_any_call(params, CONF.grub_config_template,
                                      grub_options)
         write_to_file_mock.assert_any_call(grub_file, grubcfg)
         execute_mock.assert_called_once_with(
-            'mkisofs', '-r', '-V', "VMEDIA_BOOT_ISO", '-cache-inodes', '-J',
-            '-l', '-no-emul-boot', '-boot-load-size', '4', '-boot-info-table',
-            '-b', 'isolinux/isolinux.bin', '-eltorito-alt-boot',
-            '-e', 'path/to/efiboot.img', '-no-emul-boot',
-            '-o', 'tgt_file', 'tmpdir')
+            'mkisofs', '-r', '-V', 'VMEDIA_BOOT_ISO', '-l', '-e',
+            'path/to/efiboot.img', '-no-emul-boot', '-o', 'tgt_file', 'tmpdir')
         umount_mock.assert_called_once_with('mountdir')
 
+    @mock.patch.object(utils, 'write_to_file', autospec=True)
+    @mock.patch.object(images, '_create_root_fs', autospec=True)
+    @mock.patch.object(utils, 'execute', autospec=True)
+    @mock.patch.object(utils, 'tempdir', autospec=True)
+    @mock.patch.object(images, '_generate_cfg', autospec=True)
+    def test_create_esp_image_for_uefi_with_esp_image(
+            self, gen_cfg_mock, tempdir_mock, execute_mock,
+            create_root_fs_mock, write_to_file_mock):
+
+        files_info = {
+            'path/to/kernel': 'vmlinuz',
+            'path/to/ramdisk': 'initrd',
+            'sourceabspath/to/efiboot.img': 'boot/grub/efiboot.img',
+            '/dev/null': 'EFI/MYBOOT/grub.cfg',
+        }
+
+        grub_cfg_file = '/EFI/MYBOOT/grub.cfg'
+        CONF.set_override('grub_config_path', grub_cfg_file)
+        grubcfg = "grubcfg"
+        gen_cfg_mock.side_effect = (grubcfg,)
+
+        params = ['a=b', 'c']
+        grub_options = {'linux': '/vmlinuz',
+                        'initrd': '/initrd'}
+
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
+        mock_file_handle.__enter__.return_value = 'tmpdir'
+        mock_file_handle1 = mock.MagicMock(spec=io.BytesIO)
+        mock_file_handle1.__enter__.return_value = 'mountdir'
+        tempdir_mock.side_effect = mock_file_handle, mock_file_handle1
+        mountdir_grub_cfg_path = 'tmpdir' + grub_cfg_file
+
+        images.create_esp_image_for_uefi(
+            'tgt_file', 'path/to/kernel', 'path/to/ramdisk',
+            esp_image='sourceabspath/to/efiboot.img',
+            kernel_params=params)
+        create_root_fs_mock.assert_called_once_with('tmpdir', files_info)
+        gen_cfg_mock.assert_any_call(params, CONF.grub_config_template,
+                                     grub_options)
+        write_to_file_mock.assert_any_call(mountdir_grub_cfg_path, grubcfg)
+        execute_mock.assert_called_once_with(
+            'mkisofs', '-r', '-V', 'VMEDIA_BOOT_ISO', '-l', '-e',
+            'boot/grub/efiboot.img', '-no-emul-boot', '-o', 'tgt_file',
+            'tmpdir')
+
     @mock.patch.object(images, '_create_root_fs', autospec=True)
     @mock.patch.object(utils, 'write_to_file', autospec=True)
     @mock.patch.object(utils, 'tempdir', autospec=True)
@@ -552,9 +578,10 @@ class FsImageTestCase(base.TestCase):
     @mock.patch.object(images, '_generate_cfg', autospec=True)
     def _test_create_isolinux_image_for_bios(
             self, gen_cfg_mock, execute_mock, tempdir_mock,
-            write_to_file_mock, create_root_fs_mock, ldlinux_path=None):
+            write_to_file_mock, create_root_fs_mock, ldlinux_path=None,
+            inject_files=None):
 
-        mock_file_handle = mock.MagicMock(spec=file)
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'tmpdir'
         tempdir_mock.return_value = mock_file_handle
 
@@ -569,13 +596,16 @@ class FsImageTestCase(base.TestCase):
         images.create_isolinux_image_for_bios('tgt_file',
                                               'path/to/kernel',
                                               'path/to/ramdisk',
-                                              kernel_params=params)
+                                              kernel_params=params,
+                                              inject_files=inject_files)
 
         files_info = {
             'path/to/kernel': 'vmlinuz',
             'path/to/ramdisk': 'initrd',
             CONF.isolinux_bin: 'isolinux/isolinux.bin'
         }
+        if inject_files:
+            files_info.update(inject_files)
         if ldlinux_path:
             files_info[ldlinux_path] = 'isolinux/ldlinux.c32'
         create_root_fs_mock.assert_called_once_with('tmpdir', files_info)
@@ -606,29 +636,34 @@ class FsImageTestCase(base.TestCase):
         self._test_create_isolinux_image_for_bios(
             ldlinux_path='/usr/share/syslinux/ldlinux.c32')
 
+    @mock.patch.object(os.path, 'isfile', autospec=True)
+    def test_create_isolinux_image_for_bios_inject_files(self, mock_isfile):
+        mock_isfile.return_value = False
+        self._test_create_isolinux_image_for_bios(
+            inject_files={'/source': 'target'})
+
     @mock.patch.object(images, '_umount_without_raise', autospec=True)
     @mock.patch.object(images, '_create_root_fs', autospec=True)
     @mock.patch.object(utils, 'tempdir', autospec=True)
     @mock.patch.object(utils, 'execute', autospec=True)
     @mock.patch.object(os, 'walk', autospec=True)
-    def test_create_isolinux_image_uefi_rootfs_fails(self, walk_mock,
-                                                     utils_mock,
-                                                     tempdir_mock,
-                                                     create_root_fs_mock,
-                                                     umount_mock):
+    def test_create_esp_image_uefi_rootfs_fails(
+            self, walk_mock, utils_mock, tempdir_mock,
+            create_root_fs_mock, umount_mock):
 
-        mock_file_handle = mock.MagicMock(spec=file)
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'tmpdir'
-        mock_file_handle1 = mock.MagicMock(spec=file)
+        mock_file_handle1 = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle1.__enter__.return_value = 'mountdir'
         tempdir_mock.side_effect = mock_file_handle, mock_file_handle1
         create_root_fs_mock.side_effect = IOError
 
         self.assertRaises(exception.ImageCreationFailed,
-                          images.create_isolinux_image_for_uefi,
-                          'tgt_file', 'path/to/deployiso',
+                          images.create_esp_image_for_uefi,
+                          'tgt_file',
                           'path/to/kernel',
-                          'path/to/ramdisk')
+                          'path/to/ramdisk',
+                          deploy_iso='path/to/deployiso')
         umount_mock.assert_called_once_with('mountdir')
 
     @mock.patch.object(images, '_create_root_fs', autospec=True)
@@ -653,27 +688,23 @@ class FsImageTestCase(base.TestCase):
     @mock.patch.object(utils, 'execute', autospec=True)
     @mock.patch.object(images, '_mount_deploy_iso', autospec=True)
     @mock.patch.object(images, '_generate_cfg', autospec=True)
-    def test_create_isolinux_image_mkisofs_fails(self,
-                                                 gen_cfg_mock,
-                                                 mount_mock,
-                                                 utils_mock,
-                                                 tempdir_mock,
-                                                 write_to_file_mock,
-                                                 create_root_fs_mock,
-                                                 umount_mock):
-        mock_file_handle = mock.MagicMock(spec=file)
+    def test_create_esp_image_mkisofs_fails(
+            self, gen_cfg_mock, mount_mock, utils_mock, tempdir_mock,
+            write_to_file_mock, create_root_fs_mock, umount_mock):
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'tmpdir'
-        mock_file_handle1 = mock.MagicMock(spec=file)
+        mock_file_handle1 = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle1.__enter__.return_value = 'mountdir'
         tempdir_mock.side_effect = mock_file_handle, mock_file_handle1
         mount_mock.return_value = ({'a': 'a'}, 'b', 'c')
         utils_mock.side_effect = processutils.ProcessExecutionError
 
         self.assertRaises(exception.ImageCreationFailed,
-                          images.create_isolinux_image_for_uefi,
-                          'tgt_file', 'path/to/deployiso',
+                          images.create_esp_image_for_uefi,
+                          'tgt_file',
                           'path/to/kernel',
-                          'path/to/ramdisk')
+                          'path/to/ramdisk',
+                          deploy_iso='path/to/deployiso')
         umount_mock.assert_called_once_with('mountdir')
 
     @mock.patch.object(images, '_create_root_fs', autospec=True)
@@ -687,7 +718,7 @@ class FsImageTestCase(base.TestCase):
                                                       tempdir_mock,
                                                       write_to_file_mock,
                                                       create_root_fs_mock):
-        mock_file_handle = mock.MagicMock(spec=file)
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'tmpdir'
         tempdir_mock.return_value = mock_file_handle
         utils_mock.side_effect = processutils.ProcessExecutionError
@@ -697,18 +728,20 @@ class FsImageTestCase(base.TestCase):
                           'tgt_file', 'path/to/kernel',
                           'path/to/ramdisk')
 
-    @mock.patch.object(images, 'create_isolinux_image_for_uefi', autospec=True)
+    @mock.patch.object(images, 'create_esp_image_for_uefi', autospec=True)
     @mock.patch.object(images, 'fetch', autospec=True)
     @mock.patch.object(utils, 'tempdir', autospec=True)
-    def test_create_boot_iso_for_uefi(
+    def test_create_boot_iso_for_uefi_deploy_iso(
             self, tempdir_mock, fetch_images_mock, create_isolinux_mock):
-        mock_file_handle = mock.MagicMock(spec=file)
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'tmpdir'
         tempdir_mock.return_value = mock_file_handle
 
-        images.create_boot_iso('ctx', 'output_file', 'kernel-uuid',
-                               'ramdisk-uuid', 'deploy_iso-uuid',
-                               'root-uuid', 'kernel-params', 'uefi')
+        images.create_boot_iso(
+            'ctx', 'output_file', 'kernel-uuid',
+            'ramdisk-uuid', deploy_iso_href='deploy_iso-uuid',
+            root_uuid='root-uuid', kernel_params='kernel-params',
+            boot_mode='uefi')
 
         fetch_images_mock.assert_any_call(
             'ctx', 'kernel-uuid', 'tmpdir/kernel-uuid')
@@ -719,21 +752,53 @@ class FsImageTestCase(base.TestCase):
 
         params = ['root=UUID=root-uuid', 'kernel-params']
         create_isolinux_mock.assert_called_once_with(
-            'output_file', 'tmpdir/deploy_iso-uuid', 'tmpdir/kernel-uuid',
-            'tmpdir/ramdisk-uuid', params)
+            'output_file', 'tmpdir/kernel-uuid', 'tmpdir/ramdisk-uuid',
+            deploy_iso='tmpdir/deploy_iso-uuid',
+            esp_image=None, kernel_params=params, inject_files=None)
 
-    @mock.patch.object(images, 'create_isolinux_image_for_uefi', autospec=True)
+    @mock.patch.object(images, 'create_esp_image_for_uefi', autospec=True)
     @mock.patch.object(images, 'fetch', autospec=True)
     @mock.patch.object(utils, 'tempdir', autospec=True)
-    def test_create_boot_iso_for_uefi_for_hrefs(
+    def test_create_boot_iso_for_uefi_esp_image(
             self, tempdir_mock, fetch_images_mock, create_isolinux_mock):
-        mock_file_handle = mock.MagicMock(spec=file)
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'tmpdir'
         tempdir_mock.return_value = mock_file_handle
 
-        images.create_boot_iso('ctx', 'output_file', 'http://kernel-href',
-                               'http://ramdisk-href', 'http://deploy_iso-href',
-                               'root-uuid', 'kernel-params', 'uefi')
+        images.create_boot_iso(
+            'ctx', 'output_file', 'kernel-uuid',
+            'ramdisk-uuid', esp_image_href='efiboot-uuid',
+            root_uuid='root-uuid', kernel_params='kernel-params',
+            boot_mode='uefi')
+
+        fetch_images_mock.assert_any_call(
+            'ctx', 'kernel-uuid', 'tmpdir/kernel-uuid')
+        fetch_images_mock.assert_any_call(
+            'ctx', 'ramdisk-uuid', 'tmpdir/ramdisk-uuid')
+        fetch_images_mock.assert_any_call(
+            'ctx', 'efiboot-uuid', 'tmpdir/efiboot-uuid')
+
+        params = ['root=UUID=root-uuid', 'kernel-params']
+        create_isolinux_mock.assert_called_once_with(
+            'output_file', 'tmpdir/kernel-uuid', 'tmpdir/ramdisk-uuid',
+            deploy_iso=None, esp_image='tmpdir/efiboot-uuid',
+            kernel_params=params, inject_files=None)
+
+    @mock.patch.object(images, 'create_esp_image_for_uefi', autospec=True)
+    @mock.patch.object(images, 'fetch', autospec=True)
+    @mock.patch.object(utils, 'tempdir', autospec=True)
+    def test_create_boot_iso_for_uefi_deploy_iso_for_hrefs(
+            self, tempdir_mock, fetch_images_mock, create_isolinux_mock):
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
+        mock_file_handle.__enter__.return_value = 'tmpdir'
+        tempdir_mock.return_value = mock_file_handle
+
+        images.create_boot_iso(
+            'ctx', 'output_file', 'http://kernel-href', 'http://ramdisk-href',
+            deploy_iso_href='http://deploy_iso-href',
+            root_uuid='root-uuid', kernel_params='kernel-params',
+            boot_mode='uefi')
+
         expected_calls = [mock.call('ctx', 'http://kernel-href',
                                     'tmpdir/kernel-href'),
                           mock.call('ctx', 'http://ramdisk-href',
@@ -743,26 +808,57 @@ class FsImageTestCase(base.TestCase):
         fetch_images_mock.assert_has_calls(expected_calls)
         params = ['root=UUID=root-uuid', 'kernel-params']
         create_isolinux_mock.assert_called_once_with(
-            'output_file', 'tmpdir/deploy_iso-href', 'tmpdir/kernel-href',
-            'tmpdir/ramdisk-href', params)
+            'output_file', 'tmpdir/kernel-href', 'tmpdir/ramdisk-href',
+            deploy_iso='tmpdir/deploy_iso-href',
+            esp_image=None, kernel_params=params, inject_files=None)
+
+    @mock.patch.object(images, 'create_esp_image_for_uefi', autospec=True)
+    @mock.patch.object(images, 'fetch', autospec=True)
+    @mock.patch.object(utils, 'tempdir', autospec=True)
+    def test_create_boot_iso_for_uefi_esp_image_for_hrefs(
+            self, tempdir_mock, fetch_images_mock, create_isolinux_mock):
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
+        mock_file_handle.__enter__.return_value = 'tmpdir'
+        tempdir_mock.return_value = mock_file_handle
+
+        images.create_boot_iso(
+            'ctx', 'output_file', 'http://kernel-href', 'http://ramdisk-href',
+            esp_image_href='http://efiboot-href',
+            root_uuid='root-uuid', kernel_params='kernel-params',
+            boot_mode='uefi')
+
+        expected_calls = [mock.call('ctx', 'http://kernel-href',
+                                    'tmpdir/kernel-href'),
+                          mock.call('ctx', 'http://ramdisk-href',
+                                    'tmpdir/ramdisk-href'),
+                          mock.call('ctx', 'http://efiboot-href',
+                                    'tmpdir/efiboot-href')]
+        fetch_images_mock.assert_has_calls(expected_calls)
+        params = ['root=UUID=root-uuid', 'kernel-params']
+        create_isolinux_mock.assert_called_once_with(
+            'output_file', 'tmpdir/kernel-href', 'tmpdir/ramdisk-href',
+            deploy_iso=None, esp_image='tmpdir/efiboot-href',
+            kernel_params=params, inject_files=None)
 
     @mock.patch.object(images, 'create_isolinux_image_for_bios', autospec=True)
     @mock.patch.object(images, 'fetch', autospec=True)
     @mock.patch.object(utils, 'tempdir', autospec=True)
     def test_create_boot_iso_for_bios(
             self, tempdir_mock, fetch_images_mock, create_isolinux_mock):
-        mock_file_handle = mock.MagicMock(spec=file)
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'tmpdir'
         tempdir_mock.return_value = mock_file_handle
 
         images.create_boot_iso('ctx', 'output_file', 'kernel-uuid',
                                'ramdisk-uuid', 'deploy_iso-uuid',
-                               'root-uuid', 'kernel-params', 'bios')
+                               'efiboot-uuid', 'root-uuid',
+                               'kernel-params', 'bios')
 
         fetch_images_mock.assert_any_call(
             'ctx', 'kernel-uuid', 'tmpdir/kernel-uuid')
         fetch_images_mock.assert_any_call(
             'ctx', 'ramdisk-uuid', 'tmpdir/ramdisk-uuid')
+
         # Note (NobodyCam): the original assert asserted that fetch_images
         #                   was not called with parameters, this did not
         #                   work, So I instead assert that there were only
@@ -771,10 +867,9 @@ class FsImageTestCase(base.TestCase):
         self.assertEqual(2, fetch_images_mock.call_count)
 
         params = ['root=UUID=root-uuid', 'kernel-params']
-        create_isolinux_mock.assert_called_once_with('output_file',
-                                                     'tmpdir/kernel-uuid',
-                                                     'tmpdir/ramdisk-uuid',
-                                                     params)
+        create_isolinux_mock.assert_called_once_with(
+            'output_file', 'tmpdir/kernel-uuid', 'tmpdir/ramdisk-uuid',
+            kernel_params=params, inject_files=None)
 
     @mock.patch.object(images, 'create_isolinux_image_for_bios', autospec=True)
     @mock.patch.object(images, 'fetch', autospec=True)
@@ -782,13 +877,14 @@ class FsImageTestCase(base.TestCase):
     def test_create_boot_iso_for_bios_with_no_boot_mode(self, tempdir_mock,
                                                         fetch_images_mock,
                                                         create_isolinux_mock):
-        mock_file_handle = mock.MagicMock(spec=file)
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'tmpdir'
         tempdir_mock.return_value = mock_file_handle
 
         images.create_boot_iso('ctx', 'output_file', 'kernel-uuid',
                                'ramdisk-uuid', 'deploy_iso-uuid',
-                               'root-uuid', 'kernel-params', None)
+                               'efiboot-uuid', 'root-uuid',
+                               'kernel-params', None)
 
         fetch_images_mock.assert_any_call(
             'ctx', 'kernel-uuid', 'tmpdir/kernel-uuid')
@@ -796,10 +892,29 @@ class FsImageTestCase(base.TestCase):
             'ctx', 'ramdisk-uuid', 'tmpdir/ramdisk-uuid')
 
         params = ['root=UUID=root-uuid', 'kernel-params']
-        create_isolinux_mock.assert_called_once_with('output_file',
-                                                     'tmpdir/kernel-uuid',
-                                                     'tmpdir/ramdisk-uuid',
-                                                     params)
+        create_isolinux_mock.assert_called_once_with(
+            'output_file', 'tmpdir/kernel-uuid', 'tmpdir/ramdisk-uuid',
+            kernel_params=params, inject_files=None)
+
+    @mock.patch.object(images, 'create_isolinux_image_for_bios', autospec=True)
+    @mock.patch.object(images, 'fetch', autospec=True)
+    @mock.patch.object(utils, 'tempdir', autospec=True)
+    def test_create_boot_iso_for_existing_iso(self, tempdir_mock,
+                                              fetch_images_mock,
+                                              create_isolinux_mock):
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
+        mock_file_handle.__enter__.return_value = 'tmpdir'
+        tempdir_mock.return_value = mock_file_handle
+        base_iso = 'http://fake.local:1234/fake.iso'
+        images.create_boot_iso('ctx', 'output_file', 'kernel-uuid',
+                               'ramdisk-uuid', 'deploy_iso-uuid',
+                               'efiboot-uuid', None,
+                               None, None, base_iso=base_iso)
+
+        fetch_images_mock.assert_any_call(
+            'ctx', base_iso, 'output_file')
+
+        create_isolinux_mock.assert_not_called()
 
     @mock.patch.object(image_service, 'get_image_service', autospec=True)
     def test_get_glance_image_properties_no_such_prop(self,
