@@ -13,12 +13,13 @@
 """Test class for Ironic BaseConductorManager."""
 
 import collections
+from unittest import mock
 import uuid
 
 import eventlet
 import futurist
 from futurist import periodics
-import mock
+from ironic_lib import mdns
 from oslo_config import cfg
 from oslo_db import exception as db_exception
 from oslo_utils import uuidutils
@@ -30,8 +31,10 @@ from ironic.conductor import base_manager
 from ironic.conductor import manager
 from ironic.conductor import notification_utils
 from ironic.conductor import task_manager
+from ironic.db import api as dbapi
 from ironic.drivers import fake_hardware
 from ironic.drivers import generic
+from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules import fake
 from ironic import objects
 from ironic.objects import fields
@@ -62,6 +65,17 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         node.refresh()
         self.assertIsNone(node.reservation)
 
+    def test_stop_clears_conductor_locks(self):
+        node = obj_utils.create_test_node(self.context,
+                                          reservation=self.hostname)
+        node.save()
+        self._start_service()
+        res = objects.Conductor.get_by_hostname(self.context, self.hostname)
+        self.assertEqual(self.hostname, res['hostname'])
+        self.service.del_host()
+        node.refresh()
+        self.assertIsNone(node.reservation)
+
     def test_stop_unregisters_conductor(self):
         self._start_service()
         res = objects.Conductor.get_by_hostname(self.context, self.hostname)
@@ -79,7 +93,7 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         res = objects.Conductor.get_by_hostname(self.context, self.hostname)
         self.assertEqual(self.hostname, res['hostname'])
 
-    @mock.patch.object(manager.ConductorManager, 'init_host')
+    @mock.patch.object(manager.ConductorManager, 'init_host', autospec=True)
     def test_stop_uninitialized_conductor(self, mock_init):
         self._start_service()
         self.service.del_host()
@@ -94,7 +108,8 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         mock_def_iface.return_value = 'fake'
 
         df = driver_factory.HardwareTypesFactory()
-        with mock.patch.object(df._extension_manager, 'names') as mock_names:
+        with mock.patch.object(df._extension_manager, 'names',
+                               autospec=True) as mock_names:
             # verify driver names are registered
             self.config(enabled_hardware_types=init_names)
             mock_names.return_value = init_names
@@ -167,10 +182,12 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         self.assertTrue(periodics.is_periodic(hw_type.task))
         self.assertNotIn(hw_type.task, tasks)
 
-    @mock.patch.object(driver_factory.HardwareTypesFactory, '__init__')
+    @mock.patch.object(driver_factory.HardwareTypesFactory, '__init__',
+                       autospec=True)
     def test_start_fails_on_missing_driver(self, mock_df):
         mock_df.side_effect = exception.DriverNotFound('test')
-        with mock.patch.object(self.dbapi, 'register_conductor') as mock_reg:
+        with mock.patch.object(self.dbapi, 'register_conductor',
+                               autospec=True) as mock_reg:
             self.assertRaises(exception.DriverNotFound,
                               self.service.init_host)
             self.assertTrue(mock_df.called)
@@ -182,8 +199,8 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
                                'options enabled_boot_interfaces',
                                self.service.init_host)
 
-    @mock.patch.object(base_manager, 'LOG')
-    @mock.patch.object(driver_factory, 'HardwareTypesFactory')
+    @mock.patch.object(base_manager, 'LOG', autospec=True)
+    @mock.patch.object(driver_factory, 'HardwareTypesFactory', autospec=True)
     def test_start_fails_on_hw_types(self, ht_mock, log_mock):
         driver_factory_mock = mock.MagicMock(names=[])
         ht_mock.return_value = driver_factory_mock
@@ -192,16 +209,18 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         self.assertTrue(log_mock.error.called)
         ht_mock.assert_called_once_with()
 
-    @mock.patch.object(base_manager, 'LOG')
+    @mock.patch.object(base_manager, 'LOG', autospec=True)
     @mock.patch.object(base_manager.BaseConductorManager,
-                       '_register_and_validate_hardware_interfaces')
-    @mock.patch.object(base_manager.BaseConductorManager, 'del_host')
+                       '_register_and_validate_hardware_interfaces',
+                       autospec=True)
+    @mock.patch.object(base_manager.BaseConductorManager, 'del_host',
+                       autospec=True)
     def test_start_fails_hw_type_register(self, del_mock, reg_mock, log_mock):
         reg_mock.side_effect = exception.DriverNotFound('hw-type')
         self.assertRaises(exception.DriverNotFound,
                           self.service.init_host)
         self.assertTrue(log_mock.error.called)
-        del_mock.assert_called_once_with()
+        del_mock.assert_called_once()
 
     def test_prevent_double_start(self):
         self._start_service()
@@ -217,6 +236,7 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
             (states.ADOPTING, states.ADOPTFAIL),
             (states.RESCUING, states.RESCUEFAIL),
             (states.UNRESCUING, states.UNRESCUEFAIL),
+            (states.DELETING, states.ERROR),
         ]
         nodes = [obj_utils.create_test_node(self.context, uuid=uuid.uuid4(),
                                             driver='fake-hardware',
@@ -229,13 +249,13 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
             self.assertEqual(state[1], node.provision_state,
                              'Test failed when recovering from %s' % state[0])
 
-    @mock.patch.object(base_manager, 'LOG')
+    @mock.patch.object(base_manager, 'LOG', autospec=True)
     def test_warning_on_low_workers_pool(self, log_mock):
         CONF.set_override('workers_pool_size', 3, 'conductor')
         self._start_service()
         self.assertTrue(log_mock.warning.called)
 
-    @mock.patch.object(eventlet.greenpool.GreenPool, 'waitall')
+    @mock.patch.object(eventlet.greenpool.GreenPool, 'waitall', autospec=True)
     def test_del_host_waits_on_workerpool(self, wait_mock):
         self._start_service()
         self.service.del_host()
@@ -247,6 +267,49 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         self.service.del_host()
         self.assertTrue(self.service._shutdown)
 
+    @mock.patch.object(deploy_utils, 'get_ironic_api_url', autospec=True)
+    @mock.patch.object(mdns, 'Zeroconf', autospec=True)
+    def test_start_with_mdns(self, mock_zc, mock_api_url):
+        CONF.set_override('debug', False)
+        CONF.set_override('enable_mdns', True, 'conductor')
+        self._start_service()
+        res = objects.Conductor.get_by_hostname(self.context, self.hostname)
+        self.assertEqual(self.hostname, res['hostname'])
+        mock_zc.return_value.register_service.assert_called_once_with(
+            'baremetal',
+            mock_api_url.return_value,
+            params={})
+
+    @mock.patch.object(deploy_utils, 'get_ironic_api_url', autospec=True)
+    @mock.patch.object(mdns, 'Zeroconf', autospec=True)
+    def test_start_with_mdns_and_debug(self, mock_zc, mock_api_url):
+        CONF.set_override('debug', True)
+        CONF.set_override('enable_mdns', True, 'conductor')
+        self._start_service()
+        res = objects.Conductor.get_by_hostname(self.context, self.hostname)
+        self.assertEqual(self.hostname, res['hostname'])
+        mock_zc.return_value.register_service.assert_called_once_with(
+            'baremetal',
+            mock_api_url.return_value,
+            params={'ipa_debug': True})
+
+    def test_del_host_with_mdns(self):
+        mock_zc = mock.Mock(spec=mdns.Zeroconf)
+        self.service._zeroconf = mock_zc
+        self._start_service()
+        self.service.del_host()
+        mock_zc.close.assert_called_once_with()
+        self.assertIsNone(self.service._zeroconf)
+
+    @mock.patch.object(dbapi, 'get_instance', autospec=True)
+    def test_start_dbapi_single_call(self, mock_dbapi):
+        self._start_service()
+        # NOTE(TheJulia): This seems like it should only be 1, but
+        # the hash ring initailization pulls it's own database connection
+        # instance, which is likely a good thing, thus this is 2 instead of
+        # 3 without reuse of the database connection.
+        self.assertEqual(2, mock_dbapi.call_count)
+
 
 class CheckInterfacesTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
     def test__check_enabled_interfaces_success(self):
@@ -264,9 +327,10 @@ class KeepAliveTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         self._start_service()
         # avoid wasting time at the event.wait()
         CONF.set_override('heartbeat_interval', 0, 'conductor')
-        with mock.patch.object(self.dbapi, 'touch_conductor') as mock_touch:
+        with mock.patch.object(self.dbapi, 'touch_conductor',
+                               autospec=True) as mock_touch:
             with mock.patch.object(self.service._keepalive_evt,
-                                   'is_set') as mock_is_set:
+                                   'is_set', autospec=True) as mock_is_set:
                 mock_is_set.side_effect = [False, True]
                 self.service._conductor_service_record_keepalive()
             mock_touch.assert_called_once_with(self.hostname)
@@ -275,11 +339,12 @@ class KeepAliveTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         self._start_service()
         # avoid wasting time at the event.wait()
         CONF.set_override('heartbeat_interval', 0, 'conductor')
-        with mock.patch.object(self.dbapi, 'touch_conductor') as mock_touch:
+        with mock.patch.object(self.dbapi, 'touch_conductor',
+                               autospec=True) as mock_touch:
             mock_touch.side_effect = [None, db_exception.DBConnectionError(),
                                       None]
             with mock.patch.object(self.service._keepalive_evt,
-                                   'is_set') as mock_is_set:
+                                   'is_set', autospec=True) as mock_is_set:
                 mock_is_set.side_effect = [False, False, False, True]
                 self.service._conductor_service_record_keepalive()
             self.assertEqual(3, mock_touch.call_count)
@@ -288,11 +353,12 @@ class KeepAliveTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         self._start_service()
         # avoid wasting time at the event.wait()
         CONF.set_override('heartbeat_interval', 0, 'conductor')
-        with mock.patch.object(self.dbapi, 'touch_conductor') as mock_touch:
+        with mock.patch.object(self.dbapi, 'touch_conductor',
+                               autospec=True) as mock_touch:
             mock_touch.side_effect = [None, Exception(),
                                       None]
             with mock.patch.object(self.service._keepalive_evt,
-                                   'is_set') as mock_is_set:
+                                   'is_set', autospec=True) as mock_is_set:
                 mock_is_set.side_effect = [False, False, False, True]
                 self.service._conductor_service_record_keepalive()
             self.assertEqual(3, mock_touch.call_count)
@@ -400,7 +466,8 @@ class RegisterInterfacesTestCase(mgr_utils.ServiceSetUpMixin,
 
 
 @mock.patch.object(fake.FakeConsole, 'start_console', autospec=True)
-@mock.patch.object(notification_utils, 'emit_console_notification')
+@mock.patch.object(notification_utils, 'emit_console_notification',
+                   autospec=True)
 class StartConsolesTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
     def test__start_consoles(self, mock_notify, mock_start_console):
         obj_utils.create_test_node(self.context,
@@ -453,7 +520,7 @@ class StartConsolesTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
              mock.call(mock.ANY, 'console_restore',
                        fields.NotificationStatus.ERROR)])
 
-    @mock.patch.object(base_manager, 'LOG')
+    @mock.patch.object(base_manager, 'LOG', autospec=True)
     def test__start_consoles_node_locked(self, log_mock, mock_notify,
                                          mock_start_console):
         test_node = obj_utils.create_test_node(self.context,
@@ -469,14 +536,15 @@ class StartConsolesTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         self.assertTrue(log_mock.warning.called)
         self.assertFalse(mock_notify.called)
 
-    @mock.patch.object(base_manager, 'LOG')
+    @mock.patch.object(base_manager, 'LOG', autospec=True)
     def test__start_consoles_node_not_found(self, log_mock, mock_notify,
                                             mock_start_console):
         test_node = obj_utils.create_test_node(self.context,
                                                driver='fake-hardware',
                                                console_enabled=True)
         self._start_service()
-        with mock.patch.object(task_manager, 'acquire') as mock_acquire:
+        with mock.patch.object(task_manager, 'acquire',
+                               autospec=True) as mock_acquire:
             mock_acquire.side_effect = exception.NodeNotFound(node='not found')
             self.service._start_consoles(self.context)
             self.assertFalse(mock_start_console.called)
@@ -485,3 +553,26 @@ class StartConsolesTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
             self.assertIsNone(test_node.last_error)
             self.assertTrue(log_mock.warning.called)
             self.assertFalse(mock_notify.called)
+
+
+class MiscTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
+    def setUp(self):
+        super(MiscTestCase, self).setUp()
+        self._start_service()
+
+    def test__fail_transient_state(self):
+        node = obj_utils.create_test_node(self.context,
+                                          driver='fake-hardware',
+                                          provision_state=states.DEPLOYING)
+        self.service._fail_transient_state(states.DEPLOYING, 'unknown err')
+        node.refresh()
+        self.assertEqual(states.DEPLOYFAIL, node.provision_state)
+
+    def test__fail_transient_state_maintenance(self):
+        node = obj_utils.create_test_node(self.context,
+                                          driver='fake-hardware',
+                                          maintenance=True,
+                                          provision_state=states.DEPLOYING)
+        self.service._fail_transient_state(states.DEPLOYING, 'unknown err')
+        node.refresh()
+        self.assertEqual(states.DEPLOYFAIL, node.provision_state)
