@@ -19,14 +19,16 @@ Unit Tests for :py:class:`ironic.conductor.rpcapi.ConductorAPI`.
 """
 
 import copy
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 import oslo_messaging as messaging
 from oslo_messaging import _utils as messaging_utils
 
 from ironic.common import boot_devices
+from ironic.common import components
 from ironic.common import exception
+from ironic.common import indicator_states
 from ironic.common import release_mappings
 from ironic.common import states
 from ironic.conductor import manager as conductor_manager
@@ -46,14 +48,15 @@ class ConductorRPCAPITestCase(tests_base.TestCase):
             conductor_manager.ConductorManager.RPC_API_VERSION,
             conductor_rpcapi.ConductorAPI.RPC_API_VERSION)
 
-    @mock.patch('ironic.common.rpc.get_client')
+    @mock.patch('ironic.common.rpc.get_client', autospec=True)
     def test_version_cap(self, mock_get_client):
         conductor_rpcapi.ConductorAPI()
         self.assertEqual(conductor_rpcapi.ConductorAPI.RPC_API_VERSION,
                          mock_get_client.call_args[1]['version_cap'])
 
-    @mock.patch('ironic.common.release_mappings.RELEASE_MAPPING')
-    @mock.patch('ironic.common.rpc.get_client')
+    @mock.patch('ironic.common.release_mappings.RELEASE_MAPPING',
+                autospec=True)
+    @mock.patch('ironic.common.rpc.get_client', autospec=True)
     def test_version_capped(self, mock_get_client, mock_release_mapping):
         CONF.set_override('pin_release_version',
                           release_mappings.RELEASE_VERSIONS[0])
@@ -130,14 +133,16 @@ class RPCAPITestCase(db_base.DbTestCase):
 
     def test_get_topic_for_driver_unknown_driver(self):
         CONF.set_override('host', 'fake-host')
-        self.dbapi.register_conductor({
+        c = self.dbapi.register_conductor({
             'hostname': 'fake-host',
             'drivers': [],
         })
+        self.dbapi.register_conductor_hardware_interfaces(
+            c.id, 'fake-driver', 'deploy', ['iscsi', 'direct'], 'iscsi')
         rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
         self.assertRaises(exception.DriverNotFound,
                           rpcapi.get_topic_for_driver,
-                          'fake-driver')
+                          'fake-driver-2')
 
     def test_get_topic_for_driver_doesnt_cache(self):
         CONF.set_override('host', 'fake-host')
@@ -156,10 +161,35 @@ class RPCAPITestCase(db_base.DbTestCase):
         self.assertEqual('fake-topic.fake-host',
                          rpcapi.get_topic_for_driver('fake-driver'))
 
+    def test_get_conductor_for(self):
+        CONF.set_override('host', 'fake-host')
+        c = self.dbapi.register_conductor({'hostname': 'fake-host',
+                                           'drivers': []})
+        self.dbapi.register_conductor_hardware_interfaces(
+            c.id, 'fake-driver', 'deploy', ['iscsi', 'direct'], 'iscsi')
+        rpcapi = conductor_rpcapi.ConductorAPI()
+        self.assertEqual(rpcapi.get_conductor_for(self.fake_node_obj),
+                         'fake-host')
+
+    def test_get_random_topic(self):
+        CONF.set_override('host', 'fake-host')
+        self.dbapi.register_conductor({'hostname': 'fake-host', 'drivers': []})
+
+        rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
+        expected_topic = 'fake-topic.fake-host'
+        self.assertEqual(expected_topic, rpcapi.get_random_topic())
+
+    def test_get_random_topic_no_conductors(self):
+        CONF.set_override('host', 'fake-host')
+
+        rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
+        self.assertRaises(exception.TemporaryFailure, rpcapi.get_random_topic)
+
     def _test_can_send_create_port(self, can_send):
         rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
         with mock.patch.object(rpcapi.client,
-                               "can_send_version") as mock_can_send_version:
+                               "can_send_version",
+                               autospec=True) as mock_can_send_version:
             mock_can_send_version.return_value = can_send
             result = rpcapi.can_send_create_port()
             self.assertEqual(can_send, result)
@@ -205,13 +235,16 @@ class RPCAPITestCase(db_base.DbTestCase):
                 return expected_retval
 
         with mock.patch.object(rpcapi.client,
-                               "can_send_version") as mock_can_send_version:
+                               "can_send_version",
+                               autospec=True) as mock_can_send_version:
             mock_can_send_version.side_effect = _fake_can_send_version_method
-            with mock.patch.object(rpcapi.client, "prepare") as mock_prepared:
+            with mock.patch.object(rpcapi.client, "prepare",
+                                   autospec=True) as mock_prepared:
                 mock_prepared.side_effect = _fake_prepare_method
 
                 with mock.patch.object(rpcapi.client,
-                                       rpc_method) as mock_method:
+                                       rpc_method,
+                                       autospec=True) as mock_method:
                     mock_method.side_effect = _fake_rpc_method
                     retval = getattr(rpcapi, method)(self.context, **kwargs)
                     self.assertEqual(retval, expected_retval)
@@ -336,6 +369,29 @@ class RPCAPITestCase(db_base.DbTestCase):
                           version='1.17',
                           node_id=self.fake_node['uuid'])
 
+    def test_set_indicator_state(self):
+        self._test_rpcapi('set_indicator_state',
+                          'call',
+                          version='1.50',
+                          node_id=self.fake_node['uuid'],
+                          component=components.CHASSIS,
+                          indicator='led',
+                          state=indicator_states.ON)
+
+    def test_get_indicator_state(self):
+        self._test_rpcapi('get_indicator_state',
+                          'call',
+                          version='1.50',
+                          node_id=self.fake_node['uuid'],
+                          component=components.CHASSIS,
+                          indicator='led')
+
+    def test_get_supported_indicators(self):
+        self._test_rpcapi('get_supported_indicators',
+                          'call',
+                          version='1.50',
+                          node_id=self.fake_node['uuid'])
+
     def test_get_node_vendor_passthru_methods(self):
         self._test_rpcapi('get_node_vendor_passthru_methods',
                           'call',
@@ -460,7 +516,16 @@ class RPCAPITestCase(db_base.DbTestCase):
                           node_id='fake-node',
                           callback_url='http://ramdisk.url:port',
                           agent_version=None,
-                          version='1.42')
+                          version='1.51')
+
+    def test_heartbeat_agent_token(self):
+        self._test_rpcapi('heartbeat',
+                          'call',
+                          node_id='fake-node',
+                          callback_url='http://ramdisk.url:port',
+                          agent_version=None,
+                          agent_token='xyz1',
+                          version='1.51')
 
     def test_destroy_volume_connector(self):
         fake_volume_connector = db_utils.get_test_volume_connector()
@@ -529,10 +594,17 @@ class RPCAPITestCase(db_base.DbTestCase):
                           version='1.43',
                           node_id=self.fake_node['uuid'])
 
+    def test_get_node_with_token(self):
+        self._test_rpcapi('get_node_with_token',
+                          'call',
+                          version='1.49',
+                          node_id=self.fake_node['uuid'])
+
     def _test_can_send_rescue(self, can_send):
         rpcapi = conductor_rpcapi.ConductorAPI(topic='fake-topic')
         with mock.patch.object(rpcapi.client,
-                               "can_send_version") as mock_can_send_version:
+                               "can_send_version",
+                               autospec=True) as mock_can_send_version:
             mock_can_send_version.return_value = can_send
             result = rpcapi.can_send_rescue()
             self.assertEqual(can_send, result)
@@ -572,3 +644,15 @@ class RPCAPITestCase(db_base.DbTestCase):
                           node_id='fake-node',
                           traits=None,
                           version='1.44')
+
+    def test_create_allocation(self):
+        self._test_rpcapi('create_allocation',
+                          'call',
+                          allocation='fake-allocation',
+                          version='1.48')
+
+    def test_destroy_allocation(self):
+        self._test_rpcapi('destroy_allocation',
+                          'call',
+                          allocation='fake-allocation',
+                          version='1.48')
