@@ -17,8 +17,9 @@
 
 """Tests for :class:`ironic.conductor.task_manager`."""
 
+from unittest import mock
+
 import futurist
-import mock
 from oslo_utils import uuidutils
 
 from ironic.common import driver_factory
@@ -34,14 +35,14 @@ from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.objects import utils as obj_utils
 
 
-@mock.patch.object(objects.Node, 'get')
-@mock.patch.object(objects.Node, 'release')
-@mock.patch.object(objects.Node, 'reserve')
-@mock.patch.object(driver_factory, 'build_driver_for_task')
-@mock.patch.object(objects.Port, 'list_by_node_id')
-@mock.patch.object(objects.Portgroup, 'list_by_node_id')
-@mock.patch.object(objects.VolumeConnector, 'list_by_node_id')
-@mock.patch.object(objects.VolumeTarget, 'list_by_node_id')
+@mock.patch.object(objects.Node, 'get', autospec=True)
+@mock.patch.object(objects.Node, 'release', autospec=True)
+@mock.patch.object(objects.Node, 'reserve', autospec=True)
+@mock.patch.object(driver_factory, 'build_driver_for_task', autospec=True)
+@mock.patch.object(objects.Port, 'list_by_node_id', autospec=True)
+@mock.patch.object(objects.Portgroup, 'list_by_node_id', autospec=True)
+@mock.patch.object(objects.VolumeConnector, 'list_by_node_id', autospec=True)
+@mock.patch.object(objects.VolumeTarget, 'list_by_node_id', autospec=True)
 class TaskManagerTestCase(db_base.DbTestCase):
     def setUp(self):
         super(TaskManagerTestCase, self).setUp()
@@ -79,6 +80,24 @@ class TaskManagerTestCase(db_base.DbTestCase):
         release_mock.assert_called_once_with(self.context, self.host,
                                              self.node.id)
 
+    def test_no_driver(self, get_voltgt_mock, get_volconn_mock,
+                       get_portgroups_mock, get_ports_mock,
+                       build_driver_mock, reserve_mock, release_mock,
+                       node_get_mock):
+        reserve_mock.return_value = self.node
+        with task_manager.TaskManager(self.context, 'fake-node-id',
+                                      load_driver=False) as task:
+            self.assertEqual(self.context, task.context)
+            self.assertEqual(self.node, task.node)
+            self.assertEqual(get_ports_mock.return_value, task.ports)
+            self.assertEqual(get_portgroups_mock.return_value, task.portgroups)
+            self.assertEqual(get_volconn_mock.return_value,
+                             task.volume_connectors)
+            self.assertEqual(get_voltgt_mock.return_value, task.volume_targets)
+            self.assertIsNone(task.driver)
+            self.assertFalse(task.shared)
+        self.assertFalse(build_driver_mock.called)
+
     def test_excl_nested_acquire(
             self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
             get_ports_mock, build_driver_mock,
@@ -159,6 +178,50 @@ class TaskManagerTestCase(db_base.DbTestCase):
         reserve_mock.assert_has_calls(expected_calls)
         self.assertEqual(2, reserve_mock.call_count)
 
+    def test_excl_lock_exception_no_retries(
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
+            reserve_mock, release_mock, node_get_mock):
+        retry_attempts = 3
+        self.config(node_locked_retry_attempts=retry_attempts,
+                    group='conductor')
+
+        # Fail on the first lock attempt, succeed on the second.
+        reserve_mock.side_effect = [exception.NodeLocked(node='foo',
+                                                         host='foo'),
+                                    self.node]
+
+        self.assertRaises(exception.NodeLocked,
+                          task_manager.TaskManager,
+                          self.context,
+                          'fake-node-id',
+                          retry=False)
+
+        reserve_mock.assert_called_once_with(self.context, self.host,
+                                             'fake-node-id')
+
+    def test_excl_lock_upgade_exception_no_retries(
+            self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
+            get_ports_mock, build_driver_mock,
+            reserve_mock, release_mock, node_get_mock):
+        retry_attempts = 3
+        self.config(node_locked_retry_attempts=retry_attempts,
+                    group='conductor')
+
+        node_get_mock.return_value = self.node
+        # Fail on the first lock attempt, succeed on the second.
+        reserve_mock.side_effect = [exception.NodeLocked(node='foo',
+                                                         host='foo'),
+                                    self.node]
+
+        task = task_manager.TaskManager(self.context, 'fake-node-id',
+                                        shared=True)
+        self.assertRaises(exception.NodeLocked,
+                          task.upgrade_lock, retry=False)
+
+        reserve_mock.assert_called_once_with(self.context, self.host,
+                                             'fake-node-id')
+
     def test_excl_lock_reserve_exception(
             self, get_voltgt_mock, get_volconn_mock, get_portgroups_mock,
             get_ports_mock, build_driver_mock,
@@ -632,7 +695,7 @@ class TaskManagerTestCase(db_base.DbTestCase):
         on_error_handler.assert_called_once_with(expected_exception,
                                                  'fake-argument')
 
-    @mock.patch.object(states.machine, 'copy')
+    @mock.patch.object(states.machine, 'copy', autospec=True)
     def test_init_prepares_fsm(
             self, copy_mock, get_volconn_mock, get_voltgt_mock,
             get_portgroups_mock, get_ports_mock,
