@@ -36,10 +36,10 @@ For postgres on Ubuntu this can be done with the following commands:
 
 import collections
 import contextlib
+from unittest import mock
 
 from alembic import script
 import fixtures
-import mock
 from oslo_db import exception as db_exc
 from oslo_db.sqlalchemy import enginefacade
 from oslo_db.sqlalchemy import test_fixtures
@@ -65,8 +65,8 @@ MIGRATIONS_TIMEOUT = 300
 
 @contextlib.contextmanager
 def patch_with_engine(engine):
-    with mock.patch.object(enginefacade.writer,
-                           'get_engine') as patch_engine:
+    with mock.patch.object(enginefacade.writer, 'get_engine',
+                           autospec=True) as patch_engine:
         patch_engine.return_value = engine
         yield
 
@@ -150,10 +150,10 @@ class TestWalkVersions(base.TestCase, WalkVersionsMixin):
         self._pre_upgrade_141.assert_called_with(self.engine)
         self._check_141.assert_called_with(self.engine, test_value)
 
-    @mock.patch.object(script, 'ScriptDirectory')
-    @mock.patch.object(WalkVersionsMixin, '_migrate_up')
+    @mock.patch.object(script, 'ScriptDirectory', autospec=True)
+    @mock.patch.object(WalkVersionsMixin, '_migrate_up', autospec=True)
     def test_walk_versions_all_default(self, _migrate_up, script_directory):
-        fc = script_directory.from_config()
+        fc = script_directory.from_config.return_value
         fc.walk_revisions.return_value = self.versions
         self.migration_api.version.return_value = None
 
@@ -161,20 +161,20 @@ class TestWalkVersions(base.TestCase, WalkVersionsMixin):
 
         self.migration_api.version.assert_called_with(self.config)
 
-        upgraded = [mock.call(self.engine, self.config, v.revision,
+        upgraded = [mock.call(self, self.engine, self.config, v.revision,
                     with_data=True) for v in reversed(self.versions)]
         self.assertEqual(self._migrate_up.call_args_list, upgraded)
 
-    @mock.patch.object(script, 'ScriptDirectory')
-    @mock.patch.object(WalkVersionsMixin, '_migrate_up')
+    @mock.patch.object(script, 'ScriptDirectory', autospec=True)
+    @mock.patch.object(WalkVersionsMixin, '_migrate_up', autospec=True)
     def test_walk_versions_all_false(self, _migrate_up, script_directory):
-        fc = script_directory.from_config()
+        fc = script_directory.from_config.return_value
         fc.walk_revisions.return_value = self.versions
         self.migration_api.version.return_value = None
 
         self._walk_versions(self.engine, self.config)
 
-        upgraded = [mock.call(self.engine, self.config, v.revision,
+        upgraded = [mock.call(self, self.engine, self.config, v.revision,
                     with_data=True) for v in reversed(self.versions)]
         self.assertEqual(upgraded, self._migrate_up.call_args_list)
 
@@ -944,8 +944,8 @@ class MigrationCheckersMixin(object):
         deploy_template_steps.insert().execute(step)
         # Query by deploy template ID.
         result = deploy_template_steps.select(
-            deploy_template_steps.c.deploy_template_id ==
-            template_id).execute().first()
+            deploy_template_steps.c.deploy_template_id
+            == template_id).execute().first()
         self.assertEqual(template_id, result['deploy_template_id'])
         self.assertEqual(interface, result['interface'])
         self.assertEqual(step_name, result['step'])
@@ -964,6 +964,44 @@ class MigrationCheckersMixin(object):
         self.assertIsInstance(deploy_templates.c.extra.type,
                               sqlalchemy.types.TEXT)
 
+    def _check_ce6c4b3cf5a2(self, engine, data):
+        allocations = db_utils.get_table(engine, 'allocations')
+        col_names = [column.name for column in allocations.c]
+        self.assertIn('owner', col_names)
+
+    def _check_cf1a80fdb352(self, engine, data):
+        nodes = db_utils.get_table(engine, 'nodes')
+        col_names = [column.name for column in nodes.c]
+        self.assertIn('network_data', col_names)
+        self.assertIsInstance(
+            nodes.c.network_data.type, sqlalchemy.types.String)
+
+    def _pre_upgrade_cd2c80feb331(self, engine):
+        data = {
+            'node_uuid': uuidutils.generate_uuid(),
+        }
+
+        nodes = db_utils.get_table(engine, 'nodes')
+        nodes.insert().execute({'uuid': data['node_uuid']})
+
+        return data
+
+    def _check_cd2c80feb331(self, engine, data):
+        nodes = db_utils.get_table(engine, 'nodes')
+        col_names = [column.name for column in nodes.c]
+        self.assertIn('retired', col_names)
+        self.assertIn('retired_reason', col_names)
+
+        node = nodes.select(
+            nodes.c.uuid == data['node_uuid']).execute().first()
+        self.assertFalse(node['retired'])
+        self.assertIsNone(node['retired_reason'])
+
+    def _check_b2ad35726bb0(self, engine, data):
+        nodes = db_utils.get_table(engine, 'nodes')
+        col_names = [column.name for column in nodes.c]
+        self.assertIn('lessee', col_names)
+
     def test_upgrade_and_version(self):
         with patch_with_engine(self.engine):
             self.migration_api.upgrade('head')
