@@ -16,11 +16,10 @@
 """Tests for manipulating Nodes via the DB API"""
 
 import datetime
+from unittest import mock
 
-import mock
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
-import six
 
 from ironic.common import exception
 from ironic.common import states
@@ -73,8 +72,8 @@ class DbNodeTestCase(base.DbTestCase):
         res = self.dbapi.get_node_by_id(node.id)
         self.assertEqual(node.id, res.id)
         self.assertEqual(node.uuid, res.uuid)
-        self.assertItemsEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
-        self.assertItemsEqual(['trait1', 'trait2'],
+        self.assertCountEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
+        self.assertCountEqual(['trait1', 'trait2'],
                               [trait.trait for trait in res.traits])
 
     def test_get_node_by_uuid(self):
@@ -85,8 +84,8 @@ class DbNodeTestCase(base.DbTestCase):
         res = self.dbapi.get_node_by_uuid(node.uuid)
         self.assertEqual(node.id, res.id)
         self.assertEqual(node.uuid, res.uuid)
-        self.assertItemsEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
-        self.assertItemsEqual(['trait1', 'trait2'],
+        self.assertCountEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
+        self.assertCountEqual(['trait1', 'trait2'],
                               [trait.trait for trait in res.traits])
 
     def test_get_node_by_name(self):
@@ -98,8 +97,8 @@ class DbNodeTestCase(base.DbTestCase):
         self.assertEqual(node.id, res.id)
         self.assertEqual(node.uuid, res.uuid)
         self.assertEqual(node.name, res.name)
-        self.assertItemsEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
-        self.assertItemsEqual(['trait1', 'trait2'],
+        self.assertCountEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
+        self.assertCountEqual(['trait1', 'trait2'],
                               [trait.trait for trait in res.traits])
 
     def test_get_node_that_does_not_exist(self):
@@ -231,7 +230,8 @@ class DbNodeTestCase(base.DbTestCase):
 
         # node with provision_updated timeout
         node1 = utils.create_test_node(uuid=uuidutils.generate_uuid(),
-                                       provision_updated_at=past)
+                                       provision_updated_at=past,
+                                       provision_state=states.DEPLOYING)
         # node with None in provision_updated_at
         node2 = utils.create_test_node(uuid=uuidutils.generate_uuid(),
                                        provision_state=states.DEPLOYWAIT)
@@ -247,6 +247,10 @@ class DbNodeTestCase(base.DbTestCase):
                                                     states.DEPLOYWAIT})
         self.assertEqual([node2.id], [r[0] for r in res])
 
+        res = self.dbapi.get_nodeinfo_list(
+            filters={'provision_state_in': [states.ACTIVE, states.DEPLOYING]})
+        self.assertEqual([node1.id], [r[0] for r in res])
+
     @mock.patch.object(timeutils, 'utcnow', autospec=True)
     def test_get_nodeinfo_list_inspection(self, mock_utcnow):
         past = datetime.datetime(2000, 1, 1, 0, 0)
@@ -273,14 +277,27 @@ class DbNodeTestCase(base.DbTestCase):
                                                     states.INSPECTING})
         self.assertEqual([node2.id], [r[0] for r in res])
 
+    def test_get_nodeinfo_list_description(self):
+        node1 = utils.create_test_node(uuid=uuidutils.generate_uuid(),
+                                       description='Hello')
+        node2 = utils.create_test_node(uuid=uuidutils.generate_uuid(),
+                                       description='World!')
+        res = self.dbapi.get_nodeinfo_list(
+            filters={'description_contains': 'Hello'})
+        self.assertEqual([node1.id], [r[0] for r in res])
+
+        res = self.dbapi.get_nodeinfo_list(filters={'description_contains':
+                                                    'World!'})
+        self.assertEqual([node2.id], [r[0] for r in res])
+
     def test_get_node_list(self):
         uuids = []
         for i in range(1, 6):
             node = utils.create_test_node(uuid=uuidutils.generate_uuid())
-            uuids.append(six.text_type(node['uuid']))
+            uuids.append(str(node['uuid']))
         res = self.dbapi.get_node_list()
         res_uuids = [r.uuid for r in res]
-        six.assertCountEqual(self, uuids, res_uuids)
+        self.assertCountEqual(uuids, res_uuids)
         for r in res:
             self.assertEqual([], r.tags)
             self.assertEqual([], r.traits)
@@ -302,7 +319,8 @@ class DbNodeTestCase(base.DbTestCase):
             maintenance=True,
             fault='boom',
             resource_class='foo',
-            conductor_group='group1')
+            conductor_group='group1',
+            power_state='power on')
 
         res = self.dbapi.get_node_list(filters={'chassis_uuid': ch1['uuid']})
         self.assertEqual([node1.id], [r.id for r in res])
@@ -355,6 +373,18 @@ class DbNodeTestCase(base.DbTestCase):
         res = self.dbapi.get_node_list(filters={'uuid': node1.uuid})
         self.assertEqual([node1.id], [r.id for r in res])
 
+        uuids = [uuidutils.generate_uuid(),
+                 node1.uuid,
+                 uuidutils.generate_uuid()]
+        res = self.dbapi.get_node_list(filters={'uuid_in': uuids})
+        self.assertEqual([node1.id], [r.id for r in res])
+
+        res = self.dbapi.get_node_list(filters={'with_power_state': True})
+        self.assertEqual([node2.id], [r.id for r in res])
+
+        res = self.dbapi.get_node_list(filters={'with_power_state': False})
+        self.assertEqual([node1.id], [r.id for r in res])
+
         # ensure unknown filters explode
         filters = {'bad_filter': 'foo'}
         self.assertRaisesRegex(ValueError,
@@ -369,6 +399,48 @@ class DbNodeTestCase(base.DbTestCase):
                                self.dbapi.get_node_list,
                                filters=filters)
 
+    def test_get_node_list_filter_by_project(self):
+        utils.create_test_node(uuid=uuidutils.generate_uuid())
+        node2 = utils.create_test_node(
+            uuid=uuidutils.generate_uuid(),
+            owner='project1',
+            lessee='project2',
+        )
+        node3 = utils.create_test_node(
+            uuid=uuidutils.generate_uuid(),
+            owner='project2',
+        )
+        node4 = utils.create_test_node(
+            uuid=uuidutils.generate_uuid(),
+            owner='project1',
+            lessee='project3',
+        )
+
+        res = self.dbapi.get_node_list(filters={'project': 'project1'})
+        self.assertEqual([node2.id, node4.id], [r.id for r in res])
+
+        res = self.dbapi.get_node_list(filters={'project': 'project2'})
+        self.assertEqual([node2.id, node3.id], [r.id for r in res])
+
+        res = self.dbapi.get_node_list(filters={'project': 'project3'})
+        self.assertEqual([node4.id], [r.id for r in res])
+
+        res = self.dbapi.get_node_list(filters={'project': 'flargle'})
+        self.assertEqual([], [r.id for r in res])
+
+    def test_get_node_list_description(self):
+        node1 = utils.create_test_node(uuid=uuidutils.generate_uuid(),
+                                       description='Hello')
+        node2 = utils.create_test_node(uuid=uuidutils.generate_uuid(),
+                                       description='World!')
+        res = self.dbapi.get_node_list(filters={
+            'description_contains': 'Hello'})
+        self.assertEqual([node1.id], [r.id for r in res])
+
+        res = self.dbapi.get_node_list(filters={
+            'description_contains': 'World!'})
+        self.assertEqual([node2.id], [r.id for r in res])
+
     def test_get_node_list_chassis_not_found(self):
         self.assertRaises(exception.ChassisNotFound,
                           self.dbapi.get_node_list,
@@ -383,8 +455,8 @@ class DbNodeTestCase(base.DbTestCase):
 
         res = self.dbapi.get_node_by_instance(node.instance_uuid)
         self.assertEqual(node.uuid, res.uuid)
-        self.assertItemsEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
-        self.assertItemsEqual(['trait1', 'trait2'],
+        self.assertCountEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
+        self.assertCountEqual(['trait1', 'trait2'],
                               [trait.trait for trait in res.traits])
 
     def test_get_node_by_instance_wrong_uuid(self):
@@ -519,6 +591,17 @@ class DbNodeTestCase(base.DbTestCase):
         self.assertRaises(exception.NodeNotFound,
                           self.dbapi.node_trait_exists, node.id, trait.trait)
 
+    def test_allocations_get_destroyed_after_destroying_a_node_by_uuid(self):
+        node = utils.create_test_node()
+
+        allocation = utils.create_test_allocation(node_id=node.id)
+        node = self.dbapi.update_node(node.id,
+                                      {'allocation_id': allocation.id})
+
+        self.dbapi.destroy_node(node.uuid)
+        self.assertRaises(exception.AllocationNotFound,
+                          self.dbapi.get_allocation_by_id, allocation.id)
+
     def test_update_node(self):
         node = utils.create_test_node()
 
@@ -642,8 +725,8 @@ class DbNodeTestCase(base.DbTestCase):
 
         # reserve the node
         res = self.dbapi.reserve_node(r1, uuid)
-        self.assertItemsEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
-        self.assertItemsEqual(['trait1', 'trait2'],
+        self.assertCountEqual(['tag1', 'tag2'], [tag.tag for tag in res.tags])
+        self.assertCountEqual(['trait1', 'trait2'],
                               [trait.trait for trait in res.traits])
 
         # check reservation
@@ -805,3 +888,38 @@ class DbNodeTestCase(base.DbTestCase):
                                'Multiple nodes',
                                self.dbapi.get_node_by_port_addresses,
                                addresses)
+
+    def test_check_node_list(self):
+        node1 = utils.create_test_node(uuid=uuidutils.generate_uuid())
+        node2 = utils.create_test_node(uuid=uuidutils.generate_uuid(),
+                                       name='node_2')
+        node3 = utils.create_test_node(uuid=uuidutils.generate_uuid(),
+                                       name='node_3')
+
+        mapping = self.dbapi.check_node_list([node1.uuid, node2.name,
+                                              node3.uuid])
+        self.assertEqual({node1.uuid: node1.uuid,
+                          node2.name: node2.uuid,
+                          node3.uuid: node3.uuid},
+                         mapping)
+
+    def test_check_node_list_non_existing(self):
+        node1 = utils.create_test_node(uuid=uuidutils.generate_uuid())
+        node2 = utils.create_test_node(uuid=uuidutils.generate_uuid(),
+                                       name='node_2')
+        uuid = uuidutils.generate_uuid()
+
+        exc = self.assertRaises(exception.NodeNotFound,
+                                self.dbapi.check_node_list,
+                                [node1.uuid, uuid, 'could-be-a-name',
+                                 node2.name])
+        self.assertIn(uuid, str(exc))
+        self.assertIn('could-be-a-name', str(exc))
+
+    def test_check_node_list_impossible(self):
+        node1 = utils.create_test_node(uuid=uuidutils.generate_uuid())
+
+        exc = self.assertRaises(exception.NodeNotFound,
+                                self.dbapi.check_node_list,
+                                [node1.uuid, 'this/cannot/be/a/name'])
+        self.assertIn('this/cannot/be/a/name', str(exc))
