@@ -14,8 +14,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
-from neutronclient.common import exceptions as neutron_client_exc
+from unittest import mock
+
+from openstack.connection import exceptions as openstack_exc
 from oslo_utils import uuidutils
 
 from ironic.common import dhcp_factory
@@ -49,7 +50,8 @@ class TestNeutron(db_base.DbTestCase):
         dhcp_factory.DHCPFactory._dhcp_provider = None
 
     @mock.patch('ironic.common.neutron.get_client', autospec=True)
-    def test_update_port_dhcp_opts(self, client_mock):
+    @mock.patch('ironic.common.neutron.update_neutron_port', autospec=True)
+    def test_update_port_dhcp_opts(self, update_mock, client_mock):
         opts = [{'opt_name': 'bootfile-name',
                  'opt_value': 'pxelinux.0'},
                 {'opt_name': 'tftp-server',
@@ -57,21 +59,82 @@ class TestNeutron(db_base.DbTestCase):
                 {'opt_name': 'server-ip-address',
                  'opt_value': '1.1.1.1'}]
         port_id = 'fake-port-id'
-        expected = {'port': {'extra_dhcp_opts': opts}}
+        expected = {'extra_dhcp_opts': opts}
+        port_data = {
+            "id": port_id,
+            "fixed_ips": [
+                {
+                    "ip_address": "192.168.1.3",
+                }
+            ],
+        }
+        client_mock.return_value.get_port.return_value = port_data
 
         api = dhcp_factory.DHCPFactory()
         with task_manager.acquire(self.context, self.node.uuid) as task:
             api.provider.update_port_dhcp_opts(port_id, opts,
                                                context=task.context)
-        client_mock.return_value.update_port.assert_called_once_with(
-            port_id, expected)
+        update_mock.assert_called_once_with(
+            self.context, port_id, expected)
 
     @mock.patch('ironic.common.neutron.get_client', autospec=True)
-    def test_update_port_dhcp_opts_with_exception(self, client_mock):
+    @mock.patch('ironic.common.neutron.update_neutron_port', autospec=True)
+    def test_update_port_dhcp_opts_v6(self, update_mock, client_mock):
+        opts = [{'opt_name': 'bootfile-name',
+                 'opt_value': 'pxelinux.0',
+                 'ip_version': 4},
+                {'opt_name': 'tftp-server',
+                 'opt_value': '1.1.1.1',
+                 'ip_version': 4},
+                {'opt_name': 'server-ip-address',
+                 'opt_value': '1.1.1.1',
+                 'ip_version': 4},
+                {'opt_name': 'bootfile-url',
+                 'opt_value': 'tftp://::1/file.name',
+                 'ip_version': 6}]
+        port_id = 'fake-port-id'
+        expected = {
+            'extra_dhcp_opts': [
+                {
+                    'opt_name': 'bootfile-url',
+                    'opt_value': 'tftp://::1/file.name',
+                    'ip_version': 6
+                }
+            ]
+        }
+        port_data = {
+            "id": port_id,
+            "fixed_ips": [
+                {
+                    "ip_address": "2001:db8::201",
+                }
+            ],
+        }
+        client_mock.return_value.get_port.return_value = port_data
+
+        api = dhcp_factory.DHCPFactory()
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            api.provider.update_port_dhcp_opts(port_id, opts,
+                                               context=task.context)
+        update_mock.assert_called_once_with(
+            task.context, port_id, expected)
+
+    @mock.patch('ironic.common.neutron.get_client', autospec=True)
+    @mock.patch('ironic.common.neutron.update_neutron_port', autospec=True)
+    def test_update_port_dhcp_opts_with_exception(self, update_mock,
+                                                  client_mock):
         opts = [{}]
         port_id = 'fake-port-id'
-        client_mock.return_value.update_port.side_effect = (
-            neutron_client_exc.NeutronClientException())
+        port_data = {
+            "id": port_id,
+            "fixed_ips": [
+                {
+                    "ip_address": "192.168.1.3",
+                }
+            ],
+        }
+        client_mock.return_value.get_port.return_value = port_data
+        update_mock.side_effect = openstack_exc.OpenStackCloudException()
 
         api = dhcp_factory.DHCPFactory()
         with task_manager.acquire(self.context, self.node.uuid) as task:
@@ -199,10 +262,34 @@ class TestNeutron(db_base.DbTestCase):
             "device_id": 'bece68a3-2f8b-4e66-9092-244493d6aba7',
         }
         fake_client = mock.Mock()
-        fake_client.show_port.return_value = {'port': port_data}
+        fake_client.get_port.return_value = port_data
         result = api._get_fixed_ip_address(port_id, fake_client)
         self.assertEqual(expected, result)
-        fake_client.show_port.assert_called_once_with(port_id)
+        fake_client.get_port.assert_called_once_with(port_id)
+
+    def test__get_fixed_ip_address_ipv6(self):
+        port_id = 'fake-port-id'
+        expected = "2001:dead:beef::1234"
+        api = dhcp_factory.DHCPFactory().provider
+        port_data = {
+            "id": port_id,
+            "network_id": "3cb9bc59-5699-4588-a4b1-b87f96708bc6",
+            "admin_state_up": True,
+            "status": "ACTIVE",
+            "mac_address": "fa:16:3e:4c:2c:30",
+            "fixed_ips": [
+                {
+                    "ip_address": "2001:dead:beef::1234",
+                    "subnet_id": "f8a6e8f8-c2ec-497c-9f23-da9616de54ef"
+                }
+            ],
+            "device_id": 'bece68a3-2f8b-4e66-9092-244493d6aba7',
+        }
+        fake_client = mock.Mock()
+        fake_client.get_port.return_value = port_data
+        result = api._get_fixed_ip_address(port_id, fake_client)
+        self.assertEqual(expected, result)
+        fake_client.get_port.assert_called_once_with(port_id)
 
     def test__get_fixed_ip_address_invalid_ip(self):
         port_id = 'fake-port-id'
@@ -222,22 +309,23 @@ class TestNeutron(db_base.DbTestCase):
             "device_id": 'bece68a3-2f8b-4e66-9092-244493d6aba7',
         }
         fake_client = mock.Mock()
-        fake_client.show_port.return_value = {'port': port_data}
-        self.assertRaises(exception.InvalidIPv4Address,
+        fake_client.get_port.return_value = port_data
+        self.assertRaises(exception.InvalidIPAddress,
                           api._get_fixed_ip_address,
                           port_id, fake_client)
-        fake_client.show_port.assert_called_once_with(port_id)
+        fake_client.get_port.assert_called_once_with(port_id)
 
     def test__get_fixed_ip_address_with_exception(self):
         port_id = 'fake-port-id'
         api = dhcp_factory.DHCPFactory().provider
 
         fake_client = mock.Mock()
-        fake_client.show_port.side_effect = (
-            neutron_client_exc.NeutronClientException())
+        fake_client.get_port.side_effect = (
+            openstack_exc.OpenStackCloudException())
+
         self.assertRaises(exception.NetworkError,
                           api._get_fixed_ip_address, port_id, fake_client)
-        fake_client.show_port.assert_called_once_with(port_id)
+        fake_client.get_port.assert_called_once_with(port_id)
 
     @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi._get_fixed_ip_address',
                 autospec=True)
