@@ -11,11 +11,10 @@
 # limitations under the License.
 
 import json
+from unittest import mock
 
 from ironic_lib import utils as irlib_utils
-import mock
 from oslo_concurrency import processutils
-import six
 
 from ironic.common import exception
 from ironic.common import states
@@ -235,7 +234,7 @@ class TestAnsibleMethods(AnsibleDeployTestCaseBase):
                                 ansible_deploy._run_playbook,
                                 self.node, 'deploy', extra_vars,
                                 '/path/to/key')
-        self.assertIn('VIKINGS!', six.text_type(exc))
+        self.assertIn('VIKINGS!', str(exc))
         execute_mock.assert_called_once_with(
             'env', 'ANSIBLE_CONFIG=/path/to/config',
             'ansible-playbook', '/path/to/playbooks/deploy', '-i',
@@ -357,6 +356,35 @@ class TestAnsibleMethods(AnsibleDeployTestCaseBase):
             self.assertEqual(
                 expected, ansible_deploy._parse_root_device_hints(task.node))
 
+    def test__parse_root_device_hints_iinfo(self):
+        hints = {"wwn": "fake wwn", "size": "12345", "rotational": True,
+                 "serial": "HELLO"}
+        expected = {"wwn": "fake wwn", "size": 12345, "rotational": True,
+                    "serial": "hello"}
+        iinfo = self.node.instance_info
+        iinfo['root_device'] = hints
+        self.node.instance_info = iinfo
+        self.node.save()
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            self.assertEqual(
+                expected, ansible_deploy._parse_root_device_hints(task.node))
+
+    def test__parse_root_device_hints_override(self):
+        hints = {"wwn": "fake wwn", "size": "12345", "rotational": True,
+                 "serial": "HELLO"}
+        expected = {"wwn": "fake wwn", "size": 12345, "rotational": True,
+                    "serial": "hello"}
+        props = self.node.properties
+        props['root_device'] = {'size': 'no idea'}
+        self.node.properties = props
+        iinfo = self.node.instance_info
+        iinfo['root_device'] = hints
+        self.node.instance_info = iinfo
+        self.node.save()
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            self.assertEqual(
+                expected, ansible_deploy._parse_root_device_hints(task.node))
+
     def test__parse_root_device_hints_fail_advanced(self):
         hints = {"wwn": "s!= fake wwn",
                  "size": ">= 12345",
@@ -374,8 +402,8 @@ class TestAnsibleMethods(AnsibleDeployTestCaseBase):
                 exception.InvalidParameterValue,
                 ansible_deploy._parse_root_device_hints, task.node)
             for key, value in expected.items():
-                self.assertIn(six.text_type(key), six.text_type(exc))
-                self.assertIn(six.text_type(value), six.text_type(exc))
+                self.assertIn(str(key), str(exc))
+                self.assertIn(str(value), str(exc))
 
     def test__prepare_variables(self):
         i_info = self.node.instance_info
@@ -486,9 +514,9 @@ class TestAnsibleMethods(AnsibleDeployTestCaseBase):
         exc = self.assertRaises(exception.NodeCleaningFailure,
                                 ansible_deploy._validate_clean_steps,
                                 steps, self.node.uuid)
-        self.assertIn("name foo, field ham.value", six.text_type(exc))
-        self.assertIn("name bar, field interface", six.text_type(exc))
-        self.assertIn("name undefined, field name", six.text_type(exc))
+        self.assertIn("name foo, field ham.value", str(exc))
+        self.assertIn("name bar, field interface", str(exc))
+        self.assertIn("name undefined, field name", str(exc))
 
     def test__validate_clean_steps_names_not_unique(self):
         steps = [{"name": "foo",
@@ -498,7 +526,7 @@ class TestAnsibleMethods(AnsibleDeployTestCaseBase):
         exc = self.assertRaises(exception.NodeCleaningFailure,
                                 ansible_deploy._validate_clean_steps,
                                 steps, self.node.uuid)
-        self.assertIn("unique names", six.text_type(exc))
+        self.assertIn("unique names", str(exc))
 
     @mock.patch.object(ansible_deploy.yaml, 'safe_load', autospec=True)
     def test__get_clean_steps(self, load_mock):
@@ -548,7 +576,7 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
     def test_get_properties(self):
         self.assertEqual(
             set(list(ansible_deploy.COMMON_PROPERTIES)
-                + ['deploy_forces_oob_reboot']),
+                + ['agent_verify_ca', 'deploy_forces_oob_reboot']),
             set(self.driver.get_properties()))
 
     @mock.patch.object(deploy_utils, 'check_for_missing_params',
@@ -609,7 +637,7 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
     @mock.patch('ironic.drivers.modules.deploy_utils.'
                 'build_instance_info_for_deploy',
                 return_value={'test': 'test'}, autospec=True)
-    @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk')
+    @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk', autospec=True)
     def test_prepare(self, pxe_prepare_ramdisk_mock,
                      build_instance_info_mock, build_options_mock,
                      power_action_mock):
@@ -628,7 +656,7 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
             build_instance_info_mock.assert_called_once_with(task)
             build_options_mock.assert_called_once_with(task.node)
             pxe_prepare_ramdisk_mock.assert_called_once_with(
-                task, {'op1': 'test1'})
+                task.driver.boot, task, {'op1': 'test1'})
 
         self.node.refresh()
         self.assertEqual('test', self.node.instance_info['test'])
@@ -636,13 +664,13 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
     @mock.patch.object(ansible_deploy, '_get_configdrive_path',
                        return_value='/path/test', autospec=True)
     @mock.patch.object(irlib_utils, 'unlink_without_raise', autospec=True)
-    @mock.patch.object(pxe.PXEBoot, 'clean_up_ramdisk')
+    @mock.patch.object(pxe.PXEBoot, 'clean_up_ramdisk', autospec=True)
     def test_clean_up(self, pxe_clean_up_mock, unlink_mock,
                       get_cfdrive_path_mock):
         with task_manager.acquire(
                 self.context, self.node['uuid'], shared=False) as task:
             self.driver.clean_up(task)
-            pxe_clean_up_mock.assert_called_once_with(task)
+            pxe_clean_up_mock.assert_called_once_with(task.driver.boot, task)
             get_cfdrive_path_mock.assert_called_once_with(self.node['uuid'])
             unlink_mock.assert_called_once_with('/path/test')
 
@@ -736,7 +764,7 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
     @mock.patch.object(utils, 'node_power_action', autospec=True)
     @mock.patch('ironic.drivers.modules.deploy_utils.build_agent_options',
                 return_value={'op1': 'test1'}, autospec=True)
-    @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk')
+    @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk', autospec=True)
     def test_prepare_cleaning(
             self, prepare_ramdisk_mock, buid_options_mock, power_action_mock,
             set_node_cleaning_steps, run_playbook_mock):
@@ -757,7 +785,7 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
                 task)
             buid_options_mock.assert_called_once_with(task.node)
             prepare_ramdisk_mock.assert_called_once_with(
-                task, {'op1': 'test1'})
+                task.driver.boot, task, {'op1': 'test1'})
             power_action_mock.assert_called_once_with(task, states.REBOOT)
             self.assertFalse(run_playbook_mock.called)
             self.assertEqual(states.CLEANWAIT, state)
@@ -774,7 +802,7 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
             self.assertFalse(task.driver.network.add_cleaning_network.called)
 
     @mock.patch.object(utils, 'node_power_action', autospec=True)
-    @mock.patch.object(pxe.PXEBoot, 'clean_up_ramdisk')
+    @mock.patch.object(pxe.PXEBoot, 'clean_up_ramdisk', autospec=True)
     def test_tear_down_cleaning(self, clean_ramdisk_mock, power_action_mock):
         with task_manager.acquire(self.context, self.node.uuid) as task:
             task.driver.network.remove_cleaning_network = mock.Mock()
@@ -782,7 +810,7 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
             self.driver.tear_down_cleaning(task)
 
             power_action_mock.assert_called_once_with(task, states.POWER_OFF)
-            clean_ramdisk_mock.assert_called_once_with(task)
+            clean_ramdisk_mock.assert_called_once_with(task.driver.boot, task)
             (task.driver.network.remove_cleaning_network
                 .assert_called_once_with(task))
 
@@ -862,13 +890,11 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
             run_playbook_mock.assert_called_once_with(
                 task.node, 'test_pl', ironic_nodes, 'test_k')
 
-    @mock.patch.object(utils, 'power_on_node_if_needed', autospec=True)
     @mock.patch.object(fake.FakePower, 'get_power_state',
-                       return_value=states.POWER_OFF)
+                       return_value=states.POWER_OFF, autospec=True)
     @mock.patch.object(utils, 'node_power_action', autospec=True)
-    def test_reboot_and_finish_deploy_force_reboot(
-            self, power_action_mock, get_pow_state_mock,
-            power_on_node_if_needed_mock):
+    def test_tear_down_agent_force_reboot(
+            self, power_action_mock, get_pow_state_mock):
         d_info = self.node.driver_info
         d_info['deploy_forces_oob_reboot'] = True
         self.node.driver_info = d_info
@@ -878,26 +904,15 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
         self.node.provision_state = states.DEPLOYING
         self.node.save()
 
-        power_on_node_if_needed_mock.return_value = None
         with task_manager.acquire(self.context, self.node.uuid) as task:
-            with mock.patch.object(task.driver, 'network') as net_mock:
-                self.driver.reboot_and_finish_deploy(task)
-                net_mock.remove_provisioning_network.assert_called_once_with(
-                    task)
-                net_mock.configure_tenant_networks.assert_called_once_with(
-                    task)
-            expected_power_calls = [((task, states.POWER_OFF),),
-                                    ((task, states.POWER_ON),)]
-            self.assertEqual(expected_power_calls,
-                             power_action_mock.call_args_list)
+            self.driver.tear_down_agent(task)
+            power_action_mock.assert_called_once_with(task, states.POWER_OFF)
         get_pow_state_mock.assert_not_called()
 
-    @mock.patch.object(utils, 'power_on_node_if_needed', autospec=True)
     @mock.patch.object(ansible_deploy, '_run_playbook', autospec=True)
     @mock.patch.object(utils, 'node_power_action', autospec=True)
-    def test_reboot_and_finish_deploy_soft_poweroff_retry(
-            self, power_action_mock, run_playbook_mock,
-            power_on_node_if_needed_mock):
+    def test_tear_down_agent_soft_poweroff_retry(
+            self, power_action_mock, run_playbook_mock):
         self.config(group='ansible',
                     post_deploy_get_power_state_retry_interval=0)
         self.config(group='ansible',
@@ -908,106 +923,39 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
         self.node.driver_internal_info = di_info
         self.node.save()
 
-        power_on_node_if_needed_mock.return_value = None
         with task_manager.acquire(self.context, self.node.uuid) as task:
-            with mock.patch.object(task.driver, 'network') as net_mock:
-                with mock.patch.object(task.driver.power,
-                                       'get_power_state',
-                                       return_value=states.POWER_ON) as p_mock:
-                    self.driver.reboot_and_finish_deploy(task)
-                    p_mock.assert_called_with(task)
-                    self.assertEqual(2, len(p_mock.mock_calls))
-                net_mock.remove_provisioning_network.assert_called_once_with(
-                    task)
-                net_mock.configure_tenant_networks.assert_called_once_with(
-                    task)
-            power_action_mock.assert_has_calls(
-                [mock.call(task, states.POWER_OFF),
-                 mock.call(task, states.POWER_ON)])
-            expected_power_calls = [((task, states.POWER_OFF),),
-                                    ((task, states.POWER_ON),)]
-            self.assertEqual(expected_power_calls,
-                             power_action_mock.call_args_list)
+            with mock.patch.object(task.driver.power,
+                                   'get_power_state',
+                                   return_value=states.POWER_ON,
+                                   autospec=True) as p_mock:
+                self.driver.tear_down_agent(task)
+                p_mock.assert_called_with(task)
+                self.assertEqual(2, len(p_mock.mock_calls))
+            power_action_mock.assert_called_once_with(task, states.POWER_OFF)
             run_playbook_mock.assert_called_once_with(
                 task.node, 'shutdown.yaml', mock.ANY, mock.ANY)
 
+    @mock.patch.object(utils, 'node_set_boot_device', autospec=True)
     @mock.patch.object(ansible_deploy, '_get_node_ip', autospec=True,
                        return_value='1.2.3.4')
-    def test_continue_deploy(self, getip_mock):
-        self.node.provision_state = states.DEPLOYWAIT
+    def test_write_image(self, getip_mock, bootdev_mock):
+        self.node.provision_state = states.DEPLOYING
         self.node.target_provision_state = states.ACTIVE
         self.node.save()
         with task_manager.acquire(self.context, self.node.uuid) as task:
             with mock.patch.multiple(self.driver, autospec=True,
                                      _ansible_deploy=mock.DEFAULT,
                                      reboot_to_instance=mock.DEFAULT):
-                self.driver.continue_deploy(task)
+                result = self.driver.write_image(task)
+                self.assertIsNone(result)
                 getip_mock.assert_called_once_with(task)
                 self.driver._ansible_deploy.assert_called_once_with(
                     task, '1.2.3.4')
-                self.driver.reboot_to_instance.assert_called_once_with(task)
+                bootdev_mock.assert_called_once_with(task, 'disk',
+                                                     persistent=True)
             self.assertEqual(states.ACTIVE, task.node.target_provision_state)
             self.assertEqual(states.DEPLOYING, task.node.provision_state)
 
-    @mock.patch.object(utils, 'notify_conductor_resume_deploy', autospec=True)
-    @mock.patch.object(utils, 'node_set_boot_device', autospec=True)
-    def test_reboot_to_instance(self, bootdev_mock, resume_mock):
-        self.node.provision_state = states.DEPLOYING
-        self.node.deploy_step = {
-            'step': 'deploy', 'priority': 100, 'interface': 'deploy'}
-        self.node.save()
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            with mock.patch.object(self.driver, 'reboot_and_finish_deploy',
-                                   autospec=True):
-                task.driver.boot = mock.Mock()
-                self.driver.reboot_to_instance(task)
-                bootdev_mock.assert_called_once_with(task, 'disk',
-                                                     persistent=True)
-                resume_mock.assert_called_once_with(task)
-                self.driver.reboot_and_finish_deploy.assert_called_once_with(
-                    task)
-                task.driver.boot.clean_up_ramdisk.assert_called_once_with(
-                    task)
-
-    @mock.patch.object(utils, 'notify_conductor_resume_deploy', autospec=True)
-    @mock.patch.object(utils, 'node_set_boot_device', autospec=True)
-    def test_reboot_to_instance_deprecated(self, bootdev_mock, resume_mock):
-        # TODO(rloo): no deploy steps; delete this when we remove support
-        # for handling no deploy steps.
-        self.node.provision_state = states.DEPLOYING
-        self.node.deploy_step = None
-        self.node.save()
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            with mock.patch.object(self.driver, 'reboot_and_finish_deploy',
-                                   autospec=True):
-                task.driver.boot = mock.Mock()
-                task.process_event = mock.Mock()
-                self.driver.reboot_to_instance(task)
-                bootdev_mock.assert_called_once_with(task, 'disk',
-                                                     persistent=True)
-                self.assertFalse(resume_mock.called)
-                task.process_event.assert_called_once_with('done')
-                self.driver.reboot_and_finish_deploy.assert_called_once_with(
-                    task)
-                task.driver.boot.clean_up_ramdisk.assert_called_once_with(
-                    task)
-
-    @mock.patch.object(utils, 'restore_power_state_if_needed', autospec=True)
-    @mock.patch.object(utils, 'power_on_node_if_needed')
-    @mock.patch.object(utils, 'node_power_action', autospec=True)
-    def test_tear_down_with_smartnic_port(
-            self, power_mock, power_on_node_if_needed_mock,
-            restore_power_state_mock):
-        with task_manager.acquire(
-                self.context, self.node['uuid'], shared=False) as task:
-            power_on_node_if_needed_mock.return_value = states.POWER_OFF
-            driver_return = self.driver.tear_down(task)
-            power_mock.assert_called_once_with(task, states.POWER_OFF)
-            self.assertEqual(driver_return, states.DELETED)
-            power_on_node_if_needed_mock.assert_called_once_with(task)
-            restore_power_state_mock.assert_called_once_with(
-                task, states.POWER_OFF)
-
     @mock.patch.object(flat_network.FlatNetwork, 'add_provisioning_network',
                        autospec=True)
     @mock.patch.object(utils, 'restore_power_state_if_needed', autospec=True)
@@ -1016,7 +964,7 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
     @mock.patch.object(deploy_utils, 'build_agent_options', autospec=True)
     @mock.patch.object(deploy_utils, 'build_instance_info_for_deploy',
                        autospec=True)
-    @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk')
+    @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk', autospec=True)
     def test_prepare_with_smartnic_port(
             self, pxe_prepare_ramdisk_mock,
             build_instance_info_mock, build_options_mock,
@@ -1034,7 +982,7 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
             build_instance_info_mock.assert_called_once_with(task)
             build_options_mock.assert_called_once_with(task.node)
             pxe_prepare_ramdisk_mock.assert_called_once_with(
-                task, {'op1': 'test1'})
+                task.driver.boot, task, {'op1': 'test1'})
             power_on_node_if_needed_mock.assert_called_once_with(task)
             restore_power_state_mock.assert_called_once_with(
                 task, states.POWER_OFF)
@@ -1048,7 +996,7 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
     @mock.patch.object(steps, 'set_node_cleaning_steps', autospec=True)
     @mock.patch.object(utils, 'node_power_action', autospec=True)
     @mock.patch.object(deploy_utils, 'build_agent_options', autospec=True)
-    @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk')
+    @mock.patch.object(pxe.PXEBoot, 'prepare_ramdisk', autospec=True)
     def test_prepare_cleaning_with_smartnic_port(
             self, prepare_ramdisk_mock, build_options_mock, power_action_mock,
             set_node_cleaning_steps, run_playbook_mock,
@@ -1070,7 +1018,7 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
                 task)
             build_options_mock.assert_called_once_with(task.node)
             prepare_ramdisk_mock.assert_called_once_with(
-                task, {'op1': 'test1'})
+                task.driver.boot, task, {'op1': 'test1'})
             power_action_mock.assert_called_once_with(task, states.REBOOT)
             self.assertFalse(run_playbook_mock.called)
             self.assertEqual(states.CLEANWAIT, state)
@@ -1081,7 +1029,7 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
     @mock.patch.object(utils, 'restore_power_state_if_needed', autospec=True)
     @mock.patch.object(utils, 'power_on_node_if_needed', autospec=True)
     @mock.patch.object(utils, 'node_power_action', autospec=True)
-    @mock.patch.object(pxe.PXEBoot, 'clean_up_ramdisk')
+    @mock.patch.object(pxe.PXEBoot, 'clean_up_ramdisk', autospec=True)
     def test_tear_down_cleaning_with_smartnic_port(
             self, clean_ramdisk_mock, power_action_mock,
             power_on_node_if_needed_mock, restore_power_state_mock):
@@ -1090,42 +1038,10 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
             power_on_node_if_needed_mock.return_value = states.POWER_OFF
             self.driver.tear_down_cleaning(task)
             power_action_mock.assert_called_once_with(task, states.POWER_OFF)
-            clean_ramdisk_mock.assert_called_once_with(task)
+            power_action_mock.assert_called_once_with(task, states.POWER_OFF)
+            clean_ramdisk_mock.assert_called_once_with(task.driver.boot, task)
             (task.driver.network.remove_cleaning_network
                 .assert_called_once_with(task))
             power_on_node_if_needed_mock.assert_called_once_with(task)
             restore_power_state_mock.assert_called_once_with(
                 task, states.POWER_OFF)
-
-    @mock.patch.object(flat_network.FlatNetwork, 'remove_provisioning_network',
-                       autospec=True)
-    @mock.patch.object(flat_network.FlatNetwork, 'configure_tenant_networks',
-                       autospec=True)
-    @mock.patch.object(utils, 'restore_power_state_if_needed', autospec=True)
-    @mock.patch.object(utils, 'power_on_node_if_needed', autospec=True)
-    @mock.patch.object(fake.FakePower, 'get_power_state',
-                       return_value=states.POWER_OFF)
-    @mock.patch.object(utils, 'node_power_action', autospec=True)
-    def test_reboot_and_finish_deploy_with_smartnic_port(
-            self, power_action_mock, get_pow_state_mock,
-            power_on_node_if_needed_mock, restore_power_state_mock,
-            configure_tenant_networks_mock, remove_provisioning_network_mock):
-        d_info = self.node.driver_info
-        d_info['deploy_forces_oob_reboot'] = True
-        self.node.driver_info = d_info
-        self.node.save()
-        self.config(group='ansible',
-                    post_deploy_get_power_state_retry_interval=0)
-        self.node.provision_state = states.DEPLOYING
-        self.node.save()
-        power_on_node_if_needed_mock.return_value = states.POWER_OFF
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            self.driver.reboot_and_finish_deploy(task)
-            expected_power_calls = [((task, states.POWER_OFF),),
-                                    ((task, states.POWER_ON),)]
-            self.assertEqual(
-                expected_power_calls, power_action_mock.call_args_list)
-            power_on_node_if_needed_mock.assert_called_once_with(task)
-            restore_power_state_mock.assert_called_once_with(
-                task, states.POWER_OFF)
-        get_pow_state_mock.assert_not_called()
