@@ -15,17 +15,18 @@
 
 """Test class for common methods used by iLO modules."""
 
+import builtins
 import hashlib
+import io
 import os
 import shutil
 import tempfile
+from unittest import mock
 
 from ironic_lib import utils as ironic_utils
-import mock
 from oslo_config import cfg
 from oslo_utils import importutils
-import six
-import six.moves.builtins as __builtin__
+from oslo_utils import uuidutils
 
 from ironic.common import boot_devices
 from ironic.common import exception
@@ -35,7 +36,6 @@ from ironic.conductor import task_manager
 from ironic.conductor import utils as manager_utils
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules.ilo import common as ilo_common
-from ironic.tests.unit.conductor import mgr_utils
 from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.db import utils as db_utils
 from ironic.tests.unit.objects import utils as obj_utils
@@ -45,21 +45,33 @@ INFO_DICT = db_utils.get_test_ilo_info()
 ilo_client = importutils.try_import('proliantutils.ilo.client')
 ilo_error = importutils.try_import('proliantutils.exception')
 
-if six.PY3:
-    import io
-    file = io.BytesIO
-
-
 CONF = cfg.CONF
 
 
-class IloValidateParametersTestCase(db_base.DbTestCase):
+class BaseIloTest(db_base.DbTestCase):
+
+    boot_interface = None
 
     def setUp(self):
-        super(IloValidateParametersTestCase, self).setUp()
+        super(BaseIloTest, self).setUp()
+        self.config(enabled_hardware_types=['ilo', 'fake-hardware'],
+                    enabled_boot_interfaces=['ilo-pxe', 'ilo-virtual-media',
+                                             'fake'],
+                    enabled_bios_interfaces=['ilo', 'no-bios'],
+                    enabled_power_interfaces=['ilo', 'fake'],
+                    enabled_management_interfaces=['ilo', 'fake'],
+                    enabled_inspect_interfaces=['ilo', 'fake', 'no-inspect'],
+                    enabled_console_interfaces=['ilo', 'fake', 'no-console'],
+                    enabled_vendor_interfaces=['ilo', 'fake', 'no-vendor'])
+        self.info = INFO_DICT.copy()
         self.node = obj_utils.create_test_node(
-            self.context, driver='fake_ilo',
-            driver_info=INFO_DICT)
+            self.context, uuid=uuidutils.generate_uuid(),
+            driver='ilo', boot_interface=self.boot_interface,
+            bios_interface='ilo',
+            driver_info=self.info)
+
+
+class IloValidateParametersTestCase(BaseIloTest):
 
     @mock.patch.object(os.path, 'isfile', return_value=True, autospec=True)
     def _test_parse_driver_info(self, isFile_mock):
@@ -71,15 +83,163 @@ class IloValidateParametersTestCase(db_base.DbTestCase):
         self.assertEqual(INFO_DICT['ilo_password'], info['ilo_password'])
         self.assertEqual(60, info['client_timeout'])
         self.assertEqual(443, info['client_port'])
-        self.assertEqual('/home/user/cafile.pem', info['ca_file'])
+        self.assertEqual('/home/user/cafile.pem', info['verify_ca'])
+        self.assertEqual('user', info['snmp_auth_user'])
+        self.assertEqual('1234', info['snmp_auth_prot_password'])
+        self.assertEqual('4321', info['snmp_auth_priv_password'])
+        self.assertEqual('SHA', info['snmp_auth_protocol'])
+        self.assertEqual('AES', info['snmp_auth_priv_protocol'])
 
-    def test_parse_driver_info_ca_file_in_driver_info(self):
-        self.node.driver_info['ca_file'] = '/home/user/cafile.pem'
+    @mock.patch.object(os.path, 'isfile', return_value=True, autospec=True)
+    def test_parse_driver_info_snmp_inspection_false(self, isFile_mock):
+        info = ilo_common.parse_driver_info(self.node)
+        self.assertEqual(INFO_DICT['ilo_address'], info['ilo_address'])
+        self.assertEqual(INFO_DICT['ilo_username'], info['ilo_username'])
+        self.assertEqual(INFO_DICT['ilo_password'], info['ilo_password'])
+        self.assertEqual(60, info['client_timeout'])
+        self.assertEqual(443, info['client_port'])
+
+    @mock.patch.object(os.path, 'isdir', autospec=True)
+    def test_parse_driver_info_path_verify_ca_dir(self, mock_isdir):
+        mock_isdir.return_value = True
+        fake_path = '/path/to/a/valid/CA'
+        self.node.driver_info['ilo_verify_ca'] = fake_path
+        info = ilo_common.parse_driver_info(self.node)
+        self.assertEqual(INFO_DICT['ilo_address'], info['ilo_address'])
+        self.assertEqual(INFO_DICT['ilo_username'], info['ilo_username'])
+        self.assertEqual(INFO_DICT['ilo_password'], info['ilo_password'])
+        self.assertEqual(60, info['client_timeout'])
+        self.assertEqual(443, info['client_port'])
+        self.assertEqual(fake_path, info['verify_ca'])
+        mock_isdir.assert_called_once_with(fake_path)
+
+    @mock.patch.object(os.path, 'isfile', autospec=True)
+    def test_parse_driver_info_path_verify_ca_file(self, mock_isfile):
+        mock_isfile.return_value = True
+        fake_path = '/path/to/a/valid/CA.pem'
+        self.node.driver_info['ilo_verify_ca'] = fake_path
+        info = ilo_common.parse_driver_info(self.node)
+        self.assertEqual(INFO_DICT['ilo_address'], info['ilo_address'])
+        self.assertEqual(INFO_DICT['ilo_username'], info['ilo_username'])
+        self.assertEqual(INFO_DICT['ilo_password'], info['ilo_password'])
+        self.assertEqual(60, info['client_timeout'])
+        self.assertEqual(443, info['client_port'])
+        self.assertEqual(fake_path, info['verify_ca'])
+        mock_isfile.assert_called_once_with(fake_path)
+
+    def test_parse_driver_info_verify_ca_default_value(self):
+        info = ilo_common.parse_driver_info(self.node)
+        self.assertEqual(INFO_DICT['ilo_address'], info['ilo_address'])
+        self.assertEqual(INFO_DICT['ilo_username'], info['ilo_username'])
+        self.assertEqual(INFO_DICT['ilo_password'], info['ilo_password'])
+        self.assertEqual(60, info['client_timeout'])
+        self.assertEqual(443, info['client_port'])
+        self.assertEqual(True, info['verify_ca'])
+
+    def test_parse_driver_info_verify_ca_string_false(self):
+        self.config(verify_ca="False", group='ilo')
+        info = ilo_common.parse_driver_info(self.node)
+        self.assertEqual(INFO_DICT['ilo_address'], info['ilo_address'])
+        self.assertEqual(INFO_DICT['ilo_username'], info['ilo_username'])
+        self.assertEqual(INFO_DICT['ilo_password'], info['ilo_password'])
+        self.assertEqual(60, info['client_timeout'])
+        self.assertEqual(443, info['client_port'])
+        self.assertEqual(False, info['verify_ca'])
+
+    def test_parse_driver_info_verify_ca_boolean_true(self):
+        self.config(verify_ca=True, group='ilo')
+        info = ilo_common.parse_driver_info(self.node)
+        self.assertEqual(INFO_DICT['ilo_address'], info['ilo_address'])
+        self.assertEqual(INFO_DICT['ilo_username'], info['ilo_username'])
+        self.assertEqual(INFO_DICT['ilo_password'], info['ilo_password'])
+        self.assertEqual(60, info['client_timeout'])
+        self.assertEqual(443, info['client_port'])
+        self.assertEqual(True, info['verify_ca'])
+
+    def test_parse_driver_info_verify_ca_boolean_false(self):
+        self.config(verify_ca=False, group='ilo')
+        info = ilo_common.parse_driver_info(self.node)
+        self.assertEqual(INFO_DICT['ilo_address'], info['ilo_address'])
+        self.assertEqual(INFO_DICT['ilo_username'], info['ilo_username'])
+        self.assertEqual(INFO_DICT['ilo_password'], info['ilo_password'])
+        self.assertEqual(60, info['client_timeout'])
+        self.assertEqual(443, info['client_port'])
+        self.assertEqual(False, info['verify_ca'])
+
+    def test_parse_driver_info_invalid_value_verify_ca(self):
+        # Integers are not supported
+        self.node.driver_info['ilo_verify_ca'] = 123456
+        self.assertRaisesRegex(exception.InvalidParameterValue,
+                               'Invalid value type',
+                               ilo_common._parse_optional_driver_info,
+                               self.node)
+
+    @mock.patch.object(os.path, 'isfile', return_value=True, autospec=True)
+    def test_parse_driver_info_snmp_true_no_auth_priv_protocols(self,
+                                                                isFile_mock):
+        d_info = {'ca_file': '/home/user/cafile.pem',
+                  'snmp_auth_prot_password': '1234',
+                  'snmp_auth_user': 'user',
+                  'snmp_auth_priv_password': '4321'}
+        self.node.driver_info.update(d_info)
+        info = ilo_common.parse_driver_info(self.node)
+        self.assertEqual(INFO_DICT['ilo_address'], info['ilo_address'])
+        self.assertEqual(INFO_DICT['ilo_username'], info['ilo_username'])
+        self.assertEqual(INFO_DICT['ilo_password'], info['ilo_password'])
+        self.assertEqual(60, info['client_timeout'])
+        self.assertEqual(443, info['client_port'])
+        self.assertEqual('/home/user/cafile.pem', info['verify_ca'])
+        self.assertEqual('user', info['snmp_auth_user'])
+        self.assertEqual('1234', info['snmp_auth_prot_password'])
+        self.assertEqual('4321', info['snmp_auth_priv_password'])
+
+    def test_parse_driver_info_ca_file_and_snmp_inspection_true(self):
+        d_info = {'ca_file': '/home/user/cafile.pem',
+                  'snmp_auth_prot_password': '1234',
+                  'snmp_auth_user': 'user',
+                  'snmp_auth_priv_password': '4321',
+                  'snmp_auth_protocol': 'SHA',
+                  'snmp_auth_priv_protocol': 'AES'}
+        self.node.driver_info.update(d_info)
         self._test_parse_driver_info()
 
-    def test_parse_driver_info_ca_file_in_conf_file(self):
-        self.config(ca_file='/home/user/cafile.pem', group='ilo')
-        self._test_parse_driver_info()
+    def test_parse_driver_info_snmp_true_invalid_auth_protocol(self):
+        d_info = {'ca_file': '/home/user/cafile.pem',
+                  'snmp_auth_prot_password': '1234',
+                  'snmp_auth_user': 'user',
+                  'snmp_auth_priv_password': '4321',
+                  'snmp_auth_protocol': 'abc',
+                  'snmp_auth_priv_protocol': 'AES'}
+        self.node.driver_info.update(d_info)
+        self.assertRaises(exception.InvalidParameterValue,
+                          ilo_common.parse_driver_info, self.node)
+
+    def test_parse_driver_info_snmp_true_invalid_priv_protocol(self):
+        d_info = {'ca_file': '/home/user/cafile.pem',
+                  'snmp_auth_prot_password': '1234',
+                  'snmp_auth_user': 'user',
+                  'snmp_auth_priv_password': '4321',
+                  'snmp_auth_protocol': 'SHA',
+                  'snmp_auth_priv_protocol': 'xyz'}
+        self.node.driver_info.update(d_info)
+        self.assertRaises(exception.InvalidParameterValue,
+                          ilo_common.parse_driver_info, self.node)
+
+    def test_parse_driver_info_snmp_true_integer_auth_protocol(self):
+        d_info = {'ca_file': '/home/user/cafile.pem',
+                  'snmp_auth_prot_password': '1234',
+                  'snmp_auth_user': 'user',
+                  'snmp_auth_priv_password': '4321',
+                  'snmp_auth_protocol': 12,
+                  'snmp_auth_priv_protocol': 'AES'}
+        self.node.driver_info.update(d_info)
+        self.assertRaises(exception.InvalidParameterValue,
+                          ilo_common.parse_driver_info, self.node)
+
+    def test_parse_driver_info_snmp_inspection_true_raises(self):
+        self.node.driver_info['snmp_auth_user'] = 'abc'
+        self.assertRaises(exception.MissingParameterValue,
+                          ilo_common.parse_driver_info, self.node)
 
     def test_parse_driver_info_missing_address(self):
         del self.node.driver_info['ilo_address']
@@ -125,30 +285,19 @@ class IloValidateParametersTestCase(db_base.DbTestCase):
     def test_parse_driver_info_missing_multiple_params(self):
         del self.node.driver_info['ilo_password']
         del self.node.driver_info['ilo_address']
-        try:
-            ilo_common.parse_driver_info(self.node)
-            self.fail("parse_driver_info did not throw exception.")
-        except exception.MissingParameterValue as e:
-            self.assertIn('ilo_password', str(e))
-            self.assertIn('ilo_address', str(e))
+        e = self.assertRaises(exception.MissingParameterValue,
+                              ilo_common.parse_driver_info, self.node)
+        self.assertIn('ilo_password', str(e))
+        self.assertIn('ilo_address', str(e))
 
     def test_parse_driver_info_invalid_multiple_params(self):
         self.node.driver_info['client_timeout'] = 'qwe'
-        try:
-            ilo_common.parse_driver_info(self.node)
-            self.fail("parse_driver_info did not throw exception.")
-        except exception.InvalidParameterValue as e:
-            self.assertIn('client_timeout', str(e))
+        e = self.assertRaises(exception.InvalidParameterValue,
+                              ilo_common.parse_driver_info, self.node)
+        self.assertIn('client_timeout', str(e))
 
 
-class IloCommonMethodsTestCase(db_base.DbTestCase):
-
-    def setUp(self):
-        super(IloCommonMethodsTestCase, self).setUp()
-        mgr_utils.mock_the_extension_manager(driver="fake_ilo")
-        self.info = db_utils.get_test_ilo_info()
-        self.node = obj_utils.create_test_node(
-            self.context, driver='fake_ilo', driver_info=self.info)
+class IloCommonMethodsTestCase(BaseIloTest):
 
     @mock.patch.object(os.path, 'isfile', return_value=True, autospec=True)
     @mock.patch.object(ilo_client, 'IloClient', spec_set=True,
@@ -156,7 +305,7 @@ class IloCommonMethodsTestCase(db_base.DbTestCase):
     def _test_get_ilo_object(self, ilo_client_mock, isFile_mock, ca_file=None):
         self.info['client_timeout'] = 600
         self.info['client_port'] = 4433
-        self.info['ca_file'] = ca_file
+        self.info['ilo_verify_ca'] = ca_file
         self.node.driver_info = self.info
         ilo_client_mock.return_value = 'ilo_object'
         returned_ilo_object = ilo_common.get_ilo_object(self.node)
@@ -166,40 +315,47 @@ class IloCommonMethodsTestCase(db_base.DbTestCase):
             self.info['ilo_password'],
             self.info['client_timeout'],
             self.info['client_port'],
-            cacert=self.info['ca_file'])
+            cacert=self.info['ilo_verify_ca'],
+            snmp_credentials=None)
+        self.assertEqual('ilo_object', returned_ilo_object)
+
+    @mock.patch.object(os.path, 'isfile', return_value=True, autospec=True)
+    @mock.patch.object(ilo_client, 'IloClient', spec_set=True,
+                       autospec=True)
+    def test_get_ilo_object_snmp(self, ilo_client_mock, isFile_mock):
+        info = {'auth_user': 'user',
+                'auth_prot_pp': '1234',
+                'auth_priv_pp': '4321',
+                'auth_protocol': 'SHA',
+                'priv_protocol': 'AES',
+                'snmp_inspection': True}
+        d_info = {'client_timeout': 600,
+                  'client_port': 4433,
+                  'verify_ca': 'True',
+                  'snmp_auth_user': 'user',
+                  'snmp_auth_prot_password': '1234',
+                  'snmp_auth_priv_password': '4321',
+                  'snmp_auth_protocol': 'SHA',
+                  'snmp_auth_priv_protocol': 'AES'}
+        self.info.update(d_info)
+        self.node.driver_info = self.info
+        ilo_client_mock.return_value = 'ilo_object'
+        returned_ilo_object = ilo_common.get_ilo_object(self.node)
+        ilo_client_mock.assert_called_with(
+            self.info['ilo_address'],
+            self.info['ilo_username'],
+            self.info['ilo_password'],
+            self.info['client_timeout'],
+            self.info['client_port'],
+            cacert=self.info['verify_ca'],
+            snmp_credentials=info)
         self.assertEqual('ilo_object', returned_ilo_object)
 
     def test_get_ilo_object_cafile(self):
         self._test_get_ilo_object(ca_file='/home/user/ilo.pem')
 
-    def test_get_ilo_object_no_cafile(self):
-        self._test_get_ilo_object()
-
-    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
-                       autospec=True)
-    def test_get_ilo_license(self, get_ilo_object_mock):
-        ilo_advanced_license = {'LICENSE_TYPE': 'iLO 3 Advanced'}
-        ilo_standard_license = {'LICENSE_TYPE': 'iLO 3'}
-
-        ilo_mock_object = get_ilo_object_mock.return_value
-        ilo_mock_object.get_all_licenses.return_value = ilo_advanced_license
-
-        license = ilo_common.get_ilo_license(self.node)
-        self.assertEqual(ilo_common.ADVANCED_LICENSE, license)
-
-        ilo_mock_object.get_all_licenses.return_value = ilo_standard_license
-        license = ilo_common.get_ilo_license(self.node)
-        self.assertEqual(ilo_common.STANDARD_LICENSE, license)
-
-    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
-                       autospec=True)
-    def test_get_ilo_license_fail(self, get_ilo_object_mock):
-        ilo_mock_object = get_ilo_object_mock.return_value
-        exc = ilo_error.IloError('error')
-        ilo_mock_object.get_all_licenses.side_effect = exc
-        self.assertRaises(exception.IloOperationError,
-                          ilo_common.get_ilo_license,
-                          self.node)
+    def test_get_ilo_object_cafile_boolean(self):
+        self._test_get_ilo_object(ca_file=True)
 
     def test_update_ipmi_properties(self):
         with task_manager.acquire(self.context, self.node.uuid,
@@ -229,8 +385,8 @@ class IloCommonMethodsTestCase(db_base.DbTestCase):
                        autospec=True)
     def test__prepare_floppy_image(self, tempfile_mock, fatimage_mock,
                                    swift_api_mock):
-        mock_image_file_handle = mock.MagicMock(spec=file)
-        mock_image_file_obj = mock.MagicMock(spec=file)
+        mock_image_file_handle = mock.MagicMock(spec=io.BytesIO)
+        mock_image_file_obj = mock.MagicMock(spec=io.BytesIO)
         mock_image_file_obj.name = 'image-tmp-file'
         mock_image_file_handle.__enter__.return_value = mock_image_file_obj
 
@@ -269,8 +425,8 @@ class IloCommonMethodsTestCase(db_base.DbTestCase):
     def test__prepare_floppy_image_use_webserver(self, tempfile_mock,
                                                  fatimage_mock,
                                                  copy_mock):
-        mock_image_file_handle = mock.MagicMock(spec=file)
-        mock_image_file_obj = mock.MagicMock(spec=file)
+        mock_image_file_handle = mock.MagicMock(spec=io.BytesIO)
+        mock_image_file_obj = mock.MagicMock(spec=io.BytesIO)
         mock_image_file_obj.name = 'image-tmp-file'
         mock_image_file_handle.__enter__.return_value = mock_image_file_obj
 
@@ -355,6 +511,29 @@ class IloCommonMethodsTestCase(db_base.DbTestCase):
         get_ilo_object_mock.assert_called_once_with(self.node)
         get_pending_boot_mode_mock.assert_called_once_with()
 
+    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
+                       autospec=True)
+    def test_get_current_boot_mode(self, get_ilo_object_mock):
+        ilo_object_mock = get_ilo_object_mock.return_value
+        get_current_boot_mode_mock = ilo_object_mock.get_current_boot_mode
+        get_current_boot_mode_mock.return_value = 'LEGACY'
+        ret = ilo_common.get_current_boot_mode(self.node)
+        self.assertEqual('bios', ret)
+        get_ilo_object_mock.assert_called_once_with(self.node)
+        get_current_boot_mode_mock.assert_called_once_with()
+
+    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
+                       autospec=True)
+    def test_get_current_boot_mode_fail(self, get_ilo_object_mock):
+        ilo_object_mock = get_ilo_object_mock.return_value
+        get_current_boot_mode_mock = ilo_object_mock.get_current_boot_mode
+        exc = ilo_error.IloError('error')
+        get_current_boot_mode_mock.side_effect = exc
+        self.assertRaises(exception.IloOperationError,
+                          ilo_common.get_current_boot_mode, self.node)
+        get_ilo_object_mock.assert_called_once_with(self.node)
+        get_current_boot_mode_mock.assert_called_once_with()
+
     @mock.patch.object(ilo_common, 'set_boot_mode', spec_set=True,
                        autospec=True)
     def test_update_boot_mode_instance_info_exists(self,
@@ -496,7 +675,6 @@ class IloCommonMethodsTestCase(db_base.DbTestCase):
         swift_obj_mock = swift_api_mock.return_value
         boot_iso = 'swift:object-name'
         swift_obj_mock.get_temp_url.return_value = 'image_url'
-        CONF.keystone_authtoken.auth_uri = 'http://authurl'
         CONF.ilo.swift_ilo_container = 'ilo_cont'
         CONF.ilo.swift_object_expiry_timeout = 1
         with task_manager.acquire(self.context, self.node.uuid,
@@ -560,7 +738,7 @@ class IloCommonMethodsTestCase(db_base.DbTestCase):
             self.assertTrue(log_mock.called)
             eject_mock.assert_called_once_with(task)
 
-    @mock.patch.object(ilo_common.LOG, 'warning', spec_set=True,
+    @mock.patch.object(ilo_common.LOG, 'info', spec_set=True,
                        autospec=True)
     @mock.patch.object(ilo_common, 'eject_vmedia_devices',
                        spec_set=True, autospec=True)
@@ -776,8 +954,7 @@ class IloCommonMethodsTestCase(db_base.DbTestCase):
          assert_called_once_with("/webserver/tmp_image_file"))
 
     @mock.patch.object(swift, 'SwiftAPI', spec_set=True, autospec=True)
-    @mock.patch.object(ilo_common, 'LOG')
-    def test_copy_image_to_swift(self, LOG_mock, swift_api_mock):
+    def test_copy_image_to_swift(self, swift_api_mock):
         # | GIVEN |
         self.config(swift_ilo_container='ilo_container', group='ilo')
         self.config(swift_object_expiry_timeout=1, group='ilo')
@@ -830,7 +1007,7 @@ class IloCommonMethodsTestCase(db_base.DbTestCase):
         swift_obj_mock.delete_object.assert_called_once_with(
             container, object_name)
 
-    @mock.patch.object(ilo_common, 'LOG')
+    @mock.patch.object(ilo_common, 'LOG', spec_set=True, autospec=True)
     @mock.patch.object(swift, 'SwiftAPI', spec_set=True, autospec=True)
     def test_remove_image_from_swift_suppresses_notfound_exc(
             self, swift_api_mock, LOG_mock):
@@ -841,15 +1018,15 @@ class IloCommonMethodsTestCase(db_base.DbTestCase):
         swift_obj_mock = swift_api_mock.return_value
         object_name = 'object_name'
         raised_exc = exception.SwiftObjectNotFoundError(
-            operation='delete_object', object=object_name, container=container)
+            operation='delete_object', obj=object_name, container=container)
         swift_obj_mock.delete_object.side_effect = raised_exc
         # | WHEN |
         ilo_common.remove_image_from_swift(object_name)
         # | THEN |
-        LOG_mock.warning.assert_called_once_with(
+        LOG_mock.info.assert_called_once_with(
             mock.ANY, {'associated_with_msg': "", 'err': raised_exc})
 
-    @mock.patch.object(ilo_common, 'LOG')
+    @mock.patch.object(ilo_common, 'LOG', spec_set=True, autospec=True)
     @mock.patch.object(swift, 'SwiftAPI', spec_set=True, autospec=True)
     def test_remove_image_from_swift_suppresses_operror_exc(
             self, swift_api_mock, LOG_mock):
@@ -902,6 +1079,21 @@ class IloCommonMethodsTestCase(db_base.DbTestCase):
             func_set_boot_device.assert_called_once_with(task,
                                                          boot_devices.CDROM)
 
+    @mock.patch.object(manager_utils, 'node_set_boot_device', spec_set=True,
+                       autospec=True)
+    @mock.patch.object(ilo_common, 'setup_vmedia_for_boot', spec_set=True,
+                       autospec=True)
+    def test_setup_vmedia_without_params(self,
+                                         func_setup_vmedia_for_boot,
+                                         func_set_boot_device):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            iso = '733d1c44-a2ea-414b-aca7-69decf20d810'
+            ilo_common.setup_vmedia(task, iso)
+            func_setup_vmedia_for_boot.assert_called_once_with(task, iso, None)
+            func_set_boot_device.assert_called_once_with(task,
+                                                         boot_devices.CDROM)
+
     @mock.patch.object(deploy_utils, 'is_secure_boot_requested', spec_set=True,
                        autospec=True)
     @mock.patch.object(ilo_common, 'set_secure_boot_mode', spec_set=True,
@@ -953,11 +1145,11 @@ class IloCommonMethodsTestCase(db_base.DbTestCase):
         # | THEN |
         unlink_mock.assert_called_once_with('/any_path1/any_file')
 
-    @mock.patch.object(__builtin__, 'open', autospec=True)
+    @mock.patch.object(builtins, 'open', autospec=True)
     def test_verify_image_checksum(self, open_mock):
         # | GIVEN |
         data = b'Yankee Doodle went to town riding on a pony;'
-        file_like_object = six.BytesIO(data)
+        file_like_object = io.BytesIO(data)
         open_mock().__enter__.return_value = file_like_object
         actual_hash = hashlib.md5(data).hexdigest()
         # | WHEN |
@@ -973,12 +1165,12 @@ class IloCommonMethodsTestCase(db_base.DbTestCase):
                           ilo_common.verify_image_checksum,
                           invalid_file_path, 'hash_xxx')
 
-    @mock.patch.object(__builtin__, 'open', autospec=True)
+    @mock.patch.object(builtins, 'open', autospec=True)
     def test_verify_image_checksum_throws_for_failed_validation(self,
                                                                 open_mock):
         # | GIVEN |
         data = b'Yankee Doodle went to town riding on a pony;'
-        file_like_object = six.BytesIO(data)
+        file_like_object = io.BytesIO(data)
         open_mock().__enter__.return_value = file_like_object
         invalid_hash = 'invalid_hash_value'
         # | WHEN | & | THEN |
@@ -986,3 +1178,121 @@ class IloCommonMethodsTestCase(db_base.DbTestCase):
                           ilo_common.verify_image_checksum,
                           file_like_object,
                           invalid_hash)
+
+    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
+                       autospec=True)
+    def test_get_server_post_state(self,
+                                   get_ilo_object_mock):
+        ilo_object_mock = get_ilo_object_mock.return_value
+        post_state = 'FinishedPost'
+        ilo_object_mock.get_host_post_state.return_value = post_state
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            ret = ilo_common.get_server_post_state(task.node)
+            ilo_object_mock.get_host_post_state.assert_called_once_with()
+            self.assertEqual(post_state, ret)
+
+    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
+                       autospec=True)
+    def test_get_server_post_state_fail(self,
+                                        get_ilo_object_mock):
+        ilo_mock_object = get_ilo_object_mock.return_value
+        exc = ilo_error.IloError('error')
+        ilo_mock_object.get_host_post_state.side_effect = exc
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            self.assertRaises(exception.IloOperationError,
+                              ilo_common.get_server_post_state, task.node)
+        ilo_mock_object.get_host_post_state.assert_called_once_with()
+
+    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
+                       autospec=True)
+    def test_get_server_post_state_not_supported(self,
+                                                 ilo_object_mock):
+        ilo_mock_object = ilo_object_mock.return_value
+        exc = ilo_error.IloCommandNotSupportedError('error')
+        ilo_mock_object.get_host_post_state.side_effect = exc
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            self.assertRaises(exception.IloOperationNotSupported,
+                              ilo_common.get_server_post_state,
+                              task.node)
+        ilo_mock_object.get_host_post_state.assert_called_once_with()
+
+    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
+                       autospec=True)
+    def test_setup_uefi_https_scheme_http(self, get_ilo_object_mock):
+        ilo_mock_object = get_ilo_object_mock.return_value
+        iso = "http://1.1.1.1/image.iso"
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            self.assertRaises(exception.IloOperationNotSupported,
+                              ilo_common.setup_uefi_https,
+                              task, iso, True)
+
+        ilo_mock_object.set_http_boot_url.assert_not_called()
+        ilo_mock_object.set_one_time_boot.assert_not_called()
+        ilo_mock_object.update_persistent_boot.assert_not_called()
+
+    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
+                       autospec=True)
+    def _test_setup_uefi_https(self, get_ilo_object_mock, persistent):
+
+        ilo_mock_object = get_ilo_object_mock.return_value
+
+        iso = "https://1.1.1.1/image.iso"
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            ilo_common.setup_uefi_https(task, iso, persistent=persistent)
+
+        ilo_mock_object.set_http_boot_url.assert_called_once_with(iso)
+        if persistent:
+            ilo_mock_object.set_one_time_boot.assert_not_called()
+            ilo_mock_object.update_persistent_boot.assert_called_once_with(
+                ['UEFIHTTP'])
+        else:
+            ilo_mock_object.set_one_time_boot.assert_called_once_with(
+                'UEFIHTTP')
+            ilo_mock_object.update_persistent_boot.assert_not_called()
+
+    def test_setup_uefi_https_persistent_true(self):
+        self._test_setup_uefi_https(persistent=True)
+
+    def test_setup_uefi_https_persistent_false(self):
+        self._test_setup_uefi_https(persistent=False)
+
+    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
+                       autospec=True)
+    def test_setup_uefi_https_raises_not_supported(self, get_ilo_object_mock):
+        ilo_mock_object = get_ilo_object_mock.return_value
+
+        exc = ilo_error.IloCommandNotSupportedInBiosError('error')
+        ilo_mock_object.set_http_boot_url.side_effect = exc
+
+        iso = "https://1.1.1.1/image.iso"
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            self.assertRaises(exception.IloOperationNotSupported,
+                              ilo_common.setup_uefi_https,
+                              task, iso, True)
+
+    @mock.patch.object(ilo_common, 'get_ilo_object', spec_set=True,
+                       autospec=True)
+    def test_setup_uefi_https_raises_ilo_error(self, get_ilo_object_mock):
+        ilo_mock_object = get_ilo_object_mock.return_value
+
+        exc = ilo_error.IloError('error')
+        ilo_mock_object.set_http_boot_url.side_effect = exc
+
+        iso = "https://1.1.1.1/image.iso"
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            self.assertRaises(exception.IloOperationError,
+                              ilo_common.setup_uefi_https,
+                              task, iso, True)
