@@ -16,7 +16,7 @@
 Test class for iRMC Power Driver
 """
 
-import mock
+from unittest import mock
 
 from ironic.common import exception
 from ironic.common import states
@@ -50,7 +50,8 @@ class IRMCPowerInternalMethodsTestCase(test_common.BaseIRMCTest):
         self.assertFalse(irmc_power._is_expected_power_state(
             target_state, boot_status_value))
 
-    @mock.patch('eventlet.greenthread.sleep', lambda n: None)
+    @mock.patch('oslo_utils.eventletutils.EventletEvent.wait',
+                lambda *args, **kwargs: None)
     @mock.patch('ironic.drivers.modules.irmc.power.snmp.SNMPClient',
                 spec_set=True, autospec=True)
     def test__wait_power_state_soft_power_off(self, snmpclient_mock):
@@ -69,7 +70,8 @@ class IRMCPowerInternalMethodsTestCase(test_common.BaseIRMCTest):
             self.assertEqual(states.POWER_OFF, task.node.power_state)
             self.assertEqual(states.NOSTATE, task.node.target_power_state)
 
-    @mock.patch('eventlet.greenthread.sleep', lambda n: None)
+    @mock.patch('oslo_utils.eventletutils.EventletEvent.wait',
+                lambda *args, **kwargs: None)
     @mock.patch('ironic.drivers.modules.irmc.power.snmp.SNMPClient',
                 spec_set=True, autospec=True)
     def test__wait_power_state_soft_reboot(self, snmpclient_mock):
@@ -88,7 +90,8 @@ class IRMCPowerInternalMethodsTestCase(test_common.BaseIRMCTest):
             self.assertEqual(states.POWER_ON, task.node.power_state)
             self.assertEqual(states.NOSTATE, task.node.target_power_state)
 
-    @mock.patch('eventlet.greenthread.sleep', lambda n: None)
+    @mock.patch('oslo_utils.eventletutils.EventletEvent.wait',
+                lambda *args, **kwargs: None)
     @mock.patch('ironic.drivers.modules.irmc.power.snmp.SNMPClient',
                 spec_set=True, autospec=True)
     def test__wait_power_state_timeout(self, snmpclient_mock):
@@ -115,7 +118,8 @@ class IRMCPowerInternalMethodsTestCase(test_common.BaseIRMCTest):
                        autospec=True)
     @mock.patch.object(irmc_common, 'get_irmc_client', spec_set=True,
                        autospec=True)
-    @mock.patch.object(irmc_boot, 'attach_boot_iso_if_needed')
+    @mock.patch.object(irmc_boot, 'attach_boot_iso_if_needed',
+                       autospec=True)
     def test__set_power_state_power_on_ok(
             self,
             attach_boot_iso_if_needed_mock,
@@ -149,7 +153,8 @@ class IRMCPowerInternalMethodsTestCase(test_common.BaseIRMCTest):
                        autospec=True)
     @mock.patch.object(irmc_common, 'get_irmc_client', spec_set=True,
                        autospec=True)
-    @mock.patch.object(irmc_boot, 'attach_boot_iso_if_needed')
+    @mock.patch.object(irmc_boot, 'attach_boot_iso_if_needed',
+                       autospec=True)
     def test__set_power_state_reboot_ok(
             self,
             attach_boot_iso_if_needed_mock,
@@ -168,7 +173,8 @@ class IRMCPowerInternalMethodsTestCase(test_common.BaseIRMCTest):
                        autospec=True)
     @mock.patch.object(irmc_common, 'get_irmc_client', spec_set=True,
                        autospec=True)
-    @mock.patch.object(irmc_boot, 'attach_boot_iso_if_needed')
+    @mock.patch.object(irmc_boot, 'attach_boot_iso_if_needed',
+                       autospec=True)
     def test__set_power_state_soft_reboot_ok(
             self,
             attach_boot_iso_if_needed_mock,
@@ -189,7 +195,8 @@ class IRMCPowerInternalMethodsTestCase(test_common.BaseIRMCTest):
                        autospec=True)
     @mock.patch.object(irmc_common, 'get_irmc_client', spec_set=True,
                        autospec=True)
-    @mock.patch.object(irmc_boot, 'attach_boot_iso_if_needed')
+    @mock.patch.object(irmc_boot, 'attach_boot_iso_if_needed',
+                       autospec=True)
     def test__set_power_state_soft_power_off_ok(self,
                                                 attach_boot_iso_if_needed_mock,
                                                 get_irmc_client_mock,
@@ -206,7 +213,8 @@ class IRMCPowerInternalMethodsTestCase(test_common.BaseIRMCTest):
 
     @mock.patch.object(irmc_power, '_wait_power_state', spec_set=True,
                        autospec=True)
-    @mock.patch.object(irmc_boot, 'attach_boot_iso_if_needed')
+    @mock.patch.object(irmc_boot, 'attach_boot_iso_if_needed',
+                       autospec=True)
     def test__set_power_state_invalid_target_state(
             self,
             attach_boot_iso_if_needed_mock,
@@ -224,7 +232,8 @@ class IRMCPowerInternalMethodsTestCase(test_common.BaseIRMCTest):
                        autospec=True)
     @mock.patch.object(irmc_common, 'get_irmc_client', spec_set=True,
                        autospec=True)
-    @mock.patch.object(irmc_boot, 'attach_boot_iso_if_needed')
+    @mock.patch.object(irmc_boot, 'attach_boot_iso_if_needed',
+                       autospec=True)
     def test__set_power_state_scci_exception(self,
                                              attach_boot_iso_if_needed_mock,
                                              get_irmc_client_mock,
@@ -247,7 +256,8 @@ class IRMCPowerInternalMethodsTestCase(test_common.BaseIRMCTest):
                        autospec=True)
     @mock.patch.object(irmc_common, 'get_irmc_client', spec_set=True,
                        autospec=True)
-    @mock.patch.object(irmc_boot, 'attach_boot_iso_if_needed')
+    @mock.patch.object(irmc_boot, 'attach_boot_iso_if_needed',
+                       autospec=True)
     def test__set_power_state_snmp_exception(self,
                                              attach_boot_iso_if_needed_mock,
                                              get_irmc_client_mock,
