@@ -10,7 +10,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+import json
+import os
+from unittest import mock
+
 from oslo_config import cfg
 from oslo_utils import uuidutils
 
@@ -93,7 +96,7 @@ class TestCommonFunctions(db_base.DbTestCase):
             free_port_like_objs = (
                 common._get_free_portgroups_and_ports(task, self.vif_id,
                                                       {'anyphysnet'}))
-        self.assertItemsEqual(
+        self.assertCountEqual(
             [pg1.uuid, self.port.uuid] + [p.uuid for p in pg2_ports[:2]],
             [p.uuid for p in free_port_like_objs])
 
@@ -106,7 +109,7 @@ class TestCommonFunctions(db_base.DbTestCase):
             free_port_like_objs = (
                 common._get_free_portgroups_and_ports(task, self.vif_id,
                                                       set()))
-        self.assertItemsEqual(
+        self.assertCountEqual(
             [pg1.uuid, self.port.uuid] + [p.uuid for p in pg2_ports[:2]],
             [p.uuid for p in free_port_like_objs])
 
@@ -119,7 +122,7 @@ class TestCommonFunctions(db_base.DbTestCase):
             free_port_like_objs = (
                 common._get_free_portgroups_and_ports(task, self.vif_id,
                                                       {'notaphysnet'}))
-        self.assertItemsEqual(
+        self.assertCountEqual(
             [self.port.uuid],
             [p.uuid for p in free_port_like_objs])
 
@@ -132,7 +135,7 @@ class TestCommonFunctions(db_base.DbTestCase):
             free_port_like_objs = (
                 common._get_free_portgroups_and_ports(task, self.vif_id,
                                                       {'physnet1'}))
-        self.assertItemsEqual(
+        self.assertCountEqual(
             [pg1.uuid, self.port.uuid],
             [p.uuid for p in free_port_like_objs])
 
@@ -145,7 +148,7 @@ class TestCommonFunctions(db_base.DbTestCase):
             free_port_like_objs = (
                 common._get_free_portgroups_and_ports(task, self.vif_id,
                                                       {'physnet2'}))
-        self.assertItemsEqual(
+        self.assertCountEqual(
             [self.port.uuid] + [p.uuid for p in pg2_ports[:2]],
             [p.uuid for p in free_port_like_objs])
 
@@ -158,7 +161,7 @@ class TestCommonFunctions(db_base.DbTestCase):
             free_port_like_objs = (
                 common._get_free_portgroups_and_ports(task, self.vif_id,
                                                       {'physnet3'}))
-        self.assertItemsEqual(
+        self.assertCountEqual(
             [self.port.uuid], [p.uuid for p in free_port_like_objs])
 
     def test__get_free_portgroups_and_ports_all_physnets(self):
@@ -171,7 +174,7 @@ class TestCommonFunctions(db_base.DbTestCase):
             free_port_like_objs = (
                 common._get_free_portgroups_and_ports(task, self.vif_id,
                                                       physnets))
-        self.assertItemsEqual(
+        self.assertCountEqual(
             [pg1.uuid, self.port.uuid] + [p.uuid for p in pg2_ports[:2]],
             [p.uuid for p in free_port_like_objs])
 
@@ -182,7 +185,7 @@ class TestCommonFunctions(db_base.DbTestCase):
             free_port_like_objs = (
                 common._get_free_portgroups_and_ports(task, self.vif_id,
                                                       {'anyphysnet'}))
-        self.assertItemsEqual([], free_port_like_objs)
+        self.assertCountEqual([], free_port_like_objs)
 
     @mock.patch.object(neutron_common, 'validate_port_info', autospec=True)
     def test__get_free_portgroups_and_ports_neutron(self, vpi_mock):
@@ -191,7 +194,7 @@ class TestCommonFunctions(db_base.DbTestCase):
             free_port_like_objs = (
                 common._get_free_portgroups_and_ports(task, self.vif_id,
                                                       {'anyphysnet'}))
-        self.assertItemsEqual(
+        self.assertCountEqual(
             [self.port.uuid], [p.uuid for p in free_port_like_objs])
 
     @mock.patch.object(neutron_common, 'validate_port_info', autospec=True)
@@ -203,9 +206,62 @@ class TestCommonFunctions(db_base.DbTestCase):
             free_port_like_objs = (
                 common._get_free_portgroups_and_ports(task, self.vif_id,
                                                       {'anyphysnet'}))
-        self.assertItemsEqual(
+        self.assertCountEqual(
             [self.port.uuid], [p.uuid for p in free_port_like_objs])
 
+    def test__get_free_portgroups_and_ports_port_uuid(self):
+        self.node.network_interface = 'flat'
+        self.node.save()
+        pg1, pg1_ports, pg2, pg2_ports, pg3, pg3_ports = self._objects_setup(
+            set_physnets=False)
+        with task_manager.acquire(self.context, self.node.id) as task:
+            free_port_like_objs = (
+                common._get_free_portgroups_and_ports(
+                    task, self.vif_id, {}, {'port_uuid': self.port.uuid}))
+        self.assertCountEqual(
+            [self.port.uuid],
+            [p.uuid for p in free_port_like_objs])
+
+    def test__get_free_portgroups_and_ports_portgroup_uuid(self):
+        self.node.network_interface = 'flat'
+        self.node.save()
+        pg1, pg1_ports, pg2, pg2_ports, pg3, pg3_ports = self._objects_setup(
+            set_physnets=False)
+        with task_manager.acquire(self.context, self.node.id) as task:
+            free_port_like_objs = (
+                common._get_free_portgroups_and_ports(
+                    task, self.vif_id, {}, {'portgroup_uuid': pg1.uuid}))
+        self.assertCountEqual(
+            [pg1.uuid],
+            [p.uuid for p in free_port_like_objs])
+
+    def test__get_free_portgroups_and_ports_portgroup_uuid_attached_vifs(self):
+        self.node.network_interface = 'flat'
+        self.node.save()
+        pg1, pg1_ports, pg2, pg2_ports, pg3, pg3_ports = self._objects_setup(
+            set_physnets=False)
+        with task_manager.acquire(self.context, self.node.id) as task:
+            free_port_like_objs = (
+                common._get_free_portgroups_and_ports(
+                    task, self.vif_id, {}, {'portgroup_uuid': pg2.uuid}))
+        self.assertCountEqual(
+            [],
+            [p.uuid for p in free_port_like_objs])
+
+    def test__get_free_portgroups_and_ports_no_matching_uuid(self):
+        self.node.network_interface = 'flat'
+        self.node.save()
+        pg1, pg1_ports, pg2, pg2_ports, pg3, pg3_ports = self._objects_setup(
+            set_physnets=False)
+        with task_manager.acquire(self.context, self.node.id) as task:
+            free_port_like_objs = (
+                common._get_free_portgroups_and_ports(
+                    task, self.vif_id, {},
+                    {'port_uuid': uuidutils.generate_uuid()}))
+        self.assertCountEqual(
+            [],
+            [p.uuid for p in free_port_like_objs])
+
     @mock.patch.object(neutron_common, 'validate_port_info', autospec=True,
                        return_value=True)
     def test_get_free_port_like_object_ports(self, vpi_mock):
@@ -400,22 +456,26 @@ class TestCommonFunctions(db_base.DbTestCase):
                               common.get_free_port_like_object,
                               task, self.vif_id, {'physnet2'})
 
+    @mock.patch.object(neutron_common, 'update_neutron_port', autospec=True)
     @mock.patch.object(neutron_common, 'get_client', autospec=True)
-    def test_plug_port_to_tenant_network_client(self, mock_gc):
+    def test_plug_port_to_tenant_network_client(self, mock_gc, mock_update):
         self.port.internal_info = {common.TENANT_VIF_KEY: self.vif_id}
         self.port.save()
         with task_manager.acquire(self.context, self.node.id) as task:
             common.plug_port_to_tenant_network(task, self.port,
                                                client=mock.MagicMock())
         self.assertFalse(mock_gc.called)
+        self.assertTrue(mock_update.called)
 
+    @mock.patch.object(neutron_common, 'update_neutron_port', autospec=True)
     @mock.patch.object(neutron_common, 'get_client', autospec=True)
-    def test_plug_port_to_tenant_network_no_client(self, mock_gc):
+    def test_plug_port_to_tenant_network_no_client(self, mock_gc, mock_update):
         self.port.internal_info = {common.TENANT_VIF_KEY: self.vif_id}
         self.port.save()
         with task_manager.acquire(self.context, self.node.id) as task:
             common.plug_port_to_tenant_network(task, self.port)
         self.assertTrue(mock_gc.called)
+        self.assertTrue(mock_update.called)
 
     @mock.patch.object(neutron_common, 'get_client', autospec=True)
     def test_plug_port_to_tenant_network_no_tenant_vif(self, mock_gc):
@@ -430,6 +490,28 @@ class TestCommonFunctions(db_base.DbTestCase):
                 common.plug_port_to_tenant_network,
                 task, self.port)
 
+    @mock.patch.object(neutron_common, 'wait_for_host_agent', autospec=True)
+    @mock.patch.object(neutron_common, 'wait_for_port_status', autospec=True)
+    @mock.patch.object(neutron_common, 'update_neutron_port', autospec=True)
+    @mock.patch.object(neutron_common, 'get_client', autospec=True)
+    def test_plug_port_to_tenant_network_smartnic_port(
+            self, mock_gc, mock_update, wait_port_mock, wait_agent_mock):
+        nclient = mock.MagicMock()
+        mock_gc.return_value = nclient
+        local_link_connection = self.port.local_link_connection
+        local_link_connection['hostname'] = 'hostname'
+        self.port.local_link_connection = local_link_connection
+        self.port.internal_info = {common.TENANT_VIF_KEY: self.vif_id}
+        self.port.is_smartnic = True
+        self.port.save()
+        with task_manager.acquire(self.context, self.node.id) as task:
+            common.plug_port_to_tenant_network(task, self.port)
+            wait_agent_mock.assert_called_once_with(
+                nclient, 'hostname')
+            wait_port_mock.assert_called_once_with(
+                nclient, self.vif_id, 'ACTIVE')
+            self.assertTrue(mock_update.called)
+
 
 class TestVifPortIDMixin(db_base.DbTestCase):
 
@@ -443,6 +525,10 @@ class TestVifPortIDMixin(db_base.DbTestCase):
             address='52:54:00:cf:2d:32',
             extra={'vif_port_id': uuidutils.generate_uuid(),
                    'client-id': 'fake1'})
+        network_data_file = os.path.join(
+            os.path.dirname(__file__), 'json_samples', 'network_data.json')
+        with open(network_data_file, 'rb') as fl:
+            self.network_data = json.load(fl)
 
     def test__save_vif_to_port_like_obj_port(self):
         self.port.extra = {}
@@ -573,7 +659,7 @@ class TestVifPortIDMixin(db_base.DbTestCase):
             address='52:54:00:cf:2d:01', uuid=uuidutils.generate_uuid())
         with task_manager.acquire(self.context, self.node.id) as task:
             vifs = self.interface.vif_list(task)
-            self.assertItemsEqual([{'id': pg_vif_id}, {'id': vif_id}], vifs)
+            self.assertCountEqual([{'id': pg_vif_id}, {'id': vif_id}], vifs)
 
     def test_vif_list_internal(self):
         vif_id = uuidutils.generate_uuid()
@@ -589,7 +675,7 @@ class TestVifPortIDMixin(db_base.DbTestCase):
             address='52:54:00:cf:2d:01', uuid=uuidutils.generate_uuid())
         with task_manager.acquire(self.context, self.node.id) as task:
             vifs = self.interface.vif_list(task)
-            self.assertItemsEqual([{'id': pg_vif_id}, {'id': vif_id}], vifs)
+            self.assertCountEqual([{'id': pg_vif_id}, {'id': vif_id}], vifs)
 
     def test_vif_list_extra_and_internal_priority(self):
         vif_id = uuidutils.generate_uuid()
@@ -669,7 +755,8 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
         self.neutron_port = {'id': '132f871f-eaec-4fed-9475-0d54465e0f00',
                              'mac_address': '52:54:00:cf:2d:32'}
 
-    @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj')
+    @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj',
+                       autospec=True)
     @mock.patch.object(common, 'get_free_port_like_object', autospec=True)
     @mock.patch.object(neutron_common, 'get_client', autospec=True)
     @mock.patch.object(neutron_common, 'update_port_address', autospec=True)
@@ -685,10 +772,12 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             mock_upa.assert_called_once_with(
                 "fake_vif_id", self.port.address, context=task.context)
         self.assertFalse(mock_gpbpi.called)
-        mock_gfp.assert_called_once_with(task, 'fake_vif_id', set())
+        mock_gfp.assert_called_once_with(task, 'fake_vif_id', set(),
+                                         {'id': 'fake_vif_id'})
         mock_save.assert_called_once_with(self.port, "fake_vif_id")
 
-    @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj')
+    @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj',
+                       autospec=True)
     @mock.patch.object(common, 'get_free_port_like_object', autospec=True)
     @mock.patch.object(neutron_common, 'get_client', autospec=True)
     @mock.patch.object(neutron_common, 'update_port_address', autospec=True)
@@ -701,10 +790,12 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
         with task_manager.acquire(self.context, self.node.id) as task:
             self.assertRaises(exception.NoFreePhysicalPorts,
                               self.interface.vif_attach, task, vif)
-        mock_gfp.assert_called_once_with(task, 'fake_vif_id', set())
+        mock_gfp.assert_called_once_with(task, 'fake_vif_id', set(),
+                                         {'id': 'fake_vif_id'})
         self.assertFalse(mock_save.called)
 
-    @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj')
+    @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj',
+                       autospec=True)
     @mock.patch.object(common, 'get_free_port_like_object', autospec=True)
     @mock.patch.object(neutron_common, 'get_client', autospec=True)
     @mock.patch.object(neutron_common, 'update_port_address', autospec=True)
@@ -724,10 +815,12 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
                 "fake_vif_id", self.port.address, context=task.context)
         mock_gpbpi.assert_called_once_with(mock_client.return_value,
                                            'fake_vif_id')
-        mock_gfp.assert_called_once_with(task, 'fake_vif_id', {'physnet1'})
+        mock_gfp.assert_called_once_with(task, 'fake_vif_id', {'physnet1'},
+                                         {'id': 'fake_vif_id'})
         mock_save.assert_called_once_with(self.port, "fake_vif_id")
 
-    @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj')
+    @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj',
+                       autospec=True)
     @mock.patch.object(common, 'plug_port_to_tenant_network', autospec=True)
     @mock.patch.object(common, 'get_free_port_like_object', autospec=True)
     @mock.patch.object(neutron_common, 'get_client', autospec=True)
@@ -746,11 +839,13 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             mock_upa.assert_called_once_with(
                 "fake_vif_id", self.port.address, context=task.context)
         self.assertFalse(mock_gpbpi.called)
-        mock_gfp.assert_called_once_with(task, 'fake_vif_id', set())
+        mock_gfp.assert_called_once_with(task, 'fake_vif_id', set(),
+                                         {'id': 'fake_vif_id'})
         mock_save.assert_called_once_with(self.port, "fake_vif_id")
         mock_plug.assert_called_once_with(task, self.port, mock.ANY)
 
-    @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj')
+    @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj',
+                       autospec=True)
     @mock.patch.object(common, 'plug_port_to_tenant_network', autospec=True)
     @mock.patch.object(common, 'get_free_port_like_object', autospec=True)
     @mock.patch.object(neutron_common, 'get_client', autospec=True)
@@ -772,14 +867,16 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             mock_upa.assert_called_once_with(
                 "fake_vif_id", self.port.address, context=task.context)
         self.assertFalse(mock_gpbpi.called)
-        mock_gfp.assert_called_once_with(task, 'fake_vif_id', set())
+        mock_gfp.assert_called_once_with(task, 'fake_vif_id', set(),
+                                         {'id': 'fake_vif_id'})
         mock_save.assert_called_once_with(self.port, "fake_vif_id")
         mock_plug.assert_called_once_with(task, self.port, mock.ANY)
 
-    @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj')
+    @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj',
+                       autospec=True)
     @mock.patch.object(common, 'get_free_port_like_object', autospec=True)
     @mock.patch.object(neutron_common, 'get_client', autospec=True)
-    @mock.patch.object(neutron_common, 'update_port_address')
+    @mock.patch.object(neutron_common, 'update_port_address', autospec=True)
     @mock.patch.object(neutron_common, 'get_physnets_by_port_uuid',
                        autospec=True)
     def test_vif_attach_portgroup_no_address(self, mock_gpbpi, mock_upa,
@@ -792,14 +889,16 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             self.interface.vif_attach(task, vif)
             mock_client.assert_called_once_with(context=task.context)
         self.assertFalse(mock_gpbpi.called)
-        mock_gfp.assert_called_once_with(task, 'fake_vif_id', set())
+        mock_gfp.assert_called_once_with(task, 'fake_vif_id', set(),
+                                         {'id': 'fake_vif_id'})
         self.assertFalse(mock_client.return_value.show_port.called)
         self.assertFalse(mock_upa.called)
         mock_save.assert_called_once_with(pg, "fake_vif_id")
 
-    @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj')
+    @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj',
+                       autospec=True)
     @mock.patch.object(neutron_common, 'get_client', autospec=True)
-    @mock.patch.object(neutron_common, 'update_port_address')
+    @mock.patch.object(neutron_common, 'update_port_address', autospec=True)
     @mock.patch.object(neutron_common, 'get_physnets_by_port_uuid',
                        autospec=True)
     def test_vif_attach_update_port_exception(self, mock_gpbpi, mock_upa,
@@ -820,10 +919,11 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
                                            'fake_vif_id')
         self.assertFalse(mock_save.called)
 
-    @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj')
+    @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj',
+                       autospec=True)
     @mock.patch.object(common, 'get_free_port_like_object', autospec=True)
-    @mock.patch.object(neutron_common, 'get_client')
-    @mock.patch.object(neutron_common, 'update_port_address')
+    @mock.patch.object(neutron_common, 'get_client', autospec=True)
+    @mock.patch.object(neutron_common, 'update_port_address', autospec=True)
     @mock.patch.object(neutron_common, 'get_physnets_by_port_uuid',
                        autospec=True)
     def test_vif_attach_portgroup_physnet_inconsistent(self, mock_gpbpi,
@@ -845,10 +945,11 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
         self.assertFalse(mock_upa.called)
         self.assertFalse(mock_save.called)
 
-    @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj')
+    @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj',
+                       autospec=True)
     @mock.patch.object(common, 'get_free_port_like_object', autospec=True)
-    @mock.patch.object(neutron_common, 'get_client')
-    @mock.patch.object(neutron_common, 'update_port_address')
+    @mock.patch.object(neutron_common, 'get_client', autospec=True)
+    @mock.patch.object(neutron_common, 'update_port_address', autospec=True)
     @mock.patch.object(neutron_common, 'get_physnets_by_port_uuid',
                        autospec=True)
     def test_vif_attach_multiple_segment_mappings(self, mock_gpbpi, mock_upa,
@@ -872,33 +973,39 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
         self.assertFalse(mock_upa.called)
         self.assertFalse(mock_save.called)
 
-    @mock.patch.object(common.VIFPortIDMixin, '_clear_vif_from_port_like_obj')
+    @mock.patch.object(common.VIFPortIDMixin, '_clear_vif_from_port_like_obj',
+                       autospec=True)
     @mock.patch.object(neutron_common, 'unbind_neutron_port', autospec=True)
-    @mock.patch.object(common.VIFPortIDMixin, '_get_port_like_obj_by_vif_id')
+    @mock.patch.object(common.VIFPortIDMixin, '_get_port_like_obj_by_vif_id',
+                       autospec=True)
     def test_vif_detach(self, mock_get, mock_unp, mock_clear):
         mock_get.return_value = self.port
         with task_manager.acquire(self.context, self.node.id) as task:
             self.interface.vif_detach(task, 'fake_vif_id')
-        mock_get.assert_called_once_with(task, 'fake_vif_id')
+        mock_get.assert_called_once_with(self.interface, task, 'fake_vif_id')
         self.assertFalse(mock_unp.called)
         mock_clear.assert_called_once_with(self.port)
 
-    @mock.patch.object(common.VIFPortIDMixin, '_clear_vif_from_port_like_obj')
+    @mock.patch.object(common.VIFPortIDMixin, '_clear_vif_from_port_like_obj',
+                       autospec=True)
     @mock.patch.object(neutron_common, 'unbind_neutron_port', autospec=True)
-    @mock.patch.object(common.VIFPortIDMixin, '_get_port_like_obj_by_vif_id')
+    @mock.patch.object(common.VIFPortIDMixin, '_get_port_like_obj_by_vif_id',
+                       autospec=True)
     def test_vif_detach_portgroup(self, mock_get, mock_unp, mock_clear):
         pg = obj_utils.create_test_portgroup(
             self.context, node_id=self.node.id)
         mock_get.return_value = pg
         with task_manager.acquire(self.context, self.node.id) as task:
             self.interface.vif_detach(task, 'fake_vif_id')
-        mock_get.assert_called_once_with(task, 'fake_vif_id')
+        mock_get.assert_called_once_with(self.interface, task, 'fake_vif_id')
         self.assertFalse(mock_unp.called)
         mock_clear.assert_called_once_with(pg)
 
-    @mock.patch.object(common.VIFPortIDMixin, '_clear_vif_from_port_like_obj')
+    @mock.patch.object(common.VIFPortIDMixin, '_clear_vif_from_port_like_obj',
+                       autospec=True)
     @mock.patch.object(neutron_common, 'unbind_neutron_port', autospec=True)
-    @mock.patch.object(common.VIFPortIDMixin, '_get_port_like_obj_by_vif_id')
+    @mock.patch.object(common.VIFPortIDMixin, '_get_port_like_obj_by_vif_id',
+                       autospec=True)
     def test_vif_detach_not_attached(self, mock_get, mock_unp, mock_clear):
         mock_get.side_effect = exception.VifNotAttached(vif='fake-vif',
                                                         node='fake-node')
@@ -906,13 +1013,15 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             self.assertRaisesRegex(
                 exception.VifNotAttached, "it is not attached to it.",
                 self.interface.vif_detach, task, 'fake_vif_id')
-        mock_get.assert_called_once_with(task, 'fake_vif_id')
+        mock_get.assert_called_once_with(self.interface, task, 'fake_vif_id')
         self.assertFalse(mock_unp.called)
         self.assertFalse(mock_clear.called)
 
-    @mock.patch.object(common.VIFPortIDMixin, '_clear_vif_from_port_like_obj')
+    @mock.patch.object(common.VIFPortIDMixin, '_clear_vif_from_port_like_obj',
+                       autospec=True)
     @mock.patch.object(neutron_common, 'unbind_neutron_port', autospec=True)
-    @mock.patch.object(common.VIFPortIDMixin, '_get_port_like_obj_by_vif_id')
+    @mock.patch.object(common.VIFPortIDMixin, '_get_port_like_obj_by_vif_id',
+                       autospec=True)
     def test_vif_detach_active_node(self, mock_get, mock_unp, mock_clear):
         self.node.provision_state = states.ACTIVE
         self.node.save()
@@ -921,12 +1030,14 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             self.interface.vif_detach(task, 'fake_vif_id')
             mock_unp.assert_called_once_with('fake_vif_id',
                                              context=task.context)
-        mock_get.assert_called_once_with(task, 'fake_vif_id')
+        mock_get.assert_called_once_with(self.interface, task, 'fake_vif_id')
         mock_clear.assert_called_once_with(self.port)
 
-    @mock.patch.object(common.VIFPortIDMixin, '_clear_vif_from_port_like_obj')
+    @mock.patch.object(common.VIFPortIDMixin, '_clear_vif_from_port_like_obj',
+                       autospec=True)
     @mock.patch.object(neutron_common, 'unbind_neutron_port', autospec=True)
-    @mock.patch.object(common.VIFPortIDMixin, '_get_port_like_obj_by_vif_id')
+    @mock.patch.object(common.VIFPortIDMixin, '_get_port_like_obj_by_vif_id',
+                       autospec=True)
     def test_vif_detach_deleting_node(self, mock_get, mock_unp, mock_clear):
         self.node.provision_state = states.DELETING
         self.node.save()
@@ -935,12 +1046,14 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             self.interface.vif_detach(task, 'fake_vif_id')
             mock_unp.assert_called_once_with('fake_vif_id',
                                              context=task.context)
-        mock_get.assert_called_once_with(task, 'fake_vif_id')
+        mock_get.assert_called_once_with(self.interface, task, 'fake_vif_id')
         mock_clear.assert_called_once_with(self.port)
 
-    @mock.patch.object(common.VIFPortIDMixin, '_clear_vif_from_port_like_obj')
+    @mock.patch.object(common.VIFPortIDMixin, '_clear_vif_from_port_like_obj',
+                       autospec=True)
     @mock.patch.object(neutron_common, 'unbind_neutron_port', autospec=True)
-    @mock.patch.object(common.VIFPortIDMixin, '_get_port_like_obj_by_vif_id')
+    @mock.patch.object(common.VIFPortIDMixin, '_get_port_like_obj_by_vif_id',
+                       autospec=True)
     def test_vif_detach_active_node_failure(self, mock_get, mock_unp,
                                             mock_clear):
         self.node.provision_state = states.ACTIVE
@@ -952,7 +1065,7 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
                               self.interface.vif_detach, task, 'fake_vif_id')
             mock_unp.assert_called_once_with('fake_vif_id',
                                              context=task.context)
-        mock_get.assert_called_once_with(task, 'fake_vif_id')
+        mock_get.assert_called_once_with(self.interface, task, 'fake_vif_id')
         mock_clear.assert_called_once_with(self.port)
 
     @mock.patch.object(neutron_common, 'update_port_address', autospec=True)
@@ -962,7 +1075,8 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
         with task_manager.acquire(self.context, self.node.id) as task:
             self.interface.port_changed(task, self.port)
             mac_update_mock.assert_called_once_with(
-                self.port.extra['vif_port_id'], new_address,
+                self.port.extra['vif_port_id'],
+                new_address,
                 context=task.context)
 
     @mock.patch.object(neutron_common, 'update_port_address', autospec=True)
@@ -988,7 +1102,8 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             self.interface.port_changed(task, self.port)
             self.assertFalse(mac_update_mock.called)
 
-    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts')
+    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts',
+                autospec=True)
     def test_port_changed_client_id(self, dhcp_update_mock):
         expected_extra = {'vif_port_id': 'fake-id', 'client-id': 'fake2'}
         expected_dhcp_opts = [{'opt_name': '61', 'opt_value': 'fake2'}]
@@ -996,9 +1111,10 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
         with task_manager.acquire(self.context, self.node.id) as task:
             self.interface.port_changed(task, self.port)
             dhcp_update_mock.assert_called_once_with(
-                'fake-id', expected_dhcp_opts, context=task.context)
+                mock.ANY, 'fake-id', expected_dhcp_opts, context=task.context)
 
-    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts')
+    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts',
+                autospec=True)
     def test_port_changed_extra_add_new_key(self, dhcp_update_mock):
         self.port.extra = {'vif_port_id': 'fake-id'}
         self.port.save()
@@ -1009,7 +1125,8 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             self.interface.port_changed(task, self.port)
             self.assertFalse(dhcp_update_mock.called)
 
-    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts')
+    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts',
+                autospec=True)
     def test_port_changed_client_id_fail(self, dhcp_update_mock):
         self.port.extra = {'vif_port_id': 'fake-id', 'client-id': 'fake2'}
         dhcp_update_mock.side_effect = (
@@ -1019,7 +1136,8 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
                               self.interface.port_changed,
                               task, self.port)
 
-    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts')
+    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts',
+                autospec=True)
     def test_port_changed_client_id_no_vif_id(self, dhcp_update_mock):
         self.port.extra = {'client-id': 'fake1'}
         self.port.save()
@@ -1028,7 +1146,8 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             self.interface.port_changed(task, self.port)
             self.assertFalse(dhcp_update_mock.called)
 
-    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts')
+    @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts',
+                autospec=True)
     def test_port_changed_message_format_failure(self, dhcp_update_mock):
         pg = obj_utils.create_test_portgroup(
             self.context, node_id=self.node.id,
