@@ -18,18 +18,21 @@
 """Test class for console_utils driver module."""
 
 import errno
+import fcntl
+import ipaddress
 import os
 import random
 import signal
+import socket
 import string
 import subprocess
 import tempfile
+import time
+from unittest import mock
 
 from ironic_lib import utils as ironic_utils
-import mock
 from oslo_config import cfg
 from oslo_service import loopingcall
-from oslo_utils import netutils
 import psutil
 
 from ironic.common import exception
@@ -145,8 +148,28 @@ class ConsoleUtilsTestCase(db_base.DbTestCase):
         console_utils._stop_console(self.info['uuid'])
 
         mock_pid.assert_called_once_with(self.info['uuid'])
-        mock_kill.assert_called_once_with(mock_pid.return_value,
-                                          signal.SIGTERM)
+
+        # a check if process still exist (signal 0) in a loop
+        mock_kill.assert_any_call(mock_pid.return_value, signal.SIG_DFL)
+        # and that it receives the SIGTERM
+        mock_kill.assert_any_call(mock_pid.return_value, signal.SIGTERM)
+        mock_unlink.assert_called_once_with(pid_file)
+
+    @mock.patch.object(ironic_utils, 'unlink_without_raise', autospec=True)
+    @mock.patch.object(os, 'kill', autospec=True)
+    @mock.patch.object(psutil, 'pid_exists', autospec=True, return_value=True)
+    @mock.patch.object(console_utils, '_get_console_pid', autospec=True)
+    def test__stop_console_forced_kill(self, mock_pid, mock_psutil, mock_kill,
+                                       mock_unlink):
+        pid_file = console_utils._get_console_pid_file(self.info['uuid'])
+        mock_pid.return_value = 12345
+
+        console_utils._stop_console(self.info['uuid'])
+
+        mock_pid.assert_called_once_with(self.info['uuid'])
+
+        # Make sure console process receives hard SIGKILL
+        mock_kill.assert_any_call(mock_pid.return_value, signal.SIGKILL)
         mock_unlink.assert_called_once_with(pid_file)
 
     @mock.patch.object(ironic_utils, 'unlink_without_raise', autospec=True)
@@ -201,7 +224,7 @@ class ConsoleUtilsTestCase(db_base.DbTestCase):
         generated_url = (
             console_utils.get_shellinabox_console_url(self.info['port']))
         console_host = CONF.my_ip
-        if netutils.is_valid_ipv6(console_host):
+        if ipaddress.ip_address(console_host).version == 6:
             console_host = '[%s]' % console_host
         http_url = "%s://%s:%s" % (scheme, console_host, self.info['port'])
         self.assertEqual(http_url, generated_url)
@@ -242,10 +265,11 @@ class ConsoleUtilsTestCase(db_base.DbTestCase):
                           filepath,
                           'password')
 
+    @mock.patch.object(fcntl, 'fcntl', autospec=True)
     @mock.patch.object(console_utils, 'open',
                        mock.mock_open(read_data='12345\n'))
     @mock.patch.object(os.path, 'exists', autospec=True)
-    @mock.patch.object(subprocess, 'Popen', autospec=True)
+    @mock.patch.object(subprocess, 'Popen')
     @mock.patch.object(psutil, 'pid_exists', autospec=True)
     @mock.patch.object(console_utils, '_ensure_console_pid_dir_exists',
                        autospec=True)
@@ -254,8 +278,10 @@ class ConsoleUtilsTestCase(db_base.DbTestCase):
                                        mock_dir_exists,
                                        mock_pid_exists,
                                        mock_popen,
-                                       mock_path_exists):
+                                       mock_path_exists, mock_fcntl):
         mock_popen.return_value.poll.return_value = 0
+        mock_popen.return_value.stdout.return_value.fileno.return_value = 0
+        mock_popen.return_value.stderr.return_value.fileno.return_value = 1
         mock_pid_exists.return_value = True
         mock_path_exists.return_value = True
 
@@ -271,10 +297,11 @@ class ConsoleUtilsTestCase(db_base.DbTestCase):
                                            stderr=subprocess.PIPE)
         mock_popen.return_value.poll.assert_called_once_with()
 
+    @mock.patch.object(fcntl, 'fcntl', autospec=True)
     @mock.patch.object(console_utils, 'open',
                        mock.mock_open(read_data='12345\n'))
     @mock.patch.object(os.path, 'exists', autospec=True)
-    @mock.patch.object(subprocess, 'Popen', autospec=True)
+    @mock.patch.object(subprocess, 'Popen')
     @mock.patch.object(psutil, 'pid_exists', autospec=True)
     @mock.patch.object(console_utils, '_ensure_console_pid_dir_exists',
                        autospec=True)
@@ -283,10 +310,12 @@ class ConsoleUtilsTestCase(db_base.DbTestCase):
                                              mock_dir_exists,
                                              mock_pid_exists,
                                              mock_popen,
-                                             mock_path_exists):
+                                             mock_path_exists, mock_fcntl):
         # no existing PID file before starting
         mock_stop.side_effect = exception.NoConsolePid('/tmp/blah')
         mock_popen.return_value.poll.return_value = 0
+        mock_popen.return_value.stdout.return_value.fileno.return_value = 0
+        mock_popen.return_value.stderr.return_value.fileno.return_value = 1
         mock_pid_exists.return_value = True
         mock_path_exists.return_value = True
 
@@ -302,44 +331,90 @@ class ConsoleUtilsTestCase(db_base.DbTestCase):
                                            stderr=subprocess.PIPE)
         mock_popen.return_value.poll.assert_called_once_with()
 
-    @mock.patch.object(subprocess, 'Popen', autospec=True)
+    @mock.patch.object(time, 'sleep', autospec=True)
+    @mock.patch.object(os, 'read', autospec=True)
+    @mock.patch.object(fcntl, 'fcntl', autospec=True)
+    @mock.patch.object(subprocess, 'Popen')
     @mock.patch.object(console_utils, '_ensure_console_pid_dir_exists',
                        autospec=True)
     @mock.patch.object(console_utils, '_stop_console', autospec=True)
-    def test_start_shellinabox_console_fail(self, mock_stop, mock_dir_exists,
-                                            mock_popen):
+    def test_start_shellinabox_console_fail(
+            self, mock_stop, mock_dir_exists, mock_popen, mock_fcntl,
+            mock_os_read, mock_sleep):
         mock_popen.return_value.poll.return_value = 1
-        mock_popen.return_value.communicate.return_value = ('output', 'error')
+        stdout = mock_popen.return_value.stdout
+        stderr = mock_popen.return_value.stderr
+        stdout.return_value.fileno.return_value = 0
+        stderr.return_value.fileno.return_value = 1
+        err_output = b'error output'
+        mock_os_read.side_effect = [err_output] * 2 + [OSError] * 2
+        mock_fcntl.side_effect = [1, mock.Mock()] * 2
 
-        self.assertRaises(exception.ConsoleSubprocessFailed,
-                          console_utils.start_shellinabox_console,
-                          self.info['uuid'],
-                          self.info['port'],
-                          'ls&')
+        self.assertRaisesRegex(
+            exception.ConsoleSubprocessFailed, "Stdout: %r" % err_output,
+            console_utils.start_shellinabox_console, self.info['uuid'],
+            self.info['port'], 'ls&')
+
+        mock_stop.assert_called_once_with(self.info['uuid'])
+        mock_sleep.assert_has_calls([mock.call(1), mock.call(1)])
+        mock_dir_exists.assert_called_once_with()
+        for obj in (stdout, stderr):
+            mock_fcntl.assert_has_calls([
+                mock.call(obj, fcntl.F_GETFL),
+                mock.call(obj, fcntl.F_SETFL, 1 | os.O_NONBLOCK)])
+        mock_popen.assert_called_once_with(mock.ANY,
+                                           stdout=subprocess.PIPE,
+                                           stderr=subprocess.PIPE)
+        mock_popen.return_value.poll.assert_called_with()
+
+    @mock.patch.object(fcntl, 'fcntl', autospec=True)
+    @mock.patch.object(subprocess, 'Popen')
+    @mock.patch.object(console_utils, '_ensure_console_pid_dir_exists',
+                       autospec=True)
+    @mock.patch.object(console_utils, '_stop_console', autospec=True)
+    def test_start_shellinabox_console_timeout(
+            self, mock_stop, mock_dir_exists, mock_popen, mock_fcntl):
+        self.config(subprocess_timeout=0, group='console')
+        self.config(subprocess_checking_interval=0, group='console')
+        mock_popen.return_value.poll.return_value = None
+        mock_popen.return_value.stdout.return_value.fileno.return_value = 0
+        mock_popen.return_value.stderr.return_value.fileno.return_value = 1
+
+        self.assertRaisesRegex(
+            exception.ConsoleSubprocessFailed, 'Timeout or error',
+            console_utils.start_shellinabox_console, self.info['uuid'],
+            self.info['port'], 'ls&')
 
         mock_stop.assert_called_once_with(self.info['uuid'])
         mock_dir_exists.assert_called_once_with()
         mock_popen.assert_called_once_with(mock.ANY,
                                            stdout=subprocess.PIPE,
                                            stderr=subprocess.PIPE)
-        mock_popen.return_value.poll.assert_called_once_with()
+        mock_popen.return_value.poll.assert_called_with()
+        self.assertEqual(0, mock_popen.return_value.communicate.call_count)
 
+    @mock.patch.object(time, 'sleep', autospec=True)
+    @mock.patch.object(os, 'read', autospec=True)
+    @mock.patch.object(fcntl, 'fcntl', autospec=True)
     @mock.patch.object(console_utils, 'open',
                        mock.mock_open(read_data='12345\n'))
     @mock.patch.object(os.path, 'exists', autospec=True)
-    @mock.patch.object(subprocess, 'Popen', autospec=True)
+    @mock.patch.object(subprocess, 'Popen')
     @mock.patch.object(psutil, 'pid_exists', autospec=True)
     @mock.patch.object(console_utils, '_ensure_console_pid_dir_exists',
                        autospec=True)
     @mock.patch.object(console_utils, '_stop_console', autospec=True)
-    def test_start_shellinabox_console_fail_no_pid(self, mock_stop,
-                                                   mock_dir_exists,
-                                                   mock_pid_exists,
-                                                   mock_popen,
-                                                   mock_path_exists):
+    def test_start_shellinabox_console_fail_no_pid(
+            self, mock_stop, mock_dir_exists, mock_pid_exists, mock_popen,
+            mock_path_exists, mock_fcntl, mock_os_read, mock_sleep):
         mock_popen.return_value.poll.return_value = 0
+        stdout = mock_popen.return_value.stdout
+        stderr = mock_popen.return_value.stderr
+        stdout.return_value.fileno.return_value = 0
+        stderr.return_value.fileno.return_value = 1
         mock_pid_exists.return_value = False
-        mock_popen.return_value.communicate.return_value = ('output', 'error')
+        mock_os_read.side_effect = [b'error output'] * 2 + [OSError] * 2
+        mock_fcntl.side_effect = [1, mock.Mock()] * 2
         mock_path_exists.return_value = True
 
         self.assertRaises(exception.ConsoleSubprocessFailed,
@@ -349,12 +424,17 @@ class ConsoleUtilsTestCase(db_base.DbTestCase):
                           'ls&')
 
         mock_stop.assert_called_once_with(self.info['uuid'])
+        mock_sleep.assert_has_calls([mock.call(1), mock.call(1)])
         mock_dir_exists.assert_called_once_with()
-        mock_pid_exists.assert_called_once_with(12345)
+        for obj in (stdout, stderr):
+            mock_fcntl.assert_has_calls([
+                mock.call(obj, fcntl.F_GETFL),
+                mock.call(obj, fcntl.F_SETFL, 1 | os.O_NONBLOCK)])
+        mock_pid_exists.assert_called_with(12345)
         mock_popen.assert_called_once_with(mock.ANY,
                                            stdout=subprocess.PIPE,
                                            stderr=subprocess.PIPE)
-        mock_popen.return_value.poll.assert_called_once_with()
+        mock_popen.return_value.poll.assert_called_with()
 
     @mock.patch.object(subprocess, 'Popen', autospec=True)
     @mock.patch.object(console_utils, '_ensure_console_pid_dir_exists',
@@ -447,13 +527,15 @@ class ConsoleUtilsTestCase(db_base.DbTestCase):
     def test_start_socat_console_check_arg_bind_addr_default_ipv4(self):
         self.config(my_ip='10.0.0.1')
         args = self._test_start_socat_console_check_arg()
-        self.assertIn('TCP4-LISTEN:%s,bind=10.0.0.1,reuseaddr' %
+        self.assertIn('TCP4-LISTEN:%s,bind=10.0.0.1,reuseaddr,fork,'
+                      'max-children=1' %
                       self.info['port'], args)
 
     def test_start_socat_console_check_arg_bind_addr_ipv4(self):
         self.config(socat_address='10.0.0.1', group='console')
         args = self._test_start_socat_console_check_arg()
-        self.assertIn('TCP4-LISTEN:%s,bind=10.0.0.1,reuseaddr' %
+        self.assertIn('TCP4-LISTEN:%s,bind=10.0.0.1,reuseaddr,fork,'
+                      'max-children=1' %
                       self.info['port'], args)
 
     @mock.patch.object(os.path, 'exists', autospec=True)
@@ -571,3 +653,80 @@ class ConsoleUtilsTestCase(db_base.DbTestCase):
         mock_stop.assert_called_once_with(self.info['uuid'])
         # LOG.warning() is called when _stop_console() raises NoConsolePid
         self.assertTrue(mock_log_warning.called)
+
+    def test_valid_console_port_range(self):
+        self.config(port_range='10000:20000', group='console')
+        start, stop = console_utils._get_port_range()
+        self.assertEqual((start, stop), (10000, 20000))
+
+    def test_invalid_console_port_range(self):
+        self.config(port_range='20000:10000', group='console')
+        self.assertRaises(exception.InvalidParameterValue,
+                          console_utils._get_port_range)
+
+    @mock.patch.object(console_utils, 'ALLOCATED_PORTS', autospec=True)
+    @mock.patch.object(console_utils, '_verify_port', autospec=True)
+    def test_allocate_port_success(self, mock_verify, mock_ports):
+        self.config(port_range='10000:10001', group='console')
+        port = console_utils.acquire_port()
+        mock_verify.assert_called_once_with(10000, host=None)
+        self.assertEqual(port, 10000)
+        mock_ports.add.assert_called_once_with(10000)
+
+    @mock.patch.object(console_utils, 'ALLOCATED_PORTS', autospec=True)
+    @mock.patch.object(console_utils, '_verify_port', autospec=True)
+    def test_allocate_port_range_retry(self, mock_verify, mock_ports):
+        self.config(port_range='10000:10003', group='console')
+        mock_verify.side_effect = (exception.Conflict, exception.Conflict,
+                                   None)
+        port = console_utils.acquire_port()
+        verify_calls = [mock.call(10000, host=None),
+                        mock.call(10001, host=None),
+                        mock.call(10002, host=None)]
+        mock_verify.assert_has_calls(verify_calls)
+        self.assertEqual(port, 10002)
+        mock_ports.add.assert_called_once_with(10002)
+
+    @mock.patch.object(console_utils, 'ALLOCATED_PORTS', autospec=True)
+    @mock.patch.object(console_utils, '_verify_port', autospec=True)
+    def test_allocate_port_no_free_ports(self, mock_verify, mock_ports):
+        self.config(port_range='10000:10005', group='console')
+        mock_verify.side_effect = exception.Conflict
+        self.assertRaises(exception.NoFreeIPMITerminalPorts,
+                          console_utils.acquire_port)
+        verify_calls = [mock.call(p, host=None) for p in range(10000, 10005)]
+        mock_verify.assert_has_calls(verify_calls)
+
+    @mock.patch.object(socket, 'socket', autospec=True)
+    def test__verify_port_default(self, mock_socket):
+        self.config(host='localhost.localdomain')
+        mock_sock = mock.MagicMock()
+        mock_socket.return_value = mock_sock
+        console_utils._verify_port(10000)
+        mock_sock.bind.assert_called_once_with(('localhost.localdomain',
+                                                10000))
+
+    @mock.patch.object(socket, 'socket', autospec=True)
+    def test__verify_port_hostname(self, mock_socket):
+        mock_sock = mock.MagicMock()
+        mock_socket.return_value = mock_sock
+        console_utils._verify_port(10000, host='localhost.localdomain')
+        mock_socket.assert_called_once_with()
+        mock_sock.bind.assert_called_once_with(('localhost.localdomain',
+                                                10000))
+
+    @mock.patch.object(socket, 'socket', autospec=True)
+    def test__verify_port_ipv4(self, mock_socket):
+        mock_sock = mock.MagicMock()
+        mock_socket.return_value = mock_sock
+        console_utils._verify_port(10000, host='1.2.3.4')
+        mock_socket.assert_called_once_with()
+        mock_sock.bind.assert_called_once_with(('1.2.3.4', 10000))
+
+    @mock.patch.object(socket, 'socket', autospec=True)
+    def test__verify_port_ipv6(self, mock_socket):
+        mock_sock = mock.MagicMock()
+        mock_socket.return_value = mock_sock
+        console_utils._verify_port(10000, host='2001:dead:beef::1')
+        mock_socket.assert_called_once_with(socket.AF_INET6)
+        mock_sock.bind.assert_called_once_with(('2001:dead:beef::1', 10000))
