@@ -14,11 +14,13 @@
 #    under the License.
 
 import json
+from unittest import mock
 
-import mock
-
+from ironic.common import components
 from ironic.common import exception
+from ironic.common import indicator_states
 from ironic.common import raid
+from ironic.common import states
 from ironic.drivers import base as driver_base
 from ironic.drivers.modules import fake
 from ironic.tests import base
@@ -100,16 +102,6 @@ class PassthruDecoratorTestCase(base.TestCase):
         self.assertNotEqual(inst1.driver_routes['driver_noexception']['func'],
                             inst2.driver_routes['driver_noexception']['func'])
 
-    @mock.patch.object(driver_base.LOG, 'warning')
-    def test_old_async_warning(self, mock_log_warning):
-        driver_base.passthru(['POST'], async=True)
-        mock_log_warning.assert_called_once()
-
-    @mock.patch.object(driver_base.LOG, 'warning')
-    def test_new_async_call_without_warning(self, mock_log_warning):
-        driver_base.passthru(['POST'], async_call=True)
-        mock_log_warning.assert_not_called()
-
 
 class CleanStepDecoratorTestCase(base.TestCase):
 
@@ -354,6 +346,111 @@ class CleanStepTestCase(base.TestCase):
         method_args_mock.assert_called_once_with(task_mock, **args)
 
 
+class DeployStepDecoratorTestCase(base.TestCase):
+
+    def setUp(self):
+        super(DeployStepDecoratorTestCase, self).setUp()
+        method_mock = mock.MagicMock()
+        del method_mock._is_deploy_step
+        del method_mock._deploy_step_priority
+        del method_mock._deploy_step_argsinfo
+        self.method = method_mock
+
+    def test_deploy_step_priority_only(self):
+        d = driver_base.deploy_step(priority=10)
+        d(self.method)
+        self.assertTrue(self.method._is_deploy_step)
+        self.assertEqual(10, self.method._deploy_step_priority)
+        self.assertIsNone(self.method._deploy_step_argsinfo)
+
+    def test_deploy_step_all_args(self):
+        argsinfo = {'arg1': {'description': 'desc1',
+                             'required': True}}
+        d = driver_base.deploy_step(priority=0, argsinfo=argsinfo)
+        d(self.method)
+        self.assertTrue(self.method._is_deploy_step)
+        self.assertEqual(0, self.method._deploy_step_priority)
+        self.assertEqual(argsinfo, self.method._deploy_step_argsinfo)
+
+    def test_deploy_step_bad_priority(self):
+        d = driver_base.deploy_step(priority='hi')
+        self.assertRaisesRegex(exception.InvalidParameterValue, 'priority',
+                               d, self.method)
+        self.assertTrue(self.method._is_deploy_step)
+        self.assertFalse(hasattr(self.method, '_deploy_step_priority'))
+        self.assertFalse(hasattr(self.method, '_deploy_step_argsinfo'))
+
+    @mock.patch.object(driver_base, '_validate_argsinfo', spec_set=True,
+                       autospec=True)
+    def test_deploy_step_bad_argsinfo(self, mock_valid):
+        mock_valid.side_effect = exception.InvalidParameterValue('bad')
+        d = driver_base.deploy_step(priority=0, argsinfo=100)
+        self.assertRaises(exception.InvalidParameterValue, d, self.method)
+        self.assertTrue(self.method._is_deploy_step)
+        self.assertEqual(0, self.method._deploy_step_priority)
+        self.assertFalse(hasattr(self.method, '_deploy_step_argsinfo'))
+
+
+class DeployAndCleanStepDecoratorTestCase(base.TestCase):
+
+    def setUp(self):
+        super(DeployAndCleanStepDecoratorTestCase, self).setUp()
+        method_mock = mock.MagicMock()
+        del method_mock._is_deploy_step
+        del method_mock._deploy_step_priority
+        del method_mock._deploy_step_argsinfo
+        del method_mock._is_clean_step
+        del method_mock._clean_step_priority
+        del method_mock._clean_step_abortable
+        del method_mock._clean_step_argsinfo
+        self.method = method_mock
+
+    def test_deploy_and_clean_step_priority_only(self):
+        dd = driver_base.deploy_step(priority=10)
+        dc = driver_base.clean_step(priority=11)
+        dd(dc(self.method))
+        self.assertTrue(self.method._is_deploy_step)
+        self.assertEqual(10, self.method._deploy_step_priority)
+        self.assertIsNone(self.method._deploy_step_argsinfo)
+        self.assertTrue(self.method._is_clean_step)
+        self.assertEqual(11, self.method._clean_step_priority)
+        self.assertFalse(self.method._clean_step_abortable)
+        self.assertIsNone(self.method._clean_step_argsinfo)
+
+    def test_deploy_and_clean_step_all_args(self):
+        dargsinfo = {'arg1': {'description': 'desc1',
+                              'required': True}}
+        cargsinfo = {'arg2': {'description': 'desc2',
+                              'required': False}}
+        dd = driver_base.deploy_step(priority=0, argsinfo=dargsinfo)
+        dc = driver_base.clean_step(priority=0, argsinfo=cargsinfo)
+        dd(dc(self.method))
+        self.assertTrue(self.method._is_deploy_step)
+        self.assertEqual(0, self.method._deploy_step_priority)
+        self.assertEqual(dargsinfo, self.method._deploy_step_argsinfo)
+        self.assertTrue(self.method._is_clean_step)
+        self.assertEqual(0, self.method._clean_step_priority)
+        self.assertFalse(self.method._clean_step_abortable)
+        self.assertEqual(cargsinfo, self.method._clean_step_argsinfo)
+
+    def test_clean_and_deploy_step_all_args(self):
+        # Opposite ordering, should make no difference.
+        dargsinfo = {'arg1': {'description': 'desc1',
+                              'required': True}}
+        cargsinfo = {'arg2': {'description': 'desc2',
+                              'required': False}}
+        dd = driver_base.deploy_step(priority=0, argsinfo=dargsinfo)
+        dc = driver_base.clean_step(priority=0, argsinfo=cargsinfo)
+        dc(dd(self.method))
+        self.assertTrue(self.method._is_deploy_step)
+        self.assertEqual(0, self.method._deploy_step_priority)
+        self.assertEqual(dargsinfo, self.method._deploy_step_argsinfo)
+        self.assertTrue(self.method._is_clean_step)
+        self.assertEqual(0, self.method._clean_step_priority)
+        self.assertFalse(self.method._clean_step_abortable)
+        self.assertEqual(cargsinfo, self.method._clean_step_argsinfo)
+
+
 class DeployStepTestCase(base.TestCase):
     def test_get_and_execute_deploy_steps(self):
         # Create a fake Driver class, create some deploy steps, make sure
@@ -477,7 +574,10 @@ class DeployStepTestCase(base.TestCase):
 
 class MyRAIDInterface(driver_base.RAIDInterface):
 
-    def create_configuration(self, task):
+    def create_configuration(self, task,
+                             create_root_volume=True,
+                             create_nonroot_volumes=True,
+                             delete_existing=True):
         pass
 
     def delete_configuration(self, task):
@@ -529,6 +629,68 @@ class RAIDInterfaceTestCase(base.TestCase):
         raid_interface.get_logical_disk_properties()
         get_properties_mock.assert_called_once_with(raid_schema)
 
+    @mock.patch.object(MyRAIDInterface, 'create_configuration', autospec=True)
+    @mock.patch.object(MyRAIDInterface, 'validate_raid_config',
+                       autospec=True)
+    def test_apply_configuration(self, mock_validate, mock_create):
+        raid_interface = MyRAIDInterface()
+        node_mock = mock.MagicMock(target_raid_config=None)
+        task_mock = mock.MagicMock(node=node_mock)
+        mock_create.return_value = states.DEPLOYWAIT
+        raid_config = 'some_raid_config'
+
+        result = raid_interface.apply_configuration(task_mock, raid_config)
+
+        self.assertEqual(states.DEPLOYWAIT, result)
+        mock_validate.assert_called_once_with(raid_interface, task_mock,
+                                              raid_config)
+        mock_create.assert_called_once_with(raid_interface, task_mock,
+                                            create_root_volume=True,
+                                            create_nonroot_volumes=True,
+                                            delete_existing=True)
+        self.assertEqual(raid_config, node_mock.target_raid_config)
+
+    @mock.patch.object(MyRAIDInterface, 'create_configuration', autospec=True)
+    @mock.patch.object(MyRAIDInterface, 'validate_raid_config',
+                       autospec=True)
+    def test_apply_configuration_delete_existing(self, mock_validate,
+                                                 mock_create):
+        raid_interface = MyRAIDInterface()
+        node_mock = mock.MagicMock(target_raid_config=None)
+        task_mock = mock.MagicMock(node=node_mock)
+        mock_create.return_value = states.DEPLOYWAIT
+        raid_config = 'some_raid_config'
+
+        result = raid_interface.apply_configuration(task_mock, raid_config,
+                                                    delete_existing=True)
+
+        self.assertEqual(states.DEPLOYWAIT, result)
+        mock_validate.assert_called_once_with(raid_interface, task_mock,
+                                              raid_config)
+        mock_create.assert_called_once_with(raid_interface, task_mock,
+                                            create_root_volume=True,
+                                            create_nonroot_volumes=True,
+                                            delete_existing=True)
+        self.assertEqual(raid_config, node_mock.target_raid_config)
+
+    @mock.patch.object(MyRAIDInterface, 'create_configuration', autospec=True)
+    @mock.patch.object(MyRAIDInterface, 'validate_raid_config',
+                       autospec=True)
+    def test_apply_configuration_invalid(self, mock_validate, mock_create):
+        raid_interface = MyRAIDInterface()
+        node_mock = mock.MagicMock(target_raid_config=None)
+        task_mock = mock.MagicMock(node=node_mock)
+        mock_validate.side_effect = exception.InvalidParameterValue('bad')
+        raid_config = 'some_raid_config'
+
+        self.assertRaises(exception.InvalidParameterValue,
+                          raid_interface.apply_configuration, task_mock,
+                          raid_config)
+        mock_validate.assert_called_once_with(raid_interface, task_mock,
+                                              raid_config)
+        self.assertFalse(mock_create.called)
+        self.assertIsNone(node_mock.target_raid_config)
+
 
 class TestDeployInterface(base.TestCase):
     @mock.patch.object(driver_base.LOG, 'warning', autospec=True)
@@ -549,11 +711,13 @@ class MyBIOSInterface(driver_base.BIOSInterface):
     def validate(self, task):
         pass
 
+    @driver_base.cache_bios_settings
     def apply_configuration(self, task, settings):
-        pass
+        return "return_value_apply_configuration"
 
+    @driver_base.cache_bios_settings
     def factory_reset(self, task):
-        pass
+        return "return_value_factory_reset"
 
     def cache_bios_settings(self, task):
         pass
@@ -566,16 +730,18 @@ class TestBIOSInterface(base.TestCase):
         bios = MyBIOSInterface()
         task_mock = mock.MagicMock()
 
-        bios.apply_configuration(task_mock, "")
+        actual = bios.apply_configuration(task_mock, "")
         cache_bios_settings_mock.assert_called_once_with(bios, task_mock)
+        self.assertEqual(actual, "return_value_apply_configuration")
 
     @mock.patch.object(MyBIOSInterface, 'cache_bios_settings', autospec=True)
     def test_factory_reset_wrapper(self, cache_bios_settings_mock):
         bios = MyBIOSInterface()
         task_mock = mock.MagicMock()
 
-        bios.factory_reset(task_mock)
+        actual = bios.factory_reset(task_mock)
         cache_bios_settings_mock.assert_called_once_with(bios, task_mock)
+        self.assertEqual(actual, "return_value_factory_reset")
 
 
 class TestBootInterface(base.TestCase):
@@ -618,6 +784,53 @@ class TestManagementInterface(base.TestCase):
         self.assertRaises(exception.UnsupportedDriverExtension,
                           management.get_boot_mode, task_mock)
 
+    def test_get_supported_indicators_default_impl(self):
+        management = fake.FakeManagement()
+        task_mock = mock.MagicMock(spec_set=['node'])
+
+        expected = {
+            components.CHASSIS: {
+                'led-0': {
+                    "readonly": True,
+                    "states": [
+                        indicator_states.OFF,
+                        indicator_states.ON
+                    ]
+                }
+            },
+            components.SYSTEM: {
+                'led': {
+                    "readonly": False,
+                    "states": [
+                        indicator_states.BLINKING,
+                        indicator_states.OFF,
+                        indicator_states.ON
+                    ]
+                }
+            }
+        }
+
+        self.assertEqual(
+            expected, management.get_supported_indicators(task_mock))
+
+    def test_set_indicator_state_default_impl(self):
+        management = fake.FakeManagement()
+        task_mock = mock.MagicMock(spec_set=['node'])
+
+        self.assertRaises(exception.UnsupportedDriverExtension,
+                          management.set_indicator_state, task_mock,
+                          components.CHASSIS, 'led-0', indicator_states.ON)
+
+    def test_get_indicator_state_default_impl(self):
+        management = fake.FakeManagement()
+        task_mock = mock.MagicMock(spec_set=['node'])
+
+        expected = indicator_states.ON
+
+        self.assertEqual(
+            expected, management.get_indicator_state(
+                task_mock, components.CHASSIS, 'led-0'))
+
 
 class TestBareDriver(base.TestCase):
 
