@@ -30,25 +30,21 @@ DRACCLIENT_CLIENT_MOD_SPEC = (
 DRACCLIENT_CONSTANTS_MOD_SPEC = (
     'POWER_OFF',
     'POWER_ON',
-    'REBOOT'
+    'REBOOT',
+    'RebootRequired',
+    'RaidStatus'
 )
 
-# ironic_inspector
-IRONIC_INSPECTOR_CLIENT_SPEC = (
-    'ClientV1',
+DRACCLIENT_CONSTANTS_REBOOT_REQUIRED_MOD_SPEC = (
+    'true',
+    'optional',
+    'false'
 )
 
-
-class InspectorClientV1Specs(object):
-    def __init__(self, session, inspector_url, api_version):
-        pass
-
-    def introspect(self, uuid):
-        pass
-
-    def get_status(self, uuid):
-        pass
-
+DRACCLIENT_CONSTANTS_RAID_STATUS_MOD_SPEC = (
+    'jbod',
+    'raid'
+)
 
 # proliantutils
 PROLIANTUTILS_SPEC = (
@@ -57,18 +53,6 @@ PROLIANTUTILS_SPEC = (
     'utils',
 )
 
-# pywsman
-PYWSMAN_SPEC = (
-    'Client',
-    'ClientOptions',
-    'EndPointReference',
-    'FLAG_ENUMERATION_OPTIMIZATION',
-    'Filter',
-    'XmlDoc',
-    'wsman_transport_set_verify_host',
-    'wsman_transport_set_verify_peer',
-)
-
 # pywsnmp
 PYWSNMP_SPEC = (
     'hlapi',
@@ -123,33 +107,30 @@ SCCICLIENT_VIOM_CONF_SPEC = (
     'terminate',
 )
 
-HPE_ONEVIEW_SPEC = (
-    'oneview_client',
-    'resources',
-    'exceptions',
-)
-
-HPE_ONEVIEW_CLS_SPEC = (
-)
-
-HPE_ONEVIEW_STATES_SPEC = (
-    'ONEVIEW_POWER_OFF',
-    'ONEVIEW_POWERING_OFF',
-    'ONEVIEW_POWER_ON',
-    'ONEVIEW_POWERING_ON',
-    'ONEVIEW_RESETTING',
-    'ONEVIEW_ERROR',
-)
-
 REDFISH_SPEC = (
     'redfish',
 )
 
-SUSHY_CONSTANTS_SPEC = (
+SUSHY_SPEC = (
+    'auth',
+    'exceptions',
+    'Sushy',
     'BOOT_SOURCE_TARGET_PXE',
     'BOOT_SOURCE_TARGET_HDD',
     'BOOT_SOURCE_TARGET_CD',
     'BOOT_SOURCE_TARGET_BIOS_SETUP',
+    'CHASSIS_INDICATOR_LED_LIT',
+    'CHASSIS_INDICATOR_LED_BLINKING',
+    'CHASSIS_INDICATOR_LED_OFF',
+    'CHASSIS_INDICATOR_LED_UNKNOWN',
+    'DRIVE_INDICATOR_LED_LIT',
+    'DRIVE_INDICATOR_LED_BLINKING',
+    'DRIVE_INDICATOR_LED_OFF',
+    'DRIVE_INDICATOR_LED_UNKNOWN',
+    'INDICATOR_LED_LIT',
+    'INDICATOR_LED_BLINKING',
+    'INDICATOR_LED_OFF',
+    'INDICATOR_LED_UNKNOWN',
     'SYSTEM_POWER_STATE_ON',
     'SYSTEM_POWER_STATE_POWERING_ON',
     'SYSTEM_POWER_STATE_OFF',
@@ -164,6 +145,26 @@ SUSHY_CONSTANTS_SPEC = (
     'BOOT_SOURCE_ENABLED_ONCE',
     'BOOT_SOURCE_MODE_BIOS',
     'BOOT_SOURCE_MODE_UEFI',
+    'PROCESSOR_ARCH_x86',
+    'PROCESSOR_ARCH_IA_64',
+    'PROCESSOR_ARCH_ARM',
+    'PROCESSOR_ARCH_MIPS',
+    'PROCESSOR_ARCH_OEM',
+    'STATE_ENABLED',
+    'STATE_DISABLED',
+    'STATE_ABSENT',
+    'VIRTUAL_MEDIA_CD',
+    'VIRTUAL_MEDIA_FLOPPY',
+    'APPLY_TIME_ON_RESET',
+    'TASK_STATE_COMPLETED',
+    'HEALTH_OK',
+    'HEALTH_WARNING'
+)
+
+SUSHY_AUTH_SPEC = (
+    'BasicAuth',
+    'SessionAuth',
+    'SessionOrBasicAuth',
 )
 
 XCLARITY_SPEC = (
@@ -183,3 +184,10 @@ XCLARITY_STATES_SPEC = (
     'STATE_POWER_OFF',
     'STATE_POWER_ON',
 )
+
+# python-ibmcclient
+IBMCCLIENT_SPEC = (
+    'connect',
+    'exceptions',
+    'constants',
+)
